@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  dir_access.cpp                                                       */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  dir_access.cpp                                                        */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "dir_access.h"
 
@@ -36,61 +36,59 @@
 #include "core/project_settings.h"
 
 String DirAccess::_get_root_path() const {
-
 	switch (_access_type) {
-
-		case ACCESS_RESOURCES: return ProjectSettings::get_singleton()->get_resource_path();
-		case ACCESS_USERDATA: return OS::get_singleton()->get_user_data_dir();
-		default: return "";
+		case ACCESS_RESOURCES:
+			return ProjectSettings::get_singleton()->get_resource_path();
+		case ACCESS_USERDATA:
+			return OS::get_singleton()->get_user_data_dir();
+		default:
+			return "";
 	}
 }
 String DirAccess::_get_root_string() const {
-
 	switch (_access_type) {
-
-		case ACCESS_RESOURCES: return "res://";
-		case ACCESS_USERDATA: return "user://";
-		default: return "";
+		case ACCESS_RESOURCES:
+			return "res://";
+		case ACCESS_USERDATA:
+			return "user://";
+		default:
+			return "";
 	}
 }
 
 int DirAccess::get_current_drive() {
-
 	String path = get_current_dir().to_lower();
 	for (int i = 0; i < get_drive_count(); i++) {
 		String d = get_drive(i).to_lower();
-		if (path.begins_with(d))
+		if (path.begins_with(d)) {
 			return i;
+		}
 	}
 
 	return 0;
 }
 
 bool DirAccess::drives_are_shortcuts() {
-
 	return false;
 }
 
 String DirAccess::get_current_dir_without_drive() {
-
 	return get_current_dir();
 }
 
 static Error _erase_recursive(DirAccess *da) {
-
 	List<String> dirs;
 	List<String> files;
 
 	da->list_dir_begin();
 	String n = da->get_next();
 	while (n != String()) {
-
 		if (n != "." && n != "..") {
-
-			if (da->current_is_dir())
+			if (da->current_is_dir()) {
 				dirs.push_back(n);
-			else
+			} else {
 				files.push_back(n);
+			}
 		}
 
 		n = da->get_next();
@@ -99,10 +97,8 @@ static Error _erase_recursive(DirAccess *da) {
 	da->list_dir_end();
 
 	for (List<String>::Element *E = dirs.front(); E; E = E->next()) {
-
 		Error err = da->change_dir(E->get());
 		if (err == OK) {
-
 			err = _erase_recursive(da);
 			if (err) {
 				da->change_dir("..");
@@ -122,7 +118,6 @@ static Error _erase_recursive(DirAccess *da) {
 	}
 
 	for (List<String>::Element *E = files.front(); E; E = E->next()) {
-
 		Error err = da->remove(da->get_current_dir().plus_file(E->get()));
 		if (err) {
 			return err;
@@ -133,12 +128,10 @@ static Error _erase_recursive(DirAccess *da) {
 }
 
 Error DirAccess::erase_contents_recursive() {
-
 	return _erase_recursive(this);
 }
 
 Error DirAccess::make_dir_recursive(String p_dir) {
-
 	if (p_dir.length() < 1) {
 		return OK;
 	};
@@ -155,17 +148,21 @@ Error DirAccess::make_dir_recursive(String p_dir) {
 
 	full_dir = full_dir.replace("\\", "/");
 
-	//int slices = full_dir.get_slice_count("/");
-
 	String base;
 
-	if (full_dir.begins_with("res://"))
+	if (full_dir.begins_with("res://")) {
 		base = "res://";
-	else if (full_dir.begins_with("user://"))
+	} else if (full_dir.begins_with("user://")) {
 		base = "user://";
-	else if (full_dir.begins_with("/"))
+	} else if (full_dir.is_network_share_path()) {
+		int pos = full_dir.find("/", 2);
+		ERR_FAIL_COND_V(pos < 0, ERR_INVALID_PARAMETER);
+		pos = full_dir.find("/", pos + 1);
+		ERR_FAIL_COND_V(pos < 0, ERR_INVALID_PARAMETER);
+		base = full_dir.substr(0, pos + 1);
+	} else if (full_dir.begins_with("/")) {
 		base = "/";
-	else if (full_dir.find(":/") != -1) {
+	} else if (full_dir.find(":/") != -1) {
 		base = full_dir.substr(0, full_dir.find(":/") + 2);
 	} else {
 		ERR_FAIL_V(ERR_INVALID_PARAMETER);
@@ -177,30 +174,27 @@ Error DirAccess::make_dir_recursive(String p_dir) {
 
 	String curpath = base;
 	for (int i = 0; i < subdirs.size(); i++) {
-
 		curpath = curpath.plus_file(subdirs[i]);
 		Error err = make_dir(curpath);
 		if (err != OK && err != ERR_ALREADY_EXISTS) {
-
-			ERR_FAIL_V(err);
+			ERR_FAIL_V_MSG(err, "Could not create directory: " + curpath);
 		}
 	}
 
 	return OK;
 }
 
+DirAccess::AccessType DirAccess::get_access_type() const {
+	return _access_type;
+}
+
 String DirAccess::fix_path(String p_path) const {
-
 	switch (_access_type) {
-
 		case ACCESS_RESOURCES: {
-
 			if (ProjectSettings::get_singleton()) {
 				if (p_path.begins_with("res://")) {
-
 					String resource_path = ProjectSettings::get_singleton()->get_resource_path();
 					if (resource_path != "") {
-
 						return p_path.replace_first("res:/", resource_path);
 					};
 					return p_path.replace_first("res://", "");
@@ -209,12 +203,9 @@ String DirAccess::fix_path(String p_path) const {
 
 		} break;
 		case ACCESS_USERDATA: {
-
 			if (p_path.begins_with("user://")) {
-
 				String data_dir = OS::get_singleton()->get_user_data_dir();
 				if (data_dir != "") {
-
 					return p_path.replace_first("user:/", data_dir);
 				};
 				return p_path.replace_first("user://", "");
@@ -222,28 +213,24 @@ String DirAccess::fix_path(String p_path) const {
 
 		} break;
 		case ACCESS_FILESYSTEM: {
-
 			return p_path;
 		} break;
-		case ACCESS_MAX: break; // Can't happen, but silences warning
+		case ACCESS_MAX:
+			break; // Can't happen, but silences warning
 	}
 
 	return p_path;
 }
 
-DirAccess::CreateFunc DirAccess::create_func[ACCESS_MAX] = { 0, 0, 0 };
+DirAccess::CreateFunc DirAccess::create_func[ACCESS_MAX] = { nullptr, nullptr, nullptr };
 
 DirAccess *DirAccess::create_for_path(const String &p_path) {
-
-	DirAccess *da = NULL;
+	DirAccess *da = nullptr;
 	if (p_path.begins_with("res://")) {
-
 		da = create(ACCESS_RESOURCES);
 	} else if (p_path.begins_with("user://")) {
-
 		da = create(ACCESS_USERDATA);
 	} else {
-
 		da = create(ACCESS_FILESYSTEM);
 	}
 
@@ -251,36 +238,41 @@ DirAccess *DirAccess::create_for_path(const String &p_path) {
 }
 
 DirAccess *DirAccess::open(const String &p_path, Error *r_error) {
-
 	DirAccess *da = create_for_path(p_path);
 
-	ERR_FAIL_COND_V_MSG(!da, NULL, "Cannot create DirAccess for path '" + p_path + "'.");
+	ERR_FAIL_COND_V_MSG(!da, nullptr, "Cannot create DirAccess for path '" + p_path + "'.");
 	Error err = da->change_dir(p_path);
-	if (r_error)
+	if (r_error) {
 		*r_error = err;
+	}
 	if (err != OK) {
 		memdelete(da);
-		return NULL;
+		return nullptr;
 	}
 
 	return da;
 }
 
 DirAccess *DirAccess::create(AccessType p_access) {
-
-	DirAccess *da = create_func[p_access] ? create_func[p_access]() : NULL;
+	DirAccess *da = create_func[p_access] ? create_func[p_access]() : nullptr;
 	if (da) {
 		da->_access_type = p_access;
+
+		if (p_access == ACCESS_RESOURCES) {
+			da->change_dir("res://");
+		} else if (p_access == ACCESS_USERDATA) {
+			da->change_dir("user://");
+		}
 	}
 
 	return da;
 };
 
 String DirAccess::get_full_path(const String &p_path, AccessType p_access) {
-
 	DirAccess *d = DirAccess::create(p_access);
-	if (!d)
+	if (!d) {
 		return p_path;
+	}
 
 	d->change_dir(p_path);
 	String full = d->get_current_dir();
@@ -289,31 +281,33 @@ String DirAccess::get_full_path(const String &p_path, AccessType p_access) {
 }
 
 Error DirAccess::copy(String p_from, String p_to, int p_chmod_flags) {
-
 	//printf("copy %s -> %s\n",p_from.ascii().get_data(),p_to.ascii().get_data());
 	Error err;
 	FileAccess *fsrc = FileAccess::open(p_from, FileAccess::READ, &err);
 
 	if (err) {
-		ERR_PRINTS("Failed to open " + p_from);
+		ERR_PRINT("Failed to open " + p_from);
 		return err;
 	}
 
 	FileAccess *fdst = FileAccess::open(p_to, FileAccess::WRITE, &err);
 	if (err) {
-
 		fsrc->close();
 		memdelete(fsrc);
-		ERR_PRINTS("Failed to open " + p_to);
+		ERR_PRINT("Failed to open " + p_to);
 		return err;
 	}
 
+	const size_t copy_buffer_limit = 65536; // 64 KB
+
 	fsrc->seek_end(0);
-	int size = fsrc->get_position();
+	uint64_t size = fsrc->get_position();
 	fsrc->seek(0);
 	err = OK;
-	while (size--) {
-
+	size_t buffer_size = MIN(size * sizeof(uint8_t), copy_buffer_limit);
+	LocalVector<uint8_t> buffer;
+	buffer.resize(buffer_size);
+	while (size > 0) {
 		if (fsrc->get_error() != OK) {
 			err = fsrc->get_error();
 			break;
@@ -323,15 +317,23 @@ Error DirAccess::copy(String p_from, String p_to, int p_chmod_flags) {
 			break;
 		}
 
-		fdst->store_8(fsrc->get_8());
+		int bytes_read = fsrc->get_buffer(buffer.ptr(), buffer_size);
+		if (bytes_read <= 0) {
+			err = FAILED;
+			break;
+		}
+		fdst->store_buffer(buffer.ptr(), bytes_read);
+
+		size -= bytes_read;
 	}
 
 	if (err == OK && p_chmod_flags != -1) {
 		fdst->close();
 		err = FileAccess::set_unix_permissions(p_to, p_chmod_flags);
 		// If running on a platform with no chmod support (i.e., Windows), don't fail
-		if (err == ERR_UNAVAILABLE)
+		if (err == ERR_UNAVAILABLE) {
 			err = OK;
+		}
 	}
 
 	memdelete(fsrc);
@@ -358,19 +360,19 @@ public:
 	}
 };
 
-Error DirAccess::_copy_dir(DirAccess *p_target_da, String p_to, int p_chmod_flags) {
+Error DirAccess::_copy_dir(DirAccess *p_target_da, String p_to, int p_chmod_flags, bool p_copy_links) {
 	List<String> dirs;
 
 	String curdir = get_current_dir();
 	list_dir_begin();
 	String n = get_next();
 	while (n != String()) {
-
 		if (n != "." && n != "..") {
-
-			if (current_is_dir())
+			if (p_copy_links && is_link(get_current_dir().plus_file(n))) {
+				create_link(read_link(get_current_dir().plus_file(n)), p_to + n);
+			} else if (current_is_dir()) {
 				dirs.push_back(n);
-			else {
+			} else {
 				const String &rel_path = n;
 				if (!n.is_rel_path()) {
 					list_dir_end();
@@ -400,7 +402,7 @@ Error DirAccess::_copy_dir(DirAccess *p_target_da, String p_to, int p_chmod_flag
 		Error err = change_dir(E->get());
 		ERR_FAIL_COND_V_MSG(err != OK, err, "Cannot change current directory to '" + E->get() + "'.");
 
-		err = _copy_dir(p_target_da, p_to + rel_path + "/", p_chmod_flags);
+		err = _copy_dir(p_target_da, p_to + rel_path + "/", p_chmod_flags, p_copy_links);
 		if (err) {
 			change_dir("..");
 			ERR_FAIL_V_MSG(err, "Failed to copy recursively.");
@@ -412,7 +414,7 @@ Error DirAccess::_copy_dir(DirAccess *p_target_da, String p_to, int p_chmod_flag
 	return OK;
 }
 
-Error DirAccess::copy_dir(String p_from, String p_to, int p_chmod_flags) {
+Error DirAccess::copy_dir(String p_from, String p_to, int p_chmod_flags, bool p_copy_links) {
 	ERR_FAIL_COND_V_MSG(!dir_exists(p_from), ERR_FILE_NOT_FOUND, "Source directory doesn't exist.");
 
 	DirAccess *target_da = DirAccess::create_for_path(p_to);
@@ -431,14 +433,13 @@ Error DirAccess::copy_dir(String p_from, String p_to, int p_chmod_flags) {
 	}
 
 	DirChanger dir_changer(this, p_from);
-	Error err = _copy_dir(target_da, p_to, p_chmod_flags);
+	Error err = _copy_dir(target_da, p_to, p_chmod_flags, p_copy_links);
 	memdelete(target_da);
 
 	return err;
 }
 
 bool DirAccess::exists(String p_dir) {
-
 	DirAccess *da = DirAccess::create_for_path(p_dir);
 	bool valid = da->change_dir(p_dir) == OK;
 	memdelete(da);
@@ -446,7 +447,6 @@ bool DirAccess::exists(String p_dir) {
 }
 
 DirAccess::DirAccess() {
-
 	_access_type = ACCESS_FILESYSTEM;
 }
 
