@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  net_socket_posix.cpp                                                 */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  net_socket_posix.cpp                                                  */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "net_socket_posix.h"
 
@@ -96,7 +96,6 @@
 #endif
 
 size_t NetSocketPosix::_set_addr_storage(struct sockaddr_storage *p_addr, const IP_Address &p_ip, uint16_t p_port, IP::Type p_ip_type) {
-
 	memset(p_addr, 0, sizeof(struct sockaddr_storage));
 	if (p_ip_type == IP::TYPE_IPV6 || p_ip_type == IP::TYPE_ANY) { // IPv6 socket
 
@@ -107,7 +106,7 @@ size_t NetSocketPosix::_set_addr_storage(struct sockaddr_storage *p_addr, const
 		addr6->sin6_family = AF_INET6;
 		addr6->sin6_port = htons(p_port);
 		if (p_ip.is_valid()) {
-			copymem(&addr6->sin6_addr.s6_addr, p_ip.get_ipv6(), 16);
+			memcpy(&addr6->sin6_addr.s6_addr, p_ip.get_ipv6(), 16);
 		} else {
 			addr6->sin6_addr = in6addr_any;
 		}
@@ -122,7 +121,7 @@ size_t NetSocketPosix::_set_addr_storage(struct sockaddr_storage *p_addr, const
 		addr4->sin_port = htons(p_port); // short, network byte order
 
 		if (p_ip.is_valid()) {
-			copymem(&addr4->sin_addr.s_addr, p_ip.get_ipv4(), 4);
+			memcpy(&addr4->sin_addr.s_addr, p_ip.get_ipv4(), 4);
 		} else {
 			addr4->sin_addr.s_addr = INADDR_ANY;
 		}
@@ -132,16 +131,13 @@ size_t NetSocketPosix::_set_addr_storage(struct sockaddr_storage *p_addr, const
 }
 
 void NetSocketPosix::_set_ip_port(struct sockaddr_storage *p_addr, IP_Address &r_ip, uint16_t &r_port) {
-
 	if (p_addr->ss_family == AF_INET) {
-
 		struct sockaddr_in *addr4 = (struct sockaddr_in *)p_addr;
 		r_ip.set_ipv4((uint8_t *)&(addr4->sin_addr.s_addr));
 
 		r_port = ntohs(addr4->sin_port);
 
 	} else if (p_addr->ss_family == AF_INET6) {
-
 		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *)p_addr;
 		r_ip.set_ipv6(addr6->sin6_addr.s6_addr);
 
@@ -202,12 +198,15 @@ NetSocketPosix::NetError NetSocketPosix::_get_socket_error() const {
 	print_verbose("Socket error: " + itos(err));
 	return ERR_NET_OTHER;
 #else
-	if (errno == EISCONN)
+	if (errno == EISCONN) {
 		return ERR_NET_IS_CONNECTED;
-	if (errno == EINPROGRESS || errno == EALREADY)
+	}
+	if (errno == EINPROGRESS || errno == EALREADY) {
 		return ERR_NET_IN_PROGRESS;
-	if (errno == EAGAIN || errno == EWOULDBLOCK)
+	}
+	if (errno == EAGAIN || errno == EWOULDBLOCK) {
 		return ERR_NET_WOULD_BLOCK;
+	}
 	print_verbose("Socket error: " + itos(errno));
 	return ERR_NET_OTHER;
 #endif
@@ -218,7 +217,6 @@ NetSocketPosix::NetError NetSocketPosix::_get_socket_error() const {
 #endif
 
 bool NetSocketPosix::_can_use_ip(const IP_Address &p_ip, const bool p_for_bind) const {
-
 	if (p_for_bind && !(p_ip.is_valid() || p_ip.is_wildcard())) {
 		return false;
 	} else if (!p_for_bind && !p_ip.is_valid()) {
@@ -230,7 +228,6 @@ bool NetSocketPosix::_can_use_ip(const IP_Address &p_ip, const bool p_for_bind)
 }
 
 _FORCE_INLINE_ Error NetSocketPosix::_change_multicast_group(IP_Address p_ip, String p_if_name, bool p_add) {
-
 	ERR_FAIL_COND_V(!is_open(), ERR_UNCONFIGURED);
 	ERR_FAIL_COND_V(!_can_use_ip(p_ip, false), ERR_INVALID_PARAMETER);
 
@@ -246,16 +243,19 @@ _FORCE_INLINE_ Error NetSocketPosix::_change_multicast_group(IP_Address p_ip, St
 	IP::get_singleton()->get_local_interfaces(&if_info);
 	for (Map<String, IP::Interface_Info>::Element *E = if_info.front(); E; E = E->next()) {
 		IP::Interface_Info &c = E->get();
-		if (c.name != p_if_name)
+		if (c.name != p_if_name) {
 			continue;
+		}
 
 		if_v6id = (uint32_t)c.index.to_int64();
-		if (type == IP::TYPE_IPV6)
+		if (type == IP::TYPE_IPV6) {
 			break; // IPv6 uses index.
+		}
 
 		for (List<IP_Address>::Element *F = c.ip_addresses.front(); F; F = F->next()) {
-			if (!F->get().is_ipv4())
+			if (!F->get().is_ipv4()) {
 				continue; // Wrong IP type
+			}
 			if_ip = F->get();
 			break;
 		}
@@ -266,13 +266,13 @@ _FORCE_INLINE_ Error NetSocketPosix::_change_multicast_group(IP_Address p_ip, St
 		ERR_FAIL_COND_V(!if_ip.is_valid(), ERR_INVALID_PARAMETER);
 		struct ip_mreq greq;
 		int sock_opt = p_add ? IP_ADD_MEMBERSHIP : IP_DROP_MEMBERSHIP;
-		copymem(&greq.imr_multiaddr, p_ip.get_ipv4(), 4);
-		copymem(&greq.imr_interface, if_ip.get_ipv4(), 4);
+		memcpy(&greq.imr_multiaddr, p_ip.get_ipv4(), 4);
+		memcpy(&greq.imr_interface, if_ip.get_ipv4(), 4);
 		ret = setsockopt(_sock, level, sock_opt, (const char *)&greq, sizeof(greq));
 	} else {
 		struct ipv6_mreq greq;
 		int sock_opt = p_add ? IPV6_ADD_MEMBERSHIP : IPV6_DROP_MEMBERSHIP;
-		copymem(&greq.ipv6mr_multiaddr, p_ip.get_ipv6(), 16);
+		memcpy(&greq.ipv6mr_multiaddr, p_ip.get_ipv6(), 16);
 		greq.ipv6mr_interface = if_v6id;
 		ret = setsockopt(_sock, level, sock_opt, (const char *)&greq, sizeof(greq));
 	}
@@ -350,11 +350,11 @@ Error NetSocketPosix::open(Type p_sock_type, IP::Type &ip_type) {
 		// recv/recvfrom and an ICMP reply was received from a previous send/sendto.
 		unsigned long disable = 0;
 		if (ioctlsocket(_sock, SIO_UDP_CONNRESET, &disable) == SOCKET_ERROR) {
-			print_verbose("Unable to turn off UDP WSAECONNRESET behaviour on Windows");
+			print_verbose("Unable to turn off UDP WSAECONNRESET behavior on Windows");
 		}
 		if (ioctlsocket(_sock, SIO_UDP_NETRESET, &disable) == SOCKET_ERROR) {
 			// This feature seems not to be supported on wine.
-			print_verbose("Unable to turn off UDP WSAENETRESET behaviour on Windows");
+			print_verbose("Unable to turn off UDP WSAENETRESET behavior on Windows");
 		}
 	}
 #endif
@@ -369,9 +369,9 @@ Error NetSocketPosix::open(Type p_sock_type, IP::Type &ip_type) {
 }
 
 void NetSocketPosix::close() {
-
-	if (_sock != SOCK_EMPTY)
+	if (_sock != SOCK_EMPTY) {
 		SOCK_CLOSE(_sock);
+	}
 
 	_sock = SOCK_EMPTY;
 	_ip_type = IP::TYPE_NONE;
@@ -379,7 +379,6 @@ void NetSocketPosix::close() {
 }
 
 Error NetSocketPosix::bind(IP_Address p_addr, uint16_t p_port) {
-
 	ERR_FAIL_COND_V(!is_open(), ERR_UNCONFIGURED);
 	ERR_FAIL_COND_V(!_can_use_ip(p_addr, true), ERR_INVALID_PARAMETER);
 
@@ -410,7 +409,6 @@ Error NetSocketPosix::listen(int p_max_pending) {
 }
 
 Error NetSocketPosix::connect_to_host(IP_Address p_host, uint16_t p_port) {
-
 	ERR_FAIL_COND_V(!is_open(), ERR_UNCONFIGURED);
 	ERR_FAIL_COND_V(!_can_use_ip(p_host, false), ERR_INVALID_PARAMETER);
 
@@ -418,7 +416,6 @@ Error NetSocketPosix::connect_to_host(IP_Address p_host, uint16_t p_port) {
 	size_t addr_size = _set_addr_storage(&addr, p_host, p_port, _ip_type);
 
 	if (SOCK_CONNECT(_sock, (struct sockaddr *)&addr, addr_size) != 0) {
-
 		NetError err = _get_socket_error();
 
 		switch (err) {
@@ -440,7 +437,6 @@ Error NetSocketPosix::connect_to_host(IP_Address p_host, uint16_t p_port) {
 }
 
 Error NetSocketPosix::poll(PollType p_type, int p_timeout) const {
-
 	ERR_FAIL_COND_V(!is_open(), ERR_UNCONFIGURED);
 
 #if defined(WINDOWS_ENABLED)
@@ -452,7 +448,7 @@ Error NetSocketPosix::poll(PollType p_type, int p_timeout) const {
 	FD_ZERO(&wr);
 	FD_ZERO(&ex);
 	FD_SET(_sock, &ex);
-	struct timeval timeout = { p_timeout, 0 };
+	struct timeval timeout = { p_timeout / 1000, (p_timeout % 1000) * 1000 };
 	// For blocking operation, pass NULL timeout pointer to select.
 	struct timeval *tp = NULL;
 	if (p_timeout >= 0) {
@@ -521,8 +517,9 @@ Error NetSocketPosix::poll(PollType p_type, int p_timeout) const {
 		return FAILED;
 	}
 
-	if (ret == 0)
+	if (ret == 0) {
 		return ERR_BUSY;
+	}
 
 	return OK;
 #endif
@@ -535,8 +532,9 @@ Error NetSocketPosix::recv(uint8_t *p_buffer, int p_len, int &r_read) {
 
 	if (r_read < 0) {
 		NetError err = _get_socket_error();
-		if (err == ERR_NET_WOULD_BLOCK)
+		if (err == ERR_NET_WOULD_BLOCK) {
 			return ERR_BUSY;
+		}
 
 		return FAILED;
 	}
@@ -555,8 +553,9 @@ Error NetSocketPosix::recvfrom(uint8_t *p_buffer, int p_len, int &r_read, IP_Add
 
 	if (r_read < 0) {
 		NetError err = _get_socket_error();
-		if (err == ERR_NET_WOULD_BLOCK)
+		if (err == ERR_NET_WOULD_BLOCK) {
 			return ERR_BUSY;
+		}
 
 		return FAILED;
 	}
@@ -582,15 +581,17 @@ Error NetSocketPosix::send(const uint8_t *p_buffer, int p_len, int &r_sent) {
 
 	int flags = 0;
 #ifdef MSG_NOSIGNAL
-	if (_is_stream)
+	if (_is_stream) {
 		flags = MSG_NOSIGNAL;
+	}
 #endif
 	r_sent = ::send(_sock, SOCK_CBUF(p_buffer), p_len, flags);
 
 	if (r_sent < 0) {
 		NetError err = _get_socket_error();
-		if (err == ERR_NET_WOULD_BLOCK)
+		if (err == ERR_NET_WOULD_BLOCK) {
 			return ERR_BUSY;
+		}
 
 		return FAILED;
 	}
@@ -607,8 +608,9 @@ Error NetSocketPosix::sendto(const uint8_t *p_buffer, int p_len, int &r_sent, IP
 
 	if (r_sent < 0) {
 		NetError err = _get_socket_error();
-		if (err == ERR_NET_WOULD_BLOCK)
+		if (err == ERR_NET_WOULD_BLOCK) {
 			return ERR_BUSY;
+		}
 
 		return FAILED;
 	}
@@ -619,8 +621,9 @@ Error NetSocketPosix::sendto(const uint8_t *p_buffer, int p_len, int &r_sent, IP
 Error NetSocketPosix::set_broadcasting_enabled(bool p_enabled) {
 	ERR_FAIL_COND_V(!is_open(), ERR_UNCONFIGURED);
 	// IPv6 has no broadcast support.
-	if (_ip_type == IP::TYPE_IPV6)
+	if (_ip_type == IP::TYPE_IPV6) {
 		return ERR_UNAVAILABLE;
+	}
 
 	int par = p_enabled ? 1 : 0;
 	if (setsockopt(_sock, SOL_SOCKET, SO_BROADCAST, SOCK_CBUF(&par), sizeof(int)) != 0) {
@@ -639,10 +642,11 @@ void NetSocketPosix::set_blocking_enabled(bool p_enabled) {
 	ret = SOCK_IOCTL(_sock, FIONBIO, &par);
 #else
 	int opts = fcntl(_sock, F_GETFL);
-	if (p_enabled)
+	if (p_enabled) {
 		ret = fcntl(_sock, F_SETFL, opts & ~O_NONBLOCK);
-	else
+	} else {
 		ret = fcntl(_sock, F_SETFL, opts | O_NONBLOCK);
+	}
 #endif
 
 	if (ret != 0)
@@ -701,7 +705,6 @@ bool NetSocketPosix::is_open() const {
 }
 
 int NetSocketPosix::get_available_bytes() const {
-
 	ERR_FAIL_COND_V(!is_open(), -1);
 
 	unsigned long len;
@@ -715,7 +718,6 @@ int NetSocketPosix::get_available_bytes() const {
 }
 
 Ref<NetSocket> NetSocketPosix::accept(IP_Address &r_ip, uint16_t &r_port) {
-
 	Ref<NetSocket> out;
 	ERR_FAIL_COND_V(!is_open(), out);
 
