@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  dependency_editor.cpp                                                */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  dependency_editor.cpp                                                 */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "dependency_editor.h"
 
@@ -37,7 +37,6 @@
 #include "scene/gui/margin_container.h"
 
 void DependencyEditor::_searched(const String &p_path) {
-
 	Map<String, String> dep_rename;
 	dep_rename[replacing] = p_path;
 
@@ -48,12 +47,14 @@ void DependencyEditor::_searched(const String &p_path) {
 }
 
 void DependencyEditor::_load_pressed(Object *p_item, int p_cell, int p_button) {
-
 	TreeItem *ti = Object::cast_to<TreeItem>(p_item);
 	replacing = ti->get_text(1);
 
 	search->set_title(TTR("Search Replacement For:") + " " + replacing.get_file());
 
+	// Set directory to closest existing directory.
+	search->set_current_dir(replacing.get_base_dir());
+
 	search->clear_filters();
 	List<String> ext;
 	ResourceLoader::get_recognized_extensions_for_type(ti->get_metadata(0), &ext);
@@ -63,22 +64,20 @@ void DependencyEditor::_load_pressed(Object *p_item, int p_cell, int p_button) {
 	search->popup_centered_ratio(0.65); // So it doesn't completely cover the dialog below it.
 }
 
-void DependencyEditor::_fix_and_find(EditorFileSystemDirectory *efsd, Map<String, Map<String, String> > &candidates) {
-
+void DependencyEditor::_fix_and_find(EditorFileSystemDirectory *efsd, Map<String, Map<String, String>> &candidates) {
 	for (int i = 0; i < efsd->get_subdir_count(); i++) {
 		_fix_and_find(efsd->get_subdir(i), candidates);
 	}
 
 	for (int i = 0; i < efsd->get_file_count(); i++) {
-
 		String file = efsd->get_file(i);
-		if (!candidates.has(file))
+		if (!candidates.has(file)) {
 			continue;
+		}
 
 		String path = efsd->get_file_path(i);
 
 		for (Map<String, String>::Element *E = candidates[file].front(); E; E = E->next()) {
-
 			if (E->get() == String()) {
 				E->get() = path;
 				continue;
@@ -100,7 +99,6 @@ void DependencyEditor::_fix_and_find(EditorFileSystemDirectory *efsd, Map<String
 			int current_score = 0;
 
 			for (int j = 0; j < lostv.size(); j++) {
-
 				if (j < existingv.size() && lostv[j] == existingv[j]) {
 					existing_score++;
 				}
@@ -110,7 +108,6 @@ void DependencyEditor::_fix_and_find(EditorFileSystemDirectory *efsd, Map<String
 			}
 
 			if (current_score > existing_score) {
-
 				//if it was the same, could track distance to new path but..
 
 				E->get() = path; //replace by more accurate
@@ -120,14 +117,13 @@ void DependencyEditor::_fix_and_find(EditorFileSystemDirectory *efsd, Map<String
 }
 
 void DependencyEditor::_fix_all() {
-
-	if (!EditorFileSystem::get_singleton()->get_filesystem())
+	if (!EditorFileSystem::get_singleton()->get_filesystem()) {
 		return;
+	}
 
-	Map<String, Map<String, String> > candidates;
+	Map<String, Map<String, String>> candidates;
 
 	for (List<String>::Element *E = missing.front(); E; E = E->next()) {
-
 		String base = E->get().get_file();
 		if (!candidates.has(base)) {
 			candidates[base] = Map<String, String>();
@@ -140,10 +136,8 @@ void DependencyEditor::_fix_all() {
 
 	Map<String, String> remaps;
 
-	for (Map<String, Map<String, String> >::Element *E = candidates.front(); E; E = E->next()) {
-
+	for (Map<String, Map<String, String>>::Element *E = candidates.front(); E; E = E->next()) {
 		for (Map<String, String>::Element *F = E->get().front(); F; F = F->next()) {
-
 			if (F->get() != String()) {
 				remaps[F->key()] = F->get();
 			}
@@ -151,7 +145,6 @@ void DependencyEditor::_fix_all() {
 	}
 
 	if (remaps.size()) {
-
 		ResourceLoader::rename_dependencies(editing, remaps);
 
 		_update_list();
@@ -160,12 +153,10 @@ void DependencyEditor::_fix_all() {
 }
 
 void DependencyEditor::_update_file() {
-
 	EditorFileSystem::get_singleton()->update_file(editing);
 }
 
 void DependencyEditor::_update_list() {
-
 	List<String> deps;
 	ResourceLoader::get_dependencies(editing, &deps, true);
 
@@ -179,7 +170,6 @@ void DependencyEditor::_update_list() {
 	bool broken = false;
 
 	for (List<String>::Element *E = deps.front(); E; E = E->next()) {
-
 		TreeItem *item = tree->create_item(root);
 
 		String n = E->get();
@@ -214,7 +204,6 @@ void DependencyEditor::_update_list() {
 }
 
 void DependencyEditor::edit(const String &p_path) {
-
 	editing = p_path;
 	set_title(TTR("Dependencies For:") + " " + p_path.get_file());
 
@@ -229,14 +218,12 @@ void DependencyEditor::edit(const String &p_path) {
 }
 
 void DependencyEditor::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_searched"), &DependencyEditor::_searched);
 	ClassDB::bind_method(D_METHOD("_load_pressed"), &DependencyEditor::_load_pressed);
 	ClassDB::bind_method(D_METHOD("_fix_all"), &DependencyEditor::_fix_all);
 }
 
 DependencyEditor::DependencyEditor() {
-
 	VBoxContainer *vb = memnew(VBoxContainer);
 	vb->set_name(TTR("Dependencies"));
 	add_child(vb);
@@ -275,7 +262,6 @@ DependencyEditor::DependencyEditor() {
 
 /////////////////////////////////////
 void DependencyEditorOwners::_list_rmb_select(int p_item, const Vector2 &p_pos) {
-
 	file_options->clear();
 	file_options->set_size(Size2(1, 1));
 	if (p_item >= 0) {
@@ -287,7 +273,6 @@ void DependencyEditorOwners::_list_rmb_select(int p_item, const Vector2 &p_pos)
 }
 
 void DependencyEditorOwners::_select_file(int p_idx) {
-
 	String fpath = owners->get_item_text(p_idx);
 
 	if (ResourceLoader::get_resource_type(fpath) == "PackedScene") {
@@ -298,35 +283,33 @@ void DependencyEditorOwners::_select_file(int p_idx) {
 }
 
 void DependencyEditorOwners::_file_option(int p_option) {
-
 	switch (p_option) {
 		case FILE_OPEN: {
 			int idx = owners->get_current();
-			if (idx < 0 || idx >= owners->get_item_count())
+			if (idx < 0 || idx >= owners->get_item_count()) {
 				break;
+			}
 			_select_file(idx);
 		} break;
 	}
 }
 
 void DependencyEditorOwners::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_list_rmb_select"), &DependencyEditorOwners::_list_rmb_select);
 	ClassDB::bind_method(D_METHOD("_file_option"), &DependencyEditorOwners::_file_option);
 	ClassDB::bind_method(D_METHOD("_select_file"), &DependencyEditorOwners::_select_file);
 }
 
 void DependencyEditorOwners::_fill_owners(EditorFileSystemDirectory *efsd) {
-
-	if (!efsd)
+	if (!efsd) {
 		return;
+	}
 
 	for (int i = 0; i < efsd->get_subdir_count(); i++) {
 		_fill_owners(efsd->get_subdir(i));
 	}
 
 	for (int i = 0; i < efsd->get_file_count(); i++) {
-
 		Vector<String> deps = efsd->get_file_deps(i);
 		bool found = false;
 		for (int j = 0; j < deps.size(); j++) {
@@ -335,8 +318,9 @@ void DependencyEditorOwners::_fill_owners(EditorFileSystemDirectory *efsd) {
 				break;
 			}
 		}
-		if (!found)
+		if (!found) {
 			continue;
+		}
 
 		Ref<Texture> icon = EditorNode::get_singleton()->get_class_icon(efsd->get_file_type(i));
 
@@ -345,17 +329,15 @@ void DependencyEditorOwners::_fill_owners(EditorFileSystemDirectory *efsd) {
 }
 
 void DependencyEditorOwners::show(const String &p_path) {
-
 	editing = p_path;
 	owners->clear();
 	_fill_owners(EditorFileSystem::get_singleton()->get_filesystem());
 	popup_centered_ratio();
 
-	set_title(TTR("Owners Of:") + " " + p_path.get_file());
+	set_title(vformat(TTR("Owners of: %s (Total: %d)"), p_path.get_file(), owners->get_item_count()));
 }
 
 DependencyEditorOwners::DependencyEditorOwners(EditorNode *p_editor) {
-
 	editor = p_editor;
 
 	file_options = memnew(PopupMenu);
@@ -373,8 +355,9 @@ DependencyEditorOwners::DependencyEditorOwners(EditorNode *p_editor) {
 ///////////////////////
 
 void DependencyRemoveDialog::_find_files_in_removed_folder(EditorFileSystemDirectory *efsd, const String &p_folder) {
-	if (!efsd)
+	if (!efsd) {
 		return;
+	}
 
 	for (int i = 0; i < efsd->get_subdir_count(); ++i) {
 		_find_files_in_removed_folder(efsd->get_subdir(i), p_folder);
@@ -387,8 +370,9 @@ void DependencyRemoveDialog::_find_files_in_removed_folder(EditorFileSystemDirec
 }
 
 void DependencyRemoveDialog::_find_all_removed_dependencies(EditorFileSystemDirectory *efsd, Vector<RemovedDependency> &p_removed) {
-	if (!efsd)
+	if (!efsd) {
 		return;
+	}
 
 	for (int i = 0; i < efsd->get_subdir_count(); i++) {
 		_find_all_removed_dependencies(efsd->get_subdir(i), p_removed);
@@ -398,8 +382,9 @@ void DependencyRemoveDialog::_find_all_removed_dependencies(EditorFileSystemDire
 		const String path = efsd->get_file_path(i);
 
 		//It doesn't matter if a file we are about to delete will have some of its dependencies removed too
-		if (all_remove_files.has(path))
+		if (all_remove_files.has(path)) {
 			continue;
+		}
 
 		Vector<String> all_deps = efsd->get_file_deps(i);
 		for (int j = 0; j < all_deps.size(); ++j) {
@@ -474,20 +459,19 @@ void DependencyRemoveDialog::show(const Vector<String> &p_folders, const Vector<
 	removed_deps.sort();
 	if (removed_deps.empty()) {
 		owners->hide();
-		text->set_text(TTR("Remove selected files from the project? (Can't be restored)"));
+		text->set_text(TTR("Remove the selected files from the project? (Cannot be undone.)\nDepending on your filesystem configuration, the files will either be moved to the system trash or deleted permanently."));
 		set_size(Size2());
 		popup_centered();
 	} else {
 		_build_removed_dependency_tree(removed_deps);
 		owners->show();
-		text->set_text(TTR("The files being removed are required by other resources in order for them to work.\nRemove them anyway? (no undo)"));
+		text->set_text(TTR("The files being removed are required by other resources in order for them to work.\nRemove them anyway? (Cannot be undone.)\nDepending on your filesystem configuration, the files will either be moved to the system trash or deleted permanently."));
 		popup_centered(Size2(500, 350));
 	}
 	EditorFileSystem::get_singleton()->scan_changes();
 }
 
 void DependencyRemoveDialog::ok_pressed() {
-
 	for (int i = 0; i < files_to_delete.size(); ++i) {
 		if (ResourceCache::has(files_to_delete[i])) {
 			Resource *res = ResourceCache::get(files_to_delete[i]);
@@ -533,10 +517,10 @@ void DependencyRemoveDialog::ok_pressed() {
 
 	if (dirs_to_delete.size() == 0) {
 		// If we only deleted files we should only need to tell the file system about the files we touched.
-		for (int i = 0; i < files_to_delete.size(); ++i)
+		for (int i = 0; i < files_to_delete.size(); ++i) {
 			EditorFileSystem::get_singleton()->update_file(files_to_delete[i]);
+		}
 	} else {
-
 		for (int i = 0; i < dirs_to_delete.size(); ++i) {
 			String path = OS::get_singleton()->get_resource_dir() + dirs_to_delete[i].replace_first("res://", "/");
 			print_verbose("Moving to trash: " + path);
@@ -557,11 +541,13 @@ void DependencyRemoveDialog::ok_pressed() {
 
 	for (int i = 0; i < previous_favorites.size(); ++i) {
 		if (previous_favorites[i].ends_with("/")) {
-			if (dirs_to_delete.find(previous_favorites[i]) < 0)
+			if (dirs_to_delete.find(previous_favorites[i]) < 0) {
 				new_favorites.push_back(previous_favorites[i]);
+			}
 		} else {
-			if (files_to_delete.find(previous_favorites[i]) < 0)
+			if (files_to_delete.find(previous_favorites[i]) < 0) {
 				new_favorites.push_back(previous_favorites[i]);
+			}
 		}
 	}
 
@@ -576,7 +562,6 @@ void DependencyRemoveDialog::_bind_methods() {
 }
 
 DependencyRemoveDialog::DependencyRemoveDialog() {
-
 	get_ok()->set_text(TTR("Remove"));
 
 	VBoxContainer *vb = memnew(VBoxContainer);
@@ -594,20 +579,19 @@ DependencyRemoveDialog::DependencyRemoveDialog() {
 //////////////
 
 void DependencyErrorDialog::show(Mode p_mode, const String &p_for_file, const Vector<String> &report) {
-
 	mode = p_mode;
 	for_file = p_for_file;
 	set_title(TTR("Error loading:") + " " + p_for_file.get_file());
 	files->clear();
 
-	TreeItem *root = files->create_item(NULL);
+	TreeItem *root = files->create_item(nullptr);
 	for (int i = 0; i < report.size(); i++) {
-
 		String dep;
 		String type = "Object";
 		dep = report[i].get_slice("::", 0);
-		if (report[i].get_slice_count("::") > 0)
+		if (report[i].get_slice_count("::") > 0) {
 			type = report[i].get_slice("::", 1);
+		}
 
 		Ref<Texture> icon = EditorNode::get_singleton()->get_class_icon(type);
 
@@ -620,7 +604,6 @@ void DependencyErrorDialog::show(Mode p_mode, const String &p_for_file, const Ve
 }
 
 void DependencyErrorDialog::ok_pressed() {
-
 	switch (mode) {
 		case MODE_SCENE:
 			EditorNode::get_singleton()->load_scene(for_file, true);
@@ -632,12 +615,10 @@ void DependencyErrorDialog::ok_pressed() {
 }
 
 void DependencyErrorDialog::custom_action(const String &) {
-
 	EditorNode::get_singleton()->fix_dependencies(for_file);
 }
 
 DependencyErrorDialog::DependencyErrorDialog() {
-
 	VBoxContainer *vb = memnew(VBoxContainer);
 	add_child(vb);
 
@@ -662,27 +643,26 @@ DependencyErrorDialog::DependencyErrorDialog() {
 //////////////////////////////////////////////////////////////////////
 
 void OrphanResourcesDialog::ok_pressed() {
-
 	paths.clear();
 
 	_find_to_delete(files->get_root(), paths);
-	if (paths.empty())
+	if (paths.empty()) {
 		return;
+	}
 
 	delete_confirm->set_text(vformat(TTR("Permanently delete %d item(s)? (No undo!)"), paths.size()));
 	delete_confirm->popup_centered_clamped(delete_confirm->get_minimum_size());
 }
 
 bool OrphanResourcesDialog::_fill_owners(EditorFileSystemDirectory *efsd, HashMap<String, int> &refs, TreeItem *p_parent) {
-
-	if (!efsd)
+	if (!efsd) {
 		return false;
+	}
 
 	bool has_children = false;
 
 	for (int i = 0; i < efsd->get_subdir_count(); i++) {
-
-		TreeItem *dir_item = NULL;
+		TreeItem *dir_item = nullptr;
 		if (p_parent) {
 			dir_item = files->create_item(p_parent);
 			dir_item->set_text(0, efsd->get_subdir(i)->get_name());
@@ -700,17 +680,14 @@ bool OrphanResourcesDialog::_fill_owners(EditorFileSystemDirectory *efsd, HashMa
 	}
 
 	for (int i = 0; i < efsd->get_file_count(); i++) {
-
 		if (!p_parent) {
 			Vector<String> deps = efsd->get_file_deps(i);
 			for (int j = 0; j < deps.size(); j++) {
-
 				if (!refs.has(deps[j])) {
 					refs[deps[j]] = 1;
 				}
 			}
 		} else {
-
 			String path = efsd->get_file_path(i);
 			if (!refs.has(path)) {
 				TreeItem *ti = files->create_item(p_parent);
@@ -738,22 +715,19 @@ bool OrphanResourcesDialog::_fill_owners(EditorFileSystemDirectory *efsd, HashMa
 
 void OrphanResourcesDialog::refresh() {
 	HashMap<String, int> refs;
-	_fill_owners(EditorFileSystem::get_singleton()->get_filesystem(), refs, NULL);
+	_fill_owners(EditorFileSystem::get_singleton()->get_filesystem(), refs, nullptr);
 	files->clear();
 	TreeItem *root = files->create_item();
 	_fill_owners(EditorFileSystem::get_singleton()->get_filesystem(), refs, root);
 }
 
 void OrphanResourcesDialog::show() {
-
 	refresh();
 	popup_centered_ratio();
 }
 
 void OrphanResourcesDialog::_find_to_delete(TreeItem *p_item, List<String> &paths) {
-
 	while (p_item) {
-
 		if (p_item->get_cell_mode(0) == TreeItem::CELL_MODE_CHECK && p_item->is_checked(0)) {
 			paths.push_back(p_item->get_metadata(0));
 		}
@@ -767,10 +741,8 @@ void OrphanResourcesDialog::_find_to_delete(TreeItem *p_item, List<String> &path
 }
 
 void OrphanResourcesDialog::_delete_confirm() {
-
 	DirAccess *da = DirAccess::create(DirAccess::ACCESS_RESOURCES);
 	for (List<String>::Element *E = paths.front(); E; E = E->next()) {
-
 		da->remove(E->get());
 		EditorFileSystem::get_singleton()->update_file(E->get());
 	}
@@ -779,7 +751,6 @@ void OrphanResourcesDialog::_delete_confirm() {
 }
 
 void OrphanResourcesDialog::_button_pressed(Object *p_item, int p_column, int p_id) {
-
 	TreeItem *ti = Object::cast_to<TreeItem>(p_item);
 
 	String path = ti->get_metadata(0);
@@ -787,13 +758,11 @@ void OrphanResourcesDialog::_button_pressed(Object *p_item, int p_column, int p_
 }
 
 void OrphanResourcesDialog::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_delete_confirm"), &OrphanResourcesDialog::_delete_confirm);
 	ClassDB::bind_method(D_METHOD("_button_pressed"), &OrphanResourcesDialog::_button_pressed);
 }
 
 OrphanResourcesDialog::OrphanResourcesDialog() {
-
 	set_title(TTR("Orphan Resource Explorer"));
 	delete_confirm = memnew(ConfirmationDialog);
 	get_ok()->set_text(TTR("Delete"));
