@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  editor_audio_buses.cpp                                               */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  editor_audio_buses.cpp                                                */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "editor_audio_buses.h"
 
@@ -40,10 +40,8 @@
 #include "servers/audio_server.h"
 
 void EditorAudioBus::_update_visible_channels() {
-
 	int i = 0;
 	for (; i < cc; i++) {
-
 		if (!channel[i].vu_l->is_visible()) {
 			channel[i].vu_l->show();
 		}
@@ -53,7 +51,6 @@ void EditorAudioBus::_update_visible_channels() {
 	}
 
 	for (; i < CHANNELS_MAX; i++) {
-
 		if (channel[i].vu_l->is_visible()) {
 			channel[i].vu_l->hide();
 		}
@@ -64,10 +61,9 @@ void EditorAudioBus::_update_visible_channels() {
 }
 
 void EditorAudioBus::_notification(int p_what) {
-
 	switch (p_what) {
-		case NOTIFICATION_READY: {
-
+		case NOTIFICATION_ENTER_TREE:
+		case NOTIFICATION_THEME_CHANGED: {
 			for (int i = 0; i < CHANNELS_MAX; i++) {
 				channel[i].vu_l->set_under_texture(get_icon("BusVuEmpty", "EditorIcons"));
 				channel[i].vu_l->set_progress_texture(get_icon("BusVuFull", "EditorIcons"));
@@ -91,11 +87,22 @@ void EditorAudioBus::_notification(int p_what) {
 
 			bus_options->set_icon(get_icon("GuiTabMenuHl", "EditorIcons"));
 
+			audio_value_preview_label->add_color_override("font_color", get_color("font_color", "TooltipLabel"));
+			audio_value_preview_label->add_color_override("font_color_shadow", get_color("font_color_shadow", "TooltipLabel"));
+			audio_value_preview_box->add_style_override("panel", get_stylebox("panel", "TooltipPanel"));
+
+			for (int i = 0; i < effect_options->get_item_count(); i++) {
+				String class_name = effect_options->get_item_metadata(i);
+				Ref<Texture> icon = EditorNode::get_singleton()->get_class_icon(class_name);
+				effect_options->set_item_icon(i, icon);
+			}
+		} break;
+
+		case NOTIFICATION_READY: {
 			update_bus();
 			set_process(true);
 		} break;
 		case NOTIFICATION_DRAW: {
-
 			if (is_master) {
 				draw_style_box(get_stylebox("disabled", "Button"), Rect2(Vector2(), get_size()));
 			} else if (has_focus()) {
@@ -111,7 +118,6 @@ void EditorAudioBus::_notification(int p_what) {
 			}
 		} break;
 		case NOTIFICATION_PROCESS: {
-
 			if (cc != AudioServer::get_singleton()->get_bus_channels(get_index())) {
 				cc = AudioServer::get_singleton()->get_bus_channels(get_index());
 				_update_visible_channels();
@@ -156,7 +162,6 @@ void EditorAudioBus::_notification(int p_what) {
 			}
 		} break;
 		case NOTIFICATION_VISIBILITY_CHANGED: {
-
 			for (int i = 0; i < CHANNELS_MAX; i++) {
 				channel[i].peak_l = -100;
 				channel[i].peak_r = -100;
@@ -165,27 +170,9 @@ void EditorAudioBus::_notification(int p_what) {
 
 			set_process(is_visible_in_tree());
 		} break;
-		case NOTIFICATION_THEME_CHANGED: {
 
-			for (int i = 0; i < CHANNELS_MAX; i++) {
-				channel[i].vu_l->set_under_texture(get_icon("BusVuEmpty", "EditorIcons"));
-				channel[i].vu_l->set_progress_texture(get_icon("BusVuFull", "EditorIcons"));
-				channel[i].vu_r->set_under_texture(get_icon("BusVuEmpty", "EditorIcons"));
-				channel[i].vu_r->set_progress_texture(get_icon("BusVuFull", "EditorIcons"));
-				channel[i].prev_active = true;
-			}
-
-			disabled_vu = get_icon("BusVuFrozen", "EditorIcons");
-
-			solo->set_icon(get_icon("AudioBusSolo", "EditorIcons"));
-			mute->set_icon(get_icon("AudioBusMute", "EditorIcons"));
-			bypass->set_icon(get_icon("AudioBusBypass", "EditorIcons"));
-
-			bus_options->set_icon(get_icon("GuiTabMenuHl", "EditorIcons"));
-		} break;
 		case NOTIFICATION_MOUSE_EXIT:
 		case NOTIFICATION_DRAG_END: {
-
 			if (hovering_drop) {
 				hovering_drop = false;
 				update();
@@ -195,7 +182,6 @@ void EditorAudioBus::_notification(int p_what) {
 }
 
 void EditorAudioBus::update_send() {
-
 	send->clear();
 	if (is_master) {
 		send->set_disabled(true);
@@ -218,9 +204,9 @@ void EditorAudioBus::update_send() {
 }
 
 void EditorAudioBus::update_bus() {
-
-	if (updating_bus)
+	if (updating_bus) {
 		return;
+	}
 
 	updating_bus = true;
 
@@ -229,8 +215,9 @@ void EditorAudioBus::update_bus() {
 	float db_value = AudioServer::get_singleton()->get_bus_volume_db(index);
 	slider->set_value(_scaled_db_to_normalized_volume(db_value));
 	track_name->set_text(AudioServer::get_singleton()->get_bus_name(index));
-	if (is_master)
+	if (is_master) {
 		track_name->set_editable(false);
+	}
 
 	solo->set_pressed(AudioServer::get_singleton()->is_bus_solo(index));
 	mute->set_pressed(AudioServer::get_singleton()->is_bus_mute(index));
@@ -240,7 +227,6 @@ void EditorAudioBus::update_bus() {
 
 	TreeItem *root = effects->create_item();
 	for (int i = 0; i < AudioServer::get_singleton()->get_bus_effect_count(index); i++) {
-
 		Ref<AudioEffect> afx = AudioServer::get_singleton()->get_bus_effect(index, i);
 
 		TreeItem *fx = effects->create_item(root);
@@ -263,18 +249,16 @@ void EditorAudioBus::update_bus() {
 }
 
 void EditorAudioBus::_name_changed(const String &p_new_name) {
-
-	if (p_new_name == AudioServer::get_singleton()->get_bus_name(get_index()))
+	if (p_new_name == AudioServer::get_singleton()->get_bus_name(get_index())) {
 		return;
+	}
 
 	String attempt = p_new_name;
 	int attempts = 1;
 
 	while (true) {
-
 		bool name_free = true;
 		for (int i = 0; i < AudioServer::get_singleton()->get_bus_count(); i++) {
-
 			if (AudioServer::get_singleton()->get_bus_name(i) == attempt) {
 				name_free = false;
 				break;
@@ -317,9 +301,9 @@ void EditorAudioBus::_name_changed(const String &p_new_name) {
 }
 
 void EditorAudioBus::_volume_changed(float p_normalized) {
-
-	if (updating_bus)
+	if (updating_bus) {
 		return;
+	}
 
 	updating_bus = true;
 
@@ -346,7 +330,7 @@ float EditorAudioBus::_normalized_volume_to_scaled_db(float normalized) {
 	/* There are three different formulas for the conversion from normalized
 	 * values to relative decibal values.
 	 * One formula is an exponential graph which intends to counteract
-	 * the logorithmic nature of human hearing. This is an approximation
+	 * the logarithmic nature of human hearing. This is an approximation
 	 * of the behaviour of a 'logarithmic potentiometer' found on most
 	 * musical instruments and also emulated in popular software.
 	 * The other two equations are hand-tuned linear tapers that intend to
@@ -384,7 +368,6 @@ float EditorAudioBus::_scaled_db_to_normalized_volume(float db) {
 }
 
 void EditorAudioBus::_show_value(float slider_value) {
-
 	float db;
 	if (Input::get_singleton()->is_key_pressed(KEY_CONTROL)) {
 		// Display the correct (snapped) value when holding Ctrl
@@ -393,15 +376,24 @@ void EditorAudioBus::_show_value(float slider_value) {
 		db = _normalized_volume_to_scaled_db(slider_value);
 	}
 
-	String text = vformat("%10.1f dB", db);
+	String text;
+	if (Math::is_zero_approx(Math::stepify(db, 0.1))) {
+		// Prevent displaying `-0.0 dB` and show ` 0.0 dB` instead.
+		// The leading space makes the text visually line up with its positive/negative counterparts.
+		text = " 0.0 dB";
+	} else {
+		// Show an explicit `+` sign if positive.
+		text = vformat("%+.1f dB", db);
+	}
 
+	// Also set the preview text as a standard Control tooltip.
+	// This way, it can be seen when the slider is merely hovered (instead of dragged).
 	slider->set_tooltip(text);
 	audio_value_preview_label->set_text(text);
-	Vector2 slider_size = slider->get_size();
-	Vector2 slider_position = slider->get_global_position();
-	float left_padding = 5.0f;
-	float vert_padding = 10.0f;
-	Vector2 box_position = Vector2(slider_size.x + left_padding, (slider_size.y - vert_padding) * (1.0f - slider->get_value()) - vert_padding);
+	const Vector2 slider_size = slider->get_size();
+	const Vector2 slider_position = slider->get_global_position();
+	const float vert_padding = 10.0f;
+	const Vector2 box_position = Vector2(slider_size.x, (slider_size.y - vert_padding) * (1.0f - slider->get_value()) - vert_padding);
 	audio_value_preview_box->set_position(slider_position + box_position);
 	audio_value_preview_box->set_size(audio_value_preview_label->get_size());
 	if (slider->has_focus() && !audio_value_preview_box->is_visible()) {
@@ -415,7 +407,6 @@ void EditorAudioBus::_hide_value_preview() {
 }
 
 void EditorAudioBus::_solo_toggled() {
-
 	updating_bus = true;
 
 	UndoRedo *ur = EditorNode::get_undo_redo();
@@ -429,7 +420,6 @@ void EditorAudioBus::_solo_toggled() {
 	updating_bus = false;
 }
 void EditorAudioBus::_mute_toggled() {
-
 	updating_bus = true;
 
 	UndoRedo *ur = EditorNode::get_undo_redo();
@@ -443,7 +433,6 @@ void EditorAudioBus::_mute_toggled() {
 	updating_bus = false;
 }
 void EditorAudioBus::_bypass_toggled() {
-
 	updating_bus = true;
 
 	UndoRedo *ur = EditorNode::get_undo_redo();
@@ -458,7 +447,6 @@ void EditorAudioBus::_bypass_toggled() {
 }
 
 void EditorAudioBus::_send_selected(int p_which) {
-
 	updating_bus = true;
 
 	UndoRedo *ur = EditorNode::get_undo_redo();
@@ -473,14 +461,13 @@ void EditorAudioBus::_send_selected(int p_which) {
 }
 
 void EditorAudioBus::_effect_selected() {
-
 	TreeItem *effect = effects->get_selected();
-	if (!effect)
+	if (!effect) {
 		return;
+	}
 	updating_bus = true;
 
 	if (effect->get_metadata(0) != Variant()) {
-
 		int index = effect->get_metadata(0);
 		Ref<AudioEffect> effect2 = AudioServer::get_singleton()->get_bus_effect(get_index(), index);
 		if (effect2.is_valid()) {
@@ -492,13 +479,14 @@ void EditorAudioBus::_effect_selected() {
 }
 
 void EditorAudioBus::_effect_edited() {
-
-	if (updating_bus)
+	if (updating_bus) {
 		return;
+	}
 
 	TreeItem *effect = effects->get_edited();
-	if (!effect)
+	if (!effect) {
 		return;
+	}
 
 	if (effect->get_metadata(0) == Variant()) {
 		Rect2 area = effects->get_item_rect(effect);
@@ -523,9 +511,9 @@ void EditorAudioBus::_effect_edited() {
 }
 
 void EditorAudioBus::_effect_add(int p_which) {
-
-	if (updating_bus)
+	if (updating_bus) {
 		return;
+	}
 
 	StringName name = effect_options->get_item_metadata(p_which);
 
@@ -547,7 +535,6 @@ void EditorAudioBus::_effect_add(int p_which) {
 }
 
 void EditorAudioBus::_gui_input(const Ref<InputEvent> &p_event) {
-
 	Ref<InputEventKey> k = p_event;
 	if (k.is_valid() && k->is_pressed() && k->get_scancode() == KEY_DELETE && !k->is_echo()) {
 		accept_event();
@@ -556,7 +543,6 @@ void EditorAudioBus::_gui_input(const Ref<InputEvent> &p_event) {
 
 	Ref<InputEventMouseButton> mb = p_event;
 	if (mb.is_valid() && mb->get_button_index() == 2 && mb->is_pressed()) {
-
 		Vector2 pos = Vector2(mb->get_position().x, mb->get_position().y);
 		bus_popup->set_position(get_global_position() + pos);
 		bus_popup->popup();
@@ -575,7 +561,6 @@ void EditorAudioBus::_unhandled_key_input(Ref<InputEvent> p_event) {
 }
 
 void EditorAudioBus::_bus_popup_pressed(int p_option) {
-
 	if (p_option == 2) {
 		// Reset volume
 		emit_signal("vol_reset_request");
@@ -588,7 +573,6 @@ void EditorAudioBus::_bus_popup_pressed(int p_option) {
 }
 
 Variant EditorAudioBus::get_drag_data(const Point2 &p_point) {
-
 	if (get_index() == 0) {
 		return Variant();
 	}
@@ -613,7 +597,6 @@ Variant EditorAudioBus::get_drag_data(const Point2 &p_point) {
 }
 
 bool EditorAudioBus::can_drop_data(const Point2 &p_point, const Variant &p_data) const {
-
 	if (get_index() == 0) {
 		return false;
 	}
@@ -628,13 +611,11 @@ bool EditorAudioBus::can_drop_data(const Point2 &p_point, const Variant &p_data)
 }
 
 void EditorAudioBus::drop_data(const Point2 &p_point, const Variant &p_data) {
-
 	Dictionary d = p_data;
 	emit_signal("dropped", d["index"], get_index());
 }
 
 Variant EditorAudioBus::get_drag_data_fw(const Point2 &p_point, Control *p_from) {
-
 	TreeItem *item = effects->get_item_at_position(p_point);
 	if (!item) {
 		return Variant();
@@ -658,14 +639,15 @@ Variant EditorAudioBus::get_drag_data_fw(const Point2 &p_point, Control *p_from)
 }
 
 bool EditorAudioBus::can_drop_data_fw(const Point2 &p_point, const Variant &p_data, Control *p_from) const {
-
 	Dictionary d = p_data;
-	if (!d.has("type") || String(d["type"]) != "audio_bus_effect")
+	if (!d.has("type") || String(d["type"]) != "audio_bus_effect") {
 		return false;
+	}
 
 	TreeItem *item = effects->get_item_at_position(p_point);
-	if (!item)
+	if (!item) {
 		return false;
+	}
 
 	effects->set_drop_mode_flags(Tree::DROP_MODE_INBETWEEN);
 
@@ -673,12 +655,12 @@ bool EditorAudioBus::can_drop_data_fw(const Point2 &p_point, const Variant &p_da
 }
 
 void EditorAudioBus::drop_data_fw(const Point2 &p_point, const Variant &p_data, Control *p_from) {
-
 	Dictionary d = p_data;
 
 	TreeItem *item = effects->get_item_at_position(p_point);
-	if (!item)
+	if (!item) {
 		return;
+	}
 	int pos = effects->get_drop_section_at_position(p_point);
 	Variant md = item->get_metadata(0);
 
@@ -688,8 +670,9 @@ void EditorAudioBus::drop_data_fw(const Point2 &p_point, const Variant &p_data,
 
 	if (md.get_type() == Variant::INT) {
 		paste_at = md;
-		if (pos > 0)
+		if (pos > 0) {
 			paste_at++;
+		}
 
 		if (bus == get_index() && paste_at > effect) {
 			paste_at--;
@@ -731,13 +714,14 @@ void EditorAudioBus::drop_data_fw(const Point2 &p_point, const Variant &p_data,
 }
 
 void EditorAudioBus::_delete_effect_pressed(int p_option) {
-
 	TreeItem *item = effects->get_selected();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	if (item->get_metadata(0).get_type() != Variant::INT)
+	if (item->get_metadata(0).get_type() != Variant::INT) {
 		return;
+	}
 
 	int index = item->get_metadata(0);
 
@@ -752,20 +736,20 @@ void EditorAudioBus::_delete_effect_pressed(int p_option) {
 }
 
 void EditorAudioBus::_effect_rmb(const Vector2 &p_pos) {
-
 	TreeItem *item = effects->get_selected();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
-	if (item->get_metadata(0).get_type() != Variant::INT)
+	if (item->get_metadata(0).get_type() != Variant::INT) {
 		return;
+	}
 
 	delete_effect_popup->set_position(get_global_mouse_position());
 	delete_effect_popup->popup();
 }
 
 void EditorAudioBus::_bind_methods() {
-
 	ClassDB::bind_method("update_bus", &EditorAudioBus::update_bus);
 	ClassDB::bind_method("update_send", &EditorAudioBus::update_send);
 	ClassDB::bind_method("_name_changed", &EditorAudioBus::_name_changed);
@@ -797,7 +781,6 @@ void EditorAudioBus::_bind_methods() {
 }
 
 EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
-
 	buses = p_buses;
 	updating_bus = false;
 	is_master = p_is_master;
@@ -841,7 +824,7 @@ EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
 	bus_options = memnew(MenuButton);
 	bus_options->set_h_size_flags(SIZE_SHRINK_END);
 	bus_options->set_anchor(MARGIN_RIGHT, 0.0);
-	bus_options->set_tooltip(TTR("Bus options"));
+	bus_options->set_tooltip(TTR("Bus Options"));
 	hbc->add_child(bus_options);
 
 	Ref<StyleBoxEmpty> sbempty = memnew(StyleBoxEmpty);
@@ -866,6 +849,11 @@ EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
 	slider->set_clip_contents(false);
 
 	audio_value_preview_box = memnew(Panel);
+	slider->add_child(audio_value_preview_box);
+	audio_value_preview_box->set_as_toplevel(true);
+	audio_value_preview_box->set_mouse_filter(MOUSE_FILTER_PASS);
+	audio_value_preview_box->hide();
+
 	HBoxContainer *audioprev_hbc = memnew(HBoxContainer);
 	audioprev_hbc->set_v_size_flags(SIZE_EXPAND_FILL);
 	audioprev_hbc->set_h_size_flags(SIZE_EXPAND_FILL);
@@ -876,17 +864,8 @@ EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
 	audio_value_preview_label->set_v_size_flags(SIZE_EXPAND_FILL);
 	audio_value_preview_label->set_h_size_flags(SIZE_EXPAND_FILL);
 	audio_value_preview_label->set_mouse_filter(MOUSE_FILTER_PASS);
-
 	audioprev_hbc->add_child(audio_value_preview_label);
 
-	slider->add_child(audio_value_preview_box);
-	audio_value_preview_box->set_as_toplevel(true);
-	Ref<StyleBoxFlat> panel_style = memnew(StyleBoxFlat);
-	panel_style->set_bg_color(Color(0.0f, 0.0f, 0.0f, 0.8f));
-	audio_value_preview_box->add_style_override("panel", panel_style);
-	audio_value_preview_box->set_mouse_filter(MOUSE_FILTER_PASS);
-	audio_value_preview_box->hide();
-
 	preview_timer = memnew(Timer);
 	preview_timer->set_wait_time(0.8f);
 	preview_timer->set_one_shot(true);
@@ -955,14 +934,13 @@ EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
 	ClassDB::get_inheriters_from_class("AudioEffect", &effects);
 	effects.sort_custom<StringName::AlphCompare>();
 	for (List<StringName>::Element *E = effects.front(); E; E = E->next()) {
-		if (!ClassDB::can_instance(E->get()))
+		if (!ClassDB::can_instance(E->get())) {
 			continue;
+		}
 
-		Ref<Texture> icon = EditorNode::get_singleton()->get_class_icon(E->get());
 		String name = E->get().operator String().replace("AudioEffect", "");
 		effect_options->add_item(name);
 		effect_options->set_item_metadata(effect_options->get_item_count() - 1, E->get());
-		effect_options->set_item_icon(effect_options->get_item_count() - 1, icon);
 	}
 
 	bus_popup = bus_options->get_popup();
@@ -979,7 +957,6 @@ EditorAudioBus::EditorAudioBus(EditorAudioBuses *p_buses, bool p_is_master) {
 }
 
 void EditorAudioBusDrop::_notification(int p_what) {
-
 	switch (p_what) {
 		case NOTIFICATION_DRAW: {
 			draw_style_box(get_stylebox("normal", "Button"), Rect2(Vector2(), get_size()));
@@ -991,7 +968,6 @@ void EditorAudioBusDrop::_notification(int p_what) {
 			}
 		} break;
 		case NOTIFICATION_MOUSE_ENTER: {
-
 			if (!hovering_drop) {
 				hovering_drop = true;
 				update();
@@ -999,7 +975,6 @@ void EditorAudioBusDrop::_notification(int p_what) {
 		} break;
 		case NOTIFICATION_MOUSE_EXIT:
 		case NOTIFICATION_DRAG_END: {
-
 			if (hovering_drop) {
 				hovering_drop = false;
 				update();
@@ -1009,37 +984,31 @@ void EditorAudioBusDrop::_notification(int p_what) {
 }
 
 bool EditorAudioBusDrop::can_drop_data(const Point2 &p_point, const Variant &p_data) const {
-
 	Dictionary d = p_data;
 	return (d.has("type") && String(d["type"]) == "move_audio_bus");
 }
 
 void EditorAudioBusDrop::drop_data(const Point2 &p_point, const Variant &p_data) {
-
 	Dictionary d = p_data;
 	emit_signal("dropped", d["index"], AudioServer::get_singleton()->get_bus_count());
 }
 
 void EditorAudioBusDrop::_bind_methods() {
-
 	ADD_SIGNAL(MethodInfo("dropped"));
 }
 
 EditorAudioBusDrop::EditorAudioBusDrop() {
-
 	hovering_drop = false;
 }
 
 void EditorAudioBuses::_update_buses() {
-
 	while (bus_hb->get_child_count() > 0) {
 		memdelete(bus_hb->get_child(0));
 	}
 
-	drop_end = NULL;
+	drop_end = nullptr;
 
 	for (int i = 0; i < AudioServer::get_singleton()->get_bus_count(); i++) {
-
 		bool is_master = (i == 0);
 		EditorAudioBus *audio_bus = memnew(EditorAudioBus(this, is_master));
 		bus_hb->add_child(audio_bus);
@@ -1052,33 +1021,27 @@ void EditorAudioBuses::_update_buses() {
 }
 
 EditorAudioBuses *EditorAudioBuses::register_editor() {
-
 	EditorAudioBuses *audio_buses = memnew(EditorAudioBuses);
 	EditorNode::get_singleton()->add_bottom_panel_item(TTR("Audio"), audio_buses);
 	return audio_buses;
 }
 
 void EditorAudioBuses::_notification(int p_what) {
-
 	switch (p_what) {
 		case NOTIFICATION_ENTER_TREE:
 		case NOTIFICATION_THEME_CHANGED: {
-
 			bus_scroll->add_style_override("bg", get_stylebox("bg", "Tree"));
 		} break;
 		case NOTIFICATION_READY: {
-
 			_update_buses();
 		} break;
 		case NOTIFICATION_DRAG_END: {
-
 			if (drop_end) {
 				drop_end->queue_delete();
-				drop_end = NULL;
+				drop_end = nullptr;
 			}
 		} break;
 		case NOTIFICATION_PROCESS: {
-
 			// Check if anything was edited.
 			bool edited = AudioServer::get_singleton()->is_edited();
 			for (int i = 0; i < AudioServer::get_singleton()->get_bus_count(); i++) {
@@ -1101,7 +1064,6 @@ void EditorAudioBuses::_notification(int p_what) {
 }
 
 void EditorAudioBuses::_add_bus() {
-
 	UndoRedo *ur = EditorNode::get_undo_redo();
 
 	ur->create_action(TTR("Add Audio Bus"));
@@ -1113,22 +1075,20 @@ void EditorAudioBuses::_add_bus() {
 }
 
 void EditorAudioBuses::_update_bus(int p_index) {
-
-	if (p_index >= bus_hb->get_child_count())
+	if (p_index >= bus_hb->get_child_count()) {
 		return;
+	}
 
 	bus_hb->get_child(p_index)->call("update_bus");
 }
 
 void EditorAudioBuses::_update_sends() {
-
 	for (int i = 0; i < bus_hb->get_child_count(); i++) {
 		bus_hb->get_child(i)->call("update_send");
 	}
 }
 
 void EditorAudioBuses::_delete_bus(Object *p_which) {
-
 	EditorAudioBus *bus = Object::cast_to<EditorAudioBus>(p_which);
 	int index = bus->get_index();
 	if (index == 0) {
@@ -1148,7 +1108,6 @@ void EditorAudioBuses::_delete_bus(Object *p_which) {
 	ur->add_undo_method(AudioServer::get_singleton(), "set_bus_mute", index, AudioServer::get_singleton()->is_bus_mute(index));
 	ur->add_undo_method(AudioServer::get_singleton(), "set_bus_bypass_effects", index, AudioServer::get_singleton()->is_bus_bypassing_effects(index));
 	for (int i = 0; i < AudioServer::get_singleton()->get_bus_effect_count(index); i++) {
-
 		ur->add_undo_method(AudioServer::get_singleton(), "add_bus_effect", index, AudioServer::get_singleton()->get_bus_effect(index, i));
 		ur->add_undo_method(AudioServer::get_singleton(), "set_bus_effect_enabled", index, i, AudioServer::get_singleton()->is_bus_effect_enabled(index, i));
 	}
@@ -1158,7 +1117,6 @@ void EditorAudioBuses::_delete_bus(Object *p_which) {
 }
 
 void EditorAudioBuses::_duplicate_bus(int p_which) {
-
 	int add_at_pos = p_which + 1;
 	UndoRedo *ur = EditorNode::get_undo_redo();
 	ur->create_action(TTR("Duplicate Audio Bus"));
@@ -1170,7 +1128,6 @@ void EditorAudioBuses::_duplicate_bus(int p_which) {
 	ur->add_do_method(AudioServer::get_singleton(), "set_bus_mute", add_at_pos, AudioServer::get_singleton()->is_bus_mute(p_which));
 	ur->add_do_method(AudioServer::get_singleton(), "set_bus_bypass_effects", add_at_pos, AudioServer::get_singleton()->is_bus_bypassing_effects(p_which));
 	for (int i = 0; i < AudioServer::get_singleton()->get_bus_effect_count(p_which); i++) {
-
 		ur->add_do_method(AudioServer::get_singleton(), "add_bus_effect", add_at_pos, AudioServer::get_singleton()->get_bus_effect(p_which, i));
 		ur->add_do_method(AudioServer::get_singleton(), "set_bus_effect_enabled", add_at_pos, i, AudioServer::get_singleton()->is_bus_effect_enabled(p_which, i));
 	}
@@ -1181,7 +1138,6 @@ void EditorAudioBuses::_duplicate_bus(int p_which) {
 }
 
 void EditorAudioBuses::_reset_bus_volume(Object *p_which) {
-
 	EditorAudioBus *bus = Object::cast_to<EditorAudioBus>(p_which);
 	int index = bus->get_index();
 
@@ -1195,7 +1151,6 @@ void EditorAudioBuses::_reset_bus_volume(Object *p_which) {
 }
 
 void EditorAudioBuses::_request_drop_end() {
-
 	if (!drop_end && bus_hb->get_child_count()) {
 		drop_end = memnew(EditorAudioBusDrop);
 
@@ -1206,7 +1161,6 @@ void EditorAudioBuses::_request_drop_end() {
 }
 
 void EditorAudioBuses::_drop_at_index(int p_bus, int p_index) {
-
 	UndoRedo *ur = EditorNode::get_undo_redo();
 	ur->create_action(TTR("Move Audio Bus"));
 
@@ -1221,18 +1175,15 @@ void EditorAudioBuses::_drop_at_index(int p_bus, int p_index) {
 }
 
 void EditorAudioBuses::_server_save() {
-
 	Ref<AudioBusLayout> state = AudioServer::get_singleton()->generate_bus_layout();
 	ResourceSaver::save(edited_path, state);
 }
 
 void EditorAudioBuses::_select_layout() {
-
 	EditorNode::get_singleton()->get_filesystem_dock()->select_file(edited_path);
 }
 
 void EditorAudioBuses::_save_as_layout() {
-
 	file_dialog->set_mode(EditorFileDialog::MODE_SAVE_FILE);
 	file_dialog->set_title(TTR("Save Audio Bus Layout As..."));
 	file_dialog->set_current_path(edited_path);
@@ -1241,7 +1192,6 @@ void EditorAudioBuses::_save_as_layout() {
 }
 
 void EditorAudioBuses::_new_layout() {
-
 	file_dialog->set_mode(EditorFileDialog::MODE_SAVE_FILE);
 	file_dialog->set_title(TTR("Location for New Layout..."));
 	file_dialog->set_current_path(edited_path);
@@ -1250,7 +1200,6 @@ void EditorAudioBuses::_new_layout() {
 }
 
 void EditorAudioBuses::_load_layout() {
-
 	file_dialog->set_mode(EditorFileDialog::MODE_OPEN_FILE);
 	file_dialog->set_title(TTR("Open Audio Bus Layout"));
 	file_dialog->set_current_path(edited_path);
@@ -1259,7 +1208,6 @@ void EditorAudioBuses::_load_layout() {
 }
 
 void EditorAudioBuses::_load_default_layout() {
-
 	String layout_path = ProjectSettings::get_singleton()->get("audio/default_bus_layout");
 
 	Ref<AudioBusLayout> state = ResourceLoader::load(layout_path, "", true);
@@ -1269,7 +1217,7 @@ void EditorAudioBuses::_load_default_layout() {
 	}
 
 	edited_path = layout_path;
-	file->set_text(String(TTR("Layout")) + ": " + layout_path.get_file());
+	file->set_text(String(TTR("Layout:")) + " " + layout_path.get_file());
 	AudioServer::get_singleton()->set_bus_layout(state);
 	_update_buses();
 	EditorNode::get_singleton()->get_undo_redo()->clear_history();
@@ -1277,7 +1225,6 @@ void EditorAudioBuses::_load_default_layout() {
 }
 
 void EditorAudioBuses::_file_dialog_callback(const String &p_string) {
-
 	if (file_dialog->get_mode() == EditorFileDialog::MODE_OPEN_FILE) {
 		Ref<AudioBusLayout> state = ResourceLoader::load(p_string, "", true);
 		if (state.is_null()) {
@@ -1286,14 +1233,13 @@ void EditorAudioBuses::_file_dialog_callback(const String &p_string) {
 		}
 
 		edited_path = p_string;
-		file->set_text(String(TTR("Layout")) + ": " + p_string.get_file());
+		file->set_text(String(TTR("Layout:")) + " " + p_string.get_file());
 		AudioServer::get_singleton()->set_bus_layout(state);
 		_update_buses();
 		EditorNode::get_singleton()->get_undo_redo()->clear_history();
 		call_deferred("_select_layout");
 
 	} else if (file_dialog->get_mode() == EditorFileDialog::MODE_SAVE_FILE) {
-
 		if (new_layout) {
 			Ref<AudioBusLayout> empty_state;
 			empty_state.instance();
@@ -1308,7 +1254,7 @@ void EditorAudioBuses::_file_dialog_callback(const String &p_string) {
 		}
 
 		edited_path = p_string;
-		file->set_text(String(TTR("Layout")) + ": " + p_string.get_file());
+		file->set_text(String(TTR("Layout:")) + " " + p_string.get_file());
 		_update_buses();
 		EditorNode::get_singleton()->get_undo_redo()->clear_history();
 		call_deferred("_select_layout");
@@ -1316,7 +1262,6 @@ void EditorAudioBuses::_file_dialog_callback(const String &p_string) {
 }
 
 void EditorAudioBuses::_bind_methods() {
-
 	ClassDB::bind_method("_add_bus", &EditorAudioBuses::_add_bus);
 	ClassDB::bind_method("_update_buses", &EditorAudioBuses::_update_buses);
 	ClassDB::bind_method("_update_bus", &EditorAudioBuses::_update_bus);
@@ -1337,14 +1282,13 @@ void EditorAudioBuses::_bind_methods() {
 }
 
 EditorAudioBuses::EditorAudioBuses() {
-
-	drop_end = NULL;
+	drop_end = nullptr;
 	top_hb = memnew(HBoxContainer);
 	add_child(top_hb);
 
 	file = memnew(Label);
 	String layout_path = ProjectSettings::get_singleton()->get("audio/default_bus_layout");
-	file->set_text(String(TTR("Layout")) + ": " + layout_path.get_file());
+	file->set_text(String(TTR("Layout:")) + " " + layout_path.get_file());
 	file->set_clip_text(true);
 	file->set_h_size_flags(SIZE_EXPAND_FILL);
 	top_hb->add_child(file);
@@ -1405,7 +1349,7 @@ EditorAudioBuses::EditorAudioBuses() {
 	List<String> ext;
 	ResourceLoader::get_recognized_extensions_for_type("AudioBusLayout", &ext);
 	for (List<String>::Element *E = ext.front(); E; E = E->next()) {
-		file_dialog->add_filter("*." + E->get() + "; Audio Bus Layout");
+		file_dialog->add_filter(vformat("*.%s; %s", E->get(), TTR("Audio Bus Layout")));
 	}
 	add_child(file_dialog);
 	file_dialog->connect("file_selected", this, "_file_dialog_callback");
@@ -1414,7 +1358,6 @@ EditorAudioBuses::EditorAudioBuses() {
 }
 
 void EditorAudioBuses::open_layout(const String &p_path) {
-
 	EditorNode::get_singleton()->make_bottom_panel_item_visible(this);
 
 	Ref<AudioBusLayout> state = ResourceLoader::load(p_path, "", true);
@@ -1432,9 +1375,7 @@ void EditorAudioBuses::open_layout(const String &p_path) {
 }
 
 void AudioBusesEditorPlugin::edit(Object *p_node) {
-
 	if (Object::cast_to<AudioBusLayout>(p_node)) {
-
 		String path = Object::cast_to<AudioBusLayout>(p_node)->get_path();
 		if (path.is_resource_file()) {
 			audio_bus_editor->open_layout(path);
@@ -1443,15 +1384,13 @@ void AudioBusesEditorPlugin::edit(Object *p_node) {
 }
 
 bool AudioBusesEditorPlugin::handles(Object *p_node) const {
-
-	return (Object::cast_to<AudioBusLayout>(p_node) != NULL);
+	return (Object::cast_to<AudioBusLayout>(p_node) != nullptr);
 }
 
 void AudioBusesEditorPlugin::make_visible(bool p_visible) {
 }
 
 AudioBusesEditorPlugin::AudioBusesEditorPlugin(EditorAudioBuses *p_node) {
-
 	audio_bus_editor = p_node;
 }
 
@@ -1459,12 +1398,10 @@ AudioBusesEditorPlugin::~AudioBusesEditorPlugin() {
 }
 
 void EditorAudioMeterNotches::add_notch(float p_normalized_offset, float p_db_value, bool p_render_value) {
-
 	notches.push_back(AudioNotch(p_normalized_offset, p_db_value, p_render_value));
 }
 
 Size2 EditorAudioMeterNotches::get_minimum_size() const {
-
 	Ref<Font> font = get_font("font", "Label");
 	float font_height = font->get_height();
 
@@ -1483,16 +1420,14 @@ Size2 EditorAudioMeterNotches::get_minimum_size() const {
 }
 
 void EditorAudioMeterNotches::_bind_methods() {
-
 	ClassDB::bind_method("add_notch", &EditorAudioMeterNotches::add_notch);
 	ClassDB::bind_method("_draw_audio_notches", &EditorAudioMeterNotches::_draw_audio_notches);
 }
 
 void EditorAudioMeterNotches::_notification(int p_what) {
-
 	switch (p_what) {
 		case NOTIFICATION_THEME_CHANGED: {
-			notch_color = EditorSettings::get_singleton()->is_dark_theme() ? Color(1, 1, 1) : Color(0, 0, 0);
+			notch_color = get_color("font_color", "Editor");
 		} break;
 		case NOTIFICATION_DRAW: {
 			_draw_audio_notches();
@@ -1501,20 +1436,19 @@ void EditorAudioMeterNotches::_notification(int p_what) {
 }
 
 void EditorAudioMeterNotches::_draw_audio_notches() {
-
 	Ref<Font> font = get_font("font", "Label");
 	float font_height = font->get_height();
 
 	for (int i = 0; i < notches.size(); i++) {
 		AudioNotch n = notches[i];
 		draw_line(Vector2(0, (1.0f - n.relative_position) * (get_size().y - btm_padding - top_padding) + top_padding),
-				Vector2(line_length, (1.0f - n.relative_position) * (get_size().y - btm_padding - top_padding) + top_padding),
+				Vector2(line_length * EDSCALE, (1.0f - n.relative_position) * (get_size().y - btm_padding - top_padding) + top_padding),
 				notch_color,
-				1);
+				Math::round(EDSCALE));
 
 		if (n.render_db_value) {
 			draw_string(font,
-					Vector2(line_length + label_space,
+					Vector2((line_length + label_space) * EDSCALE,
 							(1.0f - n.relative_position) * (get_size().y - btm_padding - top_padding) + (font_height / 4) + top_padding),
 					String::num(Math::abs(n.db_value)) + "dB",
 					notch_color);
@@ -1522,11 +1456,6 @@ void EditorAudioMeterNotches::_draw_audio_notches() {
 	}
 }
 
-EditorAudioMeterNotches::EditorAudioMeterNotches() :
-		line_length(5.0f),
-		label_space(2.0f),
-		btm_padding(9.0f),
-		top_padding(5.0f) {
-
-	notch_color = EditorSettings::get_singleton()->is_dark_theme() ? Color(1, 1, 1) : Color(0, 0, 0);
+EditorAudioMeterNotches::EditorAudioMeterNotches() {
+	notch_color = get_color("font_color", "Editor");
 }
