@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  editor_help_search.cpp                                               */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  editor_help_search.cpp                                                */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "editor_help_search.h"
 
@@ -36,33 +36,33 @@
 #include "editor_scale.h"
 
 void EditorHelpSearch::_update_icons() {
-
 	search_box->set_right_icon(get_icon("Search", "EditorIcons"));
 	search_box->set_clear_button_enabled(true);
 	search_box->add_icon_override("right_icon", get_icon("Search", "EditorIcons"));
 	case_sensitive_button->set_icon(get_icon("MatchCase", "EditorIcons"));
 	hierarchy_button->set_icon(get_icon("ClassList", "EditorIcons"));
 
-	if (is_visible_in_tree())
+	if (is_visible_in_tree()) {
 		_update_results();
+	}
 }
 
 void EditorHelpSearch::_update_results() {
-
 	String term = search_box->get_text();
 
 	int search_flags = filter_combo->get_selected_id();
-	if (case_sensitive_button->is_pressed())
+	if (case_sensitive_button->is_pressed()) {
 		search_flags |= SEARCH_CASE_SENSITIVE;
-	if (hierarchy_button->is_pressed())
+	}
+	if (hierarchy_button->is_pressed()) {
 		search_flags |= SEARCH_SHOW_HIERARCHY;
+	}
 
 	search = Ref<Runner>(memnew(Runner(this, results_tree, term, search_flags)));
 	set_process(true);
 }
 
 void EditorHelpSearch::_search_box_gui_input(const Ref<InputEvent> &p_event) {
-
 	// Redirect up and down navigational key events to the results list.
 	Ref<InputEventKey> key = p_event;
 	if (key.is_valid()) {
@@ -79,20 +79,18 @@ void EditorHelpSearch::_search_box_gui_input(const Ref<InputEvent> &p_event) {
 }
 
 void EditorHelpSearch::_search_box_text_changed(const String &p_text) {
-
 	_update_results();
 }
 
 void EditorHelpSearch::_filter_combo_item_selected(int p_option) {
-
 	_update_results();
 }
 
 void EditorHelpSearch::_confirmed() {
-
 	TreeItem *item = results_tree->get_selected();
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	// Activate the script editor and emit the signal with the documentation link to display.
 	EditorNode::get_singleton()->set_visible_editor(EditorNode::EDITOR_SCRIPT);
@@ -103,35 +101,31 @@ void EditorHelpSearch::_confirmed() {
 }
 
 void EditorHelpSearch::_notification(int p_what) {
-
 	switch (p_what) {
 		case EditorSettings::NOTIFICATION_EDITOR_SETTINGS_CHANGED: {
-
 			_update_icons();
 		} break;
 		case NOTIFICATION_ENTER_TREE: {
-
 			connect("confirmed", this, "_confirmed");
 			_update_icons();
 		} break;
 		case NOTIFICATION_POPUP_HIDE: {
-
 			results_tree->call_deferred("clear"); // Wait for the Tree's mouse event propagation.
 			get_ok()->set_disabled(true);
 			EditorSettings::get_singleton()->set_project_metadata("dialog_bounds", "search_help", get_rect());
 		} break;
 		case NOTIFICATION_PROCESS: {
-
 			// Update background search.
 			if (search.is_valid()) {
 				if (search->work()) {
 					// Search done.
 
-					// Only point to the perfect match if it's a new search, and not just reopening a old one.
-					if (!old_search)
+					// Only point to the match if it's a new search, and not just reopening a old one.
+					if (!old_search) {
 						results_tree->ensure_cursor_is_visible();
-					else
+					} else {
 						old_search = false;
+					}
 
 					get_ok()->set_disabled(!results_tree->get_selected());
 
@@ -146,7 +140,6 @@ void EditorHelpSearch::_notification(int p_what) {
 }
 
 void EditorHelpSearch::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_update_results"), &EditorHelpSearch::_update_results);
 	ClassDB::bind_method(D_METHOD("_search_box_gui_input"), &EditorHelpSearch::_search_box_gui_input);
 	ClassDB::bind_method(D_METHOD("_search_box_text_changed"), &EditorHelpSearch::_search_box_text_changed);
@@ -156,26 +149,26 @@ void EditorHelpSearch::_bind_methods() {
 }
 
 void EditorHelpSearch::popup_dialog() {
-
 	popup_dialog(search_box->get_text());
 }
 
 void EditorHelpSearch::popup_dialog(const String &p_term) {
-
 	// Restore valid window bounds or pop up at default size.
 	Rect2 saved_size = EditorSettings::get_singleton()->get_project_metadata("dialog_bounds", "search_help", Rect2());
-	if (saved_size != Rect2())
+	if (saved_size != Rect2()) {
 		popup(saved_size);
-	else
+	} else {
 		popup_centered_ratio(0.5F);
+	}
 
 	if (p_term == "") {
 		search_box->clear();
 	} else {
-		if (old_term == p_term)
+		if (old_term == p_term) {
 			old_search = true;
-		else
+		} else {
 			old_term = p_term;
+		}
 
 		search_box->set_text(p_term);
 		search_box->select_all();
@@ -185,7 +178,6 @@ void EditorHelpSearch::popup_dialog(const String &p_term) {
 }
 
 EditorHelpSearch::EditorHelpSearch() {
-
 	old_search = false;
 
 	set_hide_on_ok(false);
@@ -257,7 +249,6 @@ EditorHelpSearch::EditorHelpSearch() {
 }
 
 bool EditorHelpSearch::Runner::_is_class_disabled_by_feature_profile(const StringName &p_class) {
-
 	Ref<EditorFeatureProfile> profile = EditorFeatureProfileManager::get_singleton()->get_current_profile();
 	if (profile.is_null()) {
 		return false;
@@ -265,7 +256,6 @@ bool EditorHelpSearch::Runner::_is_class_disabled_by_feature_profile(const Strin
 
 	StringName class_name = p_class;
 	while (class_name != StringName()) {
-
 		if (!ClassDB::class_exists(class_name)) {
 			return false;
 		}
@@ -280,7 +270,6 @@ bool EditorHelpSearch::Runner::_is_class_disabled_by_feature_profile(const Strin
 }
 
 bool EditorHelpSearch::Runner::_slice() {
-
 	bool phase_done = false;
 	switch (phase) {
 		case PHASE_MATCH_CLASSES_INIT:
@@ -307,41 +296,41 @@ bool EditorHelpSearch::Runner::_slice() {
 		case PHASE_MAX:
 			return true;
 		default:
-			WARN_PRINTS("Invalid or unhandled phase in EditorHelpSearch::Runner, aborting search.");
+			WARN_PRINT("Invalid or unhandled phase in EditorHelpSearch::Runner, aborting search.");
 			return true;
 	};
 
-	if (phase_done)
+	if (phase_done) {
 		phase++;
+	}
 	return false;
 }
 
 bool EditorHelpSearch::Runner::_phase_match_classes_init() {
-
 	iterator_doc = EditorHelp::get_doc_data()->class_list.front();
 	matches.clear();
-	matched_item = NULL;
+	matched_item = nullptr;
+	match_highest_score = 0;
 
 	return true;
 }
 
 bool EditorHelpSearch::Runner::_phase_match_classes() {
-
 	DocData::ClassDoc &class_doc = iterator_doc->value();
 	if (!_is_class_disabled_by_feature_profile(class_doc.name)) {
-
 		matches[class_doc.name] = ClassMatch();
 		ClassMatch &match = matches[class_doc.name];
 
 		match.doc = &class_doc;
 
 		// Match class name.
-		if (search_flags & SEARCH_CLASSES)
+		if (search_flags & SEARCH_CLASSES) {
 			match.name = term == "" || _match_string(term, class_doc.name);
+		}
 
 		// Match members if the term is long enough.
 		if (term.length() > 1) {
-			if (search_flags & SEARCH_METHODS)
+			if (search_flags & SEARCH_METHODS) {
 				for (int i = 0; i < class_doc.methods.size(); i++) {
 					String method_name = (search_flags & SEARCH_CASE_SENSITIVE) ? class_doc.methods[i].name : class_doc.methods[i].name.to_lower();
 					if (method_name.find(term) > -1 ||
@@ -351,22 +340,35 @@ bool EditorHelpSearch::Runner::_phase_match_classes() {
 						match.methods.push_back(const_cast<DocData::MethodDoc *>(&class_doc.methods[i]));
 					}
 				}
-			if (search_flags & SEARCH_SIGNALS)
-				for (int i = 0; i < class_doc.signals.size(); i++)
-					if (_match_string(term, class_doc.signals[i].name))
+			}
+			if (search_flags & SEARCH_SIGNALS) {
+				for (int i = 0; i < class_doc.signals.size(); i++) {
+					if (_match_string(term, class_doc.signals[i].name)) {
 						match.signals.push_back(const_cast<DocData::MethodDoc *>(&class_doc.signals[i]));
-			if (search_flags & SEARCH_CONSTANTS)
-				for (int i = 0; i < class_doc.constants.size(); i++)
-					if (_match_string(term, class_doc.constants[i].name))
+					}
+				}
+			}
+			if (search_flags & SEARCH_CONSTANTS) {
+				for (int i = 0; i < class_doc.constants.size(); i++) {
+					if (_match_string(term, class_doc.constants[i].name)) {
 						match.constants.push_back(const_cast<DocData::ConstantDoc *>(&class_doc.constants[i]));
-			if (search_flags & SEARCH_PROPERTIES)
-				for (int i = 0; i < class_doc.properties.size(); i++)
-					if (_match_string(term, class_doc.properties[i].name) || _match_string(term, class_doc.properties[i].getter) || _match_string(term, class_doc.properties[i].setter))
+					}
+				}
+			}
+			if (search_flags & SEARCH_PROPERTIES) {
+				for (int i = 0; i < class_doc.properties.size(); i++) {
+					if (_match_string(term, class_doc.properties[i].name) || _match_string(term, class_doc.properties[i].getter) || _match_string(term, class_doc.properties[i].setter)) {
 						match.properties.push_back(const_cast<DocData::PropertyDoc *>(&class_doc.properties[i]));
-			if (search_flags & SEARCH_THEME_ITEMS)
-				for (int i = 0; i < class_doc.theme_properties.size(); i++)
-					if (_match_string(term, class_doc.theme_properties[i].name))
-						match.theme_properties.push_back(const_cast<DocData::PropertyDoc *>(&class_doc.theme_properties[i]));
+					}
+				}
+			}
+			if (search_flags & SEARCH_THEME_ITEMS) {
+				for (int i = 0; i < class_doc.theme_properties.size(); i++) {
+					if (_match_string(term, class_doc.theme_properties[i].name)) {
+						match.theme_properties.push_back(const_cast<DocData::ThemeItemDoc *>(&class_doc.theme_properties[i]));
+					}
+				}
+			}
 		}
 	}
 
@@ -375,7 +377,6 @@ bool EditorHelpSearch::Runner::_phase_match_classes() {
 }
 
 bool EditorHelpSearch::Runner::_phase_class_items_init() {
-
 	iterator_match = matches.front();
 
 	results_tree->clear();
@@ -386,15 +387,16 @@ bool EditorHelpSearch::Runner::_phase_class_items_init() {
 }
 
 bool EditorHelpSearch::Runner::_phase_class_items() {
-
 	ClassMatch &match = iterator_match->value();
 
 	if (search_flags & SEARCH_SHOW_HIERARCHY) {
-		if (match.required())
+		if (match.required()) {
 			_create_class_hierarchy(match);
+		}
 	} else {
-		if (match.name)
+		if (match.name) {
 			_create_class_item(root_item, match.doc, false);
+		}
 	}
 
 	iterator_match = iterator_match->next();
@@ -402,36 +404,39 @@ bool EditorHelpSearch::Runner::_phase_class_items() {
 }
 
 bool EditorHelpSearch::Runner::_phase_member_items_init() {
-
 	iterator_match = matches.front();
 
 	return true;
 }
 
 bool EditorHelpSearch::Runner::_phase_member_items() {
-
 	ClassMatch &match = iterator_match->value();
 
 	TreeItem *parent = (search_flags & SEARCH_SHOW_HIERARCHY) ? class_items[match.doc->name] : root_item;
-	for (int i = 0; i < match.methods.size(); i++)
+	for (int i = 0; i < match.methods.size(); i++) {
 		_create_method_item(parent, match.doc, match.methods[i]);
-	for (int i = 0; i < match.signals.size(); i++)
+	}
+	for (int i = 0; i < match.signals.size(); i++) {
 		_create_signal_item(parent, match.doc, match.signals[i]);
-	for (int i = 0; i < match.constants.size(); i++)
+	}
+	for (int i = 0; i < match.constants.size(); i++) {
 		_create_constant_item(parent, match.doc, match.constants[i]);
-	for (int i = 0; i < match.properties.size(); i++)
+	}
+	for (int i = 0; i < match.properties.size(); i++) {
 		_create_property_item(parent, match.doc, match.properties[i]);
-	for (int i = 0; i < match.theme_properties.size(); i++)
+	}
+	for (int i = 0; i < match.theme_properties.size(); i++) {
 		_create_theme_property_item(parent, match.doc, match.theme_properties[i]);
+	}
 
 	iterator_match = iterator_match->next();
 	return !iterator_match;
 }
 
 bool EditorHelpSearch::Runner::_phase_select_match() {
-
-	if (matched_item)
+	if (matched_item) {
 		matched_item->select(0);
+	}
 	return true;
 }
 
@@ -444,22 +449,27 @@ bool EditorHelpSearch::Runner::_match_string(const String &p_term, const String
 }
 
 void EditorHelpSearch::Runner::_match_item(TreeItem *p_item, const String &p_text) {
+	float inverse_length = 1.f / float(p_text.length());
 
-	if (!matched_item) {
-		if (search_flags & SEARCH_CASE_SENSITIVE) {
-			if (p_text.casecmp_to(term) == 0)
-				matched_item = p_item;
-		} else {
-			if (p_text.nocasecmp_to(term) == 0)
-				matched_item = p_item;
-		}
+	// Favor types where search term is a substring close to the start of the type.
+	float w = 0.5f;
+	int pos = p_text.findn(term);
+	float score = (pos > -1) ? 1.0f - w * MIN(1, 3 * pos * inverse_length) : MAX(0.f, .9f - w);
+
+	// Favor shorter items: they resemble the search term more.
+	w = 0.1f;
+	score *= (1 - w) + w * (term.length() * inverse_length);
+
+	if (match_highest_score == 0 || score > match_highest_score) {
+		matched_item = p_item;
+		match_highest_score = score;
 	}
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_class_hierarchy(const ClassMatch &p_match) {
-
-	if (class_items.has(p_match.doc->name))
+	if (class_items.has(p_match.doc->name)) {
 		return class_items[p_match.doc->name];
+	}
 
 	// Ensure parent nodes are created first.
 	TreeItem *parent = root_item;
@@ -478,13 +488,13 @@ TreeItem *EditorHelpSearch::Runner::_create_class_hierarchy(const ClassMatch &p_
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_class_item(TreeItem *p_parent, const DocData::ClassDoc *p_doc, bool p_gray) {
-
 	Ref<Texture> icon = empty_icon;
-	if (ui_service->has_icon(p_doc->name, "EditorIcons"))
+	if (ui_service->has_icon(p_doc->name, "EditorIcons")) {
 		icon = ui_service->get_icon(p_doc->name, "EditorIcons");
-	else if (ClassDB::class_exists(p_doc->name) && ClassDB::is_parent_class(p_doc->name, "Object"))
+	} else if (ClassDB::class_exists(p_doc->name) && ClassDB::is_parent_class(p_doc->name, "Object")) {
 		icon = ui_service->get_icon("Object", "EditorIcons");
-	String tooltip = p_doc->brief_description.strip_edges();
+	}
+	String tooltip = DTR(p_doc->brief_description.strip_edges());
 
 	TreeItem *item = results_tree->create_item(p_parent);
 	item->set_icon(0, icon);
@@ -504,57 +514,55 @@ TreeItem *EditorHelpSearch::Runner::_create_class_item(TreeItem *p_parent, const
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_method_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::MethodDoc *p_doc) {
-
 	String tooltip = p_doc->return_type + " " + p_class_doc->name + "." + p_doc->name + "(";
 	for (int i = 0; i < p_doc->arguments.size(); i++) {
 		const DocData::ArgumentDoc &arg = p_doc->arguments[i];
 		tooltip += arg.type + " " + arg.name;
-		if (arg.default_value != "")
+		if (arg.default_value != "") {
 			tooltip += " = " + arg.default_value;
-		if (i < p_doc->arguments.size() - 1)
+		}
+		if (i < p_doc->arguments.size() - 1) {
 			tooltip += ", ";
+		}
 	}
 	tooltip += ")";
 	return _create_member_item(p_parent, p_class_doc->name, "MemberMethod", p_doc->name, TTRC("Method"), "method", tooltip);
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_signal_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::MethodDoc *p_doc) {
-
 	String tooltip = p_doc->return_type + " " + p_class_doc->name + "." + p_doc->name + "(";
 	for (int i = 0; i < p_doc->arguments.size(); i++) {
 		const DocData::ArgumentDoc &arg = p_doc->arguments[i];
 		tooltip += arg.type + " " + arg.name;
-		if (arg.default_value != "")
+		if (arg.default_value != "") {
 			tooltip += " = " + arg.default_value;
-		if (i < p_doc->arguments.size() - 1)
+		}
+		if (i < p_doc->arguments.size() - 1) {
 			tooltip += ", ";
+		}
 	}
 	tooltip += ")";
 	return _create_member_item(p_parent, p_class_doc->name, "MemberSignal", p_doc->name, TTRC("Signal"), "signal", tooltip);
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_constant_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::ConstantDoc *p_doc) {
-
 	String tooltip = p_class_doc->name + "." + p_doc->name;
 	return _create_member_item(p_parent, p_class_doc->name, "MemberConstant", p_doc->name, TTRC("Constant"), "constant", tooltip);
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_property_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::PropertyDoc *p_doc) {
-
 	String tooltip = p_doc->type + " " + p_class_doc->name + "." + p_doc->name;
 	tooltip += "\n    " + p_class_doc->name + "." + p_doc->setter + "(value) setter";
 	tooltip += "\n    " + p_class_doc->name + "." + p_doc->getter + "() getter";
 	return _create_member_item(p_parent, p_class_doc->name, "MemberProperty", p_doc->name, TTRC("Property"), "property", tooltip);
 }
 
-TreeItem *EditorHelpSearch::Runner::_create_theme_property_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::PropertyDoc *p_doc) {
-
+TreeItem *EditorHelpSearch::Runner::_create_theme_property_item(TreeItem *p_parent, const DocData::ClassDoc *p_class_doc, const DocData::ThemeItemDoc *p_doc) {
 	String tooltip = p_doc->type + " " + p_class_doc->name + "." + p_doc->name;
 	return _create_member_item(p_parent, p_class_doc->name, "MemberTheme", p_doc->name, TTRC("Theme Property"), "theme_item", tooltip);
 }
 
 TreeItem *EditorHelpSearch::Runner::_create_member_item(TreeItem *p_parent, const String &p_class_name, const String &p_icon, const String &p_name, const String &p_type, const String &p_metatype, const String &p_tooltip) {
-
 	Ref<Texture> icon;
 	String text;
 	if (search_flags & SEARCH_SHOW_HIERARCHY) {
@@ -584,12 +592,13 @@ TreeItem *EditorHelpSearch::Runner::_create_member_item(TreeItem *p_parent, cons
 }
 
 bool EditorHelpSearch::Runner::work(uint64_t slot) {
-
 	// Return true when the search has been completed, otherwise false.
 	const uint64_t until = OS::get_singleton()->get_ticks_usec() + slot;
-	while (!_slice())
-		if (OS::get_singleton()->get_ticks_usec() > until)
+	while (!_slice()) {
+		if (OS::get_singleton()->get_ticks_usec() > until) {
 			return false;
+		}
+	}
 	return true;
 }
 
