@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  find_in_files.cpp                                                    */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  find_in_files.cpp                                                     */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "find_in_files.h"
 
@@ -59,14 +59,14 @@ static bool is_text_char(CharType c) {
 }
 
 static bool find_next(const String &line, String pattern, int from, bool match_case, bool whole_words, int &out_begin, int &out_end) {
-
 	int end = from;
 
 	while (true) {
 		int begin = match_case ? line.find(pattern, end) : line.findn(pattern, end);
 
-		if (begin == -1)
+		if (begin == -1) {
 			return false;
+		}
 
 		end = begin + pattern.length();
 		out_begin = begin;
@@ -153,19 +153,18 @@ void FindInFiles::_process() {
 	// This part can be moved to a thread if needed
 
 	OS &os = *OS::get_singleton();
-	float time_before = os.get_ticks_msec();
+	uint64_t time_before = os.get_ticks_msec();
 	while (is_processing()) {
 		_iterate();
-		float elapsed = (os.get_ticks_msec() - time_before);
-		if (elapsed > 1000.0 / 120.0)
+		uint64_t elapsed = (os.get_ticks_msec() - time_before);
+		if (elapsed > 8) { // Process again after waiting 8 ticks
 			break;
+		}
 	}
 }
 
 void FindInFiles::_iterate() {
-
 	if (_folders_stack.size() != 0) {
-
 		// Scan folders first so we can build a list of files and have progress info later
 
 		PoolStringArray &folders_to_scan = _folders_stack.write[_folders_stack.size() - 1];
@@ -196,7 +195,6 @@ void FindInFiles::_iterate() {
 		}
 
 	} else if (_files_to_scan.size() != 0) {
-
 		// Then scan files
 
 		String fpath = _files_to_scan[_files_to_scan.size() - 1];
@@ -220,7 +218,6 @@ float FindInFiles::get_progress() const {
 }
 
 void FindInFiles::_scan_dir(String path, PoolStringArray &out_folders) {
-
 	DirAccessRef dir = DirAccess::open(path);
 	if (!dir) {
 		print_verbose("Cannot open directory! " + path);
@@ -232,19 +229,28 @@ void FindInFiles::_scan_dir(String path, PoolStringArray &out_folders) {
 	for (int i = 0; i < 1000; ++i) {
 		String file = dir->get_next();
 
-		if (file == "")
+		if (file == "") {
 			break;
+		}
 
-		// Ignore special dirs (such as .git and .import)
-		if (file == "." || file == ".." || file.begins_with("."))
-			continue;
-		if (dir->current_is_hidden())
-			continue;
+		// If there is a .gdignore file in the directory, don't bother searching it
+		if (file == ".gdignore") {
+			break;
+		}
 
-		if (dir->current_is_dir())
+		// Ignore special dirs (such as .git and project data directory)
+		String project_data_dir_name = ProjectSettings::get_singleton()->get_project_data_dir_name();
+		if (file.begins_with(".") || file == project_data_dir_name) {
+			continue;
+		}
+		if (dir->current_is_hidden()) {
+			continue;
+		}
+
+		if (dir->current_is_dir()) {
 			out_folders.append(file);
 
-		else {
+		} else {
 			String file_ext = file.get_extension();
 			if (_extension_filter.has(file_ext)) {
 				_files_to_scan.push_back(path.plus_file(file));
@@ -254,7 +260,6 @@ void FindInFiles::_scan_dir(String path, PoolStringArray &out_folders) {
 }
 
 void FindInFiles::_scan_file(String fpath) {
-
 	FileAccessRef f = FileAccess::open(fpath, FileAccess::READ);
 	if (!f) {
 		print_verbose(String("Cannot open file ") + fpath);
@@ -264,7 +269,6 @@ void FindInFiles::_scan_file(String fpath) {
 	int line_number = 0;
 
 	while (!f->eof_reached()) {
-
 		// line number starts at 1
 		++line_number;
 
@@ -282,7 +286,6 @@ void FindInFiles::_scan_file(String fpath) {
 }
 
 void FindInFiles::_bind_methods() {
-
 	ADD_SIGNAL(MethodInfo(SIGNAL_RESULT_FOUND,
 			PropertyInfo(Variant::STRING, "path"),
 			PropertyInfo(Variant::INT, "line_number"),
@@ -298,7 +301,6 @@ const char *FindInFilesDialog::SIGNAL_FIND_REQUESTED = "find_requested";
 const char *FindInFilesDialog::SIGNAL_REPLACE_REQUESTED = "replace_requested";
 
 FindInFilesDialog::FindInFilesDialog() {
-
 	set_custom_minimum_size(Size2(500 * EDSCALE, 0));
 	set_title(TTR("Find in Files"));
 
@@ -323,6 +325,17 @@ FindInFilesDialog::FindInFilesDialog() {
 	_search_text_line_edit->connect("text_entered", this, "_on_search_text_entered");
 	gc->add_child(_search_text_line_edit);
 
+	_replace_label = memnew(Label);
+	_replace_label->set_text(TTR("Replace:"));
+	_replace_label->hide();
+	gc->add_child(_replace_label);
+
+	_replace_text_line_edit = memnew(LineEdit);
+	_replace_text_line_edit->set_h_size_flags(SIZE_EXPAND_FILL);
+	_replace_text_line_edit->connect("text_entered", this, "_on_replace_text_entered");
+	_replace_text_line_edit->hide();
+	gc->add_child(_replace_text_line_edit);
+
 	gc->add_child(memnew(Control)); // Space to maintain the grid aligned.
 
 	{
@@ -383,6 +396,8 @@ FindInFilesDialog::FindInFilesDialog() {
 
 	Button *cancel_button = get_ok();
 	cancel_button->set_text(TTR("Cancel"));
+
+	_mode = SEARCH_MODE;
 }
 
 void FindInFilesDialog::set_search_text(String text) {
@@ -390,11 +405,40 @@ void FindInFilesDialog::set_search_text(String text) {
 	_on_search_text_modified(text);
 }
 
+void FindInFilesDialog::set_replace_text(String text) {
+	_replace_text_line_edit->set_text(text);
+}
+
+void FindInFilesDialog::set_find_in_files_mode(FindInFilesMode p_mode) {
+	if (_mode == p_mode) {
+		return;
+	}
+
+	_mode = p_mode;
+
+	if (p_mode == SEARCH_MODE) {
+		set_title(TTR("Find in Files"));
+		_replace_label->hide();
+		_replace_text_line_edit->hide();
+	} else if (p_mode == REPLACE_MODE) {
+		set_title(TTR("Replace in Files"));
+		_replace_label->show();
+		_replace_text_line_edit->show();
+	}
+
+	// After hiding some child controls, recalculate proper dialog size.
+	set_size(Size2(get_size().x, 0));
+}
+
 String FindInFilesDialog::get_search_text() const {
 	String text = _search_text_line_edit->get_text();
 	return text.strip_edges();
 }
 
+String FindInFilesDialog::get_replace_text() const {
+	return _replace_text_line_edit->get_text();
+}
+
 bool FindInFilesDialog::is_match_case() const {
 	return _match_case_checkbox->is_pressed();
 }
@@ -463,7 +507,6 @@ void FindInFilesDialog::custom_action(const String &p_action) {
 }
 
 void FindInFilesDialog::_on_search_text_modified(String text) {
-
 	ERR_FAIL_COND(!_find_button);
 	ERR_FAIL_COND(!_replace_button);
 
@@ -472,24 +515,43 @@ void FindInFilesDialog::_on_search_text_modified(String text) {
 }
 
 void FindInFilesDialog::_on_search_text_entered(String text) {
-	// This allows to trigger a global search without leaving the keyboard
-	if (!_find_button->is_disabled())
-		custom_action("find");
+	// This allows to trigger a global search without leaving the keyboard.
+	if (!_find_button->is_disabled()) {
+		if (_mode == SEARCH_MODE) {
+			custom_action("find");
+		}
+	}
+
+	if (!_replace_button->is_disabled()) {
+		if (_mode == REPLACE_MODE) {
+			custom_action("replace");
+		}
+	}
+}
+
+void FindInFilesDialog::_on_replace_text_entered(String text) {
+	// This allows to trigger a global search without leaving the keyboard.
+	if (!_replace_button->is_disabled()) {
+		if (_mode == REPLACE_MODE) {
+			custom_action("replace");
+		}
+	}
 }
 
 void FindInFilesDialog::_on_folder_selected(String path) {
 	int i = path.find("://");
-	if (i != -1)
+	if (i != -1) {
 		path = path.right(i + 3);
+	}
 	_folder_line_edit->set_text(path);
 }
 
 void FindInFilesDialog::_bind_methods() {
-
 	ClassDB::bind_method("_on_folder_button_pressed", &FindInFilesDialog::_on_folder_button_pressed);
 	ClassDB::bind_method("_on_folder_selected", &FindInFilesDialog::_on_folder_selected);
 	ClassDB::bind_method("_on_search_text_modified", &FindInFilesDialog::_on_search_text_modified);
 	ClassDB::bind_method("_on_search_text_entered", &FindInFilesDialog::_on_search_text_entered);
+	ClassDB::bind_method("_on_replace_text_entered", &FindInFilesDialog::_on_replace_text_entered);
 
 	ADD_SIGNAL(MethodInfo(SIGNAL_FIND_REQUESTED));
 	ADD_SIGNAL(MethodInfo(SIGNAL_REPLACE_REQUESTED));
@@ -500,7 +562,6 @@ const char *FindInFilesPanel::SIGNAL_RESULT_SELECTED = "result_selected";
 const char *FindInFilesPanel::SIGNAL_FILES_MODIFIED = "files_modified";
 
 FindInFilesPanel::FindInFilesPanel() {
-
 	_finder = memnew(FindInFiles);
 	_finder->connect(FindInFiles::SIGNAL_RESULT_FOUND, this, "_on_result_found");
 	_finder->connect(FindInFiles::SIGNAL_FINISHED, this, "_on_finished");
@@ -517,7 +578,7 @@ FindInFilesPanel::FindInFilesPanel() {
 		HBoxContainer *hbc = memnew(HBoxContainer);
 
 		Label *find_label = memnew(Label);
-		find_label->set_text(TTR("Find: "));
+		find_label->set_text(TTR("Find:"));
 		hbc->add_child(find_label);
 
 		_search_text_label = memnew(Label);
@@ -565,7 +626,7 @@ FindInFilesPanel::FindInFilesPanel() {
 		_replace_container = memnew(HBoxContainer);
 
 		Label *replace_label = memnew(Label);
-		replace_label->set_text(TTR("Replace: "));
+		replace_label->set_text(TTR("Replace:"));
 		_replace_container->add_child(replace_label);
 
 		_replace_line_edit = memnew(LineEdit);
@@ -574,7 +635,7 @@ FindInFilesPanel::FindInFilesPanel() {
 		_replace_container->add_child(_replace_line_edit);
 
 		_replace_all_button = memnew(Button);
-		_replace_all_button->set_text(TTR("Replace all (no undo)"));
+		_replace_all_button->set_text(TTR("Replace All (NO UNDO)"));
 		_replace_all_button->connect("pressed", this, "_on_replace_all_clicked");
 		_replace_container->add_child(_replace_all_button);
 
@@ -585,7 +646,6 @@ FindInFilesPanel::FindInFilesPanel() {
 }
 
 void FindInFilesPanel::set_with_replace(bool with_replace) {
-
 	_with_replace = with_replace;
 	_replace_container->set_visible(with_replace);
 
@@ -602,6 +662,10 @@ void FindInFilesPanel::set_with_replace(bool with_replace) {
 	}
 }
 
+void FindInFilesPanel::set_replace_text(String text) {
+	_replace_line_edit->set_text(text);
+}
+
 void FindInFilesPanel::clear() {
 	_file_items.clear();
 	_result_items.clear();
@@ -610,7 +674,6 @@ void FindInFilesPanel::clear() {
 }
 
 void FindInFilesPanel::start_search() {
-
 	clear();
 
 	_status_label->set_text(TTR("Searching..."));
@@ -627,7 +690,6 @@ void FindInFilesPanel::start_search() {
 }
 
 void FindInFilesPanel::stop_search() {
-
 	_finder->stop();
 
 	_status_label->set_text("");
@@ -640,15 +702,17 @@ void FindInFilesPanel::stop_search() {
 void FindInFilesPanel::_notification(int p_what) {
 	if (p_what == NOTIFICATION_PROCESS) {
 		_progress_bar->set_as_ratio(_finder->get_progress());
+	} else if (p_what == NOTIFICATION_THEME_CHANGED) {
+		_search_text_label->add_font_override("font", get_font("source", "EditorFonts"));
+		_results_display->add_font_override("font", get_font("source", "EditorFonts"));
 	}
 }
 
 void FindInFilesPanel::_on_result_found(String fpath, int line_number, int begin, int end, String text) {
-
 	TreeItem *file_item;
 	Map<String, TreeItem *>::Element *E = _file_items.find(fpath);
 
-	if (E == NULL) {
+	if (E == nullptr) {
 		file_item = _results_display->create_item();
 		file_item->set_text(0, fpath);
 		file_item->set_metadata(0, fpath);
@@ -694,14 +758,15 @@ void FindInFilesPanel::_on_result_found(String fpath, int line_number, int begin
 }
 
 void FindInFilesPanel::draw_result_text(Object *item_obj, Rect2 rect) {
-
 	TreeItem *item = Object::cast_to<TreeItem>(item_obj);
-	if (!item)
+	if (!item) {
 		return;
+	}
 
 	Map<TreeItem *, Result>::Element *E = _result_items.find(item);
-	if (!E)
+	if (!E) {
 		return;
+	}
 	Result r = E->value();
 	String item_text = item->get_text(_with_replace ? 1 : 0);
 	Ref<Font> font = _results_display->get_font("font");
@@ -712,12 +777,13 @@ void FindInFilesPanel::draw_result_text(Object *item_obj, Rect2 rect) {
 	match_rect.position.y += 1 * EDSCALE;
 	match_rect.size.y -= 2 * EDSCALE;
 
-	_results_display->draw_rect(match_rect, Color(0, 0, 0, 0.5));
-	// Text is drawn by Tree already
+	// Use the inverted accent color to help match rectangles stand out even on the currently selected line.
+	_results_display->draw_rect(match_rect, get_color("accent_color", "Editor").inverted() * Color(1, 1, 1, 0.5));
+
+	// Text is drawn by Tree already.
 }
 
 void FindInFilesPanel::_on_item_edited() {
-
 	TreeItem *item = _results_display->get_selected();
 
 	if (item->is_checked(0)) {
@@ -732,8 +798,19 @@ void FindInFilesPanel::_on_item_edited() {
 }
 
 void FindInFilesPanel::_on_finished() {
+	String results_text;
+	int result_count = _result_items.size();
+	int file_count = _file_items.size();
 
-	_status_label->set_text(TTR("Search complete"));
+	if (result_count == 1 && file_count == 1) {
+		results_text = vformat(TTR("%d match in %d file."), result_count, file_count);
+	} else if (result_count != 1 && file_count == 1) {
+		results_text = vformat(TTR("%d matches in %d file."), result_count, file_count);
+	} else {
+		results_text = vformat(TTR("%d matches in %d files."), result_count, file_count);
+	}
+
+	_status_label->set_text(results_text);
 	update_replace_buttons();
 	set_progress_visible(false);
 	_refresh_button->show();
@@ -749,12 +826,12 @@ void FindInFilesPanel::_on_cancel_button_clicked() {
 }
 
 void FindInFilesPanel::_on_result_selected() {
-
 	TreeItem *item = _results_display->get_selected();
 	Map<TreeItem *, Result>::Element *E = _result_items.find(item);
 
-	if (E == NULL)
+	if (E == nullptr) {
 		return;
+	}
 	Result r = E->value();
 
 	TreeItem *file_item = item->get_parent();
@@ -768,24 +845,22 @@ void FindInFilesPanel::_on_replace_text_changed(String text) {
 }
 
 void FindInFilesPanel::_on_replace_all_clicked() {
-
 	String replace_text = get_replace_text();
 
 	PoolStringArray modified_files;
 
 	for (Map<String, TreeItem *>::Element *E = _file_items.front(); E; E = E->next()) {
-
 		TreeItem *file_item = E->value();
 		String fpath = file_item->get_metadata(0);
 
 		Vector<Result> locations;
 		for (TreeItem *item = file_item->get_children(); item; item = item->get_next()) {
-
-			if (!item->is_checked(0))
+			if (!item->is_checked(0)) {
 				continue;
+			}
 
 			Map<TreeItem *, Result>::Element *F = _result_items.find(item);
-			ERR_FAIL_COND(F == NULL);
+			ERR_FAIL_COND(F == nullptr);
 			locations.push_back(F->value());
 		}
 
@@ -806,13 +881,11 @@ void FindInFilesPanel::_on_replace_all_clicked() {
 class ConservativeGetLine {
 public:
 	String get_line(FileAccess *f) {
-
 		_line_buffer.clear();
 
 		CharType c = f->get_8();
 
 		while (!f->eof_reached()) {
-
 			if (c == '\n') {
 				_line_buffer.push_back(c);
 				_line_buffer.push_back(0);
@@ -838,7 +911,6 @@ private:
 };
 
 void FindInFilesPanel::apply_replaces_in_file(String fpath, const Vector<Result> &locations, String new_text) {
-
 	// If the file is already open, I assume the editor will reload it.
 	// If there are unsaved changes, the user will be asked on focus,
 	// however that means either losing changes or losing replaces.
@@ -857,7 +929,6 @@ void FindInFilesPanel::apply_replaces_in_file(String fpath, const Vector<Result>
 	int offset = 0;
 
 	for (int i = 0; i < locations.size(); ++i) {
-
 		int repl_line_number = locations[i].line_number;
 
 		while (current_line < repl_line_number) {
@@ -899,11 +970,10 @@ void FindInFilesPanel::apply_replaces_in_file(String fpath, const Vector<Result>
 }
 
 String FindInFilesPanel::get_replace_text() {
-	return _replace_line_edit->get_text().strip_edges();
+	return _replace_line_edit->get_text();
 }
 
 void FindInFilesPanel::update_replace_buttons() {
-
 	bool disabled = _finder->is_searching();
 
 	_replace_all_button->set_disabled(disabled);
@@ -914,7 +984,6 @@ void FindInFilesPanel::set_progress_visible(bool visible) {
 }
 
 void FindInFilesPanel::_bind_methods() {
-
 	ClassDB::bind_method("_on_result_found", &FindInFilesPanel::_on_result_found);
 	ClassDB::bind_method("_on_item_edited", &FindInFilesPanel::_on_item_edited);
 	ClassDB::bind_method("_on_finished", &FindInFilesPanel::_on_finished);
