@@ -1,44 +1,53 @@
-/*************************************************************************/
-/*  asset_library_editor_plugin.cpp                                      */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  asset_library_editor_plugin.cpp                                       */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "asset_library_editor_plugin.h"
 
 #include "core/io/json.h"
+#include "core/io/stream_peer_ssl.h"
 #include "core/version.h"
 #include "editor/editor_node.h"
 #include "editor/editor_scale.h"
 #include "editor/editor_settings.h"
 #include "editor/project_settings_editor.h"
 
-void EditorAssetLibraryItem::configure(const String &p_title, int p_asset_id, const String &p_category, int p_category_id, const String &p_author, int p_author_id, const String &p_cost) {
+static inline void setup_http_request(HTTPRequest *request) {
+	request->set_use_threads(EDITOR_DEF("asset_library/use_threads", true));
 
+	const String proxy_host = EDITOR_GET("network/http_proxy/host");
+	const int proxy_port = EDITOR_GET("network/http_proxy/port");
+	request->set_http_proxy(proxy_host, proxy_port);
+	request->set_https_proxy(proxy_host, proxy_port);
+}
+
+void EditorAssetLibraryItem::configure(const String &p_title, int p_asset_id, const String &p_category, int p_category_id, const String &p_author, int p_author_id, const String &p_cost) {
 	title->set_text(p_title);
 	asset_id = p_asset_id;
 	category->set_text(p_category);
@@ -49,7 +58,6 @@ void EditorAssetLibraryItem::configure(const String &p_title, int p_asset_id, co
 }
 
 void EditorAssetLibraryItem::set_image(int p_type, int p_index, const Ref<Texture> &p_image) {
-
 	ERR_FAIL_COND(p_type != EditorAssetLibrary::IMAGE_QUEUE_ICON);
 	ERR_FAIL_COND(p_index != 0);
 
@@ -57,9 +65,7 @@ void EditorAssetLibraryItem::set_image(int p_type, int p_index, const Ref<Textur
 }
 
 void EditorAssetLibraryItem::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_ENTER_TREE) {
-
 		icon->set_normal_texture(get_icon("ProjectIconLoading", "EditorIcons"));
 		category->add_color_override("font_color", Color(0.5, 0.5, 0.5));
 		author->add_color_override("font_color", Color(0.5, 0.5, 0.5));
@@ -68,21 +74,17 @@ void EditorAssetLibraryItem::_notification(int p_what) {
 }
 
 void EditorAssetLibraryItem::_asset_clicked() {
-
 	emit_signal("asset_selected", asset_id);
 }
 
 void EditorAssetLibraryItem::_category_clicked() {
-
 	emit_signal("category_selected", category_id);
 }
 void EditorAssetLibraryItem::_author_clicked() {
-
 	emit_signal("author_selected", author_id);
 }
 
 void EditorAssetLibraryItem::_bind_methods() {
-
 	ClassDB::bind_method("set_image", &EditorAssetLibraryItem::set_image);
 	ClassDB::bind_method("_asset_clicked", &EditorAssetLibraryItem::_asset_clicked);
 	ClassDB::bind_method("_category_clicked", &EditorAssetLibraryItem::_category_clicked);
@@ -93,7 +95,6 @@ void EditorAssetLibraryItem::_bind_methods() {
 }
 
 EditorAssetLibraryItem::EditorAssetLibraryItem() {
-
 	Ref<StyleBoxEmpty> border;
 	border.instance();
 	border->set_default_margin(MARGIN_LEFT, 5 * EDSCALE);
@@ -146,16 +147,12 @@ EditorAssetLibraryItem::EditorAssetLibraryItem() {
 //////////////////////////////////////////////////////////////////////////////
 
 void EditorAssetLibraryItemDescription::set_image(int p_type, int p_index, const Ref<Texture> &p_image) {
-
 	switch (p_type) {
-
 		case EditorAssetLibrary::IMAGE_QUEUE_ICON: {
-
 			item->call("set_image", p_type, p_index, p_image);
 			icon = p_image;
 		} break;
 		case EditorAssetLibrary::IMAGE_QUEUE_THUMBNAIL: {
-
 			for (int i = 0; i < preview_images.size(); i++) {
 				if (preview_images[i].id == p_index) {
 					if (preview_images[i].is_video) {
@@ -185,7 +182,6 @@ void EditorAssetLibraryItemDescription::set_image(int p_type, int p_index, const
 			}
 		} break;
 		case EditorAssetLibrary::IMAGE_QUEUE_SCREENSHOT: {
-
 			for (int i = 0; i < preview_images.size(); i++) {
 				if (preview_images[i].id == p_index) {
 					preview_images.write[i].image = p_image;
@@ -237,7 +233,6 @@ void EditorAssetLibraryItemDescription::_preview_click(int p_id) {
 }
 
 void EditorAssetLibraryItemDescription::configure(const String &p_title, int p_asset_id, const String &p_category, int p_category_id, const String &p_author, int p_author_id, const String &p_cost, int p_version, const String &p_version_string, const String &p_description, const String &p_download_url, const String &p_browse_url, const String &p_sha256_hash) {
-
 	asset_id = p_asset_id;
 	title = p_title;
 	download_url = p_download_url;
@@ -251,11 +246,11 @@ void EditorAssetLibraryItemDescription::configure(const String &p_title, int p_a
 	description->pop();
 	description->add_text("\n" + TTR("Description:") + "\n\n");
 	description->append_bbcode(p_description);
+	description->set_selection_enabled(true);
 	set_title(p_title);
 }
 
 void EditorAssetLibraryItemDescription::add_preview(int p_id, bool p_video, const String &p_url) {
-
 	Preview preview;
 	preview.id = p_id;
 	preview.video_link = p_url;
@@ -276,7 +271,6 @@ void EditorAssetLibraryItemDescription::add_preview(int p_id, bool p_video, cons
 }
 
 EditorAssetLibraryItemDescription::EditorAssetLibraryItemDescription() {
-
 	HBoxContainer *hbox = memnew(HBoxContainer);
 	add_child(hbox);
 	VBoxContainer *desc_vbox = memnew(VBoxContainer);
@@ -298,12 +292,15 @@ EditorAssetLibraryItemDescription::EditorAssetLibraryItemDescription() {
 	hbox->add_child(previews_vbox);
 	previews_vbox->add_constant_override("separation", 15 * EDSCALE);
 	previews_vbox->set_v_size_flags(SIZE_EXPAND_FILL);
+	previews_vbox->set_h_size_flags(SIZE_EXPAND_FILL);
 
 	preview = memnew(TextureRect);
 	previews_vbox->add_child(preview);
 	preview->set_expand(true);
 	preview->set_stretch_mode(TextureRect::STRETCH_KEEP_ASPECT_CENTERED);
 	preview->set_custom_minimum_size(Size2(640 * EDSCALE, 345 * EDSCALE));
+	preview->set_v_size_flags(SIZE_EXPAND_FILL);
+	preview->set_h_size_flags(SIZE_EXPAND_FILL);
 
 	previews_bg = memnew(PanelContainer);
 	previews_vbox->add_child(previews_bg);
@@ -323,11 +320,9 @@ EditorAssetLibraryItemDescription::EditorAssetLibraryItemDescription() {
 ///////////////////////////////////////////////////////////////////////////////////
 
 void EditorAssetLibraryItemDownload::_http_download_completed(int p_status, int p_code, const PoolStringArray &headers, const PoolByteArray &p_data) {
-
 	String error_text;
 
 	switch (p_status) {
-
 		case HTTPRequest::RESULT_CHUNKED_BODY_SIZE_MISMATCH:
 		case HTTPRequest::RESULT_CONNECTION_ERROR:
 		case HTTPRequest::RESULT_BODY_SIZE_LIMIT_EXCEEDED: {
@@ -373,7 +368,7 @@ void EditorAssetLibraryItemDownload::_http_download_completed(int p_status, int
 				if (sha256 != download_sha256) {
 					error_text = TTR("Bad download hash, assuming file has been tampered with.") + "\n";
 					error_text += TTR("Expected:") + " " + sha256 + "\n" + TTR("Got:") + " " + download_sha256;
-					status->set_text(TTR("Failed sha256 hash check"));
+					status->set_text(TTR("Failed SHA-256 hash check"));
 				}
 			}
 		} break;
@@ -382,6 +377,8 @@ void EditorAssetLibraryItemDownload::_http_download_completed(int p_status, int
 	if (error_text != String()) {
 		download_error->set_text(TTR("Asset Download Error:") + "\n" + error_text);
 		download_error->popup_centered_minsize();
+		// Let the user retry the download.
+		retry->show();
 		return;
 	}
 
@@ -391,32 +388,31 @@ void EditorAssetLibraryItemDownload::_http_download_completed(int p_status, int
 	progress->set_modulate(Color(0, 0, 0, 0));
 
 	set_process(false);
+
+	// Automatically prompt for installation once the download is completed.
+	_install();
 }
 
 void EditorAssetLibraryItemDownload::configure(const String &p_title, int p_asset_id, const Ref<Texture> &p_preview, const String &p_download_url, const String &p_sha256_hash) {
-
 	title->set_text(p_title);
 	icon->set_texture(p_preview);
 	asset_id = p_asset_id;
-	if (!p_preview.is_valid())
+	if (!p_preview.is_valid()) {
 		icon->set_texture(get_icon("FileBrokenBigThumb", "EditorIcons"));
+	}
 	host = p_download_url;
 	sha256 = p_sha256_hash;
 	_make_request();
 }
 
 void EditorAssetLibraryItemDownload::_notification(int p_what) {
-
 	switch (p_what) {
-
 		// FIXME: The editor crashes if 'NOTICATION_THEME_CHANGED' is used.
 		case NOTIFICATION_ENTER_TREE: {
-
 			add_style_override("panel", get_stylebox("panel", "TabContainer"));
 			dismiss->set_normal_texture(get_icon("Close", "EditorIcons"));
 		} break;
 		case NOTIFICATION_PROCESS: {
-
 			// Make the progress bar visible again when retrying the download.
 			progress->set_modulate(Color(1, 1, 1, 1));
 
@@ -444,7 +440,6 @@ void EditorAssetLibraryItemDownload::_notification(int p_what) {
 
 			if (cstatus != prev_status) {
 				switch (cstatus) {
-
 					case HTTPClient::STATUS_RESOLVING: {
 						status->set_text(TTR("Resolving..."));
 						progress->set_max(1);
@@ -469,14 +464,12 @@ void EditorAssetLibraryItemDownload::_notification(int p_what) {
 	}
 }
 void EditorAssetLibraryItemDownload::_close() {
-
 	// Clean up downloaded file.
 	DirAccess::remove_file_or_error(download->get_download_file());
 	queue_delete();
 }
 
 void EditorAssetLibraryItemDownload::_install() {
-
 	String file = download->get_download_file();
 
 	if (external_install) {
@@ -484,10 +477,14 @@ void EditorAssetLibraryItemDownload::_install() {
 		return;
 	}
 
+	asset_installer->set_asset_name(title->get_text());
 	asset_installer->open(file, 1);
 }
 
 void EditorAssetLibraryItemDownload::_make_request() {
+	// Hide the Retry button if we've just pressed it.
+	retry->hide();
+
 	download->cancel_request();
 	download->set_download_file(EditorSettings::get_singleton()->get_cache_dir().plus_file("tmp_asset_" + itos(asset_id)) + ".zip");
 
@@ -500,7 +497,6 @@ void EditorAssetLibraryItemDownload::_make_request() {
 }
 
 void EditorAssetLibraryItemDownload::_bind_methods() {
-
 	ClassDB::bind_method("_http_download_completed", &EditorAssetLibraryItemDownload::_http_download_completed);
 	ClassDB::bind_method("_install", &EditorAssetLibraryItemDownload::_install);
 	ClassDB::bind_method("_close", &EditorAssetLibraryItemDownload::_close);
@@ -510,7 +506,6 @@ void EditorAssetLibraryItemDownload::_bind_methods() {
 }
 
 EditorAssetLibraryItemDownload::EditorAssetLibraryItemDownload() {
-
 	HBoxContainer *hb = memnew(HBoxContainer);
 	add_child(hb);
 	icon = memnew(TextureRect);
@@ -552,6 +547,8 @@ EditorAssetLibraryItemDownload::EditorAssetLibraryItemDownload() {
 	retry = memnew(Button);
 	retry->set_text(TTR("Retry"));
 	retry->connect("pressed", this, "_make_request");
+	// Only show the Retry button in case of a failure.
+	retry->hide();
 
 	hb2->add_child(retry);
 	hb2->add_child(install);
@@ -560,7 +557,7 @@ EditorAssetLibraryItemDownload::EditorAssetLibraryItemDownload() {
 	download = memnew(HTTPRequest);
 	add_child(download);
 	download->connect("request_completed", this, "_http_download_completed");
-	download->set_use_threads(EDITOR_DEF("asset_library/use_threads", true));
+	setup_http_request(download);
 
 	download_error = memnew(AcceptDialog);
 	add_child(download_error);
@@ -577,11 +574,8 @@ EditorAssetLibraryItemDownload::EditorAssetLibraryItemDownload() {
 
 ////////////////////////////////////////////////////////////////////////////////
 void EditorAssetLibrary::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_READY: {
-
 			error_tr->set_texture(get_icon("Error", "EditorIcons"));
 			filter->set_right_icon(get_icon("Search", "EditorIcons"));
 			filter->set_clear_button_enabled(true);
@@ -589,13 +583,18 @@ void EditorAssetLibrary::_notification(int p_what) {
 			error_label->raise();
 		} break;
 		case NOTIFICATION_VISIBILITY_CHANGED: {
+			if (is_visible()) {
+				// Focus the search box automatically when switching to the Templates tab (in the Project Manager)
+				// or switching to the AssetLib tab (in the editor).
+				// The Project Manager's project filter box is automatically focused in the project manager code.
+				filter->grab_focus();
 
-			if (is_visible() && initial_loading) {
-				_repository_changed(0); // Update when shown for the first time.
+				if (initial_loading) {
+					_repository_changed(0); // Update when shown for the first time.
+				}
 			}
 		} break;
 		case NOTIFICATION_PROCESS: {
-
 			HTTPClient::Status s = request->get_http_client_status();
 			const bool loading = s != HTTPClient::STATUS_DISCONNECTED;
 
@@ -612,27 +611,46 @@ void EditorAssetLibrary::_notification(int p_what) {
 
 		} break;
 		case NOTIFICATION_THEME_CHANGED: {
-
 			library_scroll_bg->add_style_override("panel", get_stylebox("bg", "Tree"));
 			downloads_scroll->add_style_override("bg", get_stylebox("bg", "Tree"));
 			error_tr->set_texture(get_icon("Error", "EditorIcons"));
 			filter->set_right_icon(get_icon("Search", "EditorIcons"));
 			filter->set_clear_button_enabled(true);
 		} break;
+
+		case NOTIFICATION_RESIZED: {
+			_update_asset_items_columns();
+		} break;
+
+		case EditorSettings::NOTIFICATION_EDITOR_SETTINGS_CHANGED: {
+			_update_repository_options();
+			setup_http_request(request);
+		} break;
+	}
+}
+
+void EditorAssetLibrary::_update_repository_options() {
+	Dictionary default_urls;
+	default_urls["godotengine.org (Official)"] = "https://godotengine.org/asset-library/api";
+	Dictionary available_urls = _EDITOR_DEF("asset_library/available_urls", default_urls, true);
+	repository->clear();
+	Array keys = available_urls.keys();
+	for (int i = 0; i < available_urls.size(); i++) {
+		String key = keys[i];
+		repository->add_item(key);
+		repository->set_item_metadata(i, available_urls[key]);
 	}
 }
 
 void EditorAssetLibrary::_install_asset() {
-
 	ERR_FAIL_COND(!description);
 
 	for (int i = 0; i < downloads_hb->get_child_count(); i++) {
-
 		EditorAssetLibraryItemDownload *d = Object::cast_to<EditorAssetLibraryItemDownload>(downloads_hb->get_child(i));
 		if (d && d->get_asset_id() == description->get_asset_id()) {
-
-			if (EditorNode::get_singleton() != NULL)
+			if (EditorNode::get_singleton() != nullptr) {
 				EditorNode::get_singleton()->show_warning(TTR("Download for this asset is already in progress!"));
+			}
 			return;
 		}
 	}
@@ -672,16 +690,14 @@ const char *EditorAssetLibrary::support_key[SUPPORT_MAX] = {
 };
 
 void EditorAssetLibrary::_select_author(int p_id) {
-
 	// Open author window.
 }
 
 void EditorAssetLibrary::_select_category(int p_id) {
-
 	for (int i = 0; i < categories->get_item_count(); i++) {
-
-		if (i == 0)
+		if (i == 0) {
 			continue;
+		}
 		int id = categories->get_item_metadata(i);
 		if (id == p_id) {
 			categories->select(i);
@@ -691,7 +707,6 @@ void EditorAssetLibrary::_select_category(int p_id) {
 	}
 }
 void EditorAssetLibrary::_select_asset(int p_id) {
-
 	_api_request("asset/" + itos(p_id), REQUESTING_ASSET);
 }
 
@@ -776,11 +791,9 @@ void EditorAssetLibrary::_image_update(bool use_cache, bool final, const PoolByt
 }
 
 void EditorAssetLibrary::_image_request_completed(int p_status, int p_code, const PoolStringArray &headers, const PoolByteArray &p_data, int p_queue_id) {
-
 	ERR_FAIL_COND(!image_queue.has(p_queue_id));
 
 	if (p_status == HTTPRequest::RESULT_SUCCESS && p_code < HTTPClient::RESPONSE_BAD_REQUEST) {
-
 		if (p_code != HTTPClient::RESPONSE_NOT_MODIFIED) {
 			for (int i = 0; i < headers.size(); i++) {
 				if (headers[i].findn("ETag:") == 0) { // Save etag
@@ -812,7 +825,7 @@ void EditorAssetLibrary::_image_request_completed(int p_status, int p_code, cons
 		_image_update(p_code == HTTPClient::RESPONSE_NOT_MODIFIED, true, p_data, p_queue_id);
 
 	} else {
-		WARN_PRINTS("Error getting image file from URL: " + image_queue[p_queue_id].image_url);
+		WARN_PRINT("Error getting image file from URL: " + image_queue[p_queue_id].image_url);
 		Object *obj = ObjectDB::get_instance(image_queue[p_queue_id].target);
 		if (obj) {
 			obj->call("set_image", image_queue[p_queue_id].image_type, image_queue[p_queue_id].image_index, get_icon("FileBrokenBigThumb", "EditorIcons"));
@@ -826,14 +839,12 @@ void EditorAssetLibrary::_image_request_completed(int p_status, int p_code, cons
 }
 
 void EditorAssetLibrary::_update_image_queue() {
-
 	const int max_images = 6;
 	int current_images = 0;
 
 	List<int> to_delete;
 	for (Map<int, ImageQueue>::Element *E = image_queue.front(); E; E = E->next()) {
 		if (!E->get().active && current_images < max_images) {
-
 			String cache_filename_base = EditorSettings::get_singleton()->get_cache_dir().plus_file("assetimage_" + E->get().image_url.md5_text());
 			Vector<String> headers;
 
@@ -866,13 +877,12 @@ void EditorAssetLibrary::_update_image_queue() {
 }
 
 void EditorAssetLibrary::_request_image(ObjectID p_for, String p_image_url, ImageType p_type, int p_image_index) {
-
 	ImageQueue iq;
 	iq.image_url = p_image_url;
 	iq.image_index = p_image_index;
 	iq.image_type = p_type;
 	iq.request = memnew(HTTPRequest);
-	iq.request->set_use_threads(EDITOR_DEF("asset_library/use_threads", true));
+	setup_http_request(iq.request);
 
 	iq.target = p_for;
 	iq.queue_id = ++last_queue_id;
@@ -889,6 +899,19 @@ void EditorAssetLibrary::_request_image(ObjectID p_for, String p_image_url, Imag
 }
 
 void EditorAssetLibrary::_repository_changed(int p_repository_id) {
+	library_error->hide();
+	library_info->set_text(TTR("Loading..."));
+	library_info->show();
+
+	asset_top_page->hide();
+	asset_bottom_page->hide();
+	asset_items->hide();
+
+	filter->set_editable(false);
+	sort->set_disabled(true);
+	categories->set_disabled(true);
+	support->set_disabled(true);
+
 	host = repository->get_item_metadata(p_repository_id);
 	if (templates_only) {
 		_api_request("configure", REQUESTING_CONFIG, "?type=project");
@@ -907,7 +930,6 @@ void EditorAssetLibrary::_rerun_search(int p_ignore) {
 }
 
 void EditorAssetLibrary::_search(int p_page) {
-
 	String args;
 
 	if (templates_only) {
@@ -931,7 +953,6 @@ void EditorAssetLibrary::_search(int p_page) {
 	}
 
 	if (categories->get_selected() > 0) {
-
 		args += "&category=" + itos(categories->get_item_metadata(categories->get_selected()));
 	}
 
@@ -951,31 +972,40 @@ void EditorAssetLibrary::_search(int p_page) {
 	_api_request("asset", REQUESTING_SEARCH, args);
 }
 
-void EditorAssetLibrary::_search_text_entered(const String &p_text) {
+void EditorAssetLibrary::_search_text_changed(const String &p_text) {
+	filter_debounce_timer->start();
+}
 
+void EditorAssetLibrary::_filter_debounce_timer_timeout() {
 	_search();
 }
 
-HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int p_page_len, int p_total_items, int p_current_items) {
+void EditorAssetLibrary::_request_current_config() {
+	_repository_changed(repository->get_selected());
+}
 
+HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int p_page_len, int p_total_items, int p_current_items) {
 	HBoxContainer *hbc = memnew(HBoxContainer);
 
-	if (p_page_count < 2)
+	if (p_page_count < 2) {
 		return hbc;
+	}
 
 	//do the mario
 	int from = p_page - 5;
-	if (from < 0)
+	if (from < 0) {
 		from = 0;
+	}
 	int to = from + 10;
-	if (to > p_page_count)
+	if (to > p_page_count) {
 		to = p_page_count;
+	}
 
 	hbc->add_spacer();
 	hbc->add_constant_override("separation", 5 * EDSCALE);
 
 	Button *first = memnew(Button);
-	first->set_text(TTR("First"));
+	first->set_text(TTR("First", "Pagination"));
 	if (p_page != 0) {
 		first->connect("pressed", this, "_search", varray(0));
 	} else {
@@ -985,7 +1015,7 @@ HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int
 	hbc->add_child(first);
 
 	Button *prev = memnew(Button);
-	prev->set_text(TTR("Previous"));
+	prev->set_text(TTR("Previous", "Pagination"));
 	if (p_page > 0) {
 		prev->connect("pressed", this, "_search", varray(p_page - 1));
 	} else {
@@ -996,19 +1026,18 @@ HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int
 	hbc->add_child(memnew(VSeparator));
 
 	for (int i = from; i < to; i++) {
-
 		if (i == p_page) {
-
 			Button *current = memnew(Button);
-			current->set_text(itos(i + 1));
+			// Keep the extended padding for the currently active page (see below).
+			current->set_text(vformat(" %d ", i + 1));
 			current->set_disabled(true);
 			current->set_focus_mode(Control::FOCUS_NONE);
 
 			hbc->add_child(current);
 		} else {
-
 			Button *current = memnew(Button);
-			current->set_text(itos(i + 1));
+			// Add padding to make page number buttons easier to click.
+			current->set_text(vformat(" %d ", i + 1));
 			current->connect("pressed", this, "_search", varray(i));
 
 			hbc->add_child(current);
@@ -1016,7 +1045,7 @@ HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int
 	}
 
 	Button *next = memnew(Button);
-	next->set_text(TTR("Next"));
+	next->set_text(TTR("Next", "Pagination"));
 	if (p_page < p_page_count - 1) {
 		next->connect("pressed", this, "_search", varray(p_page + 1));
 	} else {
@@ -1027,7 +1056,7 @@ HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int
 	hbc->add_child(next);
 
 	Button *last = memnew(Button);
-	last->set_text(TTR("Last"));
+	last->set_text(TTR("Last", "Pagination"));
 	if (p_page != p_page_count - 1) {
 		last->connect("pressed", this, "_search", varray(p_page_count - 1));
 	} else {
@@ -1042,7 +1071,6 @@ HBoxContainer *EditorAssetLibrary::_make_pages(int p_page, int p_page_count, int
 }
 
 void EditorAssetLibrary::_api_request(const String &p_request, RequestType p_request_type, const String &p_arguments) {
-
 	if (requesting != REQUESTING_NONE) {
 		request->cancel_request();
 	}
@@ -1054,7 +1082,6 @@ void EditorAssetLibrary::_api_request(const String &p_request, RequestType p_req
 }
 
 void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const PoolStringArray &headers, const PoolByteArray &p_data) {
-
 	String str;
 
 	{
@@ -1066,7 +1093,6 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 	bool error_abort = true;
 
 	switch (p_status) {
-
 		case HTTPRequest::RESULT_CANT_RESOLVE: {
 			error_label->set_text(TTR("Can't resolve hostname:") + " " + host);
 		} break;
@@ -1093,13 +1119,16 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 			if (p_code != 200) {
 				error_label->set_text(TTR("Request failed, return code:") + " " + itos(p_code));
 			} else {
-
 				error_abort = false;
 			}
 		} break;
 	}
 
 	if (error_abort) {
+		if (requesting == REQUESTING_CONFIG) {
+			library_info->hide();
+			library_error->show();
+		}
 		error_hb->show();
 		return;
 	}
@@ -1118,7 +1147,6 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 
 	switch (requested) {
 		case REQUESTING_CONFIG: {
-
 			categories->clear();
 			categories->add_item(TTR("All"));
 			categories->set_item_metadata(0, 0);
@@ -1126,8 +1154,9 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 				Array clist = d["categories"];
 				for (int i = 0; i < clist.size(); i++) {
 					Dictionary cat = clist[i];
-					if (!cat.has("name") || !cat.has("id"))
+					if (!cat.has("name") || !cat.has("id")) {
 						continue;
+					}
 					String name = cat["name"];
 					int id = cat["id"];
 					categories->add_item(name);
@@ -1136,18 +1165,16 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 				}
 			}
 
+			filter->set_editable(true);
+			sort->set_disabled(false);
+			categories->set_disabled(false);
+			support->set_disabled(false);
+
 			_search();
 		} break;
 		case REQUESTING_SEARCH: {
-
 			initial_loading = false;
 
-			// The loading text only needs to be displayed before the first page is loaded.
-			// Therefore, we don't need to show it again.
-			library_loading->hide();
-
-			library_error->hide();
-
 			if (asset_items) {
 				memdelete(asset_items);
 			}
@@ -1186,7 +1213,7 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 			library_vb->add_child(asset_top_page);
 
 			asset_items = memnew(GridContainer);
-			asset_items->set_columns(2);
+			_update_asset_items_columns();
 			asset_items->add_constant_override("hseparation", 10 * EDSCALE);
 			asset_items->add_constant_override("vseparation", 10 * EDSCALE);
 
@@ -1196,12 +1223,13 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 			library_vb->add_child(asset_bottom_page);
 
 			if (result.empty()) {
-				library_error->set_text(vformat(TTR("No results for \"%s\"."), filter->get_text()));
-				library_error->show();
+				library_info->set_text(vformat(TTR("No results for \"%s\"."), filter->get_text()));
+				library_info->show();
+			} else {
+				library_info->hide();
 			}
 
 			for (int i = 0; i < result.size(); i++) {
-
 				Dictionary r = result[i];
 
 				ERR_CONTINUE(!r.has("title"));
@@ -1223,6 +1251,10 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 					_request_image(item->get_instance_id(), r["icon_url"], IMAGE_QUEUE_ICON, 0);
 				}
 			}
+
+			if (!result.empty()) {
+				library_scroll->set_v_scroll(0);
+			}
 		} break;
 		case REQUESTING_ASSET: {
 			Dictionary r = d;
@@ -1260,7 +1292,6 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 				Array previews = d["previews"];
 
 				for (int i = 0; i < previews.size(); i++) {
-
 					Dictionary p = previews[i];
 
 					ERR_CONTINUE(!p.has("type"));
@@ -1284,51 +1315,59 @@ void EditorAssetLibrary::_http_request_completed(int p_status, int p_code, const
 				}
 			}
 		} break;
-		default: break;
+		default:
+			break;
 	}
 }
 
 void EditorAssetLibrary::_asset_file_selected(const String &p_file) {
-
 	if (asset_installer) {
 		memdelete(asset_installer);
-		asset_installer = NULL;
+		asset_installer = nullptr;
 	}
 
 	asset_installer = memnew(EditorAssetInstaller);
+	asset_installer->set_asset_name(p_file.get_basename());
 	add_child(asset_installer);
 	asset_installer->open(p_file);
 }
 
 void EditorAssetLibrary::_asset_open() {
-
 	asset_open->popup_centered_ratio();
 }
 
 void EditorAssetLibrary::_manage_plugins() {
-
 	ProjectSettingsEditor::get_singleton()->popup_project_settings();
 	ProjectSettingsEditor::get_singleton()->set_plugins_page();
 }
 
 void EditorAssetLibrary::_install_external_asset(String p_zip_path, String p_title) {
-
 	emit_signal("install_asset", p_zip_path, p_title);
 }
 
+void EditorAssetLibrary::_update_asset_items_columns() {
+	int new_columns = get_size().x / (450.0 * EDSCALE);
+	new_columns = MAX(1, new_columns);
+
+	if (new_columns != asset_items->get_columns()) {
+		asset_items->set_columns(new_columns);
+	}
+}
+
 void EditorAssetLibrary::disable_community_support() {
 	support->get_popup()->set_item_checked(SUPPORT_COMMUNITY, false);
 }
 
 void EditorAssetLibrary::_bind_methods() {
-
 	ClassDB::bind_method("_http_request_completed", &EditorAssetLibrary::_http_request_completed);
 	ClassDB::bind_method("_select_asset", &EditorAssetLibrary::_select_asset);
 	ClassDB::bind_method("_select_author", &EditorAssetLibrary::_select_author);
 	ClassDB::bind_method("_select_category", &EditorAssetLibrary::_select_category);
 	ClassDB::bind_method("_image_request_completed", &EditorAssetLibrary::_image_request_completed);
 	ClassDB::bind_method("_search", &EditorAssetLibrary::_search, DEFVAL(0));
-	ClassDB::bind_method("_search_text_entered", &EditorAssetLibrary::_search_text_entered);
+	ClassDB::bind_method("_search_text_changed", &EditorAssetLibrary::_search_text_changed);
+	ClassDB::bind_method("_filter_debounce_timer_timeout", &EditorAssetLibrary::_filter_debounce_timer_timeout);
+	ClassDB::bind_method("_request_current_config", &EditorAssetLibrary::_request_current_config);
 	ClassDB::bind_method("_install_asset", &EditorAssetLibrary::_install_asset);
 	ClassDB::bind_method("_manage_plugins", &EditorAssetLibrary::_manage_plugins);
 	ClassDB::bind_method("_asset_open", &EditorAssetLibrary::_asset_open);
@@ -1342,7 +1381,6 @@ void EditorAssetLibrary::_bind_methods() {
 }
 
 EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
-
 	requesting = REQUESTING_NONE;
 	templates_only = p_templates_only;
 	initial_loading = true;
@@ -1357,15 +1395,26 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	library_main->add_constant_override("separation", 10 * EDSCALE);
 
 	filter = memnew(LineEdit);
+	if (templates_only) {
+		filter->set_placeholder(TTR("Search templates, projects, and demos"));
+	} else {
+		filter->set_placeholder(TTR("Search assets (excluding templates, projects, and demos)"));
+	}
 	search_hb->add_child(filter);
 	filter->set_h_size_flags(SIZE_EXPAND_FILL);
-	filter->connect("text_entered", this, "_search_text_entered");
-	search = memnew(Button(TTR("Search")));
-	search->connect("pressed", this, "_search");
-	search_hb->add_child(search);
+	filter->connect("text_changed", this, "_search_text_changed");
 
-	if (!p_templates_only)
+	// Perform a search automatically if the user hasn't entered any text for a certain duration.
+	// This way, the user doesn't need to press Enter to initiate their search.
+	filter_debounce_timer = memnew(Timer);
+	filter_debounce_timer->set_one_shot(true);
+	filter_debounce_timer->set_wait_time(0.25);
+	filter_debounce_timer->connect("timeout", this, "_filter_debounce_timer_timeout");
+	search_hb->add_child(filter_debounce_timer);
+
+	if (!p_templates_only) {
 		search_hb->add_child(memnew(VSeparator));
+	}
 
 	Button *open_asset = memnew(Button);
 	open_asset->set_text(TTR("Import..."));
@@ -1394,8 +1443,8 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	search_hb2->add_child(sort);
 
 	sort->set_h_size_flags(SIZE_EXPAND_FILL);
+	sort->set_clip_text(true);
 	sort->connect("item_selected", this, "_rerun_search");
-
 	search_hb2->add_child(memnew(VSeparator));
 
 	search_hb2->add_child(memnew(Label(TTR("Category:") + " ")));
@@ -1403,6 +1452,7 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	categories->add_item(TTR("All"));
 	search_hb2->add_child(categories);
 	categories->set_h_size_flags(SIZE_EXPAND_FILL);
+	categories->set_clip_text(true);
 	categories->connect("item_selected", this, "_rerun_search");
 
 	search_hb2->add_child(memnew(VSeparator));
@@ -1410,21 +1460,20 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	search_hb2->add_child(memnew(Label(TTR("Site:") + " ")));
 	repository = memnew(OptionButton);
 
-	repository->add_item("godotengine.org");
-	repository->set_item_metadata(0, "https://godotengine.org/asset-library/api");
-	repository->add_item("localhost");
-	repository->set_item_metadata(1, "http://127.0.0.1/asset-library/api");
+	_update_repository_options();
 
 	repository->connect("item_selected", this, "_repository_changed");
 
 	search_hb2->add_child(repository);
 	repository->set_h_size_flags(SIZE_EXPAND_FILL);
+	repository->set_clip_text(true);
 
 	search_hb2->add_child(memnew(VSeparator));
 
 	support = memnew(MenuButton);
 	search_hb2->add_child(support);
 	support->set_text(TTR("Support"));
+	support->get_popup()->set_hide_on_checkable_item_selection(false);
 	support->get_popup()->add_check_item(TTR("Official"), SUPPORT_OFFICIAL);
 	support->get_popup()->add_check_item(TTR("Community"), SUPPORT_COMMUNITY);
 	support->get_popup()->add_check_item(TTR("Testing"), SUPPORT_TESTING);
@@ -1462,20 +1511,28 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 
 	library_vb_border->add_child(library_vb);
 
-	library_loading = memnew(Label(TTR("Loading...")));
-	library_loading->set_align(Label::ALIGN_CENTER);
-	library_vb->add_child(library_loading);
+	library_info = memnew(Label);
+	library_info->set_align(Label::ALIGN_CENTER);
+	library_vb->add_child(library_info);
 
-	library_error = memnew(Label);
-	library_error->set_align(Label::ALIGN_CENTER);
+	library_error = memnew(VBoxContainer);
 	library_error->hide();
 	library_vb->add_child(library_error);
 
+	library_error_label = memnew(Label(TTR("Failed to get repository configuration.")));
+	library_error_label->set_align(Label::ALIGN_CENTER);
+	library_error->add_child(library_error_label);
+
+	library_error_retry = memnew(Button(TTR("Retry")));
+	library_error_retry->set_h_size_flags(SIZE_SHRINK_CENTER);
+	library_error_retry->connect("pressed", this, "_request_current_config");
+	library_error->add_child(library_error_retry);
+
 	asset_top_page = memnew(HBoxContainer);
 	library_vb->add_child(asset_top_page);
 
 	asset_items = memnew(GridContainer);
-	asset_items->set_columns(2);
+	_update_asset_items_columns();
 	asset_items->add_constant_override("hseparation", 10 * EDSCALE);
 	asset_items->add_constant_override("vseparation", 10 * EDSCALE);
 
@@ -1486,7 +1543,7 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 
 	request = memnew(HTTPRequest);
 	add_child(request);
-	request->set_use_threads(EDITOR_DEF("asset_library/use_threads", true));
+	setup_http_request(request);
 	request->connect("request_completed", this, "_http_request_completed");
 
 	last_queue_id = 0;
@@ -1502,7 +1559,7 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	error_tr->set_v_size_flags(Control::SIZE_SHRINK_CENTER);
 	error_hb->add_child(error_tr);
 
-	description = NULL;
+	description = nullptr;
 
 	set_process(true);
 
@@ -1521,24 +1578,30 @@ EditorAssetLibrary::EditorAssetLibrary(bool p_templates_only) {
 	add_child(asset_open);
 	asset_open->connect("file_selected", this, "_asset_file_selected");
 
-	asset_installer = NULL;
+	asset_installer = nullptr;
 }
 
 ///////
 
+bool AssetLibraryEditorPlugin::is_available() {
+#ifdef JAVASCRIPT_ENABLED
+	// Asset Library can't work on Web editor for now as most assets are sourced
+	// directly from GitHub which does not set CORS.
+	return false;
+#else
+	return StreamPeerSSL::is_available();
+#endif
+}
+
 void AssetLibraryEditorPlugin::make_visible(bool p_visible) {
-
 	if (p_visible) {
-
 		addon_library->show();
 	} else {
-
 		addon_library->hide();
 	}
 }
 
 AssetLibraryEditorPlugin::AssetLibraryEditorPlugin(EditorNode *p_node) {
-
 	editor = p_node;
 	addon_library = memnew(EditorAssetLibrary);
 	addon_library->set_v_size_flags(Control::SIZE_EXPAND_FILL);
