@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  collision_polygon_editor_plugin.cpp                                  */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  collision_polygon_editor_plugin.cpp                                   */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "collision_polygon_editor_plugin.h"
 
@@ -39,11 +39,8 @@
 #include "spatial_editor_plugin.h"
 
 void Polygon3DEditor::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_READY: {
-
 			button_create->set_icon(get_icon("Edit", "EditorIcons"));
 			button_edit->set_icon(get_icon("MovePoint", "EditorIcons"));
 			button_edit->set_pressed(true);
@@ -64,28 +61,24 @@ void Polygon3DEditor::_notification(int p_what) {
 	}
 }
 void Polygon3DEditor::_node_removed(Node *p_node) {
-
 	if (p_node == node) {
-		node = NULL;
-		if (imgeom->get_parent() == p_node)
+		node = nullptr;
+		if (imgeom->get_parent() == p_node) {
 			p_node->remove_child(imgeom);
+		}
 		hide();
 		set_process(false);
 	}
 }
 
 void Polygon3DEditor::_menu_option(int p_option) {
-
 	switch (p_option) {
-
 		case MODE_CREATE: {
-
 			mode = MODE_CREATE;
 			button_create->set_pressed(true);
 			button_edit->set_pressed(false);
 		} break;
 		case MODE_EDIT: {
-
 			mode = MODE_EDIT;
 			button_create->set_pressed(false);
 			button_edit->set_pressed(true);
@@ -94,7 +87,6 @@ void Polygon3DEditor::_menu_option(int p_option) {
 }
 
 void Polygon3DEditor::_wip_close() {
-
 	undo_redo->create_action(TTR("Create Polygon3D"));
 	undo_redo->add_undo_method(node, "set_polygon", node->call("get_polygon"));
 	undo_redo->add_do_method(node, "set_polygon", wip);
@@ -110,9 +102,9 @@ void Polygon3DEditor::_wip_close() {
 }
 
 bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<InputEvent> &p_event) {
-
-	if (!node)
+	if (!node) {
 		return false;
+	}
 
 	Transform gt = node->get_global_transform();
 	Transform gi = gt.affine_inverse();
@@ -123,15 +115,15 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 	Ref<InputEventMouseButton> mb = p_event;
 
 	if (mb.is_valid()) {
-
 		Vector2 gpoint = mb->get_position();
 		Vector3 ray_from = p_camera->project_ray_origin(gpoint);
 		Vector3 ray_dir = p_camera->project_ray_normal(gpoint);
 
 		Vector3 spoint;
 
-		if (!p.intersects_ray(ray_from, ray_dir, &spoint))
+		if (!p.intersects_ray(ray_from, ray_dir, &spoint)) {
 			return false;
+		}
 
 		spoint = gi.xform(spoint);
 
@@ -147,13 +139,9 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 		real_t grab_threshold = EDITOR_GET("editors/poly_editor/point_grab_radius");
 
 		switch (mode) {
-
 			case MODE_CREATE: {
-
 				if (mb->get_button_index() == BUTTON_LEFT && mb->is_pressed()) {
-
 					if (!wip_active) {
-
 						wip.clear();
 						wip.push_back(cpoint);
 						wip_active = true;
@@ -163,14 +151,12 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 						edited_point = 1;
 						return true;
 					} else {
-
 						if (wip.size() > 1 && p_camera->unproject_position(gt.xform(Vector3(wip[0].x, wip[0].y, depth))).distance_to(gpoint) < grab_threshold) {
 							//wip closed
 							_wip_close();
 
 							return true;
 						} else {
-
 							wip.push_back(cpoint);
 							edited_point = wip.size();
 							snap_ignore = false;
@@ -185,14 +171,10 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 			} break;
 
 			case MODE_EDIT: {
-
 				if (mb->get_button_index() == BUTTON_LEFT) {
 					if (mb->is_pressed()) {
-
 						if (mb->get_control()) {
-
 							if (poly.size() < 3) {
-
 								undo_redo->create_action(TTR("Edit Poly"));
 								undo_redo->add_undo_method(node, "set_polygon", poly);
 								poly.push_back(cpoint);
@@ -208,15 +190,15 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 							Vector2 closest_pos;
 							real_t closest_dist = 1e10;
 							for (int i = 0; i < poly.size(); i++) {
-
 								Vector2 points[2] = {
 									p_camera->unproject_position(gt.xform(Vector3(poly[i].x, poly[i].y, depth))),
 									p_camera->unproject_position(gt.xform(Vector3(poly[(i + 1) % poly.size()].x, poly[(i + 1) % poly.size()].y, depth)))
 								};
 
 								Vector2 cp = Geometry::get_closest_point_to_segment_2d(gpoint, points);
-								if (cp.distance_squared_to(points[0]) < CMP_EPSILON2 || cp.distance_squared_to(points[1]) < CMP_EPSILON2)
+								if (cp.distance_squared_to(points[0]) < CMP_EPSILON2 || cp.distance_squared_to(points[1]) < CMP_EPSILON2) {
 									continue; //not valid to reuse point
+								}
 
 								real_t d = cp.distance_to(gpoint);
 								if (d < closest_dist && d < grab_threshold) {
@@ -227,7 +209,6 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 							}
 
 							if (closest_idx >= 0) {
-
 								pre_move_edit = poly;
 								poly.insert(closest_idx + 1, cpoint);
 								edited_point = closest_idx + 1;
@@ -239,14 +220,12 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 								return true;
 							}
 						} else {
-
 							//look for points to move
 
 							int closest_idx = -1;
 							Vector2 closest_pos;
 							real_t closest_dist = 1e10;
 							for (int i = 0; i < poly.size(); i++) {
-
 								Vector2 cp = p_camera->unproject_position(gt.xform(Vector3(poly[i].x, poly[i].y, depth)));
 
 								real_t d = cp.distance_to(gpoint);
@@ -258,7 +237,6 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 							}
 
 							if (closest_idx >= 0) {
-
 								pre_move_edit = poly;
 								edited_point = closest_idx;
 								edited_point_pos = poly[closest_idx];
@@ -268,11 +246,9 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 							}
 						}
 					} else {
-
 						snap_ignore = false;
 
 						if (edited_point != -1) {
-
 							//apply
 
 							ERR_FAIL_INDEX_V(edited_point, poly.size(), false);
@@ -290,12 +266,10 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 					}
 				}
 				if (mb->get_button_index() == BUTTON_RIGHT && mb->is_pressed() && edited_point == -1) {
-
 					int closest_idx = -1;
 					Vector2 closest_pos;
 					real_t closest_dist = 1e10;
 					for (int i = 0; i < poly.size(); i++) {
-
 						Vector2 cp = p_camera->unproject_position(gt.xform(Vector3(poly[i].x, poly[i].y, depth)));
 
 						real_t d = cp.distance_to(gpoint);
@@ -307,7 +281,6 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 					}
 
 					if (closest_idx >= 0) {
-
 						undo_redo->create_action(TTR("Edit Poly (Remove Point)"));
 						undo_redo->add_undo_method(node, "set_polygon", poly);
 						poly.remove(closest_idx);
@@ -327,7 +300,6 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 
 	if (mm.is_valid()) {
 		if (edited_point != -1 && (wip_active || mm->get_button_mask() & BUTTON_MASK_LEFT)) {
-
 			Vector2 gpoint = mm->get_position();
 
 			Vector3 ray_from = p_camera->project_ray_origin(gpoint);
@@ -335,8 +307,9 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 
 			Vector3 spoint;
 
-			if (!p.intersects_ray(ray_from, ray_dir, &spoint))
+			if (!p.intersects_ray(ray_from, ray_dir, &spoint)) {
 				return false;
+			}
 
 			spoint = gi.xform(spoint);
 
@@ -361,24 +334,25 @@ bool Polygon3DEditor::forward_spatial_gui_input(Camera *p_camera, const Ref<Inpu
 }
 
 float Polygon3DEditor::_get_depth() {
-
-	if (bool(node->call("_has_editable_3d_polygon_no_depth")))
+	if (bool(node->call("_has_editable_3d_polygon_no_depth"))) {
 		return 0;
+	}
 
 	return float(node->call("get_depth"));
 }
 
 void Polygon3DEditor::_polygon_draw() {
-
-	if (!node)
+	if (!node) {
 		return;
+	}
 
 	Vector<Vector2> poly;
 
-	if (wip_active)
+	if (wip_active) {
 		poly = wip;
-	else
+	} else {
 		poly = node->call("get_polygon");
+	}
 
 	float depth = _get_depth() * 0.5;
 
@@ -389,18 +363,19 @@ void Polygon3DEditor::_polygon_draw() {
 	Rect2 rect;
 
 	for (int i = 0; i < poly.size(); i++) {
-
 		Vector2 p, p2;
 		p = i == edited_point ? edited_point_pos : poly[i];
-		if ((wip_active && i == poly.size() - 1) || (((i + 1) % poly.size()) == edited_point))
+		if ((wip_active && i == poly.size() - 1) || (((i + 1) % poly.size()) == edited_point)) {
 			p2 = edited_point_pos;
-		else
+		} else {
 			p2 = poly[(i + 1) % poly.size()];
+		}
 
-		if (i == 0)
+		if (i == 0) {
 			rect.position = p;
-		else
+		} else {
 			rect.expand_to(p);
+		}
 
 		Vector3 point = Vector3(p.x, p.y, depth);
 		Vector3 next_point = Vector3(p2.x, p2.y, depth);
@@ -467,18 +442,17 @@ void Polygon3DEditor::_polygon_draw() {
 		m->surface_remove(0);
 	}
 
-	if (poly.size() == 0)
+	if (poly.size() == 0) {
 		return;
+	}
 
 	Array a;
 	a.resize(Mesh::ARRAY_MAX);
 	PoolVector<Vector3> va;
 	{
-
 		va.resize(poly.size());
 		PoolVector<Vector3>::Write w = va.write();
 		for (int i = 0; i < poly.size(); i++) {
-
 			Vector2 p, p2;
 			p = i == edited_point ? edited_point_pos : poly[i];
 
@@ -492,9 +466,7 @@ void Polygon3DEditor::_polygon_draw() {
 }
 
 void Polygon3DEditor::edit(Node *p_collision_polygon) {
-
 	if (p_collision_polygon) {
-
 		node = Object::cast_to<Spatial>(p_collision_polygon);
 		//Enable the pencil tool if the polygon is empty
 		if (Vector<Vector2>(node->call("get_polygon")).size() == 0) {
@@ -509,25 +481,24 @@ void Polygon3DEditor::edit(Node *p_collision_polygon) {
 		prev_depth = -1;
 
 	} else {
-		node = NULL;
+		node = nullptr;
 
-		if (imgeom->get_parent())
+		if (imgeom->get_parent()) {
 			imgeom->get_parent()->remove_child(imgeom);
+		}
 
 		set_process(false);
 	}
 }
 
 void Polygon3DEditor::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_menu_option"), &Polygon3DEditor::_menu_option);
 	ClassDB::bind_method(D_METHOD("_polygon_draw"), &Polygon3DEditor::_polygon_draw);
 	ClassDB::bind_method(D_METHOD("_node_removed"), &Polygon3DEditor::_node_removed);
 }
 
 Polygon3DEditor::Polygon3DEditor(EditorNode *p_editor) {
-
-	node = NULL;
+	node = nullptr;
 	editor = p_editor;
 	undo_redo = EditorNode::get_undo_redo();
 
@@ -575,33 +546,27 @@ Polygon3DEditor::Polygon3DEditor(EditorNode *p_editor) {
 }
 
 Polygon3DEditor::~Polygon3DEditor() {
-
 	memdelete(imgeom);
 }
 
 void Polygon3DEditorPlugin::edit(Object *p_object) {
-
 	collision_polygon_editor->edit(Object::cast_to<Node>(p_object));
 }
 
 bool Polygon3DEditorPlugin::handles(Object *p_object) const {
-
 	return Object::cast_to<Spatial>(p_object) && bool(p_object->call("_is_editable_3d_polygon"));
 }
 
 void Polygon3DEditorPlugin::make_visible(bool p_visible) {
-
 	if (p_visible) {
 		collision_polygon_editor->show();
 	} else {
-
 		collision_polygon_editor->hide();
-		collision_polygon_editor->edit(NULL);
+		collision_polygon_editor->edit(nullptr);
 	}
 }
 
 Polygon3DEditorPlugin::Polygon3DEditorPlugin(EditorNode *p_node) {
-
 	editor = p_node;
 	collision_polygon_editor = memnew(Polygon3DEditor(p_node));
 	SpatialEditor::get_singleton()->add_control_to_menu_panel(collision_polygon_editor);
