@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  curve_editor_plugin.cpp                                              */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  curve_editor_plugin.cpp                                               */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "curve_editor_plugin.h"
 
@@ -65,9 +65,9 @@ CurveEditor::CurveEditor() {
 }
 
 void CurveEditor::set_curve(Ref<Curve> curve) {
-
-	if (curve == _curve_ref)
+	if (curve == _curve_ref) {
 		return;
+	}
 
 	if (_curve_ref.is_valid()) {
 		_curve_ref->disconnect(CoreStringNames::get_singleton()->changed, this, "_curve_changed");
@@ -96,24 +96,23 @@ Size2 CurveEditor::get_minimum_size() const {
 }
 
 void CurveEditor::_notification(int p_what) {
-	if (p_what == NOTIFICATION_DRAW)
+	if (p_what == NOTIFICATION_DRAW) {
 		_draw();
+	}
 }
 
 void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
-
 	Ref<InputEventMouseButton> mb_ref = p_event;
 	if (mb_ref.is_valid()) {
-
 		const InputEventMouseButton &mb = **mb_ref;
 
 		if (mb.is_pressed() && !_dragging) {
-
 			Vector2 mpos = mb.get_position();
 
 			_selected_tangent = get_tangent_at(mpos);
-			if (_selected_tangent == TANGENT_NONE)
+			if (_selected_tangent == TANGENT_NONE) {
 				set_selected_point(get_point_at(mpos));
+			}
 
 			switch (mb.get_button_index()) {
 				case BUTTON_RIGHT:
@@ -134,7 +133,6 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 		if (!mb.is_pressed() && _dragging && mb.get_button_index() == BUTTON_LEFT) {
 			_dragging = false;
 			if (_has_undo_data) {
-
 				UndoRedo &ur = *EditorNode::get_singleton()->get_undo_redo();
 
 				ur.create_action(_selected_tangent == TANGENT_NONE ? TTR("Modify Curve Point") : TTR("Modify Curve Tangent"));
@@ -151,13 +149,11 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 
 	Ref<InputEventMouseMotion> mm_ref = p_event;
 	if (mm_ref.is_valid()) {
-
 		const InputEventMouseMotion &mm = **mm_ref;
 
 		Vector2 mpos = mm.get_position();
 
 		if (_dragging && _curve_ref.is_valid()) {
-
 			if (_selected_point != -1) {
 				Curve &curve = **_curve_ref;
 
@@ -189,10 +185,11 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 					set_selected_point(i);
 
 					// This is to prevent the user from losing a point out of view.
-					if (point_pos.y < curve.get_min_value())
+					if (point_pos.y < curve.get_min_value()) {
 						point_pos.y = curve.get_min_value();
-					else if (point_pos.y > curve.get_max_value())
+					} else if (point_pos.y > curve.get_max_value()) {
 						point_pos.y = curve.get_max_value();
+					}
 
 					curve.set_point_value(_selected_point, point_pos.y);
 
@@ -205,10 +202,11 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 					Vector2 dir = (control_pos - point_pos).normalized();
 
 					real_t tangent;
-					if (!Math::is_zero_approx(dir.x))
+					if (!Math::is_zero_approx(dir.x)) {
 						tangent = dir.y / dir.x;
-					else
+					} else {
 						tangent = 9999 * (dir.y >= 0 ? 1 : -1);
+					}
 
 					bool link = !Input::get_singleton()->is_key_pressed(KEY_SHIFT);
 
@@ -216,14 +214,16 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 						curve.set_point_left_tangent(_selected_point, tangent);
 
 						// Note: if a tangent is set to linear, it shouldn't be linked to the other
-						if (link && _selected_point != (curve.get_point_count() - 1) && curve.get_point_right_mode(_selected_point) != Curve::TANGENT_LINEAR)
+						if (link && _selected_point != (curve.get_point_count() - 1) && curve.get_point_right_mode(_selected_point) != Curve::TANGENT_LINEAR) {
 							curve.set_point_right_tangent(_selected_point, tangent);
+						}
 
 					} else {
 						curve.set_point_right_tangent(_selected_point, tangent);
 
-						if (link && _selected_point != 0 && curve.get_point_left_mode(_selected_point) != Curve::TANGENT_LINEAR)
+						if (link && _selected_point != 0 && curve.get_point_left_mode(_selected_point) != Curve::TANGENT_LINEAR) {
 							curve.set_point_left_tangent(_selected_point, tangent);
+						}
 					}
 				}
 			}
@@ -238,8 +238,9 @@ void CurveEditor::on_gui_input(const Ref<InputEvent> &p_event) {
 		const InputEventKey &key = **key_ref;
 
 		if (key.is_pressed() && _selected_point != -1) {
-			if (key.get_scancode() == KEY_DELETE)
+			if (key.get_scancode() == KEY_DELETE) {
 				remove_point(_selected_point);
+			}
 		}
 	}
 }
@@ -351,15 +352,16 @@ void CurveEditor::open_context_menu(Vector2 pos) {
 
 				_context_menu->add_check_item(TTR("Linear"), CONTEXT_LINEAR);
 
-				bool is_linear = _selected_tangent == TANGENT_LEFT ?
-										 _curve_ref->get_point_left_mode(_selected_point) == Curve::TANGENT_LINEAR :
-										 _curve_ref->get_point_right_mode(_selected_point) == Curve::TANGENT_LINEAR;
+				bool is_linear = _selected_tangent == TANGENT_LEFT
+						? _curve_ref->get_point_left_mode(_selected_point) == Curve::TANGENT_LINEAR
+						: _curve_ref->get_point_right_mode(_selected_point) == Curve::TANGENT_LINEAR;
 
 				_context_menu->set_item_checked(_context_menu->get_item_index(CONTEXT_LINEAR), is_linear);
 
 			} else {
-				if (_selected_point > 0 || _selected_point + 1 < _curve_ref->get_point_count())
+				if (_selected_point > 0 || _selected_point + 1 < _curve_ref->get_point_count()) {
 					_context_menu->add_separator();
+				}
 
 				if (_selected_point > 0) {
 					_context_menu->add_check_item(TTR("Left Linear"), CONTEXT_LEFT_LINEAR);
@@ -384,11 +386,13 @@ void CurveEditor::open_context_menu(Vector2 pos) {
 }
 
 int CurveEditor::get_point_at(Vector2 pos) const {
-	if (_curve_ref.is_null())
+	if (_curve_ref.is_null()) {
 		return -1;
+	}
 	const Curve &curve = **_curve_ref;
 
-	const float r = _hover_radius * _hover_radius;
+	const float true_hover_radius = Math::round(_hover_radius * EDSCALE);
+	const float r = true_hover_radius * true_hover_radius;
 
 	for (int i = 0; i < curve.get_point_count(); ++i) {
 		Vector2 p = get_view_pos(curve.get_point_position(i));
@@ -401,8 +405,9 @@ int CurveEditor::get_point_at(Vector2 pos) const {
 }
 
 CurveEditor::TangentIndex CurveEditor::get_tangent_at(Vector2 pos) const {
-	if (_curve_ref.is_null() || _selected_point < 0)
+	if (_curve_ref.is_null() || _selected_point < 0) {
 		return TANGENT_NONE;
+	}
 
 	if (_selected_point != 0) {
 		Vector2 control_pos = get_tangent_view_pos(_selected_point, TANGENT_LEFT);
@@ -428,10 +433,11 @@ void CurveEditor::add_point(Vector2 pos) {
 	ur.create_action(TTR("Remove Curve Point"));
 
 	Vector2 point_pos = get_world_pos(pos);
-	if (point_pos.y < 0.0)
+	if (point_pos.y < 0.0) {
 		point_pos.y = 0.0;
-	else if (point_pos.y > 1.0)
+	} else if (point_pos.y > 1.0) {
 		point_pos.y = 1.0;
+	}
 
 	// Small trick to get the point index to feed the undo method
 	int i = _curve_ref->add_point(point_pos);
@@ -454,11 +460,13 @@ void CurveEditor::remove_point(int index) {
 	ur.add_do_method(*_curve_ref, "remove_point", index);
 	ur.add_undo_method(*_curve_ref, "add_point", p.pos, p.left_tangent, p.right_tangent, p.left_mode, p.right_mode);
 
-	if (index == _selected_point)
+	if (index == _selected_point) {
 		set_selected_point(-1);
+	}
 
-	if (index == _hover_point)
+	if (index == _hover_point) {
 		set_hover_point_index(-1);
+	}
 
 	ur.commit_action();
 }
@@ -469,11 +477,11 @@ void CurveEditor::toggle_linear(TangentIndex tangent) {
 	UndoRedo &ur = *EditorNode::get_singleton()->get_undo_redo();
 	ur.create_action(TTR("Toggle Curve Linear Tangent"));
 
-	if (tangent == TANGENT_NONE)
+	if (tangent == TANGENT_NONE) {
 		tangent = _selected_tangent;
+	}
 
 	if (tangent == TANGENT_LEFT) {
-
 		bool is_linear = _curve_ref->get_point_left_mode(_selected_point) == Curve::TANGENT_LINEAR;
 
 		Curve::TangentMode prev_mode = _curve_ref->get_point_left_mode(_selected_point);
@@ -483,7 +491,6 @@ void CurveEditor::toggle_linear(TangentIndex tangent) {
 		ur.add_undo_method(*_curve_ref, "set_point_left_mode", _selected_point, prev_mode);
 
 	} else {
-
 		bool is_linear = _curve_ref->get_point_right_mode(_selected_point) == Curve::TANGENT_LINEAR;
 
 		Curve::TangentMode prev_mode = _curve_ref->get_point_right_mode(_selected_point);
@@ -538,17 +545,17 @@ void CurveEditor::update_view_transform() {
 }
 
 Vector2 CurveEditor::get_tangent_view_pos(int i, TangentIndex tangent) const {
-
 	Vector2 dir;
-	if (tangent == TANGENT_LEFT)
+	if (tangent == TANGENT_LEFT) {
 		dir = -Vector2(1, _curve_ref->get_point_left_tangent(i));
-	else
+	} else {
 		dir = Vector2(1, _curve_ref->get_point_right_tangent(i));
+	}
 
 	Vector2 point_pos = get_view_pos(_curve_ref->get_point_position(i));
 	Vector2 control_pos = get_view_pos(_curve_ref->get_point_position(i) + dir);
 
-	return point_pos + _tangents_length * (control_pos - point_pos).normalized();
+	return point_pos + Math::round(_tangents_length * EDSCALE) * (control_pos - point_pos).normalized();
 }
 
 Vector2 CurveEditor::get_view_pos(Vector2 world_pos) const {
@@ -562,7 +569,6 @@ Vector2 CurveEditor::get_world_pos(Vector2 view_pos) const {
 // Uses non-baked points, but takes advantage of ordered iteration to be faster
 template <typename T>
 static void plot_curve_accurate(const Curve &curve, float step, T plot_func) {
-
 	if (curve.get_point_count() <= 1) {
 		// Not enough points to make a curve, so it's just a straight line
 		float y = curve.interpolate(0);
@@ -600,7 +606,6 @@ static void plot_curve_accurate(const Curve &curve, float step, T plot_func) {
 }
 
 struct CanvasItemPlotCurve {
-
 	CanvasItem &ci;
 	Color color1;
 	Color color2;
@@ -617,8 +622,9 @@ struct CanvasItemPlotCurve {
 };
 
 void CurveEditor::_draw() {
-	if (_curve_ref.is_null())
+	if (_curve_ref.is_null()) {
 		return;
+	}
 	Curve &curve = **_curve_ref;
 
 	update_view_transform();
@@ -685,7 +691,6 @@ void CurveEditor::_draw() {
 	// Draw tangents for current point
 
 	if (_selected_point >= 0) {
-
 		const Color tangent_color = get_color("accent_color", "Editor");
 
 		int i = _selected_point;
@@ -694,13 +699,13 @@ void CurveEditor::_draw() {
 		if (i != 0) {
 			Vector2 control_pos = get_tangent_view_pos(i, TANGENT_LEFT);
 			draw_line(get_view_pos(pos), control_pos, tangent_color, Math::round(EDSCALE), true);
-			draw_rect(Rect2(control_pos, Vector2(1, 1)).grow(2), tangent_color);
+			draw_rect(Rect2(control_pos, Vector2(1, 1)).grow(Math::round(2 * EDSCALE)), tangent_color);
 		}
 
 		if (i != curve.get_point_count() - 1) {
 			Vector2 control_pos = get_tangent_view_pos(i, TANGENT_RIGHT);
 			draw_line(get_view_pos(pos), control_pos, tangent_color, Math::round(EDSCALE), true);
-			draw_rect(Rect2(control_pos, Vector2(1, 1)).grow(2), tangent_color);
+			draw_rect(Rect2(control_pos, Vector2(1, 1)).grow(Math::round(2 * EDSCALE)), tangent_color);
 		}
 	}
 
@@ -723,7 +728,7 @@ void CurveEditor::_draw() {
 
 	for (int i = 0; i < curve.get_point_count(); ++i) {
 		Vector2 pos = curve.get_point_position(i);
-		draw_rect(Rect2(get_view_pos(pos), Vector2(1, 1)).grow(3), i == _selected_point ? selected_point_color : point_color);
+		draw_rect(Rect2(get_view_pos(pos), Vector2(1, 1)).grow(Math::round(3 * EDSCALE)), i == _selected_point ? selected_point_color : point_color);
 		// TODO Circles are prettier. Needs a fix! Or a texture
 		//draw_circle(pos, 2, point_color);
 	}
@@ -733,7 +738,7 @@ void CurveEditor::_draw() {
 	if (_hover_point != -1) {
 		const Color hover_color = line_color;
 		Vector2 pos = curve.get_point_position(_hover_point);
-		draw_rect(Rect2(get_view_pos(pos), Vector2(1, 1)).grow(_hover_radius), hover_color, false, Math::round(EDSCALE));
+		draw_rect(Rect2(get_view_pos(pos), Vector2(1, 1)).grow(Math::round(_hover_radius * EDSCALE)), hover_color, false, Math::round(EDSCALE));
 	}
 
 	// Help text
@@ -757,12 +762,10 @@ void CurveEditor::_bind_methods() {
 //---------------
 
 bool EditorInspectorPluginCurve::can_handle(Object *p_object) {
-
-	return Object::cast_to<Curve>(p_object) != NULL;
+	return Object::cast_to<Curve>(p_object) != nullptr;
 }
 
 void EditorInspectorPluginCurve::parse_begin(Object *p_object) {
-
 	Curve *curve = Object::cast_to<Curve>(p_object);
 	ERR_FAIL_COND(!curve);
 	Ref<Curve> c(curve);
@@ -788,7 +791,6 @@ bool CurvePreviewGenerator::handles(const String &p_type) const {
 }
 
 Ref<Texture> CurvePreviewGenerator::generate(const Ref<Resource> &p_from, const Size2 &p_size) const {
-
 	Ref<Curve> curve_ref = p_from;
 	ERR_FAIL_COND_V_MSG(curve_ref.is_null(), Ref<Texture>(), "It's not a reference to a valid Resource object.");
 	Curve &curve = **curve_ref;
@@ -800,7 +802,7 @@ Ref<Texture> CurvePreviewGenerator::generate(const Ref<Resource> &p_from, const
 	img_ref.instance();
 	Image &im = **img_ref;
 
-	im.create(thumbnail_size, thumbnail_size / 2, 0, Image::FORMAT_RGBA8);
+	im.create(thumbnail_size, thumbnail_size / 2, false, Image::FORMAT_RGBA8);
 
 	im.lock();
 
@@ -816,7 +818,6 @@ Ref<Texture> CurvePreviewGenerator::generate(const Ref<Resource> &p_from, const
 
 	int prev_y = 0;
 	for (int x = 0; x < im.get_width(); ++x) {
-
 		float t = static_cast<float>(x) / im.get_width();
 		float v = (curve.interpolate_baked(t) - curve.get_min_value()) / range_y;
 		int y = CLAMP(im.get_height() - v * im.get_height(), 0, im.get_height());
