@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  path_editor_plugin.cpp                                               */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  path_editor_plugin.cpp                                                */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "path_editor_plugin.h"
 
@@ -35,13 +35,12 @@
 #include "spatial_editor_plugin.h"
 
 String PathSpatialGizmo::get_handle_name(int p_idx) const {
-
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return "";
+	}
 
 	if (p_idx < c->get_point_count()) {
-
 		return TTR("Curve Point #") + itos(p_idx);
 	}
 
@@ -50,21 +49,21 @@ String PathSpatialGizmo::get_handle_name(int p_idx) const {
 	int idx = p_idx / 2;
 	int t = p_idx % 2;
 	String n = TTR("Curve Point #") + itos(idx);
-	if (t == 0)
+	if (t == 0) {
 		n += " In";
-	else
+	} else {
 		n += " Out";
+	}
 
 	return n;
 }
 Variant PathSpatialGizmo::get_handle_value(int p_idx) {
-
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return Variant();
+	}
 
 	if (p_idx < c->get_point_count()) {
-
 		original = c->get_point_position(p_idx);
 		return original;
 	}
@@ -75,20 +74,21 @@ Variant PathSpatialGizmo::get_handle_value(int p_idx) {
 	int t = p_idx % 2;
 
 	Vector3 ofs;
-	if (t == 0)
+	if (t == 0) {
 		ofs = c->get_point_in(idx);
-	else
+	} else {
 		ofs = c->get_point_out(idx);
+	}
 
 	original = ofs + c->get_point_position(idx);
 
 	return ofs;
 }
 void PathSpatialGizmo::set_handle(int p_idx, Camera *p_camera, const Point2 &p_point) {
-
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return;
+	}
 
 	Transform gt = path->get_global_transform();
 	Transform gi = gt.affine_inverse();
@@ -97,13 +97,11 @@ void PathSpatialGizmo::set_handle(int p_idx, Camera *p_camera, const Point2 &p_p
 
 	// Setting curve point positions
 	if (p_idx < c->get_point_count()) {
-
 		Plane p(gt.xform(original), p_camera->get_transform().basis.get_axis(2));
 
 		Vector3 inters;
 
 		if (p.intersects_ray(ray_from, ray_dir, &inters)) {
-
 			if (SpatialEditor::get_singleton()->is_snap_enabled()) {
 				float snap = SpatialEditor::get_singleton()->get_translate_snap();
 				inters.snap(Vector3(snap, snap, snap));
@@ -129,7 +127,6 @@ void PathSpatialGizmo::set_handle(int p_idx, Camera *p_camera, const Point2 &p_p
 
 	// Setting curve in/out positions
 	if (p.intersects_ray(ray_from, ray_dir, &inters)) {
-
 		if (!PathEditorPlugin::singleton->is_handle_clicked()) {
 			orig_in_length = c->get_point_in(idx).length();
 			orig_out_length = c->get_point_out(idx).length();
@@ -144,28 +141,28 @@ void PathSpatialGizmo::set_handle(int p_idx, Camera *p_camera, const Point2 &p_p
 
 		if (t == 0) {
 			c->set_point_in(idx, local);
-			if (PathEditorPlugin::singleton->mirror_angle_enabled())
+			if (PathEditorPlugin::singleton->mirror_angle_enabled()) {
 				c->set_point_out(idx, PathEditorPlugin::singleton->mirror_length_enabled() ? -local : (-local.normalized() * orig_out_length));
+			}
 		} else {
 			c->set_point_out(idx, local);
-			if (PathEditorPlugin::singleton->mirror_angle_enabled())
+			if (PathEditorPlugin::singleton->mirror_angle_enabled()) {
 				c->set_point_in(idx, PathEditorPlugin::singleton->mirror_length_enabled() ? -local : (-local.normalized() * orig_in_length));
+			}
 		}
 	}
 }
 
 void PathSpatialGizmo::commit_handle(int p_idx, const Variant &p_restore, bool p_cancel) {
-
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return;
+	}
 
 	UndoRedo *ur = SpatialEditor::get_singleton()->get_undo_redo();
 
 	if (p_idx < c->get_point_count()) {
-
 		if (p_cancel) {
-
 			c->set_point_position(p_idx, p_restore);
 			return;
 		}
@@ -218,7 +215,6 @@ void PathSpatialGizmo::commit_handle(int p_idx, const Variant &p_restore, bool p
 }
 
 void PathSpatialGizmo::redraw() {
-
 	clear();
 
 	Ref<SpatialMaterial> path_material = gizmo_plugin->get_material("path_material", this);
@@ -226,21 +222,22 @@ void PathSpatialGizmo::redraw() {
 	Ref<SpatialMaterial> handles_material = gizmo_plugin->get_material("handles");
 
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return;
+	}
 
 	PoolVector<Vector3> v3a = c->tessellate();
 	//PoolVector<Vector3> v3a=c->get_baked_points();
 
 	int v3s = v3a.size();
-	if (v3s == 0)
+	if (v3s == 0) {
 		return;
+	}
 	Vector<Vector3> v3p;
 	PoolVector<Vector3>::Read r = v3a.read();
 
 	// BUG: the following won't work when v3s, avoid drawing as a temporary workaround.
 	for (int i = 0; i < v3s - 1; i++) {
-
 		v3p.push_back(r[i]);
 		v3p.push_back(r[i + 1]);
 		//v3p.push_back(r[i]);
@@ -258,7 +255,6 @@ void PathSpatialGizmo::redraw() {
 		Vector<Vector3> sec_handles;
 
 		for (int i = 0; i < c->get_point_count(); i++) {
-
 			Vector3 p = c->get_point_position(i);
 			handles.push_back(p);
 			if (i > 0) {
@@ -287,18 +283,18 @@ void PathSpatialGizmo::redraw() {
 }
 
 PathSpatialGizmo::PathSpatialGizmo(Path *p_path) {
-
 	path = p_path;
 	set_spatial_node(p_path);
 }
 
 bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<InputEvent> &p_event) {
-
-	if (!path)
+	if (!path) {
 		return false;
+	}
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return false;
+	}
 	Transform gt = path->get_global_transform();
 	Transform it = gt.affine_inverse();
 
@@ -307,11 +303,11 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 	Ref<InputEventMouseButton> mb = p_event;
 
 	if (mb.is_valid()) {
-
 		Point2 mbpos(mb->get_position().x, mb->get_position().y);
 
-		if (!mb->is_pressed())
+		if (!mb->is_pressed()) {
 			set_handle_clicked(false);
+		}
 
 		if (mb->is_pressed() && mb->get_button_index() == BUTTON_LEFT && (curve_create->is_pressed() || (curve_edit->is_pressed() && mb->get_control()))) {
 			//click into curve, break it down
@@ -325,17 +321,18 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 			if (rc >= 2) {
 				PoolVector<Vector3>::Read r = v3a.read();
 
-				if (p_camera->unproject_position(gt.xform(c->get_point_position(0))).distance_to(mbpos) < click_dist)
+				if (p_camera->unproject_position(gt.xform(c->get_point_position(0))).distance_to(mbpos) < click_dist) {
 					return false; //nope, existing
+				}
 
 				for (int i = 0; i < c->get_point_count() - 1; i++) {
 					//find the offset and point index of the place to break up
 					int j = idx;
-					if (p_camera->unproject_position(gt.xform(c->get_point_position(i + 1))).distance_to(mbpos) < click_dist)
+					if (p_camera->unproject_position(gt.xform(c->get_point_position(i + 1))).distance_to(mbpos) < click_dist) {
 						return false; //nope, existing
+					}
 
 					while (j < rc && c->get_point_position(i + 1) != r[j]) {
-
 						Vector3 from = r[j];
 						Vector3 to = r[j + 1];
 						real_t cdist = from.distance_to(to);
@@ -349,7 +346,6 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 							float d = inters.distance_to(mbpos);
 
 							if (d < 10 && d < closest_d) {
-
 								closest_d = d;
 								closest_seg = i;
 								Vector3 ray_from = p_camera->project_ray_origin(mbpos);
@@ -363,13 +359,15 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 						}
 						j++;
 					}
-					if (idx == j)
+					if (idx == j) {
 						idx++; //force next
-					else
+					} else {
 						idx = j; //swap
+					}
 
-					if (j == rc)
+					if (j == rc) {
 						break;
+					}
 				}
 			}
 
@@ -384,19 +382,18 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 				return true;
 
 			} else {
-
 				Vector3 org;
-				if (c->get_point_count() == 0)
+				if (c->get_point_count() == 0) {
 					org = path->get_transform().get_origin();
-				else
+				} else {
 					org = gt.xform(c->get_point_position(c->get_point_count() - 1));
+				}
 				Plane p(org, p_camera->get_transform().basis.get_axis(2));
 				Vector3 ray_from = p_camera->project_ray_origin(mbpos);
 				Vector3 ray_dir = p_camera->project_ray_normal(mbpos);
 
 				Vector3 inters;
 				if (p.intersects_ray(ray_from, ray_dir, &inters)) {
-
 					ur->create_action(TTR("Add Point to Curve"));
 					ur->add_do_method(c.ptr(), "add_point", it.xform(inters), Vector3(), Vector3(), -1);
 					ur->add_undo_method(c.ptr(), "remove_point", c->get_point_count());
@@ -408,7 +405,6 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 			}
 
 		} else if (mb->is_pressed() && ((mb->get_button_index() == BUTTON_LEFT && curve_del->is_pressed()) || (mb->get_button_index() == BUTTON_RIGHT && curve_edit->is_pressed()))) {
-
 			for (int i = 0; i < c->get_point_count(); i++) {
 				real_t dist_to_p = p_camera->unproject_position(gt.xform(c->get_point_position(i))).distance_to(mbpos);
 				real_t dist_to_p_out = p_camera->unproject_position(gt.xform(c->get_point_position(i) + c->get_point_out(i))).distance_to(mbpos);
@@ -417,7 +413,6 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 				// Find the offset and point index of the place to break up.
 				// Also check for the control points.
 				if (dist_to_p < click_dist) {
-
 					UndoRedo *ur = editor->get_undo_redo();
 					ur->create_action(TTR("Remove Path Point"));
 					ur->add_do_method(c.ptr(), "remove_point", i);
@@ -425,7 +420,6 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 					ur->commit_action();
 					return true;
 				} else if (dist_to_p_out < click_dist) {
-
 					UndoRedo *ur = editor->get_undo_redo();
 					ur->create_action(TTR("Remove Out-Control Point"));
 					ur->add_do_method(c.ptr(), "set_point_out", i, Vector3());
@@ -433,7 +427,6 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 					ur->commit_action();
 					return true;
 				} else if (dist_to_p_in < click_dist) {
-
 					UndoRedo *ur = editor->get_undo_redo();
 					ur->create_action(TTR("Remove In-Control Point"));
 					ur->add_do_method(c.ptr(), "set_point_in", i, Vector3());
@@ -449,18 +442,16 @@ bool PathEditorPlugin::forward_spatial_gui_input(Camera *p_camera, const Ref<Inp
 }
 
 void PathEditorPlugin::edit(Object *p_object) {
-
 	if (p_object) {
 		path = Object::cast_to<Path>(p_object);
 		if (path) {
-
 			if (path->get_curve().is_valid()) {
 				path->get_curve()->emit_signal("changed");
 			}
 		}
 	} else {
 		Path *pre = path;
-		path = NULL;
+		path = nullptr;
 		if (pre) {
 			pre->get_curve()->emit_signal("changed");
 		}
@@ -469,14 +460,11 @@ void PathEditorPlugin::edit(Object *p_object) {
 }
 
 bool PathEditorPlugin::handles(Object *p_object) const {
-
 	return p_object->is_class("Path");
 }
 
 void PathEditorPlugin::make_visible(bool p_visible) {
-
 	if (p_visible) {
-
 		curve_create->show();
 		curve_edit->show();
 		curve_del->show();
@@ -484,7 +472,6 @@ void PathEditorPlugin::make_visible(bool p_visible) {
 		handle_menu->show();
 		sep->show();
 	} else {
-
 		curve_create->hide();
 		curve_edit->hide();
 		curve_del->hide();
@@ -494,7 +481,7 @@ void PathEditorPlugin::make_visible(bool p_visible) {
 
 		{
 			Path *pre = path;
-			path = NULL;
+			path = nullptr;
 			if (pre && pre->get_curve().is_valid()) {
 				pre->get_curve()->emit_signal("changed");
 			}
@@ -503,24 +490,30 @@ void PathEditorPlugin::make_visible(bool p_visible) {
 }
 
 void PathEditorPlugin::_mode_changed(int p_idx) {
-
 	curve_create->set_pressed(p_idx == 0);
 	curve_edit->set_pressed(p_idx == 1);
 	curve_del->set_pressed(p_idx == 2);
 }
 
 void PathEditorPlugin::_close_curve() {
-
 	Ref<Curve3D> c = path->get_curve();
-	if (c.is_null())
+	if (c.is_null()) {
 		return;
-	if (c->get_point_count() < 2)
+	}
+	if (c->get_point_count() < 2) {
 		return;
-	c->add_point(c->get_point_position(0), c->get_point_in(0), c->get_point_out(0));
+	}
+	if (c->get_point_position(0) == c->get_point_position(c->get_point_count() - 1)) {
+		return;
+	}
+	UndoRedo *ur = editor->get_undo_redo();
+	ur->create_action(TTR("Close Curve"));
+	ur->add_do_method(c.ptr(), "add_point", c->get_point_position(0), c->get_point_in(0), c->get_point_out(0), -1);
+	ur->add_undo_method(c.ptr(), "remove_point", c->get_point_count());
+	ur->commit_action();
 }
 
 void PathEditorPlugin::_handle_option_pressed(int p_option) {
-
 	PopupMenu *pm;
 	pm = handle_menu->get_popup();
 
@@ -540,9 +533,7 @@ void PathEditorPlugin::_handle_option_pressed(int p_option) {
 }
 
 void PathEditorPlugin::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_ENTER_TREE) {
-
 		curve_create->connect("pressed", this, "_mode_changed", make_binds(0));
 		curve_edit->connect("pressed", this, "_mode_changed", make_binds(1));
 		curve_del->connect("pressed", this, "_mode_changed", make_binds(2));
@@ -551,17 +542,15 @@ void PathEditorPlugin::_notification(int p_what) {
 }
 
 void PathEditorPlugin::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_mode_changed"), &PathEditorPlugin::_mode_changed);
 	ClassDB::bind_method(D_METHOD("_close_curve"), &PathEditorPlugin::_close_curve);
 	ClassDB::bind_method(D_METHOD("_handle_option_pressed"), &PathEditorPlugin::_handle_option_pressed);
 }
 
-PathEditorPlugin *PathEditorPlugin::singleton = NULL;
+PathEditorPlugin *PathEditorPlugin::singleton = nullptr;
 
 PathEditorPlugin::PathEditorPlugin(EditorNode *p_node) {
-
-	path = NULL;
+	path = nullptr;
 	editor = p_node;
 	singleton = this;
 	mirror_handle_angle = true;
@@ -617,15 +606,6 @@ PathEditorPlugin::PathEditorPlugin(EditorNode *p_node) {
 	menu->connect("id_pressed", this, "_handle_option_pressed");
 
 	curve_edit->set_pressed(true);
-	/*
-    collision_polygon_editor = memnew( PathEditor(p_node) );
-    editor->get_viewport()->add_child(collision_polygon_editor);
-    collision_polygon_editor->set_margin(MARGIN_LEFT,200);
-    collision_polygon_editor->set_margin(MARGIN_RIGHT,230);
-    collision_polygon_editor->set_margin(MARGIN_TOP,0);
-    collision_polygon_editor->set_margin(MARGIN_BOTTOM,10);
-    collision_polygon_editor->hide();
-    */
 }
 
 PathEditorPlugin::~PathEditorPlugin() {
@@ -635,7 +615,9 @@ Ref<EditorSpatialGizmo> PathSpatialGizmoPlugin::create_gizmo(Spatial *p_spatial)
 	Ref<PathSpatialGizmo> ref;
 
 	Path *path = Object::cast_to<Path>(p_spatial);
-	if (path) ref = Ref<PathSpatialGizmo>(memnew(PathSpatialGizmo(path)));
+	if (path) {
+		ref = Ref<PathSpatialGizmo>(memnew(PathSpatialGizmo(path)));
+	}
 
 	return ref;
 }
@@ -649,7 +631,6 @@ int PathSpatialGizmoPlugin::get_priority() const {
 }
 
 PathSpatialGizmoPlugin::PathSpatialGizmoPlugin() {
-
 	Color path_color = EDITOR_DEF("editors/3d_gizmos/gizmo_colors/path", Color(0.5, 0.5, 1.0, 0.8));
 	create_material("path_material", path_color);
 	create_material("path_thin_material", Color(0.5, 0.5, 0.5));
