@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  text_editor.cpp                                                      */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  text_editor.cpp                                                       */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "text_editor.h"
 
@@ -41,15 +41,15 @@ void TextEditor::add_syntax_highlighter(SyntaxHighlighter *p_highlighter) {
 void TextEditor::set_syntax_highlighter(SyntaxHighlighter *p_highlighter) {
 	TextEdit *te = code_editor->get_text_edit();
 	te->_set_syntax_highlighting(p_highlighter);
-	if (p_highlighter != NULL) {
+	if (p_highlighter != nullptr) {
 		highlighter_menu->set_item_checked(highlighter_menu->get_item_idx_from_text(p_highlighter->get_name()), true);
 	} else {
-		highlighter_menu->set_item_checked(highlighter_menu->get_item_idx_from_text("Standard"), true);
+		highlighter_menu->set_item_checked(highlighter_menu->get_item_idx_from_text(TTR("Standard")), true);
 	}
 
 	// little work around. GDScript highlighter goes through text_edit for colours,
 	// so to remove all colours we need to set and unset them here.
-	if (p_highlighter == NULL) { // standard
+	if (p_highlighter == nullptr) { // standard
 		TextEdit *text_edit = code_editor->get_text_edit();
 		text_edit->add_color_override("number_color", colors_cache.font_color);
 		text_edit->add_color_override("function_color", colors_cache.font_color);
@@ -62,7 +62,7 @@ void TextEditor::set_syntax_highlighter(SyntaxHighlighter *p_highlighter) {
 
 void TextEditor::_change_syntax_highlighter(int p_idx) {
 	Map<String, SyntaxHighlighter *>::Element *el = highlighters.front();
-	while (el != NULL) {
+	while (el != nullptr) {
 		highlighter_menu->set_item_checked(highlighter_menu->get_item_idx_from_text(el->key()), false);
 		el = el->next();
 	}
@@ -70,7 +70,6 @@ void TextEditor::_change_syntax_highlighter(int p_idx) {
 }
 
 void TextEditor::_load_theme_settings() {
-
 	TextEdit *text_edit = code_editor->get_text_edit();
 	text_edit->clear_colors();
 
@@ -102,6 +101,7 @@ void TextEditor::_load_theme_settings() {
 	Color search_result_border_color = EDITOR_GET("text_editor/highlighting/search_result_border_color");
 	Color symbol_color = EDITOR_GET("text_editor/highlighting/symbol_color");
 	Color keyword_color = EDITOR_GET("text_editor/highlighting/keyword_color");
+	Color control_flow_keyword_color = EDITOR_GET("text_editor/highlighting/control_flow_keyword_color");
 	Color basetype_color = EDITOR_GET("text_editor/highlighting/base_type_color");
 	Color type_color = EDITOR_GET("text_editor/highlighting/engine_type_color");
 	Color comment_color = EDITOR_GET("text_editor/highlighting/comment_color");
@@ -135,11 +135,12 @@ void TextEditor::_load_theme_settings() {
 	text_edit->add_color_override("search_result_border_color", search_result_border_color);
 	text_edit->add_color_override("symbol_color", symbol_color);
 
-	text_edit->add_constant_override("line_spacing", EDITOR_DEF("text_editor/theme/line_spacing", 6));
+	text_edit->add_constant_override("line_spacing", EDITOR_GET("text_editor/theme/line_spacing"));
 
 	colors_cache.font_color = text_color;
 	colors_cache.symbol_color = symbol_color;
 	colors_cache.keyword_color = keyword_color;
+	colors_cache.control_flow_keyword_color = control_flow_keyword_color;
 	colors_cache.basetype_color = basetype_color;
 	colors_cache.type_color = type_color;
 	colors_cache.comment_color = comment_color;
@@ -149,23 +150,28 @@ void TextEditor::_load_theme_settings() {
 String TextEditor::get_name() {
 	String name;
 
-	if (text_file->get_path().find("local://") == -1 && text_file->get_path().find("::") == -1) {
-		name = text_file->get_path().get_file();
-		if (is_unsaved()) {
-			name += "(*)";
+	name = text_file->get_path().get_file();
+	if (name.empty()) {
+		// This appears for newly created built-in text_files before saving the scene.
+		name = TTR("[unsaved]");
+	} else if (text_file->get_path().find("local://") == -1 || text_file->get_path().find("::") == -1) {
+		const String &text_file_name = text_file->get_name();
+		if (text_file_name != "") {
+			// If the built-in text_file has a custom resource name defined,
+			// display the built-in text_file name as follows: `ResourceName (scene_file.tscn)`
+			name = vformat("%s (%s)", text_file_name, name.get_slice("::", 0));
 		}
-	} else if (text_file->get_name() != "") {
-		name = text_file->get_name();
-	} else {
-		name = text_file->get_class() + "(" + itos(text_file->get_instance_id()) + ")";
+	}
+
+	if (is_unsaved()) {
+		name += "(*)";
 	}
 
 	return name;
 }
 
 Ref<Texture> TextEditor::get_icon() {
-
-	return EditorNode::get_singleton()->get_object_icon(text_file.operator->(), "");
+	return EditorNode::get_singleton()->get_object_icon(text_file.ptr(), "");
 }
 
 RES TextEditor::get_edited_resource() const {
@@ -173,7 +179,8 @@ RES TextEditor::get_edited_resource() const {
 }
 
 void TextEditor::set_edited_resource(const RES &p_res) {
-	ERR_FAIL_COND(!text_file.is_null());
+	ERR_FAIL_COND(text_file.is_valid());
+	ERR_FAIL_COND(p_res.is_null());
 
 	text_file = p_res;
 
@@ -185,6 +192,16 @@ void TextEditor::set_edited_resource(const RES &p_res) {
 	code_editor->update_line_and_column();
 }
 
+void TextEditor::enable_editor() {
+	if (editor_enabled) {
+		return;
+	}
+
+	editor_enabled = true;
+
+	_load_theme_settings();
+}
+
 void TextEditor::add_callback(const String &p_function, PoolStringArray p_args) {
 }
 
@@ -195,7 +212,6 @@ void TextEditor::get_breakpoints(List<int> *p_breakpoints) {
 }
 
 void TextEditor::reload_text() {
-
 	ERR_FAIL_COND(text_file.is_null());
 
 	TextEdit *te = code_editor->get_text_edit();
@@ -221,7 +237,6 @@ void TextEditor::_validate_script() {
 }
 
 void TextEditor::_update_bookmark_list() {
-
 	bookmarks_menu->clear();
 
 	bookmarks_menu->add_shortcut(ED_GET_SHORTCUT("script_text_editor/toggle_bookmark"), BOOKMARK_TOGGLE);
@@ -249,7 +264,6 @@ void TextEditor::_update_bookmark_list() {
 }
 
 void TextEditor::_bookmark_item_pressed(int p_idx) {
-
 	if (p_idx < 4) { // Any item before the separator.
 		_edit_option(bookmarks_menu->get_item_id(p_idx));
 	} else {
@@ -262,17 +276,14 @@ void TextEditor::apply_code() {
 }
 
 bool TextEditor::is_unsaved() {
-
 	return code_editor->get_text_edit()->get_version() != code_editor->get_text_edit()->get_saved_version();
 }
 
 Variant TextEditor::get_edit_state() {
-
 	return code_editor->get_edit_state();
 }
 
 void TextEditor::set_edit_state(const Variant &p_state) {
-
 	code_editor->set_edit_state(p_state);
 
 	Dictionary state = p_state;
@@ -282,45 +293,39 @@ void TextEditor::set_edit_state(const Variant &p_state) {
 			_change_syntax_highlighter(idx);
 		}
 	}
+
+	ensure_focus();
 }
 
 void TextEditor::trim_trailing_whitespace() {
-
 	code_editor->trim_trailing_whitespace();
 }
 
 void TextEditor::insert_final_newline() {
-
 	code_editor->insert_final_newline();
 }
 
 void TextEditor::convert_indent_to_spaces() {
-
 	code_editor->convert_indent_to_spaces();
 }
 
 void TextEditor::convert_indent_to_tabs() {
-
 	code_editor->convert_indent_to_tabs();
 }
 
 void TextEditor::tag_saved_version() {
-
 	code_editor->get_text_edit()->tag_saved_version();
 }
 
 void TextEditor::goto_line(int p_line, bool p_with_error) {
-
 	code_editor->goto_line(p_line);
 }
 
 void TextEditor::goto_line_selection(int p_line, int p_begin, int p_end) {
-
 	code_editor->goto_line_selection(p_line, p_begin, p_end);
 }
 
 void TextEditor::set_executing_line(int p_line) {
-
 	code_editor->set_executing_line(p_line);
 }
 
@@ -329,12 +334,10 @@ void TextEditor::clear_executing_line() {
 }
 
 void TextEditor::ensure_focus() {
-
 	code_editor->get_text_edit()->grab_focus();
 }
 
 Vector<String> TextEditor::get_functions() {
-
 	return Vector<String>();
 }
 
@@ -343,17 +346,14 @@ bool TextEditor::show_members_overview() {
 }
 
 void TextEditor::update_settings() {
-
 	code_editor->update_editor_settings();
 }
 
 void TextEditor::set_tooltip_request_func(String p_method, Object *p_obj) {
-
 	code_editor->get_text_edit()->set_tooltip_request_func(p_obj, p_method, this);
 }
 
 Control *TextEditor::get_edit_menu() {
-
 	return edit_hb;
 }
 
@@ -361,166 +361,129 @@ void TextEditor::clear_edit_menu() {
 	memdelete(edit_hb);
 }
 
-void TextEditor::_notification(int p_what) {
-
-	switch (p_what) {
-		case NOTIFICATION_READY:
-			_load_theme_settings();
-			break;
-	}
-}
-
 void TextEditor::_edit_option(int p_op) {
 	TextEdit *tx = code_editor->get_text_edit();
 
 	switch (p_op) {
 		case EDIT_UNDO: {
-
 			tx->undo();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_REDO: {
-
 			tx->redo();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_CUT: {
-
 			tx->cut();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_COPY: {
-
 			tx->copy();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_PASTE: {
-
 			tx->paste();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_SELECT_ALL: {
-
 			tx->select_all();
 			tx->call_deferred("grab_focus");
 		} break;
 		case EDIT_MOVE_LINE_UP: {
-
 			code_editor->move_lines_up();
 		} break;
 		case EDIT_MOVE_LINE_DOWN: {
-
 			code_editor->move_lines_down();
 		} break;
 		case EDIT_INDENT_LEFT: {
-
 			tx->indent_left();
 		} break;
 		case EDIT_INDENT_RIGHT: {
-
 			tx->indent_right();
 		} break;
 		case EDIT_DELETE_LINE: {
-
 			code_editor->delete_lines();
 		} break;
-		case EDIT_CLONE_DOWN: {
-
-			code_editor->clone_lines_down();
+		case EDIT_DUPLICATE_SELECTION: {
+			code_editor->duplicate_selection();
 		} break;
 		case EDIT_TOGGLE_FOLD_LINE: {
-
 			tx->toggle_fold_line(tx->cursor_get_line());
 			tx->update();
 		} break;
 		case EDIT_FOLD_ALL_LINES: {
-
 			tx->fold_all_lines();
 			tx->update();
 		} break;
 		case EDIT_UNFOLD_ALL_LINES: {
-
 			tx->unhide_all_lines();
 			tx->update();
 		} break;
 		case EDIT_TRIM_TRAILING_WHITESAPCE: {
-
 			trim_trailing_whitespace();
 		} break;
 		case EDIT_CONVERT_INDENT_TO_SPACES: {
-
 			convert_indent_to_spaces();
 		} break;
 		case EDIT_CONVERT_INDENT_TO_TABS: {
-
 			convert_indent_to_tabs();
 		} break;
 		case EDIT_TO_UPPERCASE: {
-
 			_convert_case(CodeTextEditor::UPPER);
 		} break;
 		case EDIT_TO_LOWERCASE: {
-
 			_convert_case(CodeTextEditor::LOWER);
 		} break;
 		case EDIT_CAPITALIZE: {
-
 			_convert_case(CodeTextEditor::CAPITALIZE);
 		} break;
 		case SEARCH_FIND: {
-
 			code_editor->get_find_replace_bar()->popup_search();
 		} break;
 		case SEARCH_FIND_NEXT: {
-
 			code_editor->get_find_replace_bar()->search_next();
 		} break;
 		case SEARCH_FIND_PREV: {
-
 			code_editor->get_find_replace_bar()->search_prev();
 		} break;
 		case SEARCH_REPLACE: {
-
 			code_editor->get_find_replace_bar()->popup_replace();
 		} break;
 		case SEARCH_IN_FILES: {
-
 			String selected_text = code_editor->get_text_edit()->get_selection_text();
 
 			// Yep, because it doesn't make sense to instance this dialog for every single script open...
 			// So this will be delegated to the ScriptEditor.
 			emit_signal("search_in_files_requested", selected_text);
 		} break;
-		case SEARCH_GOTO_LINE: {
+		case REPLACE_IN_FILES: {
+			String selected_text = code_editor->get_text_edit()->get_selection_text();
 
+			emit_signal("replace_in_files_requested", selected_text);
+		} break;
+		case SEARCH_GOTO_LINE: {
 			goto_line_dialog->popup_find_line(tx);
 		} break;
 		case BOOKMARK_TOGGLE: {
-
 			code_editor->toggle_bookmark();
 		} break;
 		case BOOKMARK_GOTO_NEXT: {
-
 			code_editor->goto_next_bookmark();
 		} break;
 		case BOOKMARK_GOTO_PREV: {
-
 			code_editor->goto_prev_bookmark();
 		} break;
 		case BOOKMARK_REMOVE_ALL: {
-
 			code_editor->remove_all_bookmarks();
 		} break;
 	}
 }
 
 void TextEditor::_convert_case(CodeTextEditor::CaseStyle p_case) {
-
 	code_editor->convert_case(p_case);
 }
 
 void TextEditor::_bind_methods() {
-
 	ClassDB::bind_method("_validate_script", &TextEditor::_validate_script);
 	ClassDB::bind_method("_update_bookmark_list", &TextEditor::_update_bookmark_list);
 	ClassDB::bind_method("_bookmark_item_pressed", &TextEditor::_bookmark_item_pressed);
@@ -528,28 +491,25 @@ void TextEditor::_bind_methods() {
 	ClassDB::bind_method("_edit_option", &TextEditor::_edit_option);
 	ClassDB::bind_method("_change_syntax_highlighter", &TextEditor::_change_syntax_highlighter);
 	ClassDB::bind_method("_text_edit_gui_input", &TextEditor::_text_edit_gui_input);
+	ClassDB::bind_method("_prepare_edit_menu", &TextEditor::_prepare_edit_menu);
 }
 
 static ScriptEditorBase *create_editor(const RES &p_resource) {
-
 	if (Object::cast_to<TextFile>(*p_resource)) {
 		return memnew(TextEditor);
 	}
-	return NULL;
+	return nullptr;
 }
 
 void TextEditor::register_editor() {
-
 	ScriptEditor::register_create_script_editor_function(create_editor);
 }
 
 void TextEditor::_text_edit_gui_input(const Ref<InputEvent> &ev) {
-
 	Ref<InputEventMouseButton> mb = ev;
 
 	if (mb.is_valid()) {
 		if (mb->get_button_index() == BUTTON_RIGHT) {
-
 			int col, row;
 			TextEdit *tx = code_editor->get_text_edit();
 			tx->_get_mouse_pos(mb->get_global_position() - tx->get_global_position(), row, col);
@@ -560,7 +520,6 @@ void TextEditor::_text_edit_gui_input(const Ref<InputEvent> &ev) {
 
 			if (tx->is_right_click_moving_caret()) {
 				if (tx->is_selection_active()) {
-
 					int from_line = tx->get_selection_from_line();
 					int to_line = tx->get_selection_to_line();
 					int from_column = tx->get_selection_from_column();
@@ -592,8 +551,14 @@ void TextEditor::_text_edit_gui_input(const Ref<InputEvent> &ev) {
 	}
 }
 
-void TextEditor::_make_context_menu(bool p_selection, bool p_can_fold, bool p_is_folded, Vector2 p_position) {
+void TextEditor::_prepare_edit_menu() {
+	const TextEdit *tx = code_editor->get_text_edit();
+	PopupMenu *popup = edit_menu->get_popup();
+	popup->set_item_disabled(popup->get_item_index(EDIT_UNDO), !tx->has_undo());
+	popup->set_item_disabled(popup->get_item_index(EDIT_REDO), !tx->has_redo());
+}
 
+void TextEditor::_make_context_menu(bool p_selection, bool p_can_fold, bool p_is_folded, Vector2 p_position) {
 	context_menu->clear();
 	if (p_selection) {
 		context_menu->add_shortcut(ED_GET_SHORTCUT("script_text_editor/cut"), EDIT_CUT);
@@ -615,8 +580,13 @@ void TextEditor::_make_context_menu(bool p_selection, bool p_can_fold, bool p_is
 		context_menu->add_shortcut(ED_GET_SHORTCUT("script_text_editor/convert_to_uppercase"), EDIT_TO_UPPERCASE);
 		context_menu->add_shortcut(ED_GET_SHORTCUT("script_text_editor/convert_to_lowercase"), EDIT_TO_LOWERCASE);
 	}
-	if (p_can_fold || p_is_folded)
+	if (p_can_fold || p_is_folded) {
 		context_menu->add_shortcut(ED_GET_SHORTCUT("script_text_editor/toggle_fold_line"), EDIT_TOGGLE_FOLD_LINE);
+	}
+
+	const TextEdit *tx = code_editor->get_text_edit();
+	context_menu->set_item_disabled(context_menu->get_item_index(EDIT_UNDO), !tx->has_undo());
+	context_menu->set_item_disabled(context_menu->get_item_index(EDIT_REDO), !tx->has_redo());
 
 	context_menu->set_position(get_global_transform().xform(p_position));
 	context_menu->set_size(Vector2(1, 1));
@@ -624,6 +594,8 @@ void TextEditor::_make_context_menu(bool p_selection, bool p_can_fold, bool p_is
 }
 
 TextEditor::TextEditor() {
+	editor_enabled = false;
+
 	code_editor = memnew(CodeTextEditor);
 	add_child(code_editor);
 	code_editor->add_constant_override("separation", 0);
@@ -655,11 +627,13 @@ TextEditor::TextEditor() {
 	search_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/replace"), SEARCH_REPLACE);
 	search_menu->get_popup()->add_separator();
 	search_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/find_in_files"), SEARCH_IN_FILES);
+	search_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/replace_in_files"), REPLACE_IN_FILES);
 
 	edit_menu = memnew(MenuButton);
 	edit_hb->add_child(edit_menu);
 	edit_menu->set_text(TTR("Edit"));
 	edit_menu->set_switch_on_hover(true);
+	edit_menu->connect("about_to_show", this, "_prepare_edit_menu");
 	edit_menu->get_popup()->connect("id_pressed", this, "_edit_option");
 
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/undo"), EDIT_UNDO);
@@ -680,7 +654,7 @@ TextEditor::TextEditor() {
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/fold_all_lines"), EDIT_FOLD_ALL_LINES);
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/unfold_all_lines"), EDIT_UNFOLD_ALL_LINES);
 	edit_menu->get_popup()->add_separator();
-	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/clone_down"), EDIT_CLONE_DOWN);
+	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/duplicate_selection"), EDIT_DUPLICATE_SELECTION);
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/trim_trailing_whitespace"), EDIT_TRIM_TRAILING_WHITESAPCE);
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/convert_indent_to_spaces"), EDIT_CONVERT_INDENT_TO_SPACES);
 	edit_menu->get_popup()->add_shortcut(ED_GET_SHORTCUT("script_text_editor/convert_indent_to_tabs"), EDIT_CONVERT_INDENT_TO_TABS);
@@ -695,7 +669,7 @@ TextEditor::TextEditor() {
 	convert_case->add_shortcut(ED_SHORTCUT("script_text_editor/capitalize", TTR("Capitalize")), EDIT_CAPITALIZE);
 	convert_case->connect("id_pressed", this, "_edit_option");
 
-	highlighters["Standard"] = NULL;
+	highlighters[TTR("Standard")] = NULL;
 	highlighter_menu = memnew(PopupMenu);
 	highlighter_menu->set_name("highlighter_menu");
 	edit_menu->get_popup()->add_child(highlighter_menu);
