@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  texture_loader_dds.cpp                                               */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  texture_loader_dds.cpp                                                */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "texture_loader_dds.h"
 
@@ -95,18 +95,20 @@ static const DDSFormatInfo dds_format_info[DDS_MAX] = {
 };
 
 RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path, Error *r_error) {
-
-	if (r_error)
+	if (r_error) {
 		*r_error = ERR_CANT_OPEN;
+	}
 
 	Error err;
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ, &err);
-	if (!f)
+	if (!f) {
 		return RES();
+	}
 
 	FileAccessRef fref(f);
-	if (r_error)
+	if (r_error) {
 		*r_error = ERR_FILE_CORRUPT;
+	}
 
 	ERR_FAIL_COND_V_MSG(err != OK, RES(), "Unable to open DDS texture file '" + p_path + "'.");
 
@@ -120,8 +122,9 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 	uint32_t mipmaps = f->get_32();
 
 	//skip 11
-	for (int i = 0; i < 11; i++)
+	for (int i = 0; i < 11; i++) {
 		f->get_32();
+	}
 
 	//validate
 
@@ -158,70 +161,55 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 	*/
 
 	//must avoid this later
-	while (f->get_position() < 128)
+	while (f->get_position() < 128) {
 		f->get_8();
+	}
 
 	DDSFormat dds_format;
 
 	if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("DXT1")) {
-
 		dds_format = DDS_DXT1;
 	} else if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("DXT3")) {
-
 		dds_format = DDS_DXT3;
 
 	} else if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("DXT5")) {
-
 		dds_format = DDS_DXT5;
 	} else if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("ATI1")) {
-
 		dds_format = DDS_ATI1;
 	} else if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("ATI2")) {
-
 		dds_format = DDS_ATI2;
 	} else if (format_flags & DDPF_FOURCC && format_fourcc == PF_FOURCC("A2XY")) {
-
 		dds_format = DDS_A2XY;
 
 	} else if (format_flags & DDPF_RGB && format_flags & DDPF_ALPHAPIXELS && format_rgb_bits == 32 && format_red_mask == 0xff0000 && format_green_mask == 0xff00 && format_blue_mask == 0xff && format_alpha_mask == 0xff000000) {
-
 		dds_format = DDS_BGRA8;
 	} else if (format_flags & DDPF_RGB && !(format_flags & DDPF_ALPHAPIXELS) && format_rgb_bits == 24 && format_red_mask == 0xff0000 && format_green_mask == 0xff00 && format_blue_mask == 0xff) {
-
 		dds_format = DDS_BGR8;
 	} else if (format_flags & DDPF_RGB && format_flags & DDPF_ALPHAPIXELS && format_rgb_bits == 32 && format_red_mask == 0xff && format_green_mask == 0xff00 && format_blue_mask == 0xff0000 && format_alpha_mask == 0xff000000) {
-
 		dds_format = DDS_RGBA8;
 	} else if (format_flags & DDPF_RGB && !(format_flags & DDPF_ALPHAPIXELS) && format_rgb_bits == 24 && format_red_mask == 0xff && format_green_mask == 0xff00 && format_blue_mask == 0xff0000) {
-
 		dds_format = DDS_RGB8;
 
 	} else if (format_flags & DDPF_RGB && format_flags & DDPF_ALPHAPIXELS && format_rgb_bits == 16 && format_red_mask == 0x00007c00 && format_green_mask == 0x000003e0 && format_blue_mask == 0x0000001f && format_alpha_mask == 0x00008000) {
-
 		dds_format = DDS_BGR5A1;
 	} else if (format_flags & DDPF_RGB && format_flags & DDPF_ALPHAPIXELS && format_rgb_bits == 32 && format_red_mask == 0x3ff00000 && format_green_mask == 0xffc00 && format_blue_mask == 0x3ff && format_alpha_mask == 0xc0000000) {
-
 		dds_format = DDS_BGR10A2;
 	} else if (format_flags & DDPF_RGB && !(format_flags & DDPF_ALPHAPIXELS) && format_rgb_bits == 16 && format_red_mask == 0x0000f800 && format_green_mask == 0x000007e0 && format_blue_mask == 0x0000001f) {
-
 		dds_format = DDS_BGR565;
 	} else if (!(format_flags & DDPF_ALPHAPIXELS) && format_rgb_bits == 8 && format_red_mask == 0xff && format_green_mask == 0xff && format_blue_mask == 0xff) {
-
 		dds_format = DDS_LUMINANCE;
 	} else if ((format_flags & DDPF_ALPHAPIXELS) && format_rgb_bits == 16 && format_red_mask == 0xff && format_green_mask == 0xff && format_blue_mask == 0xff && format_alpha_mask == 0xff00) {
-
 		dds_format = DDS_LUMINANCE_ALPHA;
 	} else if (format_flags & DDPF_INDEXED && format_rgb_bits == 8) {
-
 		dds_format = DDS_BGR565;
 	} else {
-
 		printf("unrecognized fourcc %x format_flags: %x - rgbbits %i - red_mask %x green mask %x blue mask %x alpha mask %x\n", format_fourcc, format_flags, format_rgb_bits, format_red_mask, format_green_mask, format_blue_mask, format_alpha_mask);
 		ERR_FAIL_V_MSG(RES(), "Unrecognized or unsupported color layout in DDS '" + p_path + "'.");
 	}
 
-	if (!(flags & DDSD_MIPMAPCOUNT))
+	if (!(flags & DDSD_MIPMAPCOUNT)) {
 		mipmaps = 1;
+	}
 
 	PoolVector<uint8_t> src_data;
 
@@ -237,7 +225,6 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 		ERR_FAIL_COND_V(!(flags & DDSD_LINEARSIZE), RES());
 
 		for (uint32_t i = 1; i < mipmaps; i++) {
-
 			w = MAX(1, w >> 1);
 			h = MAX(1, h >> 1);
 			uint32_t bsize = MAX(info.divisor, w) / info.divisor * MAX(info.divisor, h) / info.divisor * info.block_size;
@@ -250,7 +237,6 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 		f->get_buffer(wb.ptr(), size);
 
 	} else if (info.palette) {
-
 		//indexed
 		ERR_FAIL_COND_V(!(flags & DDSD_PITCH), RES());
 		ERR_FAIL_COND_V(format_rgb_bits != 8, RES());
@@ -263,16 +249,15 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 
 		int colsize = 3;
 		for (int i = 0; i < 256; i++) {
-
-			if (palette[i * 4 + 3] < 255)
+			if (palette[i * 4 + 3] < 255) {
 				colsize = 4;
+			}
 		}
 
 		int w2 = width;
 		int h2 = height;
 
 		for (uint32_t i = 1; i < mipmaps; i++) {
-
 			w2 = (w2 + 1) >> 1;
 			h2 = (h2 + 1) >> 1;
 			size += w2 * h2 * info.block_size;
@@ -283,14 +268,14 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 		f->get_buffer(wb.ptr(), size);
 
 		for (int i = 0; i < 256; i++) {
-
 			int dst_ofs = size + i * colsize;
 			int src_ofs = i * 4;
 			wb[dst_ofs + 0] = palette[src_ofs + 2];
 			wb[dst_ofs + 1] = palette[src_ofs + 1];
 			wb[dst_ofs + 2] = palette[src_ofs + 0];
-			if (colsize == 4)
+			if (colsize == 4) {
 				wb[dst_ofs + 3] = palette[src_ofs + 3];
+			}
 		}
 	} else {
 		//uncompressed generic...
@@ -298,30 +283,27 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 		uint32_t size = width * height * info.block_size;
 
 		for (uint32_t i = 1; i < mipmaps; i++) {
-
 			w = (w + 1) >> 1;
 			h = (h + 1) >> 1;
 			size += w * h * info.block_size;
 		}
 
-		if (dds_format == DDS_BGR565)
+		if (dds_format == DDS_BGR565) {
 			size = size * 3 / 2;
-		else if (dds_format == DDS_BGR5A1)
+		} else if (dds_format == DDS_BGR5A1) {
 			size = size * 2;
+		}
 
 		src_data.resize(size);
 		PoolVector<uint8_t>::Write wb = src_data.write();
 		f->get_buffer(wb.ptr(), size);
 
 		switch (dds_format) {
-
 			case DDS_BGR5A1: {
-
 				// TO RGBA
 				int colcount = size / 4;
 
 				for (int i = colcount - 1; i >= 0; i--) {
-
 					int src_ofs = i * 2;
 					int dst_ofs = i * 4;
 
@@ -336,11 +318,9 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 				}
 			} break;
 			case DDS_BGR565: {
-
 				int colcount = size / 3;
 
 				for (int i = colcount - 1; i >= 0; i--) {
-
 					int src_ofs = i * 2;
 					int dst_ofs = i * 3;
 
@@ -354,12 +334,10 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 
 			} break;
 			case DDS_BGR10A2: {
-
 				// TO RGBA
 				int colcount = size / 4;
 
 				for (int i = colcount - 1; i >= 0; i--) {
-
 					int ofs = i * 4;
 
 					uint32_t w32 = uint32_t(wb[ofs + 0]) | (uint32_t(wb[ofs + 1]) << 8) | (uint32_t(wb[ofs + 2]) << 16) | (uint32_t(wb[ofs + 3]) << 24);
@@ -376,26 +354,21 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 				}
 			} break;
 			case DDS_BGRA8: {
-
 				int colcount = size / 4;
 
 				for (int i = 0; i < colcount; i++) {
-
 					SWAP(wb[i * 4 + 0], wb[i * 4 + 2]);
 				}
 
 			} break;
 			case DDS_BGR8: {
-
 				int colcount = size / 3;
 
 				for (int i = 0; i < colcount; i++) {
-
 					SWAP(wb[i * 3 + 0], wb[i * 3 + 2]);
 				}
 			} break;
 			case DDS_RGBA8: {
-
 				/* do nothing either
 				int colcount = size/4;
 
@@ -414,7 +387,6 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 				*/
 			} break;
 			case DDS_RGB8: {
-
 				// do nothing
 				/*
 				int colcount = size/3;
@@ -425,12 +397,10 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 				}*/
 			} break;
 			case DDS_LUMINANCE: {
-
 				// do nothing i guess?
 
 			} break;
 			case DDS_LUMINANCE_ALPHA: {
-
 				// do nothing i guess?
 
 			} break;
@@ -445,25 +415,24 @@ RES ResourceFormatDDS::load(const String &p_path, const String &p_original_path,
 	Ref<ImageTexture> texture = memnew(ImageTexture);
 	texture->create_from_image(img);
 
-	if (r_error)
+	if (r_error) {
 		*r_error = OK;
+	}
 
 	return texture;
 }
 
 void ResourceFormatDDS::get_recognized_extensions(List<String> *p_extensions) const {
-
 	p_extensions->push_back("dds");
 }
 
 bool ResourceFormatDDS::handles_type(const String &p_type) const {
-
 	return ClassDB::is_parent_class(p_type, "Texture");
 }
 
 String ResourceFormatDDS::get_resource_type(const String &p_path) const {
-
-	if (p_path.get_extension().to_lower() == "dds")
+	if (p_path.get_extension().to_lower() == "dds") {
 		return "ImageTexture";
+	}
 	return "";
 }
