@@ -1,39 +1,39 @@
-/*************************************************************************/
-/*  video_stream_gdnative.cpp                                            */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  video_stream_gdnative.cpp                                             */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "video_stream_gdnative.h"
 
 #include "core/project_settings.h"
 #include "servers/audio_server.h"
 
-VideoDecoderServer *VideoDecoderServer::instance = NULL;
+VideoDecoderServer *VideoDecoderServer::instance = nullptr;
 
 static VideoDecoderServer decoder_server;
 
@@ -47,11 +47,7 @@ godot_int GDAPI godot_videodecoder_file_read(void *ptr, uint8_t *buf, int buf_si
 
 	// if file exists
 	if (file) {
-		long bytes_read = file->get_buffer(buf, buf_size);
-		// No bytes to read => EOF
-		if (bytes_read == 0) {
-			return 0;
-		}
+		int64_t bytes_read = file->get_buffer(buf, buf_size);
 		return bytes_read;
 	}
 	return -1;
@@ -62,41 +58,35 @@ int64_t GDAPI godot_videodecoder_file_seek(void *ptr, int64_t pos, int whence) {
 	FileAccess *file = reinterpret_cast<FileAccess *>(ptr);
 
 	if (file) {
-		size_t len = file->get_len();
+		int64_t len = file->get_len();
 		switch (whence) {
 			case SEEK_SET: {
-				// Just for explicitness
-				size_t new_pos = static_cast<size_t>(pos);
-				if (new_pos > len) {
+				if (pos > len) {
 					return -1;
 				}
-				file->seek(new_pos);
-				pos = static_cast<int64_t>(file->get_position());
-				return pos;
+				file->seek(pos);
+				return file->get_position();
 			} break;
 			case SEEK_CUR: {
 				// Just in case it doesn't exist
-				if (pos < 0 && (size_t)-pos > file->get_position()) {
+				if (pos < 0 && -pos > (int64_t)file->get_position()) {
 					return -1;
 				}
-				pos = pos + static_cast<int>(file->get_position());
-				file->seek(pos);
-				pos = static_cast<int64_t>(file->get_position());
-				return pos;
+				file->seek(file->get_position() + pos);
+				return file->get_position();
 			} break;
 			case SEEK_END: {
 				// Just in case something goes wrong
-				if ((size_t)-pos > len) {
+				if (-pos > len) {
 					return -1;
 				}
 				file->seek_end(pos);
-				pos = static_cast<int64_t>(file->get_position());
-				return pos;
+				return file->get_position();
 			} break;
 			default: {
 				// Only 4 possible options, hence default = AVSEEK_SIZE
 				// Asks to return the length of file
-				return static_cast<int64_t>(len);
+				return len;
 			} break;
 		}
 	}
@@ -105,7 +95,6 @@ int64_t GDAPI godot_videodecoder_file_seek(void *ptr, int64_t pos, int whence) {
 }
 
 void GDAPI godot_videodecoder_register_decoder(const godot_videodecoder_interface_gdnative *p_interface) {
-
 	decoder_server.register_decoder_interface(p_interface);
 }
 }
@@ -113,7 +102,7 @@ void GDAPI godot_videodecoder_register_decoder(const godot_videodecoder_interfac
 // VideoStreamPlaybackGDNative starts here.
 
 bool VideoStreamPlaybackGDNative::open_file(const String &p_file) {
-	ERR_FAIL_COND_V(interface == NULL, false);
+	ERR_FAIL_COND_V(interface == nullptr, false);
 	file = FileAccess::open(p_file, FileAccess::READ);
 	bool file_opened = interface->open_file(data_struct, file);
 
@@ -146,7 +135,7 @@ void VideoStreamPlaybackGDNative::update(float p_delta) {
 		return;
 	}
 	time += p_delta;
-	ERR_FAIL_COND(interface == NULL);
+	ERR_FAIL_COND(interface == nullptr);
 	interface->update(data_struct, p_delta);
 
 	// Don't mix if there's no audio (num_channels == 0).
@@ -185,7 +174,7 @@ void VideoStreamPlaybackGDNative::update(float p_delta) {
 void VideoStreamPlaybackGDNative::update_texture() {
 	PoolByteArray *pba = (PoolByteArray *)interface->get_videoframe(data_struct);
 
-	if (pba == NULL) {
+	if (pba == nullptr) {
 		playing = false;
 		return;
 	}
@@ -201,44 +190,46 @@ VideoStreamPlaybackGDNative::VideoStreamPlaybackGDNative() :
 		texture(Ref<ImageTexture>(memnew(ImageTexture))),
 		playing(false),
 		paused(false),
-		mix_udata(NULL),
-		mix_callback(NULL),
+		mix_udata(nullptr),
+		mix_callback(nullptr),
 		num_channels(-1),
 		time(0),
 		seek_backward(false),
 		mix_rate(0),
 		delay_compensation(0),
-		pcm(NULL),
+		pcm(nullptr),
 		pcm_write_idx(0),
 		samples_decoded(0),
-		file(NULL),
-		interface(NULL),
-		data_struct(NULL) {}
+		file(nullptr),
+		interface(nullptr),
+		data_struct(nullptr) {}
 
 VideoStreamPlaybackGDNative::~VideoStreamPlaybackGDNative() {
 	cleanup();
 }
 
 void VideoStreamPlaybackGDNative::cleanup() {
-	if (data_struct)
+	if (data_struct) {
 		interface->destructor(data_struct);
-	if (pcm)
+	}
+	if (pcm) {
 		memfree(pcm);
+	}
 	if (file) {
 		file->close();
 		memdelete(file);
-		file = NULL;
+		file = nullptr;
 	}
-	pcm = NULL;
+	pcm = nullptr;
 	time = 0;
 	num_channels = -1;
-	interface = NULL;
-	data_struct = NULL;
+	interface = nullptr;
+	data_struct = nullptr;
 }
 
 void VideoStreamPlaybackGDNative::set_interface(const godot_videodecoder_interface_gdnative *p_interface) {
-	ERR_FAIL_COND(p_interface == NULL);
-	if (interface != NULL) {
+	ERR_FAIL_COND(p_interface == nullptr);
+	if (interface != nullptr) {
 		cleanup();
 	}
 	interface = p_interface;
@@ -256,7 +247,6 @@ bool VideoStreamPlaybackGDNative::is_paused() const {
 }
 
 void VideoStreamPlaybackGDNative::play() {
-
 	stop();
 
 	playing = true;
@@ -273,10 +263,11 @@ void VideoStreamPlaybackGDNative::stop() {
 }
 
 void VideoStreamPlaybackGDNative::seek(float p_time) {
-	ERR_FAIL_COND(interface == NULL);
+	ERR_FAIL_COND(interface == nullptr);
 	interface->seek(data_struct, p_time);
-	if (p_time < time)
+	if (p_time < time) {
 		seek_backward = true;
+	}
 	time = p_time;
 	// reset audio buffers
 	memset(pcm, 0, num_channels * AUX_BUFFER_SIZE * sizeof(float));
@@ -293,13 +284,12 @@ Ref<Texture> VideoStreamPlaybackGDNative::get_texture() const {
 }
 
 float VideoStreamPlaybackGDNative::get_length() const {
-	ERR_FAIL_COND_V(interface == NULL, 0);
+	ERR_FAIL_COND_V(interface == nullptr, 0);
 	return interface->get_length(data_struct);
 }
 
 float VideoStreamPlaybackGDNative::get_playback_position() const {
-
-	ERR_FAIL_COND_V(interface == NULL, 0);
+	ERR_FAIL_COND_V(interface == nullptr, 0);
 	return interface->get_playback_position(data_struct);
 }
 
@@ -313,24 +303,23 @@ void VideoStreamPlaybackGDNative::set_loop(bool p_enable) {
 }
 
 void VideoStreamPlaybackGDNative::set_audio_track(int p_idx) {
-	ERR_FAIL_COND(interface == NULL);
+	ERR_FAIL_COND(interface == nullptr);
 	interface->set_audio_track(data_struct, p_idx);
 }
 
 void VideoStreamPlaybackGDNative::set_mix_callback(AudioMixCallback p_callback, void *p_userdata) {
-
 	mix_udata = p_userdata;
 	mix_callback = p_callback;
 }
 
 int VideoStreamPlaybackGDNative::get_channels() const {
-	ERR_FAIL_COND_V(interface == NULL, 0);
+	ERR_FAIL_COND_V(interface == nullptr, 0);
 
 	return (num_channels > 0) ? num_channels : 0;
 }
 
 int VideoStreamPlaybackGDNative::get_mix_rate() const {
-	ERR_FAIL_COND_V(interface == NULL, 0);
+	ERR_FAIL_COND_V(interface == nullptr, 0);
 
 	return mix_rate;
 }
@@ -340,27 +329,26 @@ int VideoStreamPlaybackGDNative::get_mix_rate() const {
 Ref<VideoStreamPlayback> VideoStreamGDNative::instance_playback() {
 	Ref<VideoStreamPlaybackGDNative> pb = memnew(VideoStreamPlaybackGDNative);
 	VideoDecoderGDNative *decoder = decoder_server.get_decoder(file.get_extension().to_lower());
-	if (decoder == NULL)
-		return NULL;
+	if (decoder == nullptr) {
+		return nullptr;
+	}
 	pb->set_interface(decoder->interface);
 	pb->set_audio_track(audio_track);
-	if (pb->open_file(file))
+	if (pb->open_file(file)) {
 		return pb;
-	return NULL;
+	}
+	return nullptr;
 }
 
 void VideoStreamGDNative::set_file(const String &p_file) {
-
 	file = p_file;
 }
 
 String VideoStreamGDNative::get_file() {
-
 	return file;
 }
 
 void VideoStreamGDNative::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_file", "file"), &VideoStreamGDNative::set_file);
 	ClassDB::bind_method(D_METHOD("get_file"), &VideoStreamGDNative::get_file);
 
@@ -368,7 +356,6 @@ void VideoStreamGDNative::_bind_methods() {
 }
 
 void VideoStreamGDNative::set_audio_track(int p_track) {
-
 	audio_track = p_track;
 }
 
@@ -406,7 +393,8 @@ bool ResourceFormatLoaderVideoStreamGDNative::handles_type(const String &p_type)
 
 String ResourceFormatLoaderVideoStreamGDNative::get_resource_type(const String &p_path) const {
 	String el = p_path.get_extension().to_lower();
-	if (VideoDecoderServer::get_instance()->get_extensions().has(el))
+	if (VideoDecoderServer::get_instance()->get_extensions().has(el)) {
 		return "VideoStreamGDNative";
+	}
 	return "";
 }
