@@ -1,14 +1,14 @@
 def can_build(env, platform):
-	  return True
+    return True
+
 
 def configure(env):
-	pass
+    pass
+
 
 def get_doc_classes():
-    return [
-        "NoiseTexture",
-        "OpenSimplexNoise"
-    ]
+    return ["NoiseTexture", "OpenSimplexNoise"]
+
 
 def get_doc_path():
     return "doc_classes"
