@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  video_stream_theora.cpp                                              */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  video_stream_theora.cpp                                               */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "video_stream_theora.h"
 
@@ -36,7 +36,6 @@
 #include "thirdparty/misc/yuv2rgb.h"
 
 int VideoStreamPlaybackTheora::buffer_data() {
-
 	char *buffer = ogg_sync_buffer(&oy, 4096);
 
 #ifdef THEORA_USE_THREAD_STREAMING
@@ -44,7 +43,7 @@ int VideoStreamPlaybackTheora::buffer_data() {
 	int read;
 
 	do {
-		thread_sem->post();
+		thread_sem.post();
 		read = MIN(ring_buffer.data_left(), 4096);
 		if (read) {
 			ring_buffer.read((uint8_t *)buffer, read);
@@ -59,7 +58,7 @@ int VideoStreamPlaybackTheora::buffer_data() {
 
 #else
 
-	int bytes = file->get_buffer((uint8_t *)buffer, 4096);
+	uint64_t bytes = file->get_buffer((uint8_t *)buffer, 4096);
 	ogg_sync_wrote(&oy, bytes);
 	return (bytes);
 
@@ -69,18 +68,20 @@ int VideoStreamPlaybackTheora::buffer_data() {
 int VideoStreamPlaybackTheora::queue_page(ogg_page *page) {
 	if (theora_p) {
 		ogg_stream_pagein(&to, page);
-		if (to.e_o_s)
+		if (to.e_o_s) {
 			theora_eos = true;
+		}
 	}
 	if (vorbis_p) {
 		ogg_stream_pagein(&vo, page);
-		if (vo.e_o_s)
+		if (vo.e_o_s) {
 			vorbis_eos = true;
+		}
 	}
 	return 0;
 }
 
-void VideoStreamPlaybackTheora::video_write(void) {
+void VideoStreamPlaybackTheora::video_write() {
 	th_ycbcr_buffer yuv;
 	th_decode_ycbcr_out(td, yuv);
 
@@ -93,15 +94,12 @@ void VideoStreamPlaybackTheora::video_write(void) {
 		//uv_offset=(ti.pic_x/2)+(yuv[1].stride)*(ti.pic_y/2);
 
 		if (px_fmt == TH_PF_444) {
-
 			yuv444_2_rgb8888((uint8_t *)dst, (uint8_t *)yuv[0].data, (uint8_t *)yuv[1].data, (uint8_t *)yuv[2].data, size.x, size.y, yuv[0].stride, yuv[1].stride, size.x << 2);
 
 		} else if (px_fmt == TH_PF_422) {
-
 			yuv422_2_rgb8888((uint8_t *)dst, (uint8_t *)yuv[0].data, (uint8_t *)yuv[1].data, (uint8_t *)yuv[2].data, size.x, size.y, yuv[0].stride, yuv[1].stride, size.x << 2);
 
 		} else if (px_fmt == TH_PF_420) {
-
 			yuv420_2_rgb8888((uint8_t *)dst, (uint8_t *)yuv[0].data, (uint8_t *)yuv[1].data, (uint8_t *)yuv[2].data, size.x, size.y, yuv[0].stride, yuv[1].stride, size.x << 2);
 		};
 
@@ -116,9 +114,9 @@ void VideoStreamPlaybackTheora::video_write(void) {
 }
 
 void VideoStreamPlaybackTheora::clear() {
-
-	if (!file)
+	if (!file) {
 		return;
+	}
 
 	if (vorbis_p) {
 		ogg_stream_clear(&vo);
@@ -141,10 +139,8 @@ void VideoStreamPlaybackTheora::clear() {
 
 #ifdef THEORA_USE_THREAD_STREAMING
 	thread_exit = true;
-	thread_sem->post(); //just in case
-	Thread::wait_to_finish(thread);
-	memdelete(thread);
-	thread = NULL;
+	thread_sem.post(); //just in case
+	thread.wait_to_finish();
 	ring_buffer.clear();
 #endif
 
@@ -159,15 +155,14 @@ void VideoStreamPlaybackTheora::clear() {
 	if (file) {
 		memdelete(file);
 	}
-	file = NULL;
+	file = nullptr;
 	playing = false;
 };
 
 void VideoStreamPlaybackTheora::set_file(const String &p_file) {
-
 	ERR_FAIL_COND(playing);
 	ogg_packet op;
-	th_setup_info *ts = NULL;
+	th_setup_info *ts = nullptr;
 
 	file_name = p_file;
 	if (file) {
@@ -181,10 +176,10 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 	thread_eof = false;
 	//pre-fill buffer
 	int to_read = ring_buffer.space_left();
-	int read = file->get_buffer(read_buffer.ptr(), to_read);
+	uint64_t read = file->get_buffer(read_buffer.ptr(), to_read);
 	ring_buffer.write(read_buffer.ptr(), read);
 
-	thread = Thread::create(_streaming_thread, this);
+	thread.start(_streaming_thread, this);
 
 #endif
 
@@ -209,7 +204,9 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 
 	while (!stateflag) {
 		int ret = buffer_data();
-		if (ret == 0) break;
+		if (ret == 0) {
+			break;
+		}
 		while (ogg_sync_pageout(&oy, &og) > 0) {
 			ogg_stream_state test;
 
@@ -228,10 +225,9 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 			/* identify the codec: try theora */
 			if (!theora_p && th_decode_headerin(&ti, &tc, &ts, &op) >= 0) {
 				/* it is theora */
-				copymem(&to, &test, sizeof(test));
+				memcpy(&to, &test, sizeof(test));
 				theora_p = 1;
 			} else if (!vorbis_p && vorbis_synthesis_headerin(&vi, &vc, &op) >= 0) {
-
 				/* it is vorbis */
 				if (audio_track_skip) {
 					vorbis_info_clear(&vi);
@@ -242,7 +238,7 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 
 					audio_track_skip--;
 				} else {
-					copymem(&vo, &test, sizeof(test));
+					memcpy(&vo, &test, sizeof(test));
 					vorbis_p = 1;
 				}
 			} else {
@@ -286,7 +282,9 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 				return;
 			}
 			vorbis_p++;
-			if (vorbis_p == 3) break;
+			if (vorbis_p == 3) {
+				break;
+			}
 		}
 
 		/* The header pages/packets will arrive before anything else we
@@ -304,7 +302,7 @@ void VideoStreamPlaybackTheora::set_file(const String &p_file) {
 		}
 	}
 
-	/* and now we have it all.  initialize decoders */
+	/* And now we have it all. Initialize decoders. */
 	if (theora_p) {
 		td = th_decode_alloc(&ti, ts);
 		px_fmt = ti.pixel_fmt;
@@ -370,14 +368,13 @@ float VideoStreamPlaybackTheora::get_time() const {
 };
 
 Ref<Texture> VideoStreamPlaybackTheora::get_texture() const {
-
 	return texture;
 }
 
 void VideoStreamPlaybackTheora::update(float p_delta) {
-
-	if (!file)
+	if (!file) {
 		return;
+	}
 
 	if (!playing || paused) {
 		//printf("not playing\n");
@@ -385,7 +382,7 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 	};
 
 #ifdef THEORA_USE_THREAD_STREAMING
-	thread_sem->post();
+	thread_sem.post();
 #endif
 
 	time += p_delta;
@@ -411,13 +408,11 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 			/* if there's pending, decoded audio, grab it */
 			ret = vorbis_synthesis_pcmout(&vd, &pcm);
 			if (ret > 0) {
-
 				const int AUXBUF_LEN = 4096;
 				int to_read = ret;
 				float aux_buffer[AUXBUF_LEN];
 
 				while (to_read) {
-
 					int m = MIN(AUXBUF_LEN / vi.channels, to_read);
 
 					int count = 0;
@@ -445,7 +440,6 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 				audio_frames_wrote += ret - to_read;
 
 			} else {
-
 				/* no pending audio; is there a pending packet to decode? */
 				if (ogg_stream_packetout(&vo, &op) > 0) {
 					if (vorbis_synthesis(&vb, &op) == 0) { /* test for success! */
@@ -458,14 +452,14 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 
 			audio_done = videobuf_time < (audio_frames_wrote / float(vi.rate));
 
-			if (buffer_full)
+			if (buffer_full) {
 				break;
+			}
 		}
 
 		while (theora_p && !frame_done) {
 			/* theora is one in, one out... */
 			if (ogg_stream_packetout(&to, &op) > 0) {
-
 				if (false && pp_inc) {
 					pp_level += pp_inc;
 					th_decode_ctl(td, TH_DECCTL_SET_PPLEVEL, &pp_level,
@@ -488,10 +482,10 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 
 					//printf("frame time %f, play time %f, ready %i\n", (float)videobuf_time, get_time(), videobuf_ready);
 
-					/* is it already too old to be useful?  This is only actually
-					 useful cosmetically after a SIGSTOP.  Note that we have to
+					/* is it already too old to be useful? This is only actually
+					 useful cosmetically after a SIGSTOP. Note that we have to
 					 decode the frame even if we don't show it (for now) due to
-					 keyframing.  Soon enough libtheora will be able to deal
+					 keyframing. Soon enough libtheora will be able to deal
 					 with non-keyframe seeks.  */
 
 					if (videobuf_time >= get_time()) {
@@ -532,7 +526,6 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 
 		/* are we at or past time for this video frame? */
 		if (videobuf_ready && videobuf_time <= get_time()) {
-
 			//video_write();
 			//videobuf_ready=0;
 		} else {
@@ -552,10 +545,9 @@ void VideoStreamPlaybackTheora::update(float p_delta) {
 };
 
 void VideoStreamPlaybackTheora::play() {
-
-	if (!playing)
+	if (!playing) {
 		time = 0;
-	else {
+	} else {
 		stop();
 	}
 
@@ -565,9 +557,7 @@ void VideoStreamPlaybackTheora::play() {
 };
 
 void VideoStreamPlaybackTheora::stop() {
-
 	if (playing) {
-
 		clear();
 		set_file(file_name); //reset
 	}
@@ -576,17 +566,14 @@ void VideoStreamPlaybackTheora::stop() {
 };
 
 bool VideoStreamPlaybackTheora::is_playing() const {
-
 	return playing;
 };
 
 void VideoStreamPlaybackTheora::set_paused(bool p_paused) {
-
 	paused = p_paused;
 };
 
 bool VideoStreamPlaybackTheora::is_paused() const {
-
 	return paused;
 };
 
@@ -595,84 +582,70 @@ void VideoStreamPlaybackTheora::set_loop(bool p_enable){
 };
 
 bool VideoStreamPlaybackTheora::has_loop() const {
-
 	return false;
 };
 
 float VideoStreamPlaybackTheora::get_length() const {
-
 	return 0;
 };
 
 String VideoStreamPlaybackTheora::get_stream_name() const {
-
 	return "";
 };
 
 int VideoStreamPlaybackTheora::get_loop_count() const {
-
 	return 0;
 };
 
 float VideoStreamPlaybackTheora::get_playback_position() const {
-
 	return get_time();
 };
 
-void VideoStreamPlaybackTheora::seek(float p_time){
-
-	// no
-};
+void VideoStreamPlaybackTheora::seek(float p_time) {
+	WARN_PRINT_ONCE("Seeking in Theora and WebM videos is not implemented yet (it's only supported for GDNative-provided video streams).");
+}
 
 void VideoStreamPlaybackTheora::set_mix_callback(AudioMixCallback p_callback, void *p_userdata) {
-
 	mix_callback = p_callback;
 	mix_udata = p_userdata;
 }
 
 int VideoStreamPlaybackTheora::get_channels() const {
-
 	return vi.channels;
 }
 
 void VideoStreamPlaybackTheora::set_audio_track(int p_idx) {
-
 	audio_track = p_idx;
 }
 
 int VideoStreamPlaybackTheora::get_mix_rate() const {
-
 	return vi.rate;
 }
 
 #ifdef THEORA_USE_THREAD_STREAMING
 
 void VideoStreamPlaybackTheora::_streaming_thread(void *ud) {
-
 	VideoStreamPlaybackTheora *vs = (VideoStreamPlaybackTheora *)ud;
 
 	while (!vs->thread_exit) {
-
 		//just fill back the buffer
 		if (!vs->thread_eof) {
-
 			int to_read = vs->ring_buffer.space_left();
-			if (to_read) {
-				int read = vs->file->get_buffer(vs->read_buffer.ptr(), to_read);
+			if (to_read > 0) {
+				uint64_t read = vs->file->get_buffer(vs->read_buffer.ptr(), to_read);
 				vs->ring_buffer.write(vs->read_buffer.ptr(), read);
 				vs->thread_eof = vs->file->eof_reached();
 			}
 		}
 
-		vs->thread_sem->wait();
+		vs->thread_sem.wait();
 	}
 }
 
 #endif
 
 VideoStreamPlaybackTheora::VideoStreamPlaybackTheora() {
-
-	file = NULL;
+	file = nullptr;
 	theora_p = 0;
 	vorbis_p = 0;
 	videobuf_ready = 0;
@@ -683,8 +656,8 @@ VideoStreamPlaybackTheora::VideoStreamPlaybackTheora() {
 
 	buffering = false;
 	texture = Ref<ImageTexture>(memnew(ImageTexture));
-	mix_callback = NULL;
-	mix_udata = NULL;
+	mix_callback = nullptr;
+	mix_udata = nullptr;
 	audio_track = 0;
 	delay_compensation = 0;
 	audio_frames_wrote = 0;
@@ -693,8 +666,6 @@ VideoStreamPlaybackTheora::VideoStreamPlaybackTheora() {
 	int rb_power = nearest_shift(RB_SIZE_KB * 1024);
 	ring_buffer.resize(rb_power);
 	read_buffer.resize(RB_SIZE_KB * 1024);
-	thread_sem = Semaphore::create();
-	thread = NULL;
 	thread_exit = false;
 	thread_eof = false;
 
@@ -702,19 +673,14 @@ VideoStreamPlaybackTheora::VideoStreamPlaybackTheora() {
 };
 
 VideoStreamPlaybackTheora::~VideoStreamPlaybackTheora() {
-
-#ifdef THEORA_USE_THREAD_STREAMING
-
-	memdelete(thread_sem);
-#endif
 	clear();
 
-	if (file)
+	if (file) {
 		memdelete(file);
+	}
 };
 
 void VideoStreamTheora::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_file", "file"), &VideoStreamTheora::set_file);
 	ClassDB::bind_method(D_METHOD("get_file"), &VideoStreamTheora::get_file);
 
@@ -724,7 +690,6 @@ void VideoStreamTheora::_bind_methods() {
 ////////////
 
 RES ResourceFormatLoaderTheora::load(const String &p_path, const String &p_original_path, Error *r_error) {
-
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ);
 	if (!f) {
 		if (r_error) {
@@ -748,19 +713,17 @@ RES ResourceFormatLoaderTheora::load(const String &p_path, const String &p_origi
 }
 
 void ResourceFormatLoaderTheora::get_recognized_extensions(List<String> *p_extensions) const {
-
 	p_extensions->push_back("ogv");
 }
 
 bool ResourceFormatLoaderTheora::handles_type(const String &p_type) const {
-
 	return ClassDB::is_parent_class(p_type, "VideoStream");
 }
 
 String ResourceFormatLoaderTheora::get_resource_type(const String &p_path) const {
-
 	String el = p_path.get_extension().to_lower();
-	if (el == "ogv")
+	if (el == "ogv") {
 		return "VideoStreamTheora";
+	}
 	return "";
 }
