@@ -1,14 +1,14 @@
 def can_build(env, platform):
     return True
 
+
 def configure(env):
     pass
 
+
 def get_doc_classes():
-    return [
-        "UPNP",
-        "UPNPDevice"
-    ]
+    return ["UPNP", "UPNPDevice"]
+
 
 def get_doc_path():
     return "doc_classes"
