@@ -1,34 +1,35 @@
-/*************************************************************************/
-/*  export.cpp                                                           */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  export.cpp                                                            */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "export.h"
+
 #include "core/bind/core_bind.h"
 #include "core/crypto/crypto_core.h"
 #include "core/io/marshalls.h"
@@ -42,6 +43,12 @@
 #include "editor/editor_node.h"
 #include "platform/uwp/logo.gen.h"
 
+// Mono build doesn't support UWP, so we show a specific error.
+// We don't bypass the whole logic so that it doesn't lose potential UWP presets
+// added to export_presets.cfg from a non-Mono build (and in case third-parties
+// actually have Mono-enabled UWP templates they can use).
+#include "modules/modules_enabled.gen.h" // For mono.
+
 #include "thirdparty/minizip/unzip.h"
 #include "thirdparty/minizip/zip.h"
 
@@ -54,7 +61,7 @@ static const char *uwp_capabilities[] = {
 	"internetClient",
 	"internetClientServer",
 	"privateNetworkClientServer",
-	NULL
+	nullptr
 };
 static const char *uwp_uap_capabilities[] = {
 	"appointments",
@@ -71,7 +78,7 @@ static const char *uwp_uap_capabilities[] = {
 	"userAccountInformation",
 	"videosLibrary",
 	"voipCall",
-	NULL
+	nullptr
 };
 static const char *uwp_device_capabilities[] = {
 	"bluetooth",
@@ -79,11 +86,10 @@ static const char *uwp_device_capabilities[] = {
 	"microphone",
 	"proximity",
 	"webcam",
-	NULL
+	nullptr
 };
 
 class AppxPackager {
-
 	enum {
 		FILE_HEADER_MAGIC = 0x04034b50,
 		DATA_DESCRIPTOR_MAGIC = 0x08074b50,
@@ -107,13 +113,11 @@ class AppxPackager {
 	};
 
 	struct BlockHash {
-
 		String base64_hash;
 		size_t compressed_size;
 	};
 
 	struct FileMeta {
-
 		String name;
 		int lfh_size;
 		bool compressed;
@@ -195,7 +199,6 @@ public:
 ///////////////////////////////////////////////////////////////////////////
 
 String AppxPackager::hash_block(const uint8_t *p_block_data, size_t p_block_len) {
-
 	unsigned char hash[32];
 	char base64[45];
 
@@ -208,24 +211,22 @@ String AppxPackager::hash_block(const uint8_t *p_block_data, size_t p_block_len)
 }
 
 void AppxPackager::make_block_map(const String &p_path) {
-
 	FileAccess *tmp_file = FileAccess::open(p_path, FileAccess::WRITE);
 
 	tmp_file->store_string("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>");
 	tmp_file->store_string("<BlockMap xmlns=\"http://schemas.microsoft.com/appx/2010/blockmap\" HashMethod=\"http://www.w3.org/2001/04/xmlenc#sha256\">");
 
 	for (int i = 0; i < file_metadata.size(); i++) {
-
 		FileMeta file = file_metadata[i];
 
 		tmp_file->store_string(
 				"<File Name=\"" + file.name.replace("/", "\\") + "\" Size=\"" + itos(file.uncompressed_size) + "\" LfhSize=\"" + itos(file.lfh_size) + "\">");
 
 		for (int j = 0; j < file.hashes.size(); j++) {
-
 			tmp_file->store_string("<Block Hash=\"" + file.hashes[j].base64_hash + "\" ");
-			if (file.compressed)
+			if (file.compressed) {
 				tmp_file->store_string("Size=\"" + itos(file.hashes[j].compressed_size) + "\" ");
+			}
 			tmp_file->store_string("/>");
 		}
 
@@ -239,21 +240,20 @@ void AppxPackager::make_block_map(const String &p_path) {
 }
 
 String AppxPackager::content_type(String p_extension) {
-
-	if (p_extension == "png")
+	if (p_extension == "png") {
 		return "image/png";
-	else if (p_extension == "jpg")
+	} else if (p_extension == "jpg") {
 		return "image/jpg";
-	else if (p_extension == "xml")
+	} else if (p_extension == "xml") {
 		return "application/xml";
-	else if (p_extension == "exe" || p_extension == "dll")
+	} else if (p_extension == "exe" || p_extension == "dll") {
 		return "application/x-msdownload";
-	else
+	} else {
 		return "application/octet-stream";
+	}
 }
 
 void AppxPackager::make_content_types(const String &p_path) {
-
 	FileAccess *tmp_file = FileAccess::open(p_path, FileAccess::WRITE);
 
 	tmp_file->store_string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
@@ -262,10 +262,11 @@ void AppxPackager::make_content_types(const String &p_path) {
 	Map<String, String> types;
 
 	for (int i = 0; i < file_metadata.size(); i++) {
+		String ext = file_metadata[i].name.get_extension().to_lower();
 
-		String ext = file_metadata[i].name.get_extension();
-
-		if (types.has(ext)) continue;
+		if (types.has(ext)) {
+			continue;
+		}
 
 		types[ext] = content_type(ext);
 
@@ -288,7 +289,6 @@ void AppxPackager::make_content_types(const String &p_path) {
 }
 
 Vector<uint8_t> AppxPackager::make_file_header(FileMeta p_file_meta) {
-
 	Vector<uint8_t> buf;
 	buf.resize(BASE_FILE_HEADER_SIZE + p_file_meta.name.length());
 
@@ -331,7 +331,6 @@ Vector<uint8_t> AppxPackager::make_file_header(FileMeta p_file_meta) {
 }
 
 void AppxPackager::store_central_dir_header(const FileMeta &p_file, bool p_do_hash) {
-
 	Vector<uint8_t> &buf = central_dir_data;
 	int offs = buf.size();
 	buf.resize(buf.size() + BASE_CENTRAL_DIR_SIZE + p_file.name.length());
@@ -383,7 +382,6 @@ void AppxPackager::store_central_dir_header(const FileMeta &p_file, bool p_do_ha
 }
 
 Vector<uint8_t> AppxPackager::make_end_of_central_record() {
-
 	Vector<uint8_t> buf;
 	buf.resize(ZIP64_END_OF_CENTRAL_DIR_SIZE + 12 + END_OF_CENTRAL_DIR_SIZE); // Size plus magic
 
@@ -453,14 +451,12 @@ Vector<uint8_t> AppxPackager::make_end_of_central_record() {
 }
 
 void AppxPackager::init(FileAccess *p_fa) {
-
 	package = p_fa;
 	central_dir_offset = 0;
 	end_of_central_dir_offset = 0;
 }
 
 Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t p_len, int p_file_no, int p_total_files, bool p_compress) {
-
 	if (p_file_no >= 1 && p_total_files >= 1) {
 		if (EditorNode::progress_task_step(progress_task, "File: " + p_file_name, (p_file_no * 100) / p_total_files)) {
 			return ERR_SKIP;
@@ -478,13 +474,12 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 
 	// Data for compression
 	z_stream strm;
-	FileAccess *strm_f = NULL;
+	FileAccess *strm_f = nullptr;
 	Vector<uint8_t> strm_in;
 	strm_in.resize(BLOCK_SIZE);
 	Vector<uint8_t> strm_out;
 
 	if (p_compress) {
-
 		strm.zalloc = zipio_alloc;
 		strm.zfree = zipio_free;
 		strm.opaque = &strm_f;
@@ -497,7 +492,6 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 	int step = 0;
 
 	while (p_len - step > 0) {
-
 		size_t block_size = (p_len - step) > BLOCK_SIZE ? (size_t)BLOCK_SIZE : (p_len - step);
 
 		for (uint64_t i = 0; i < block_size; i++) {
@@ -508,7 +502,6 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 		bh.base64_hash = hash_block(strm_in.ptr(), block_size);
 
 		if (p_compress) {
-
 			strm.avail_in = block_size;
 			strm.avail_out = strm_out.size();
 			strm.next_in = (uint8_t *)strm_in.ptr();
@@ -524,15 +517,17 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 			//package->store_buffer(strm_out.ptr(), strm.total_out - total_out_before);
 			int start = file_buffer.size();
 			file_buffer.resize(file_buffer.size() + bh.compressed_size);
-			for (uint64_t i = 0; i < bh.compressed_size; i++)
+			for (uint64_t i = 0; i < bh.compressed_size; i++) {
 				file_buffer.write[start + i] = strm_out[i];
+			}
 		} else {
 			bh.compressed_size = block_size;
 			//package->store_buffer(strm_in.ptr(), block_size);
 			int start = file_buffer.size();
 			file_buffer.resize(file_buffer.size() + block_size);
-			for (uint64_t i = 0; i < bh.compressed_size; i++)
+			for (uint64_t i = 0; i < bh.compressed_size; i++) {
 				file_buffer.write[start + i] = strm_in[i];
+			}
 		}
 
 		meta.hashes.push_back(bh);
@@ -541,7 +536,6 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 	}
 
 	if (p_compress) {
-
 		strm.avail_in = 0;
 		strm.avail_out = strm_out.size();
 		strm.next_in = (uint8_t *)strm_in.ptr();
@@ -554,14 +548,14 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 		//package->store_buffer(strm_out.ptr(), strm.total_out - total_out_before);
 		int start = file_buffer.size();
 		file_buffer.resize(file_buffer.size() + (strm.total_out - total_out_before));
-		for (uint64_t i = 0; i < (strm.total_out - total_out_before); i++)
+		for (uint64_t i = 0; i < (strm.total_out - total_out_before); i++) {
 			file_buffer.write[start + i] = strm_out[i];
+		}
 
 		deflateEnd(&strm);
 		meta.compressed_size = strm.total_out;
 
 	} else {
-
 		meta.compressed_size = p_len;
 	}
 
@@ -584,7 +578,6 @@ Error AppxPackager::add_file(String p_file_name, const uint8_t *p_buffer, size_t
 }
 
 void AppxPackager::finish() {
-
 	// Create and add block map file
 	EditorNode::progress_task_step("export", "Creating block map...", 4);
 
@@ -641,7 +634,7 @@ void AppxPackager::finish() {
 
 	package->close();
 	memdelete(package);
-	package = NULL;
+	package = nullptr;
 }
 
 AppxPackager::AppxPackager() {}
@@ -651,7 +644,6 @@ AppxPackager::~AppxPackager() {}
 ////////////////////////////////////////////////////////////////////
 
 class EditorExportPlatformUWP : public EditorExportPlatform {
-
 	GDCLASS(EditorExportPlatformUWP, EditorExportPlatform);
 
 	Ref<ImageTexture> logo;
@@ -663,19 +655,24 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	};
 
 	bool _valid_resource_name(const String &p_name) const {
-
-		if (p_name.empty()) return false;
-		if (p_name.ends_with(".")) return false;
+		if (p_name.empty()) {
+			return false;
+		}
+		if (p_name.ends_with(".")) {
+			return false;
+		}
 
 		static const char *invalid_names[] = {
 			"CON", "PRN", "AUX", "NUL", "COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7",
 			"COM8", "COM9", "LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9",
-			NULL
+			nullptr
 		};
 
 		const char **t = invalid_names;
 		while (*t) {
-			if (p_name == *t) return false;
+			if (p_name == *t) {
+				return false;
+			}
 			t++;
 		}
 
@@ -683,22 +680,33 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	}
 
 	bool _valid_guid(const String &p_guid) const {
-
 		Vector<String> parts = p_guid.split("-");
 
-		if (parts.size() != 5) return false;
-		if (parts[0].length() != 8) return false;
-		for (int i = 1; i < 4; i++)
-			if (parts[i].length() != 4) return false;
-		if (parts[4].length() != 12) return false;
+		if (parts.size() != 5) {
+			return false;
+		}
+		if (parts[0].length() != 8) {
+			return false;
+		}
+		for (int i = 1; i < 4; i++) {
+			if (parts[i].length() != 4) {
+				return false;
+			}
+		}
+		if (parts[4].length() != 12) {
+			return false;
+		}
 
 		return true;
 	}
 
 	bool _valid_bgcolor(const String &p_color) const {
-
-		if (p_color.empty()) return true;
-		if (p_color.begins_with("#") && p_color.is_valid_html_color()) return true;
+		if (p_color.empty()) {
+			return true;
+		}
+		if (p_color.begins_with("#") && p_color.is_valid_html_color()) {
+			return true;
+		}
 
 		// Colors from https://msdn.microsoft.com/en-us/library/windows/apps/dn934817.aspx
 		static const char *valid_colors[] = {
@@ -726,13 +734,15 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 			"snow", "springGreen", "steelBlue", "tan", "teal", "thistle",
 			"tomato", "transparent", "turquoise", "violet", "wheat", "white",
 			"whiteSmoke", "yellow", "yellowGreen",
-			NULL
+			nullptr
 		};
 
 		const char **color = valid_colors;
 
 		while (*color) {
-			if (p_color == *color) return true;
+			if (p_color == *color) {
+				return true;
+			}
 			color++;
 		}
 
@@ -740,7 +750,6 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	}
 
 	bool _valid_image(const StreamTexture *p_image, int p_width, int p_height) const {
-
 		if (!p_image) {
 			return false;
 		}
@@ -751,7 +760,6 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	}
 
 	Vector<uint8_t> _fix_manifest(const Ref<EditorExportPreset> &p_preset, const Vector<uint8_t> &p_template, bool p_give_internet) const {
-
 		String result = String::utf8((const char *)p_template.ptr(), p_template.size());
 
 		result = result.replace("$godot_version$", VERSION_FULL_NAME);
@@ -766,7 +774,7 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 		result = result.replace("$version_string$", version);
 
 		Platform arch = (Platform)(int)p_preset->get("architecture/target");
-		String architecture = arch == ARM ? "arm" : arch == X86 ? "x86" : "x64";
+		String architecture = arch == ARM ? "arm" : (arch == X86 ? "x86" : "x64");
 		result = result.replace("$architecture$", architecture);
 
 		result = result.replace("$display_name$", String(p_preset->get("package/display_name")).empty() ? (String)ProjectSettings::get_singleton()->get("application/config/name") : String(p_preset->get("package/display_name")));
@@ -852,43 +860,44 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 		Vector<uint8_t> r_ret;
 		r_ret.resize(result.length());
 
-		for (int i = 0; i < result.length(); i++)
+		for (int i = 0; i < result.length(); i++) {
 			r_ret.write[i] = result.utf8().get(i);
+		}
 
 		return r_ret;
 	}
 
 	Vector<uint8_t> _get_image_data(const Ref<EditorExportPreset> &p_preset, const String &p_path) {
-
 		Vector<uint8_t> data;
-		StreamTexture *image = NULL;
+		StreamTexture *image = nullptr;
 
 		if (p_path.find("StoreLogo") != -1) {
-			image = p_preset->get("images/store_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/store_logo")));
+			image = p_preset->get("images/store_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/store_logo")));
 		} else if (p_path.find("Square44x44Logo") != -1) {
-			image = p_preset->get("images/square44x44_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square44x44_logo")));
+			image = p_preset->get("images/square44x44_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square44x44_logo")));
 		} else if (p_path.find("Square71x71Logo") != -1) {
-			image = p_preset->get("images/square71x71_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square71x71_logo")));
+			image = p_preset->get("images/square71x71_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square71x71_logo")));
 		} else if (p_path.find("Square150x150Logo") != -1) {
-			image = p_preset->get("images/square150x150_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square150x150_logo")));
+			image = p_preset->get("images/square150x150_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square150x150_logo")));
 		} else if (p_path.find("Square310x310Logo") != -1) {
-			image = p_preset->get("images/square310x310_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square310x310_logo")));
+			image = p_preset->get("images/square310x310_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/square310x310_logo")));
 		} else if (p_path.find("Wide310x150Logo") != -1) {
-			image = p_preset->get("images/wide310x150_logo").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/wide310x150_logo")));
+			image = p_preset->get("images/wide310x150_logo").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/wide310x150_logo")));
 		} else if (p_path.find("SplashScreen") != -1) {
-			image = p_preset->get("images/splash_screen").is_zero() ? NULL : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/splash_screen")));
+			image = p_preset->get("images/splash_screen").is_zero() ? nullptr : Object::cast_to<StreamTexture>(((Object *)p_preset->get("images/splash_screen")));
 		} else {
 			ERR_PRINT("Unable to load logo");
 		}
 
-		if (!image) return data;
+		if (!image) {
+			return data;
+		}
 
 		String tmp_path = EditorSettings::get_singleton()->get_cache_dir().plus_file("uwp_tmp_logo.png");
 
 		Error err = image->get_data()->save_png(tmp_path);
 
 		if (err != OK) {
-
 			String err_string = "Couldn't save temp logo file.";
 
 			EditorNode::add_io_error(err_string);
@@ -898,7 +907,6 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 		FileAccess *f = FileAccess::open(tmp_path, FileAccess::READ, &err);
 
 		if (err != OK) {
-
 			String err_string = "Couldn't open temp logo file.";
 			// Cleanup generated file.
 			DirAccess::remove_file_or_error(tmp_path);
@@ -917,17 +925,16 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	}
 
 	static bool _should_compress_asset(const String &p_path, const Vector<uint8_t> &p_data) {
-
 		/* TODO: This was copied verbatim from Android export. It should be
-		* refactored to the parent class and also be used for .zip export.
-		*/
+		 * refactored to the parent class and also be used for .zip export.
+		 */
 
 		/*
-		*  By not compressing files with little or not benefit in doing so,
-		*  a performance gain is expected at runtime. Moreover, if the APK is
-		*  zip-aligned, assets stored as they are can be efficiently read by
-		*  Android by memory-mapping them.
-		*/
+		 *  By not compressing files with little or not benefit in doing so,
+		 *  a performance gain is expected at runtime. Moreover, if the APK is
+		 *  zip-aligned, assets stored as they are can be efficiently read by
+		 *  Android by memory-mapping them.
+		 */
 
 		// -- Unconditional uncompress to mimic AAPT plus some other
 
@@ -946,7 +953,7 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 			".scn", // Binary scenes are usually already compressed
 			".stex", // Streamable textures are usually already compressed
 			// Trailer for easier processing
-			NULL
+			nullptr
 		};
 
 		for (const char **ext = unconditional_compress_ext; *ext; ++ext) {
@@ -968,7 +975,6 @@ class EditorExportPlatformUWP : public EditorExportPlatform {
 	}
 
 	static Error save_appx_file(void *p_userdata, const String &p_path, const Vector<uint8_t> &p_data, int p_file, int p_total) {
-
 		AppxPackager *packager = (AppxPackager *)p_userdata;
 		String dst_path = p_path.replace_first("res://", "game/");
 
@@ -1010,6 +1016,9 @@ public:
 	}
 
 	virtual void get_export_options(List<ExportOption> *r_options) {
+		r_options->push_back(ExportOption(PropertyInfo(Variant::STRING, "custom_template/debug", PROPERTY_HINT_GLOBAL_FILE, "*.zip"), ""));
+		r_options->push_back(ExportOption(PropertyInfo(Variant::STRING, "custom_template/release", PROPERTY_HINT_GLOBAL_FILE, "*.zip"), ""));
+
 		r_options->push_back(ExportOption(PropertyInfo(Variant::INT, "architecture/target", PROPERTY_HINT_ENUM, "arm,x86,x64"), 1));
 
 		r_options->push_back(ExportOption(PropertyInfo(Variant::STRING, "command_line/extra_args"), ""));
@@ -1051,31 +1060,27 @@ public:
 		r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "tiles/show_name_on_wide310x150"), false));
 		r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "tiles/show_name_on_square310x310"), false));
 
-		r_options->push_back(ExportOption(PropertyInfo(Variant::STRING, "custom_template/debug", PROPERTY_HINT_GLOBAL_FILE, "*.zip"), ""));
-		r_options->push_back(ExportOption(PropertyInfo(Variant::STRING, "custom_template/release", PROPERTY_HINT_GLOBAL_FILE, "*.zip"), ""));
-
 		// Capabilities
 		const char **basic = uwp_capabilities;
 		while (*basic) {
-			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*basic).camelcase_to_underscore(false)), false));
+			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*basic)), false));
 			basic++;
 		}
 
 		const char **uap = uwp_uap_capabilities;
 		while (*uap) {
-			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*uap).camelcase_to_underscore(false)), false));
+			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*uap)), false));
 			uap++;
 		}
 
 		const char **device = uwp_device_capabilities;
 		while (*device) {
-			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*device).camelcase_to_underscore(false)), false));
+			r_options->push_back(ExportOption(PropertyInfo(Variant::BOOL, "capabilities/" + String(*device)), false));
 			device++;
 		}
 	}
 
 	virtual bool can_export(const Ref<EditorExportPreset> &p_preset, String &r_error, bool &r_missing_templates) const {
-
 		String err;
 		bool valid = false;
 
@@ -1112,6 +1117,16 @@ public:
 		}
 
 		valid = dvalid || rvalid;
+
+#ifdef MODULE_MONO_ENABLED
+		// If this is a Mono build, provide a custom error so that users are not confused.
+		// We don't bypass the whole logic to check templates because third-parties might have
+		// Mono-enabled UWP builds using this path.
+		r_missing_templates = false; // Don't warn about those.
+		r_error = TTR("Godot's Mono version does not support the UWP platform. Use the standard build (no C# support) if you wish to target UWP.");
+		return false;
+#endif // MODULE_MONO_ENABLED
+
 		r_missing_templates = !valid;
 
 		// Validate the rest of the configuration.
@@ -1186,15 +1201,17 @@ public:
 	}
 
 	virtual Error export_project(const Ref<EditorExportPreset> &p_preset, bool p_debug, const String &p_path, int p_flags = 0) {
+		ExportNotifier notifier(*this, p_preset, p_debug, p_path, p_flags);
 
 		String src_appx;
 
 		EditorProgress ep("export", "Exporting for UWP", 7, true);
 
-		if (p_debug)
+		if (p_debug) {
 			src_appx = p_preset->get("custom_template/debug");
-		else
+		} else {
 			src_appx = p_preset->get("custom_template/release");
+		}
 
 		src_appx = src_appx.strip_edges();
 
@@ -1236,7 +1253,7 @@ public:
 		AppxPackager packager;
 		packager.init(fa_pack);
 
-		FileAccess *src_f = NULL;
+		FileAccess *src_f = nullptr;
 		zlib_filefunc_def io = zipio_create_io_from_file(&src_f);
 
 		if (ep.step("Creating package...", 0)) {
@@ -1246,7 +1263,6 @@ public:
 		unzFile pkg = unzOpen2(src_appx.utf8().get_data(), &io);
 
 		if (!pkg) {
-
 			EditorNode::add_io_error("Could not find template appx to export:\n" + src_appx);
 			return ERR_FILE_NOT_FOUND;
 		}
@@ -1264,13 +1280,12 @@ public:
 		int template_file_no = 1;
 
 		while (ret == UNZ_OK) {
-
 			// get file name
 			unz_file_info info;
-			char fname[16834];
-			ret = unzGetCurrentFileInfo(pkg, &info, fname, 16834, NULL, 0, NULL, 0);
+			char fname[16384];
+			ret = unzGetCurrentFileInfo(pkg, &info, fname, 16384, nullptr, 0, nullptr, 0);
 
-			String path = fname;
+			String path = String::utf8(fname);
 
 			if (path.ends_with("/")) {
 				// Ignore directories
@@ -1282,11 +1297,12 @@ public:
 			bool do_read = true;
 
 			if (path.begins_with("Assets/")) {
-
 				path = path.replace(".scale-100", "");
 
 				data = _get_image_data(p_preset, path);
-				if (data.size() > 0) do_read = false;
+				if (data.size() > 0) {
+					do_read = false;
+				}
 			}
 
 			//read
@@ -1298,7 +1314,6 @@ public:
 			}
 
 			if (path == "AppxManifest.xml") {
-
 				data = _fix_manifest(p_preset, data, p_flags & (DEBUG_FLAG_DUMB_CLIENT | DEBUG_FLAG_REMOTE_DEBUG));
 			}
 
@@ -1341,12 +1356,11 @@ public:
 		encode_uint32(cl.size(), clf.ptrw());
 
 		for (int i = 0; i < cl.size(); i++) {
-
 			CharString txt = cl[i].utf8();
 			int base = clf.size();
 			clf.resize(base + 4 + txt.length());
 			encode_uint32(txt.length(), &clf.write[base]);
-			copymem(&clf.write[base + 4], txt.ptr(), txt.length());
+			memcpy(&clf.write[base + 4], txt.ptr(), txt.length());
 			print_line(itos(i) + " param: " + cl[i]);
 		}
 
@@ -1362,7 +1376,7 @@ public:
 		EditorNode::progress_add_task("project_files", "Project Files", 100);
 		packager.set_progress_task("project_files");
 
-		err = export_project_files(p_preset, save_appx_file, &packager);
+		err = export_project_files(p_preset, save_appx_file, &packager, copy_shared_objects);
 
 		EditorNode::progress_end_task("project_files");
 
@@ -1382,7 +1396,7 @@ public:
 		}
 
 		if (!FileAccess::exists(signtool_path)) {
-			ERR_PRINTS("Could not find signtool executable at " + signtool_path + ", aborting.");
+			ERR_PRINT("Could not find signtool executable at " + signtool_path + ", aborting.");
 			return ERR_FILE_NOT_FOUND;
 		}
 
@@ -1403,12 +1417,12 @@ public:
 		}
 
 		if (!FileAccess::exists(cert_path)) {
-			ERR_PRINTS("Could not find certificate file at " + cert_path + ", aborting.");
+			ERR_PRINT("Could not find certificate file at " + cert_path + ", aborting.");
 			return ERR_FILE_NOT_FOUND;
 		}
 
 		if (cert_alg < 0 || cert_alg > 2) {
-			ERR_PRINTS("Invalid certificate algorithm " + itos(cert_alg) + ", aborting.");
+			ERR_PRINT("Invalid certificate algorithm " + itos(cert_alg) + ", aborting.");
 			return ERR_INVALID_DATA;
 		}
 
@@ -1430,7 +1444,6 @@ public:
 	}
 
 	virtual void get_platform_features(List<String> *r_features) {
-
 		r_features->push_back("pc");
 		r_features->push_back("UWP");
 	}
@@ -1438,6 +1451,10 @@ public:
 	virtual void resolve_platform_feature_priorities(const Ref<EditorExportPreset> &p_preset, Set<String> &p_features) {
 	}
 
+	static Error copy_shared_objects(void *p_userdata, const SharedObject &p_so) {
+		return save_appx_file(p_userdata, p_so.path, FileAccess::get_file_as_array(p_so.path), 0, 1);
+	}
+
 	EditorExportPlatformUWP() {
 		Ref<Image> img = memnew(Image(_uwp_logo));
 		logo.instance();
@@ -1446,7 +1463,6 @@ public:
 };
 
 void register_uwp_exporter() {
-
 #ifdef WINDOWS_ENABLED
 	EDITOR_DEF("export/uwp/signtool", "");
 	EditorSettings::get_singleton()->add_property_hint(PropertyInfo(Variant::STRING, "export/uwp/signtool", PROPERTY_HINT_GLOBAL_FILE, "*.exe"));
