@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  polygon_2d.cpp                                                       */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  polygon_2d.cpp                                                        */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "polygon_2d.h"
 
@@ -65,10 +65,11 @@ Rect2 Polygon2D::_edit_get_rect() const {
 		item_rect = Rect2();
 		for (int i = 0; i < l; i++) {
 			Vector2 pos = r[i] + offset;
-			if (i == 0)
+			if (i == 0) {
 				item_rect.position = pos;
-			else
+			} else {
 				item_rect.expand_to(pos);
+			}
 		}
 		rect_cache_dirty = false;
 	}
@@ -81,7 +82,6 @@ bool Polygon2D::_edit_use_rect() const {
 }
 
 bool Polygon2D::_edit_is_selected_on_click(const Point2 &p_point, double p_tolerance) const {
-
 	Vector<Vector2> polygon2d = Variant(polygon);
 	if (internal_vertices > 0) {
 		polygon2d.resize(polygon2d.size() - internal_vertices);
@@ -95,15 +95,13 @@ void Polygon2D::_skeleton_bone_setup_changed() {
 }
 
 void Polygon2D::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_DRAW: {
-
-			if (polygon.size() < 3)
+			if (polygon.size() < 3) {
 				return;
+			}
 
-			Skeleton2D *skeleton_node = NULL;
+			Skeleton2D *skeleton_node = nullptr;
 			if (has_node(skeleton)) {
 				skeleton_node = Object::cast_to<Skeleton2D>(get_node(skeleton));
 			}
@@ -147,7 +145,6 @@ void Polygon2D::_notification(int p_what) {
 			points.resize(len);
 
 			{
-
 				PoolVector<Vector2>::Read polyr = polygon.read();
 				for (int i = 0; i < len; i++) {
 					points.write[i] = polyr[i] + offset;
@@ -155,17 +152,17 @@ void Polygon2D::_notification(int p_what) {
 			}
 
 			if (invert) {
-
 				Rect2 bounds;
 				int highest_idx = -1;
 				float highest_y = -1e20;
 				float sum = 0;
 
 				for (int i = 0; i < len; i++) {
-					if (i == 0)
+					if (i == 0) {
 						bounds.position = points[i];
-					else
+					} else {
 						bounds.expand_to(points[i]);
+					}
 					if (points[i].y > highest_y) {
 						highest_idx = i;
 						highest_y = points[i].y;
@@ -195,12 +192,10 @@ void Polygon2D::_notification(int p_what) {
 
 				points.resize(points.size() + 7);
 				for (int i = points.size() - 1; i >= highest_idx + 7; i--) {
-
 					points.write[i] = points[i - 7];
 				}
 
 				for (int i = 0; i < 7; i++) {
-
 					points.write[highest_idx + i + 1] = ep[i];
 				}
 
@@ -208,7 +203,6 @@ void Polygon2D::_notification(int p_what) {
 			}
 
 			if (texture.is_valid()) {
-
 				Transform2D texmat(tex_rot, tex_ofs);
 				texmat.scale(tex_scale);
 				Size2 tex_size = texture->get_size();
@@ -216,7 +210,6 @@ void Polygon2D::_notification(int p_what) {
 				uvs.resize(len);
 
 				if (points.size() == uv.size()) {
-
 					PoolVector<Vector2>::Read uvr = uv.read();
 
 					for (int i = 0; i < len; i++) {
@@ -259,8 +252,9 @@ void Polygon2D::_notification(int p_what) {
 					int bone_index = bone->get_index_in_skeleton();
 					PoolVector<float>::Read r = bone_weights[i].weights.read();
 					for (int j = 0; j < vc; j++) {
-						if (r[j] == 0.0)
+						if (r[j] == 0.0) {
 							continue; //weight is unpainted, skip
+						}
 						//find an index with a weight
 						for (int k = 0; k < 4; k++) {
 							if (weightsw[j * 4 + k] < r[j]) {
@@ -283,8 +277,9 @@ void Polygon2D::_notification(int p_what) {
 					for (int j = 0; j < 4; j++) {
 						tw += weightsw[i * 4 + j];
 					}
-					if (tw == 0)
+					if (tw == 0) {
 						continue; //unpainted, do nothing
+					}
 
 					//normalize
 					for (int j = 0; j < 4; j++) {
@@ -318,8 +313,9 @@ void Polygon2D::_notification(int p_what) {
 				for (int i = 0; i < polygons.size(); i++) {
 					PoolVector<int> src_indices = polygons[i];
 					int ic = src_indices.size();
-					if (ic < 3)
+					if (ic < 3) {
 						continue;
+					}
 					PoolVector<int>::Read r = src_indices.read();
 
 					Vector<Vector2> tmp_points;
@@ -359,12 +355,10 @@ void Polygon2D::set_polygon(const PoolVector<Vector2> &p_polygon) {
 }
 
 PoolVector<Vector2> Polygon2D::get_polygon() const {
-
 	return polygon;
 }
 
 void Polygon2D::set_internal_vertex_count(int p_count) {
-
 	internal_vertices = p_count;
 }
 
@@ -373,49 +367,40 @@ int Polygon2D::get_internal_vertex_count() const {
 }
 
 void Polygon2D::set_uv(const PoolVector<Vector2> &p_uv) {
-
 	uv = p_uv;
 	update();
 }
 
 PoolVector<Vector2> Polygon2D::get_uv() const {
-
 	return uv;
 }
 
 void Polygon2D::set_polygons(const Array &p_polygons) {
-
 	polygons = p_polygons;
 	update();
 }
 
 Array Polygon2D::get_polygons() const {
-
 	return polygons;
 }
 
 void Polygon2D::set_color(const Color &p_color) {
-
 	color = p_color;
 	update();
 }
 Color Polygon2D::get_color() const {
-
 	return color;
 }
 
 void Polygon2D::set_vertex_colors(const PoolVector<Color> &p_colors) {
-
 	vertex_colors = p_colors;
 	update();
 }
 PoolVector<Color> Polygon2D::get_vertex_colors() const {
-
 	return vertex_colors;
 }
 
 void Polygon2D::set_texture(const Ref<Texture> &p_texture) {
-
 	texture = p_texture;
 
 	/*if (texture.is_valid()) {
@@ -429,81 +414,65 @@ void Polygon2D::set_texture(const Ref<Texture> &p_texture) {
 	update();
 }
 Ref<Texture> Polygon2D::get_texture() const {
-
 	return texture;
 }
 
 void Polygon2D::set_texture_offset(const Vector2 &p_offset) {
-
 	tex_ofs = p_offset;
 	update();
 }
 Vector2 Polygon2D::get_texture_offset() const {
-
 	return tex_ofs;
 }
 
 void Polygon2D::set_texture_rotation(float p_rot) {
-
 	tex_rot = p_rot;
 	update();
 }
 float Polygon2D::get_texture_rotation() const {
-
 	return tex_rot;
 }
 
 void Polygon2D::set_texture_rotation_degrees(float p_rot) {
-
 	set_texture_rotation(Math::deg2rad(p_rot));
 }
 float Polygon2D::get_texture_rotation_degrees() const {
-
 	return Math::rad2deg(get_texture_rotation());
 }
 
 void Polygon2D::set_texture_scale(const Size2 &p_scale) {
-
 	tex_scale = p_scale;
 	update();
 }
 Size2 Polygon2D::get_texture_scale() const {
-
 	return tex_scale;
 }
 
 void Polygon2D::set_invert(bool p_invert) {
-
 	invert = p_invert;
 	update();
 }
 bool Polygon2D::get_invert() const {
-
 	return invert;
 }
 
 void Polygon2D::set_antialiased(bool p_antialiased) {
-
 	antialiased = p_antialiased;
 	update();
 }
 bool Polygon2D::get_antialiased() const {
-
 	return antialiased;
 }
 
 void Polygon2D::set_invert_border(float p_invert_border) {
-
 	invert_border = p_invert_border;
 	update();
 }
 float Polygon2D::get_invert_border() const {
-
 	return invert_border;
 }
 
 void Polygon2D::set_offset(const Vector2 &p_offset) {
-
 	offset = p_offset;
 	rect_cache_dirty = true;
 	update();
@@ -511,12 +480,10 @@ void Polygon2D::set_offset(const Vector2 &p_offset) {
 }
 
 Vector2 Polygon2D::get_offset() const {
-
 	return offset;
 }
 
 void Polygon2D::add_bone(const NodePath &p_path, const PoolVector<float> &p_weights) {
-
 	Bone bone;
 	bone.path = p_path;
 	bone.weights = p_weights;
@@ -530,12 +497,10 @@ NodePath Polygon2D::get_bone_path(int p_index) const {
 	return bone_weights[p_index].path;
 }
 PoolVector<float> Polygon2D::get_bone_weights(int p_index) const {
-
 	ERR_FAIL_INDEX_V(p_index, bone_weights.size(), PoolVector<float>());
 	return bone_weights[p_index].weights;
 }
 void Polygon2D::erase_bone(int p_idx) {
-
 	ERR_FAIL_INDEX(p_idx, bone_weights.size());
 	bone_weights.remove(p_idx);
 }
@@ -558,23 +523,26 @@ void Polygon2D::set_bone_path(int p_index, const NodePath &p_path) {
 Array Polygon2D::_get_bones() const {
 	Array bones;
 	for (int i = 0; i < get_bone_count(); i++) {
-		bones.push_back(get_bone_path(i));
+		// Convert path property to String to avoid errors due to invalid node path in editor,
+		// because it's relative to the Skeleton2D node and not Polygon2D.
+		bones.push_back(String(get_bone_path(i)));
 		bones.push_back(get_bone_weights(i));
 	}
 	return bones;
 }
 void Polygon2D::_set_bones(const Array &p_bones) {
-
 	ERR_FAIL_COND(p_bones.size() & 1);
 	clear_bones();
 	for (int i = 0; i < p_bones.size(); i += 2) {
-		add_bone(p_bones[i], p_bones[i + 1]);
+		// Convert back from String to NodePath.
+		add_bone(NodePath(p_bones[i].operator String()), p_bones[i + 1]);
 	}
 }
 
 void Polygon2D::set_skeleton(const NodePath &p_skeleton) {
-	if (skeleton == p_skeleton)
+	if (skeleton == p_skeleton) {
 		return;
+	}
 	skeleton = p_skeleton;
 	update();
 }
@@ -584,7 +552,6 @@ NodePath Polygon2D::get_skeleton() const {
 }
 
 void Polygon2D::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_polygon", "polygon"), &Polygon2D::set_polygon);
 	ClassDB::bind_method(D_METHOD("get_polygon"), &Polygon2D::get_polygon);
 
@@ -669,13 +636,12 @@ void Polygon2D::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::POOL_VECTOR2_ARRAY, "uv"), "set_uv", "get_uv");
 	ADD_PROPERTY(PropertyInfo(Variant::POOL_COLOR_ARRAY, "vertex_colors"), "set_vertex_colors", "get_vertex_colors");
 	ADD_PROPERTY(PropertyInfo(Variant::ARRAY, "polygons"), "set_polygons", "get_polygons");
-	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "bones", PROPERTY_HINT_NONE, "", PROPERTY_USAGE_NOEDITOR), "_set_bones", "_get_bones");
+	ADD_PROPERTY(PropertyInfo(Variant::ARRAY, "bones", PROPERTY_HINT_NONE, "", PROPERTY_USAGE_NOEDITOR), "_set_bones", "_get_bones");
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "internal_vertex_count", PROPERTY_HINT_RANGE, "0,1000"), "set_internal_vertex_count", "get_internal_vertex_count");
 }
 
 Polygon2D::Polygon2D() {
-
-	invert = 0;
+	invert = false;
 	invert_border = 100;
 	antialiased = false;
 	tex_rot = 0;
