@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  touch_screen_button.cpp                                              */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  touch_screen_button.cpp                                               */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "touch_screen_button.h"
 
@@ -35,104 +35,96 @@
 #include "core/os/os.h"
 
 void TouchScreenButton::set_texture(const Ref<Texture> &p_texture) {
-
 	texture = p_texture;
 	update();
 }
 
 Ref<Texture> TouchScreenButton::get_texture() const {
-
 	return texture;
 }
 
 void TouchScreenButton::set_texture_pressed(const Ref<Texture> &p_texture_pressed) {
-
 	texture_pressed = p_texture_pressed;
 	update();
 }
 
 Ref<Texture> TouchScreenButton::get_texture_pressed() const {
-
 	return texture_pressed;
 }
 
 void TouchScreenButton::set_bitmask(const Ref<BitMap> &p_bitmask) {
-
 	bitmask = p_bitmask;
 }
 
 Ref<BitMap> TouchScreenButton::get_bitmask() const {
-
 	return bitmask;
 }
 
 void TouchScreenButton::set_shape(const Ref<Shape2D> &p_shape) {
-
-	if (shape.is_valid())
+	if (shape.is_valid()) {
 		shape->disconnect("changed", this, "update");
+	}
 
 	shape = p_shape;
 
-	if (shape.is_valid())
+	if (shape.is_valid()) {
 		shape->connect("changed", this, "update");
+	}
 
 	update();
 }
 
 Ref<Shape2D> TouchScreenButton::get_shape() const {
-
 	return shape;
 }
 
 void TouchScreenButton::set_shape_centered(bool p_shape_centered) {
-
 	shape_centered = p_shape_centered;
 	update();
 }
 
 bool TouchScreenButton::is_shape_visible() const {
-
 	return shape_visible;
 }
 
 void TouchScreenButton::set_shape_visible(bool p_shape_visible) {
-
 	shape_visible = p_shape_visible;
 	update();
 }
 
 bool TouchScreenButton::is_shape_centered() const {
-
 	return shape_centered;
 }
 
 void TouchScreenButton::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_DRAW: {
-
-			if (!is_inside_tree())
+			if (!is_inside_tree()) {
 				return;
-			if (!Engine::get_singleton()->is_editor_hint() && !OS::get_singleton()->has_touchscreen_ui_hint() && visibility == VISIBILITY_TOUCHSCREEN_ONLY)
+			}
+			if (!Engine::get_singleton()->is_editor_hint() && !OS::get_singleton()->has_touchscreen_ui_hint() && visibility == VISIBILITY_TOUCHSCREEN_ONLY) {
 				return;
-
-			if (finger_pressed != -1) {
-
-				if (texture_pressed.is_valid())
-					draw_texture(texture_pressed, Point2());
-				else if (texture.is_valid())
-					draw_texture(texture, Point2());
-
-			} else {
-				if (texture.is_valid())
-					draw_texture(texture, Point2());
 			}
 
-			if (!shape_visible)
+			if (finger_pressed != -1) {
+				if (texture_pressed.is_valid()) {
+					draw_texture(texture_pressed, Point2());
+				} else if (texture.is_valid()) {
+					draw_texture(texture, Point2());
+				}
+
+			} else {
+				if (texture.is_valid()) {
+					draw_texture(texture, Point2());
+				}
+			}
+
+			if (!shape_visible) {
 				return;
-			if (!Engine::get_singleton()->is_editor_hint() && !get_tree()->is_debugging_collisions_hint())
+			}
+			if (!Engine::get_singleton()->is_editor_hint() && !get_tree()->is_debugging_collisions_hint()) {
 				return;
+			}
 			if (shape.is_valid()) {
 				Color draw_col = get_tree()->get_debug_collisions_color();
 
@@ -144,80 +136,79 @@ void TouchScreenButton::_notification(int p_what) {
 
 		} break;
 		case NOTIFICATION_ENTER_TREE: {
-
-			if (!Engine::get_singleton()->is_editor_hint() && !OS::get_singleton()->has_touchscreen_ui_hint() && visibility == VISIBILITY_TOUCHSCREEN_ONLY)
+			if (!Engine::get_singleton()->is_editor_hint() && !OS::get_singleton()->has_touchscreen_ui_hint() && visibility == VISIBILITY_TOUCHSCREEN_ONLY) {
 				return;
+			}
 			update();
 
-			if (!Engine::get_singleton()->is_editor_hint())
+			if (!Engine::get_singleton()->is_editor_hint()) {
 				set_process_input(is_visible_in_tree());
+			}
 
 		} break;
 		case NOTIFICATION_EXIT_TREE: {
-			if (is_pressed())
+			if (is_pressed()) {
 				_release(true);
+			}
 		} break;
 		case NOTIFICATION_VISIBILITY_CHANGED: {
-			if (Engine::get_singleton()->is_editor_hint())
+			if (Engine::get_singleton()->is_editor_hint()) {
 				break;
+			}
 			if (is_visible_in_tree()) {
 				set_process_input(true);
 			} else {
 				set_process_input(false);
-				if (is_pressed())
+				if (is_pressed()) {
 					_release();
+				}
 			}
 		} break;
 		case NOTIFICATION_PAUSED: {
-			if (is_pressed())
+			if (is_pressed()) {
 				_release();
+			}
 		} break;
 	}
 }
 
 bool TouchScreenButton::is_pressed() const {
-
 	return finger_pressed != -1;
 }
 
 void TouchScreenButton::set_action(const String &p_action) {
-
 	action = p_action;
 }
 
 String TouchScreenButton::get_action() const {
-
 	return action;
 }
 
 void TouchScreenButton::_input(const Ref<InputEvent> &p_event) {
+	ERR_FAIL_COND(p_event.is_null());
 
-	if (!get_tree())
+	if (!is_visible_in_tree()) {
 		return;
+	}
 
-	if (p_event->get_device() != 0)
+	if (p_event->get_device() != 0) {
 		return;
-
-	ERR_FAIL_COND(!is_visible_in_tree());
+	}
 
 	const InputEventScreenTouch *st = Object::cast_to<InputEventScreenTouch>(*p_event);
 
 	if (passby_press) {
-
 		const InputEventScreenDrag *sd = Object::cast_to<InputEventScreenDrag>(*p_event);
 
 		if (st && !st->is_pressed() && finger_pressed == st->get_index()) {
-
 			_release();
 		}
 
 		if ((st && st->is_pressed()) || sd) {
-
 			int index = st ? st->get_index() : sd->get_index();
 			Point2 coord = st ? st->get_position() : sd->get_position();
 
 			if (finger_pressed == -1 || index == finger_pressed) {
-
 				if (_is_point_inside(coord)) {
 					if (finger_pressed == -1) {
 						_press(index);
@@ -231,14 +222,12 @@ void TouchScreenButton::_input(const Ref<InputEvent> &p_event) {
 		}
 
 	} else {
-
 		if (st) {
-
 			if (st->is_pressed()) {
-
 				const bool can_press = finger_pressed == -1;
-				if (!can_press)
+				if (!can_press) {
 					return; //already fingering
+				}
 
 				if (_is_point_inside(st->get_position())) {
 					_press(st->get_index());
@@ -267,29 +256,27 @@ bool TouchScreenButton::_is_point_inside(const Point2 &p_point) {
 	}
 
 	if (bitmask.is_valid()) {
-
 		check_rect = false;
 		if (!touched && Rect2(Point2(), bitmask->get_size()).has_point(coord)) {
-
-			if (bitmask->get_bit(coord))
+			if (bitmask->get_bit(coord)) {
 				touched = true;
+			}
 		}
 	}
 
 	if (!touched && check_rect) {
-		if (texture.is_valid())
+		if (texture.is_valid()) {
 			touched = Rect2(Size2(), texture->get_size()).has_point(coord);
+		}
 	}
 
 	return touched;
 }
 
 void TouchScreenButton::_press(int p_finger_pressed) {
-
 	finger_pressed = p_finger_pressed;
 
 	if (action != StringName()) {
-
 		Input::get_singleton()->action_press(action);
 		Ref<InputEventAction> iea;
 		iea.instance();
@@ -303,14 +290,11 @@ void TouchScreenButton::_press(int p_finger_pressed) {
 }
 
 void TouchScreenButton::_release(bool p_exiting_tree) {
-
 	finger_pressed = -1;
 
 	if (action != StringName()) {
-
 		Input::get_singleton()->action_release(action);
 		if (!p_exiting_tree) {
-
 			Ref<InputEventAction> iea;
 			iea.instance();
 			iea->set_action(action);
@@ -327,8 +311,9 @@ void TouchScreenButton::_release(bool p_exiting_tree) {
 
 #ifdef TOOLS_ENABLED
 Rect2 TouchScreenButton::_edit_get_rect() const {
-	if (texture.is_null())
+	if (texture.is_null()) {
 		return CanvasItem::_edit_get_rect();
+	}
 
 	return Rect2(Size2(), texture->get_size());
 }
@@ -339,8 +324,9 @@ bool TouchScreenButton::_edit_use_rect() const {
 #endif
 
 Rect2 TouchScreenButton::get_anchorable_rect() const {
-	if (texture.is_null())
+	if (texture.is_null()) {
 		return CanvasItem::get_anchorable_rect();
+	}
 
 	return Rect2(Size2(), texture->get_size());
 }
@@ -351,22 +337,18 @@ void TouchScreenButton::set_visibility_mode(VisibilityMode p_mode) {
 }
 
 TouchScreenButton::VisibilityMode TouchScreenButton::get_visibility_mode() const {
-
 	return visibility;
 }
 
 void TouchScreenButton::set_passby_press(bool p_enable) {
-
 	passby_press = p_enable;
 }
 
 bool TouchScreenButton::is_passby_press_enabled() const {
-
 	return passby_press;
 }
 
 void TouchScreenButton::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_texture", "texture"), &TouchScreenButton::set_texture);
 	ClassDB::bind_method(D_METHOD("get_texture"), &TouchScreenButton::get_texture);
 
@@ -416,7 +398,6 @@ void TouchScreenButton::_bind_methods() {
 }
 
 TouchScreenButton::TouchScreenButton() {
-
 	finger_pressed = -1;
 	passby_press = false;
 	visibility = VISIBILITY_ALWAYS;
