@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  audio_stream_player_3d.cpp                                           */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  audio_stream_player_3d.cpp                                            */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "audio_stream_player_3d.h"
 #include "core/engine.h"
@@ -99,10 +99,18 @@ void AudioStreamPlayer3D::_calc_output_vol(const Vector3 &source_dir, real_t tig
 	unsigned int speaker_count; // only main speakers (no LFE)
 	switch (AudioServer::get_singleton()->get_speaker_mode()) {
 		default: //fallthrough
-		case AudioServer::SPEAKER_MODE_STEREO: speaker_count = 2; break;
-		case AudioServer::SPEAKER_SURROUND_31: speaker_count = 3; break;
-		case AudioServer::SPEAKER_SURROUND_51: speaker_count = 5; break;
-		case AudioServer::SPEAKER_SURROUND_71: speaker_count = 7; break;
+		case AudioServer::SPEAKER_MODE_STEREO:
+			speaker_count = 2;
+			break;
+		case AudioServer::SPEAKER_SURROUND_31:
+			speaker_count = 3;
+			break;
+		case AudioServer::SPEAKER_SURROUND_51:
+			speaker_count = 5;
+			break;
+		case AudioServer::SPEAKER_SURROUND_71:
+			speaker_count = 7;
+			break;
 	}
 
 	Spcap spcap(speaker_count, speaker_directions); //TODO: should only be created/recreated once the speaker mode / speaker positions changes
@@ -129,16 +137,15 @@ void AudioStreamPlayer3D::_calc_output_vol(const Vector3 &source_dir, real_t tig
 }
 
 void AudioStreamPlayer3D::_mix_audio() {
-
-	if (!stream_playback.is_valid() || !active ||
+	if (!stream_playback.is_valid() || !active.is_set() ||
 			(stream_paused && !stream_paused_fade_out)) {
 		return;
 	}
 
 	bool started = false;
-	if (setseek >= 0.0) {
-		stream_playback->start(setseek);
-		setseek = -1.0; //reset seek
+	if (setseek.get() >= 0.0) {
+		stream_playback->start(setseek.get());
+		setseek.set(-1.0); //reset seek
 		started = true;
 	}
 
@@ -152,15 +159,14 @@ void AudioStreamPlayer3D::_mix_audio() {
 	}
 
 	// Mix if we're not paused or we're fading out
-	if ((output_count > 0 || out_of_range_mode == OUT_OF_RANGE_MIX)) {
-
+	if ((output_count.get() > 0 || out_of_range_mode == OUT_OF_RANGE_MIX)) {
 		float output_pitch_scale = 0.0;
-		if (output_count) {
+		if (output_count.get()) {
 			//used for doppler, not realistic but good enough
-			for (int i = 0; i < output_count; i++) {
+			for (int i = 0; i < output_count.get(); i++) {
 				output_pitch_scale += outputs[i].pitch_scale;
 			}
-			output_pitch_scale /= float(output_count);
+			output_pitch_scale /= float(output_count.get());
 		} else {
 			output_pitch_scale = 1.0;
 		}
@@ -169,8 +175,7 @@ void AudioStreamPlayer3D::_mix_audio() {
 	}
 
 	//write all outputs
-	for (int i = 0; i < output_count; i++) {
-
+	for (int i = 0; i < output_count.get(); i++) {
 		Output current = outputs[i];
 
 		//see if current output exists, to keep volume ramp
@@ -207,8 +212,9 @@ void AudioStreamPlayer3D::_mix_audio() {
 			AudioFrame vol_inc = (target_volume - vol_prev) / float(buffer_size);
 			AudioFrame vol = vol_prev;
 
-			if (!AudioServer::get_singleton()->thread_has_channel_mix_buffer(current.bus_index, k))
+			if (!AudioServer::get_singleton()->thread_has_channel_mix_buffer(current.bus_index, k)) {
 				continue; //may have been deleted, will be updated on process
+			}
 
 			AudioFrame *target = AudioServer::get_singleton()->thread_get_channel_mix_buffer(current.bus_index, k);
 			current.filter.set_mode(AudioFilterSW::HIGHSHELF);
@@ -219,7 +225,6 @@ void AudioStreamPlayer3D::_mix_audio() {
 			current.filter.set_gain(current.filter_gain);
 
 			if (interpolate_filter) {
-
 				current.filter_process[k * 2 + 0] = prev_outputs[i].filter_process[k * 2 + 0];
 				current.filter_process[k * 2 + 1] = prev_outputs[i].filter_process[k * 2 + 1];
 
@@ -229,7 +234,6 @@ void AudioStreamPlayer3D::_mix_audio() {
 				current.filter_process[k * 2 + 0].update_coeffs(buffer_size);
 				current.filter_process[k * 2 + 1].update_coeffs(buffer_size);
 				for (int j = 0; j < buffer_size; j++) {
-
 					AudioFrame f = buffer[j] * vol;
 					current.filter_process[k * 2 + 0].process_one_interp(f.l);
 					current.filter_process[k * 2 + 1].process_one_interp(f.r);
@@ -244,7 +248,6 @@ void AudioStreamPlayer3D::_mix_audio() {
 				current.filter_process[k * 2 + 0].update_coeffs();
 				current.filter_process[k * 2 + 1].update_coeffs();
 				for (int j = 0; j < buffer_size; j++) {
-
 					AudioFrame f = buffer[j] * vol;
 					current.filter_process[k * 2 + 0].process_one(f.l);
 					current.filter_process[k * 2 + 1].process_one(f.r);
@@ -255,9 +258,9 @@ void AudioStreamPlayer3D::_mix_audio() {
 			}
 
 			if (current.reverb_bus_index >= 0) {
-
-				if (!AudioServer::get_singleton()->thread_has_channel_mix_buffer(current.reverb_bus_index, k))
+				if (!AudioServer::get_singleton()->thread_has_channel_mix_buffer(current.reverb_bus_index, k)) {
 					continue; //may have been deleted, will be updated on process
+				}
 
 				AudioFrame *rtarget = AudioServer::get_singleton()->thread_get_channel_mix_buffer(current.reverb_bus_index, k);
 
@@ -266,15 +269,12 @@ void AudioStreamPlayer3D::_mix_audio() {
 					AudioFrame rvol = prev_outputs[i].reverb_vol[k];
 
 					for (int j = 0; j < buffer_size; j++) {
-
 						rtarget[j] += buffer[j] * rvol;
 						rvol += rvol_inc;
 					}
 				} else {
-
 					AudioFrame rvol = current.reverb_vol[k];
 					for (int j = 0; j < buffer_size; j++) {
-
 						rtarget[j] += buffer[j] * rvol;
 					}
 				}
@@ -284,20 +284,19 @@ void AudioStreamPlayer3D::_mix_audio() {
 		prev_outputs[i] = current;
 	}
 
-	prev_output_count = output_count;
+	prev_output_count = output_count.get();
 
 	//stream is no longer active, disable this.
 	if (!stream_playback->is_playing()) {
-		active = false;
+		active.clear();
 	}
 
-	output_ready = false;
+	output_ready.clear();
 	stream_paused_fade_in = false;
 	stream_paused_fade_out = false;
 }
 
 float AudioStreamPlayer3D::_get_attenuation_db(float p_distance) const {
-
 	float att = 0;
 	switch (attenuation_model) {
 		case ATTENUATION_INVERSE_DISTANCE: {
@@ -311,7 +310,8 @@ float AudioStreamPlayer3D::_get_attenuation_db(float p_distance) const {
 		case ATTENUATION_LOGARITHMIC: {
 			att = -20 * Math::log(p_distance / unit_size + CMP_EPSILON);
 		} break;
-		case ATTENUATION_DISABLED: break;
+		case ATTENUATION_DISABLED:
+			break;
 		default: {
 			ERR_PRINT("Unknown attenuation type");
 			break;
@@ -330,9 +330,7 @@ void _update_sound() {
 }
 
 void AudioStreamPlayer3D::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_ENTER_TREE) {
-
 		velocity_tracker->reset(get_global_transform().origin);
 		AudioServer::get_singleton()->add_callback(_mix_audios, this);
 		if (autoplay && !Engine::get_singleton()->is_editor_hint()) {
@@ -341,7 +339,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 	}
 
 	if (p_what == NOTIFICATION_EXIT_TREE) {
-
 		AudioServer::get_singleton()->remove_callback(_mix_audios, this);
 	}
 
@@ -357,18 +354,15 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 	}
 
 	if (p_what == NOTIFICATION_TRANSFORM_CHANGED) {
-
 		if (doppler_tracking != DOPPLER_TRACKING_DISABLED) {
 			velocity_tracker->update_position(get_global_transform().origin);
 		}
 	}
 
 	if (p_what == NOTIFICATION_INTERNAL_PHYSICS_PROCESS) {
-
 		//update anything related to position first, if possible of course
 
-		if (!output_ready) {
-
+		if (!output_ready.is_set()) {
 			Vector3 linear_velocity;
 
 			//compute linear velocity for doppler
@@ -392,18 +386,21 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 			PhysicsDirectSpaceState::ShapeResult sr[MAX_INTERSECT_AREAS];
 
 			int areas = space_state->intersect_point(global_pos, sr, MAX_INTERSECT_AREAS, Set<RID>(), area_mask, false, true);
-			Area *area = NULL;
+			Area *area = nullptr;
 
 			for (int i = 0; i < areas; i++) {
-				if (!sr[i].collider)
+				if (!sr[i].collider) {
 					continue;
+				}
 
 				Area *tarea = Object::cast_to<Area>(sr[i].collider);
-				if (!tarea)
+				if (!tarea) {
 					continue;
+				}
 
-				if (!tarea->is_overriding_audio_bus() && !tarea->is_using_reverb_bus())
+				if (!tarea->is_overriding_audio_bus() && !tarea->is_using_reverb_bus()) {
 					continue;
+				}
 
 				area = tarea;
 				break;
@@ -413,11 +410,11 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 			world->get_camera_list(&cameras);
 
 			for (List<Camera *>::Element *E = cameras.front(); E; E = E->next()) {
-
 				Camera *camera = E->get();
 				Viewport *vp = camera->get_viewport();
-				if (!vp->is_audio_listener())
+				if (!vp->is_audio_listener()) {
 					continue;
+				}
 
 				bool listener_is_camera = true;
 				Spatial *listener_node = camera;
@@ -441,7 +438,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 				}
 
 				if (max_distance > 0) {
-
 					float total_max = max_distance;
 
 					if (area && area->is_using_reverb_bus() && area->get_reverb_uniformity() > 0) {
@@ -468,8 +464,9 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 					Vector3 listenertopos = global_pos - listener_node->get_global_transform().origin;
 					float c = listenertopos.normalized().dot(get_global_transform().basis.get_axis(2).normalized()); //it's z negative
 					float angle = Math::rad2deg(Math::acos(c));
-					if (angle > emission_angle)
+					if (angle > emission_angle) {
 						db_att -= -emission_angle_filter_attenuation_db;
+					}
 				}
 
 				output.filter_gain = Math::db2linear(db_att);
@@ -487,7 +484,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 				int vol_index_max = AudioServer::get_singleton()->get_speaker_mode() + 1;
 
 				if (area) {
-
 					if (area->is_overriding_audio_bus()) {
 						//override audio bus
 						StringName bus_name = area->get_audio_bus();
@@ -495,7 +491,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 					}
 
 					if (area->is_using_reverb_bus()) {
-
 						filled_reverb = true;
 						StringName bus_name = area->get_reverb_bus();
 						output.reverb_bus_index = AudioServer::get_singleton()->thread_find_bus_index(bus_name);
@@ -504,7 +499,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 						float area_send = area->get_reverb_amount();
 
 						if (uniformity > 0.0) {
-
 							float distance = listener_area_pos.length();
 							float attenuation = Math::db2linear(_get_attenuation_db(distance));
 
@@ -546,26 +540,21 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 								}
 
 								for (int i = 0; i < vol_index_max; i++) {
-
 									output.reverb_vol[i] = output.reverb_vol[i].linear_interpolate(center_frame, attenuation);
 								}
 							} else {
 								for (int i = 0; i < vol_index_max; i++) {
-
 									output.reverb_vol[i] = center_frame;
 								}
 							}
 
 							for (int i = 0; i < vol_index_max; i++) {
-
 								output.reverb_vol[i] = output.vol[i].linear_interpolate(output.reverb_vol[i] * attenuation, uniformity);
 								output.reverb_vol[i] *= area_send;
 							}
 
 						} else {
-
 							for (int i = 0; i < vol_index_max; i++) {
-
 								output.reverb_vol[i] = output.vol[i] * area_send;
 							}
 						}
@@ -573,7 +562,6 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 				}
 
 				if (doppler_tracking != DOPPLER_TRACKING_DISABLED) {
-
 					Vector3 listener_velocity;
 
 					if (listener_is_camera) {
@@ -598,34 +586,33 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 				}
 
 				if (!filled_reverb) {
-
 					for (int i = 0; i < vol_index_max; i++) {
-
 						output.reverb_vol[i] = AudioFrame(0, 0);
 					}
 				}
 
 				outputs[new_output_count] = output;
 				new_output_count++;
-				if (new_output_count == MAX_OUTPUTS)
+				if (new_output_count == MAX_OUTPUTS) {
 					break;
+				}
 			}
 
-			output_count = new_output_count;
-			output_ready = true;
+			output_count.set(new_output_count);
+			output_ready.set();
 		}
 
 		//start playing if requested
-		if (setplay >= 0.0) {
-			setseek = setplay;
-			active = true;
-			setplay = -1;
+		if (setplay.get() >= 0.0) {
+			setseek.set(setplay.get());
+			active.set();
+			setplay.set(-1);
 			//do not update, this makes it easier to animate (will shut off otherwise)
 			///_change_notify("playing"); //update property in editor
 		}
 
 		//stop playing if no longer active
-		if (!active) {
+		if (!active.is_set()) {
 			set_physics_process_internal(false);
 			//do not update, this makes it easier to animate (will shut off otherwise)
 			//_change_notify("playing"); //update property in editor
@@ -635,6 +622,11 @@ void AudioStreamPlayer3D::_notification(int p_what) {
 }
 
 void AudioStreamPlayer3D::set_stream(Ref<AudioStream> p_stream) {
+	// Instancing audio streams can cause large memory allocations, do it prior to AudioServer::lock.
+	Ref<AudioStreamPlayback> pre_instanced_playback;
+	if (p_stream.is_valid()) {
+		pre_instanced_playback = p_stream->instance_playback();
+	}
 
 	AudioServer::get_singleton()->lock();
 
@@ -643,13 +635,13 @@ void AudioStreamPlayer3D::set_stream(Ref<AudioStream> p_stream) {
 	if (stream_playback.is_valid()) {
 		stream_playback.unref();
 		stream.unref();
-		active = false;
-		setseek = -1;
+		active.clear();
+		setseek.set(-1);
 	}
 
 	if (p_stream.is_valid()) {
 		stream = p_stream;
-		stream_playback = p_stream->instance_playback();
+		stream_playback = pre_instanced_playback;
 	}
 
 	AudioServer::get_singleton()->unlock();
@@ -660,34 +652,27 @@ void AudioStreamPlayer3D::set_stream(Ref<AudioStream> p_stream) {
 }
 
 Ref<AudioStream> AudioStreamPlayer3D::get_stream() const {
-
 	return stream;
 }
 
 void AudioStreamPlayer3D::set_unit_db(float p_volume) {
-
 	unit_db = p_volume;
 }
 float AudioStreamPlayer3D::get_unit_db() const {
-
 	return unit_db;
 }
 
 void AudioStreamPlayer3D::set_unit_size(float p_volume) {
-
 	unit_size = p_volume;
 }
 float AudioStreamPlayer3D::get_unit_size() const {
-
 	return unit_size;
 }
 
 void AudioStreamPlayer3D::set_max_db(float p_boost) {
-
 	max_db = p_boost;
 }
 float AudioStreamPlayer3D::get_max_db() const {
-
 	return max_db;
 }
 
@@ -700,48 +685,46 @@ float AudioStreamPlayer3D::get_pitch_scale() const {
 }
 
 void AudioStreamPlayer3D::play(float p_from_pos) {
-
 	if (!is_playing()) {
 		// Reset the prev_output_count if the stream is stopped
 		prev_output_count = 0;
 	}
 
 	if (stream_playback.is_valid()) {
-		active = true;
-		setplay = p_from_pos;
-		output_ready = false;
+		setplay.set(p_from_pos);
+		output_ready.clear();
 		set_physics_process_internal(true);
 	}
 }
 
 void AudioStreamPlayer3D::seek(float p_seconds) {
-
 	if (stream_playback.is_valid()) {
-		setseek = p_seconds;
+		setseek.set(p_seconds);
 	}
 }
 
 void AudioStreamPlayer3D::stop() {
-
 	if (stream_playback.is_valid()) {
-		active = false;
+		active.clear();
 		set_physics_process_internal(false);
-		setplay = -1;
+		setplay.set(-1);
 	}
 }
 
 bool AudioStreamPlayer3D::is_playing() const {
-
 	if (stream_playback.is_valid()) {
-		return active; // && stream_playback->is_playing();
+		return active.is_set() || setplay.get() >= 0;
 	}
 
 	return false;
 }
 
 float AudioStreamPlayer3D::get_playback_position() {
-
 	if (stream_playback.is_valid()) {
+		float ss = setseek.get();
+		if (ss >= 0.0) {
+			return ss;
+		}
 		return stream_playback->get_playback_position();
 	}
 
@@ -749,14 +732,12 @@ float AudioStreamPlayer3D::get_playback_position() {
 }
 
 void AudioStreamPlayer3D::set_bus(const StringName &p_bus) {
-
 	//if audio is active, must lock this
 	AudioServer::get_singleton()->lock();
 	bus = p_bus;
 	AudioServer::get_singleton()->unlock();
 }
 StringName AudioStreamPlayer3D::get_bus() const {
-
 	for (int i = 0; i < AudioServer::get_singleton()->get_bus_count(); i++) {
 		if (AudioServer::get_singleton()->get_bus_name(i) == bus) {
 			return bus;
@@ -766,34 +747,30 @@ StringName AudioStreamPlayer3D::get_bus() const {
 }
 
 void AudioStreamPlayer3D::set_autoplay(bool p_enable) {
-
 	autoplay = p_enable;
 }
 bool AudioStreamPlayer3D::is_autoplay_enabled() {
-
 	return autoplay;
 }
 
 void AudioStreamPlayer3D::_set_playing(bool p_enable) {
-
-	if (p_enable)
+	if (p_enable) {
 		play();
-	else
+	} else {
 		stop();
+	}
 }
 bool AudioStreamPlayer3D::_is_active() const {
-
-	return active;
+	return active.is_set();
 }
 
 void AudioStreamPlayer3D::_validate_property(PropertyInfo &property) const {
-
 	if (property.name == "bus") {
-
 		String options;
 		for (int i = 0; i < AudioServer::get_singleton()->get_bus_count(); i++) {
-			if (i > 0)
+			if (i > 0) {
 				options += ",";
+			}
 			String name = AudioServer::get_singleton()->get_bus_name(i);
 			options += name;
 		}
@@ -803,28 +780,23 @@ void AudioStreamPlayer3D::_validate_property(PropertyInfo &property) const {
 }
 
 void AudioStreamPlayer3D::_bus_layout_changed() {
-
 	_change_notify();
 }
 
 void AudioStreamPlayer3D::set_max_distance(float p_metres) {
-
 	ERR_FAIL_COND(p_metres < 0.0);
 	max_distance = p_metres;
 }
 
 float AudioStreamPlayer3D::get_max_distance() const {
-
 	return max_distance;
 }
 
 void AudioStreamPlayer3D::set_area_mask(uint32_t p_mask) {
-
 	area_mask = p_mask;
 }
 
 uint32_t AudioStreamPlayer3D::get_area_mask() const {
-
 	return area_mask;
 }
 
@@ -849,30 +821,24 @@ float AudioStreamPlayer3D::get_emission_angle() const {
 }
 
 void AudioStreamPlayer3D::set_emission_angle_filter_attenuation_db(float p_angle_attenuation_db) {
-
 	emission_angle_filter_attenuation_db = p_angle_attenuation_db;
 }
 
 float AudioStreamPlayer3D::get_emission_angle_filter_attenuation_db() const {
-
 	return emission_angle_filter_attenuation_db;
 }
 
 void AudioStreamPlayer3D::set_attenuation_filter_cutoff_hz(float p_hz) {
-
 	attenuation_filter_cutoff_hz = p_hz;
 }
 float AudioStreamPlayer3D::get_attenuation_filter_cutoff_hz() const {
-
 	return attenuation_filter_cutoff_hz;
 }
 
 void AudioStreamPlayer3D::set_attenuation_filter_db(float p_db) {
-
 	attenuation_filter_db = p_db;
 }
 float AudioStreamPlayer3D::get_attenuation_filter_db() const {
-
 	return attenuation_filter_db;
 }
 
@@ -886,20 +852,18 @@ AudioStreamPlayer3D::AttenuationModel AudioStreamPlayer3D::get_attenuation_model
 }
 
 void AudioStreamPlayer3D::set_out_of_range_mode(OutOfRangeMode p_mode) {
-
 	ERR_FAIL_INDEX((int)p_mode, 2);
 	out_of_range_mode = p_mode;
 }
 
 AudioStreamPlayer3D::OutOfRangeMode AudioStreamPlayer3D::get_out_of_range_mode() const {
-
 	return out_of_range_mode;
 }
 
 void AudioStreamPlayer3D::set_doppler_tracking(DopplerTracking p_tracking) {
-
-	if (doppler_tracking == p_tracking)
+	if (doppler_tracking == p_tracking) {
 		return;
+	}
 
 	doppler_tracking = p_tracking;
 
@@ -915,12 +879,10 @@ void AudioStreamPlayer3D::set_doppler_tracking(DopplerTracking p_tracking) {
 }
 
 AudioStreamPlayer3D::DopplerTracking AudioStreamPlayer3D::get_doppler_tracking() const {
-
 	return doppler_tracking;
 }
 
 void AudioStreamPlayer3D::set_stream_paused(bool p_pause) {
-
 	if (p_pause != stream_paused) {
 		stream_paused = p_pause;
 		stream_paused_fade_in = !stream_paused;
@@ -929,7 +891,6 @@ void AudioStreamPlayer3D::set_stream_paused(bool p_pause) {
 }
 
 bool AudioStreamPlayer3D::get_stream_paused() const {
-
 	return stream_paused;
 }
 
@@ -938,7 +899,6 @@ Ref<AudioStreamPlayback> AudioStreamPlayer3D::get_stream_playback() {
 }
 
 void AudioStreamPlayer3D::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_stream", "stream"), &AudioStreamPlayer3D::set_stream);
 	ClassDB::bind_method(D_METHOD("get_stream"), &AudioStreamPlayer3D::get_stream);
 
@@ -1008,7 +968,7 @@ void AudioStreamPlayer3D::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("_bus_layout_changed"), &AudioStreamPlayer3D::_bus_layout_changed);
 
 	ADD_PROPERTY(PropertyInfo(Variant::OBJECT, "stream", PROPERTY_HINT_RESOURCE_TYPE, "AudioStream"), "set_stream", "get_stream");
-	ADD_PROPERTY(PropertyInfo(Variant::INT, "attenuation_model", PROPERTY_HINT_ENUM, "Inverse,InverseSquare,Log,Disabled"), "set_attenuation_model", "get_attenuation_model");
+	ADD_PROPERTY(PropertyInfo(Variant::INT, "attenuation_model", PROPERTY_HINT_ENUM, "Inverse,InverseSquare,Logarithmic,Disabled"), "set_attenuation_model", "get_attenuation_model");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "unit_db", PROPERTY_HINT_RANGE, "-80,80"), "set_unit_db", "get_unit_db");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "unit_size", PROPERTY_HINT_RANGE, "0.1,100,0.1"), "set_unit_size", "get_unit_size");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "max_db", PROPERTY_HINT_RANGE, "-24,6"), "set_max_db", "get_max_db");
@@ -1016,7 +976,7 @@ void AudioStreamPlayer3D::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "playing", PROPERTY_HINT_NONE, "", PROPERTY_USAGE_EDITOR), "_set_playing", "is_playing");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "autoplay"), "set_autoplay", "is_autoplay_enabled");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "stream_paused", PROPERTY_HINT_NONE, ""), "set_stream_paused", "get_stream_paused");
-	ADD_PROPERTY(PropertyInfo(Variant::REAL, "max_distance", PROPERTY_HINT_EXP_RANGE, "0,4096,1,or_greater"), "set_max_distance", "get_max_distance");
+	ADD_PROPERTY(PropertyInfo(Variant::REAL, "max_distance", PROPERTY_HINT_RANGE, "0,4096,0.01,or_greater"), "set_max_distance", "get_max_distance");
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "out_of_range_mode", PROPERTY_HINT_ENUM, "Mix,Pause"), "set_out_of_range_mode", "get_out_of_range_mode");
 	ADD_PROPERTY(PropertyInfo(Variant::STRING, "bus", PROPERTY_HINT_ENUM, ""), "set_bus", "get_bus");
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "area_mask", PROPERTY_HINT_LAYERS_2D_PHYSICS), "set_area_mask", "get_area_mask");
@@ -1046,20 +1006,16 @@ void AudioStreamPlayer3D::_bind_methods() {
 }
 
 AudioStreamPlayer3D::AudioStreamPlayer3D() {
-
 	unit_db = 0;
 	unit_size = 1;
 	attenuation_model = ATTENUATION_INVERSE_DISTANCE;
 	max_db = 3;
 	pitch_scale = 1.0;
 	autoplay = false;
-	setseek = -1;
-	active = false;
-	output_count = 0;
+	setseek.set(-1);
 	prev_output_count = 0;
 	max_distance = 0;
-	setplay = -1;
-	output_ready = false;
+	setplay.set(-1);
 	area_mask = 1;
 	emission_angle = 45;
 	emission_angle_enabled = false;
