@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  camera.cpp                                                           */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  camera.cpp                                                            */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "camera.h"
 
@@ -39,16 +39,13 @@ void Camera::_update_audio_listener_state() {
 }
 
 void Camera::_request_camera_update() {
-
 	_update_camera();
 }
 
 void Camera::_update_camera_mode() {
-
 	force_change = true;
 	switch (mode) {
 		case PROJECTION_PERSPECTIVE: {
-
 			set_perspective(fov, near, far);
 
 		} break;
@@ -78,9 +75,9 @@ void Camera::_validate_property(PropertyInfo &p_property) const {
 }
 
 void Camera::_update_camera() {
-
-	if (!is_inside_tree())
+	if (!is_inside_tree()) {
 		return;
+	}
 
 	VisualServer::get_singleton()->camera_set_transform(camera, get_camera_transform());
 
@@ -90,8 +87,9 @@ void Camera::_update_camera() {
 		get_viewport()->_camera_transform_changed_notify();
 	*/
 
-	if (get_tree()->is_node_being_edited(this) || !is_current())
+	if (get_tree()->is_node_being_edited(this) || !is_current()) {
 		return;
+	}
 
 	get_viewport()->_camera_transform_changed_notify();
 
@@ -100,12 +98,13 @@ void Camera::_update_camera() {
 	}
 }
 
+void Camera::_physics_interpolated_changed() {
+	VisualServer::get_singleton()->camera_set_interpolated(camera, is_physics_interpolated());
+}
+
 void Camera::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_ENTER_WORLD: {
-
 			// Needs to track the Viewport  because it's needed on NOTIFICATION_EXIT_WORLD
 			// and Spatial will handle it first, including clearing its reference to the Viewport,
 			// therefore making it impossible to subclasses to access it
@@ -113,19 +112,22 @@ void Camera::_notification(int p_what) {
 			ERR_FAIL_COND(!viewport);
 
 			bool first_camera = viewport->_camera_add(this);
-			if (current || first_camera)
+			if (current || first_camera) {
 				viewport->_camera_set(this);
-
+			}
 		} break;
 		case NOTIFICATION_TRANSFORM_CHANGED: {
-
 			_request_camera_update();
 			if (doppler_tracking != DOPPLER_TRACKING_DISABLED) {
 				velocity_tracker->update_position(get_global_transform().origin);
 			}
 		} break;
+		case NOTIFICATION_RESET_PHYSICS_INTERPOLATION: {
+			if (is_physics_interpolated()) {
+				VisualServer::get_singleton()->camera_reset_physics_interpolation(camera);
+			}
+		} break;
 		case NOTIFICATION_EXIT_WORLD: {
-
 			if (!get_tree()->is_node_being_edited(this)) {
 				if (is_current()) {
 					clear_current();
@@ -138,7 +140,7 @@ void Camera::_notification(int p_what) {
 
 			if (viewport) {
 				viewport->_camera_remove(this);
-				viewport = NULL;
+				viewport = nullptr;
 			}
 
 		} break;
@@ -156,7 +158,6 @@ void Camera::_notification(int p_what) {
 }
 
 Transform Camera::get_camera_transform() const {
-
 	Transform tr = get_global_transform().orthonormalized();
 	tr.origin += tr.basis.get_axis(1) * v_offset;
 	tr.origin += tr.basis.get_axis(0) * h_offset;
@@ -164,9 +165,9 @@ Transform Camera::get_camera_transform() const {
 }
 
 void Camera::set_perspective(float p_fovy_degrees, float p_z_near, float p_z_far) {
-
-	if (!force_change && fov == p_fovy_degrees && p_z_near == near && p_z_far == far && mode == PROJECTION_PERSPECTIVE)
+	if (!force_change && fov == p_fovy_degrees && p_z_near == near && p_z_far == far && mode == PROJECTION_PERSPECTIVE) {
 		return;
+	}
 
 	fov = p_fovy_degrees;
 	near = p_z_near;
@@ -178,9 +179,9 @@ void Camera::set_perspective(float p_fovy_degrees, float p_z_near, float p_z_far
 	force_change = false;
 }
 void Camera::set_orthogonal(float p_size, float p_z_near, float p_z_far) {
-
-	if (!force_change && size == p_size && p_z_near == near && p_z_far == far && mode == PROJECTION_ORTHOGONAL)
+	if (!force_change && size == p_size && p_z_near == near && p_z_far == far && mode == PROJECTION_ORTHOGONAL) {
 		return;
+	}
 
 	size = p_size;
 
@@ -194,8 +195,9 @@ void Camera::set_orthogonal(float p_size, float p_z_near, float p_z_far) {
 }
 
 void Camera::set_frustum(float p_size, Vector2 p_offset, float p_z_near, float p_z_far) {
-	if (!force_change && size == p_size && frustum_offset == p_offset && p_z_near == near && p_z_far == far && mode == PROJECTION_FRUSTUM)
+	if (!force_change && size == p_size && frustum_offset == p_offset && p_z_near == near && p_z_far == far && mode == PROJECTION_FRUSTUM) {
 		return;
+	}
 
 	size = p_size;
 	frustum_offset = p_offset;
@@ -218,16 +220,15 @@ void Camera::set_projection(Camera::Projection p_mode) {
 }
 
 RID Camera::get_camera() const {
-
 	return camera;
 };
 
 void Camera::make_current() {
-
 	current = true;
 
-	if (!is_inside_tree())
+	if (!is_inside_tree()) {
 		return;
+	}
 
 	get_viewport()->_camera_set(this);
 
@@ -235,13 +236,13 @@ void Camera::make_current() {
 }
 
 void Camera::clear_current(bool p_enable_next) {
-
 	current = false;
-	if (!is_inside_tree())
+	if (!is_inside_tree()) {
 		return;
+	}
 
 	if (get_viewport()->get_camera() == this) {
-		get_viewport()->_camera_set(NULL);
+		get_viewport()->_camera_set(nullptr);
 
 		if (p_enable_next) {
 			get_viewport()->_camera_make_next_current(this);
@@ -258,27 +259,19 @@ void Camera::set_current(bool p_current) {
 }
 
 bool Camera::is_current() const {
-
 	if (is_inside_tree() && !get_tree()->is_node_being_edited(this)) {
-
 		return get_viewport()->get_camera() == this;
-	} else
+	} else {
 		return current;
-}
-
-bool Camera::_can_gizmo_scale() const {
-
-	return false;
+	}
 }
 
 Vector3 Camera::project_ray_normal(const Point2 &p_pos) const {
-
 	Vector3 ray = project_local_ray_normal(p_pos);
 	return get_camera_transform().basis.xform(ray).normalized();
 };
 
 Vector3 Camera::project_local_ray_normal(const Point2 &p_pos) const {
-
 	ERR_FAIL_COND_V_MSG(!is_inside_tree(), Vector3(), "Camera is not inside scene.");
 
 	Size2 viewport_size = get_viewport()->get_camera_rect_size();
@@ -286,7 +279,6 @@ Vector3 Camera::project_local_ray_normal(const Point2 &p_pos) const {
 	Vector3 ray;
 
 	if (mode == PROJECTION_ORTHOGONAL) {
-
 		ray = Vector3(0, 0, -1);
 	} else {
 		CameraMatrix cm;
@@ -299,7 +291,6 @@ Vector3 Camera::project_local_ray_normal(const Point2 &p_pos) const {
 };
 
 Vector3 Camera::project_ray_origin(const Point2 &p_pos) const {
-
 	ERR_FAIL_COND_V_MSG(!is_inside_tree(), Vector3(), "Camera is not inside scene.");
 
 	Size2 viewport_size = get_viewport()->get_camera_rect_size();
@@ -307,10 +298,8 @@ Vector3 Camera::project_ray_origin(const Point2 &p_pos) const {
 	ERR_FAIL_COND_V(viewport_size.y == 0, Vector3());
 
 	if (mode == PROJECTION_PERSPECTIVE) {
-
 		return get_camera_transform().origin;
 	} else {
-
 		Vector2 pos = cpos / viewport_size;
 		float vsize, hsize;
 		if (keep_aspect == KEEP_WIDTH) {
@@ -331,7 +320,6 @@ Vector3 Camera::project_ray_origin(const Point2 &p_pos) const {
 };
 
 bool Camera::is_position_behind(const Vector3 &p_pos) const {
-
 	Transform t = get_global_transform();
 	Vector3 eyedir = -t.basis.get_axis(2).normalized();
 	return eyedir.dot(p_pos - t.origin) < near;
@@ -344,10 +332,11 @@ Vector<Vector3> Camera::get_near_plane_points() const {
 
 	CameraMatrix cm;
 
-	if (mode == PROJECTION_ORTHOGONAL)
+	if (mode == PROJECTION_ORTHOGONAL) {
 		cm.set_orthogonal(size, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
-	else
+	} else {
 		cm.set_perspective(fov, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
+	}
 
 	Vector3 endpoints[8];
 	cm.get_endpoints(Transform(), endpoints);
@@ -361,17 +350,17 @@ Vector<Vector3> Camera::get_near_plane_points() const {
 }
 
 Point2 Camera::unproject_position(const Vector3 &p_pos) const {
-
 	ERR_FAIL_COND_V_MSG(!is_inside_tree(), Vector2(), "Camera is not inside scene.");
 
 	Size2 viewport_size = get_viewport()->get_visible_rect().size;
 
 	CameraMatrix cm;
 
-	if (mode == PROJECTION_ORTHOGONAL)
+	if (mode == PROJECTION_ORTHOGONAL) {
 		cm.set_orthogonal(size, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
-	else
+	} else {
 		cm.set_perspective(fov, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
+	}
 
 	Plane p(get_camera_transform().xform_inv(p_pos), 1.0);
 
@@ -386,7 +375,6 @@ Point2 Camera::unproject_position(const Vector3 &p_pos) const {
 }
 
 Vector3 Camera::project_position(const Point2 &p_point, float p_z_depth) const {
-
 	ERR_FAIL_COND_V_MSG(!is_inside_tree(), Vector3(), "Camera is not inside scene.");
 
 	if (p_z_depth == 0 && mode != PROJECTION_ORTHOGONAL) {
@@ -396,10 +384,11 @@ Vector3 Camera::project_position(const Point2 &p_point, float p_z_depth) const {
 
 	CameraMatrix cm;
 
-	if (mode == PROJECTION_ORTHOGONAL)
+	if (mode == PROJECTION_ORTHOGONAL) {
 		cm.set_orthogonal(size, viewport_size.aspect(), p_z_depth, far, keep_aspect == KEEP_WIDTH);
-	else
+	} else {
 		cm.set_perspective(fov, viewport_size.aspect(), p_z_depth, far, keep_aspect == KEEP_WIDTH);
+	}
 
 	Vector2 vp_he = cm.get_viewport_half_extents();
 
@@ -431,17 +420,16 @@ void Camera::_camera_make_current(Node *p_camera) {
 */
 
 void Camera::set_environment(const Ref<Environment> &p_environment) {
-
 	environment = p_environment;
-	if (environment.is_valid())
+	if (environment.is_valid()) {
 		VS::get_singleton()->camera_set_environment(camera, environment->get_rid());
-	else
+	} else {
 		VS::get_singleton()->camera_set_environment(camera, RID());
+	}
 	_update_camera_mode();
 }
 
 Ref<Environment> Camera::get_environment() const {
-
 	return environment;
 }
 
@@ -453,14 +441,13 @@ void Camera::set_keep_aspect_mode(KeepAspect p_aspect) {
 }
 
 Camera::KeepAspect Camera::get_keep_aspect_mode() const {
-
 	return keep_aspect;
 }
 
 void Camera::set_doppler_tracking(DopplerTracking p_tracking) {
-
-	if (doppler_tracking == p_tracking)
+	if (doppler_tracking == p_tracking) {
 		return;
+	}
 
 	doppler_tracking = p_tracking;
 	if (p_tracking != DOPPLER_TRACKING_DISABLED) {
@@ -477,7 +464,6 @@ Camera::DopplerTracking Camera::get_doppler_tracking() const {
 }
 
 void Camera::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("project_ray_normal", "screen_point"), &Camera::project_ray_normal);
 	ClassDB::bind_method(D_METHOD("project_local_ray_normal", "screen_point"), &Camera::project_local_ray_normal);
 	ClassDB::bind_method(D_METHOD("project_ray_origin", "screen_point"), &Camera::project_ray_origin);
@@ -489,7 +475,7 @@ void Camera::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_frustum", "size", "offset", "z_near", "z_far"), &Camera::set_frustum);
 	ClassDB::bind_method(D_METHOD("make_current"), &Camera::make_current);
 	ClassDB::bind_method(D_METHOD("clear_current", "enable_next"), &Camera::clear_current, DEFVAL(true));
-	ClassDB::bind_method(D_METHOD("set_current"), &Camera::set_current);
+	ClassDB::bind_method(D_METHOD("set_current", "enable"), &Camera::set_current);
 	ClassDB::bind_method(D_METHOD("is_current"), &Camera::is_current);
 	ClassDB::bind_method(D_METHOD("get_camera_transform"), &Camera::get_camera_transform);
 	ClassDB::bind_method(D_METHOD("get_fov"), &Camera::get_fov);
@@ -497,13 +483,13 @@ void Camera::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("get_size"), &Camera::get_size);
 	ClassDB::bind_method(D_METHOD("get_zfar"), &Camera::get_zfar);
 	ClassDB::bind_method(D_METHOD("get_znear"), &Camera::get_znear);
-	ClassDB::bind_method(D_METHOD("set_fov"), &Camera::set_fov);
-	ClassDB::bind_method(D_METHOD("set_frustum_offset"), &Camera::set_frustum_offset);
-	ClassDB::bind_method(D_METHOD("set_size"), &Camera::set_size);
-	ClassDB::bind_method(D_METHOD("set_zfar"), &Camera::set_zfar);
-	ClassDB::bind_method(D_METHOD("set_znear"), &Camera::set_znear);
+	ClassDB::bind_method(D_METHOD("set_fov", "fov"), &Camera::set_fov);
+	ClassDB::bind_method(D_METHOD("set_frustum_offset", "frustum_offset"), &Camera::set_frustum_offset);
+	ClassDB::bind_method(D_METHOD("set_size", "size"), &Camera::set_size);
+	ClassDB::bind_method(D_METHOD("set_zfar", "zfar"), &Camera::set_zfar);
+	ClassDB::bind_method(D_METHOD("set_znear", "znear"), &Camera::set_znear);
 	ClassDB::bind_method(D_METHOD("get_projection"), &Camera::get_projection);
-	ClassDB::bind_method(D_METHOD("set_projection"), &Camera::set_projection);
+	ClassDB::bind_method(D_METHOD("set_projection", "projection"), &Camera::set_projection);
 	ClassDB::bind_method(D_METHOD("set_h_offset", "ofs"), &Camera::set_h_offset);
 	ClassDB::bind_method(D_METHOD("get_h_offset"), &Camera::get_h_offset);
 	ClassDB::bind_method(D_METHOD("set_v_offset", "ofs"), &Camera::set_v_offset);
@@ -533,7 +519,7 @@ void Camera::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "projection", PROPERTY_HINT_ENUM, "Perspective,Orthogonal,Frustum"), "set_projection", "get_projection");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "current"), "set_current", "is_current");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "fov", PROPERTY_HINT_RANGE, "1,179,0.1"), "set_fov", "get_fov");
-	ADD_PROPERTY(PropertyInfo(Variant::REAL, "size", PROPERTY_HINT_RANGE, "0.1,16384,0.01"), "set_size", "get_size");
+	ADD_PROPERTY(PropertyInfo(Variant::REAL, "size", PROPERTY_HINT_RANGE, "0.001,16384,0.001"), "set_size", "get_size");
 	ADD_PROPERTY(PropertyInfo(Variant::VECTOR2, "frustum_offset"), "set_frustum_offset", "get_frustum_offset");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "near", PROPERTY_HINT_EXP_RANGE, "0.01,8192,0.01,or_greater"), "set_znear", "get_znear");
 	ADD_PROPERTY(PropertyInfo(Variant::REAL, "far", PROPERTY_HINT_EXP_RANGE, "0.1,8192,0.1,or_greater"), "set_zfar", "get_zfar");
@@ -551,17 +537,14 @@ void Camera::_bind_methods() {
 }
 
 float Camera::get_fov() const {
-
 	return fov;
 }
 
 float Camera::get_size() const {
-
 	return size;
 }
 
 float Camera::get_znear() const {
-
 	return near;
 }
 
@@ -570,12 +553,10 @@ Vector2 Camera::get_frustum_offset() const {
 }
 
 float Camera::get_zfar() const {
-
 	return far;
 }
 
 Camera::Projection Camera::get_projection() const {
-
 	return mode;
 }
 
@@ -587,7 +568,7 @@ void Camera::set_fov(float p_fov) {
 }
 
 void Camera::set_size(float p_size) {
-	ERR_FAIL_COND(p_size < 0.1 || p_size > 16384);
+	ERR_FAIL_COND(p_size < 0.001 || p_size > 16384);
 	size = p_size;
 	_update_camera_mode();
 	_change_notify("size");
@@ -615,7 +596,6 @@ void Camera::set_cull_mask(uint32_t p_layers) {
 }
 
 uint32_t Camera::get_cull_mask() const {
-
 	return layers;
 }
 
@@ -634,27 +614,25 @@ bool Camera::get_cull_mask_bit(int p_layer) const {
 }
 
 Vector<Plane> Camera::get_frustum() const {
-
 	ERR_FAIL_COND_V(!is_inside_world(), Vector<Plane>());
 
 	Size2 viewport_size = get_viewport()->get_visible_rect().size;
 	CameraMatrix cm;
-	if (mode == PROJECTION_PERSPECTIVE)
+	if (mode == PROJECTION_PERSPECTIVE) {
 		cm.set_perspective(fov, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
-	else
+	} else {
 		cm.set_orthogonal(size, viewport_size.aspect(), near, far, keep_aspect == KEEP_WIDTH);
+	}
 
 	return cm.get_projection_planes(get_camera_transform());
 }
 
 void Camera::set_v_offset(float p_offset) {
-
 	v_offset = p_offset;
 	_update_camera();
 }
 
 float Camera::get_v_offset() const {
-
 	return v_offset;
 }
 
@@ -664,12 +642,10 @@ void Camera::set_h_offset(float p_offset) {
 }
 
 float Camera::get_h_offset() const {
-
 	return h_offset;
 }
 
 Vector3 Camera::get_doppler_tracked_velocity() const {
-
 	if (doppler_tracking != DOPPLER_TRACKING_DISABLED) {
 		return velocity_tracker->get_tracked_linear_velocity();
 	} else {
@@ -677,15 +653,14 @@ Vector3 Camera::get_doppler_tracked_velocity() const {
 	}
 }
 Camera::Camera() {
-
-	camera = VisualServer::get_singleton()->camera_create();
+	camera = RID_PRIME(VisualServer::get_singleton()->camera_create());
 	size = 1;
 	fov = 0;
 	frustum_offset = Vector2();
 	near = 0;
 	far = 0;
 	current = false;
-	viewport = NULL;
+	viewport = nullptr;
 	force_change = false;
 	mode = PROJECTION_PERSPECTIVE;
 	set_perspective(70.0, 0.05, 100.0);
@@ -702,7 +677,6 @@ Camera::Camera() {
 }
 
 Camera::~Camera() {
-
 	VisualServer::get_singleton()->free(camera);
 }
 
@@ -715,7 +689,6 @@ float ClippedCamera::get_margin() const {
 	return margin;
 }
 void ClippedCamera::set_process_mode(ProcessMode p_mode) {
-
 	if (process_mode == p_mode) {
 		return;
 	}
@@ -728,7 +701,6 @@ ClippedCamera::ProcessMode ClippedCamera::get_process_mode() const {
 }
 
 Transform ClippedCamera::get_camera_transform() const {
-
 	Transform t = Camera::get_camera_transform();
 	t.origin += -t.basis.get_axis(Vector3::AXIS_Z).normalized() * clip_offset;
 	return t;
@@ -736,7 +708,6 @@ Transform ClippedCamera::get_camera_transform() const {
 
 void ClippedCamera::_notification(int p_what) {
 	if (p_what == NOTIFICATION_INTERNAL_PROCESS || p_what == NOTIFICATION_INTERNAL_PHYSICS_PROCESS) {
-
 		Spatial *parent = Object::cast_to<Spatial>(get_parent());
 		if (!parent) {
 			return;
@@ -796,90 +767,80 @@ void ClippedCamera::_notification(int p_what) {
 }
 
 void ClippedCamera::set_collision_mask(uint32_t p_mask) {
-
 	collision_mask = p_mask;
 }
 
 uint32_t ClippedCamera::get_collision_mask() const {
-
 	return collision_mask;
 }
 
 void ClippedCamera::set_collision_mask_bit(int p_bit, bool p_value) {
-
+	ERR_FAIL_INDEX_MSG(p_bit, 32, "Collision layer bit must be between 0 and 31 inclusive.");
 	uint32_t mask = get_collision_mask();
-	if (p_value)
+	if (p_value) {
 		mask |= 1 << p_bit;
-	else
+	} else {
 		mask &= ~(1 << p_bit);
+	}
 	set_collision_mask(mask);
 }
 
 bool ClippedCamera::get_collision_mask_bit(int p_bit) const {
-
+	ERR_FAIL_INDEX_V_MSG(p_bit, 32, false, "Collision mask bit must be between 0 and 31 inclusive.");
 	return get_collision_mask() & (1 << p_bit);
 }
 
 void ClippedCamera::add_exception_rid(const RID &p_rid) {
-
 	exclude.insert(p_rid);
 }
 
 void ClippedCamera::add_exception(const Object *p_object) {
-
 	ERR_FAIL_NULL(p_object);
 	const CollisionObject *co = Object::cast_to<CollisionObject>(p_object);
-	if (!co)
+	if (!co) {
 		return;
+	}
 	add_exception_rid(co->get_rid());
 }
 
 void ClippedCamera::remove_exception_rid(const RID &p_rid) {
-
 	exclude.erase(p_rid);
 }
 
 void ClippedCamera::remove_exception(const Object *p_object) {
-
 	ERR_FAIL_NULL(p_object);
 	const CollisionObject *co = Object::cast_to<CollisionObject>(p_object);
-	if (!co)
+	if (!co) {
 		return;
+	}
 	remove_exception_rid(co->get_rid());
 }
 
 void ClippedCamera::clear_exceptions() {
-
 	exclude.clear();
 }
 
 float ClippedCamera::get_clip_offset() const {
-
 	return clip_offset;
 }
 
 void ClippedCamera::set_clip_to_areas(bool p_clip) {
-
 	clip_to_areas = p_clip;
 }
 
 bool ClippedCamera::is_clip_to_areas_enabled() const {
-
 	return clip_to_areas;
 }
 
 void ClippedCamera::set_clip_to_bodies(bool p_clip) {
-
 	clip_to_bodies = p_clip;
 }
 
 bool ClippedCamera::is_clip_to_bodies_enabled() const {
-
 	return clip_to_bodies;
 }
 
 void ClippedCamera::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("set_margin", "margin"), &ClippedCamera::set_margin);
 	ClassDB::bind_method(D_METHOD("get_margin"), &ClippedCamera::get_margin);
 
@@ -927,7 +888,7 @@ ClippedCamera::ClippedCamera() {
 	collision_mask = 1;
 	set_notify_local_transform(Engine::get_singleton()->is_editor_hint());
 	points.resize(5);
-	pyramid_shape = PhysicsServer::get_singleton()->shape_create(PhysicsServer::SHAPE_CONVEX_POLYGON);
+	pyramid_shape = RID_PRIME(PhysicsServer::get_singleton()->shape_create(PhysicsServer::SHAPE_CONVEX_POLYGON));
 	clip_to_areas = false;
 	clip_to_bodies = true;
 }
