@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  script_debugger_remote.cpp                                           */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  script_debugger_remote.cpp                                            */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "script_debugger_remote.h"
 
@@ -43,10 +43,10 @@
 #include "servers/visual_server.h"
 
 void ScriptDebuggerRemote::_send_video_memory() {
-
 	List<ResourceUsage> usage;
-	if (resource_usage_func)
+	if (resource_usage_func) {
 		resource_usage_func(&usage);
+	}
 
 	usage.sort();
 
@@ -54,7 +54,6 @@ void ScriptDebuggerRemote::_send_video_memory() {
 	packet_peer_stream->put_var(usage.size() * 4);
 
 	for (List<ResourceUsage>::Element *E = usage.front(); E; E = E->next()) {
-
 		packet_peer_stream->put_var(E->get().path);
 		packet_peer_stream->put_var(E->get().type);
 		packet_peer_stream->put_var(E->get().format);
@@ -63,12 +62,12 @@ void ScriptDebuggerRemote::_send_video_memory() {
 }
 
 Error ScriptDebuggerRemote::connect_to_host(const String &p_host, uint16_t p_port) {
-
 	IP_Address ip;
-	if (p_host.is_valid_ip_address())
+	if (p_host.is_valid_ip_address()) {
 		ip = p_host;
-	else
+	} else {
 		ip = IP::get_singleton()->resolve_hostname(p_host);
+	}
 
 	int port = p_port;
 
@@ -78,12 +77,10 @@ Error ScriptDebuggerRemote::connect_to_host(const String &p_host, uint16_t p_por
 	tcp_client->connect_to_host(ip, port);
 
 	for (int i = 0; i < tries; i++) {
-
 		if (tcp_client->get_status() == StreamPeerTCP::STATUS_CONNECTED) {
 			print_verbose("Remote Debugger: Connected!");
 			break;
 		} else {
-
 			const int ms = waits[i];
 			OS::get_singleton()->delay_usec(ms * 1000);
 			print_verbose("Remote Debugger: Connection failed with status: '" + String::num(tcp_client->get_status()) + "', retrying in " + String::num(ms) + " msec.");
@@ -91,8 +88,7 @@ Error ScriptDebuggerRemote::connect_to_host(const String &p_host, uint16_t p_por
 	};
 
 	if (tcp_client->get_status() != StreamPeerTCP::STATUS_CONNECTED) {
-
-		ERR_PRINTS("Remote Debugger: Unable to connect. Status: " + String::num(tcp_client->get_status()) + ".");
+		ERR_PRINT("Remote Debugger: Unable to connect. Status: " + String::num(tcp_client->get_status()) + ".");
 		return FAILED;
 	};
 
@@ -102,16 +98,15 @@ Error ScriptDebuggerRemote::connect_to_host(const String &p_host, uint16_t p_por
 }
 
 void ScriptDebuggerRemote::_put_variable(const String &p_name, const Variant &p_variable) {
-
 	packet_peer_stream->put_var(p_name);
 
 	Variant var = p_variable;
-	if (p_variable.get_type() == Variant::OBJECT && !ObjectDB::instance_validate(p_variable)) {
+	if (p_variable.get_type() == Variant::OBJECT && p_variable.operator Object *() == nullptr) {
 		var = Variant();
 	}
 
 	int len = 0;
-	Error err = encode_variant(var, NULL, len, true);
+	Error err = encode_variant(var, nullptr, len, true);
 	if (err != OK)
 		ERR_PRINT("Failed to encode variant.");
 
@@ -123,7 +118,6 @@ void ScriptDebuggerRemote::_put_variable(const String &p_name, const Variant &p_
 }
 
 void ScriptDebuggerRemote::_save_node(ObjectID id, const String &p_path) {
-
 	Node *node = Object::cast_to<Node>(ObjectDB::get_instance(id));
 	ERR_FAIL_COND(!node);
 
@@ -133,15 +127,19 @@ void ScriptDebuggerRemote::_save_node(ObjectID id, const String &p_path) {
 }
 
 void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue, bool p_is_error_breakpoint) {
-
 	//this function is called when there is a debugger break (bug on script)
 	//or when execution is paused from editor
 
-	if (skip_breakpoints && !p_is_error_breakpoint)
+	if (skip_breakpoints && !p_is_error_breakpoint) {
 		return;
+	}
 
 	ERR_FAIL_COND_MSG(!tcp_client->is_connected_to_host(), "Script Debugger failed to connect, but being used anyway.");
 
+	if (allow_focus_steal_pid) {
+		OS::get_singleton()->enable_for_stealing_focus(allow_focus_steal_pid);
+	}
+
 	packet_peer_stream->put_var("debug_enter");
 	packet_peer_stream->put_var(2);
 	packet_peer_stream->put_var(p_can_continue);
@@ -150,8 +148,9 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 	skip_profile_frame = true; // to avoid super long frame time for the frame
 
 	Input::MouseMode mouse_mode = Input::get_singleton()->get_mouse_mode();
-	if (mouse_mode != Input::MOUSE_MODE_VISIBLE)
+	if (mouse_mode != Input::MOUSE_MODE_VISIBLE) {
 		Input::get_singleton()->set_mouse_mode(Input::MOUSE_MODE_VISIBLE);
+	}
 
 	uint64_t loop_begin_usec = 0;
 	uint64_t loop_time_sec = 0;
@@ -161,7 +160,6 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 		_get_output();
 
 		if (packet_peer_stream->get_available_packet_count() > 0) {
-
 			Variant var;
 			Error err = packet_peer_stream->get_var(var);
 
@@ -176,13 +174,11 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 			String command = cmd[0];
 
 			if (command == "get_stack_dump") {
-
 				packet_peer_stream->put_var("stack_dump");
 				int slc = p_script->debug_get_stack_level_count();
 				packet_peer_stream->put_var(slc);
 
 				for (int i = 0; i < slc; i++) {
-
 					Dictionary d;
 					d["file"] = p_script->debug_get_stack_level_source(i);
 					d["line"] = p_script->debug_get_stack_level_line(i);
@@ -194,7 +190,6 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 				}
 
 			} else if (command == "get_stack_frame_vars") {
-
 				cmd.remove(0);
 				ERR_CONTINUE(cmd.size() != 1);
 				int lv = cmd[0];
@@ -242,7 +237,6 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 					List<Variant>::Element *F = member_vals.front();
 
 					while (E) {
-
 						_put_variable(E->get(), F->get());
 
 						E = E->next();
@@ -265,12 +259,10 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 				}
 
 			} else if (command == "step") {
-
 				set_depth(-1);
 				set_lines_left(1);
 				break;
 			} else if (command == "next") {
-
 				set_depth(0);
 				set_lines_left(1);
 				break;
@@ -284,20 +276,17 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 				ERR_PRINT("Got break when already broke!");
 				break;
 			} else if (command == "request_scene_tree") {
-
 #ifdef DEBUG_ENABLED
-				if (scene_tree)
+				if (scene_tree) {
 					scene_tree->_debugger_request_tree();
+				}
 #endif
 			} else if (command == "request_video_mem") {
-
 				_send_video_memory();
 			} else if (command == "inspect_object") {
-
 				ObjectID id = cmd[1];
 				_send_object_id(id);
 			} else if (command == "set_object_property") {
-
 				_set_object_property(cmd[1], cmd[2], cmd[3]);
 
 			} else if (command == "override_camera_2D:set") {
@@ -337,12 +326,12 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 			} else if (command == "reload_scripts") {
 				reload_all_scripts = true;
 			} else if (command == "breakpoint") {
-
 				bool set = cmd[3];
-				if (set)
+				if (set) {
 					insert_breakpoint(cmd[2], cmd[1]);
-				else
+				} else {
 					remove_breakpoint(cmd[2], cmd[1]);
+				}
 
 			} else if (command == "save_node") {
 				_save_node(cmd[1], cmd[2]);
@@ -368,15 +357,14 @@ void ScriptDebuggerRemote::debug(ScriptLanguage *p_script, bool p_can_continue,
 	packet_peer_stream->put_var("debug_exit");
 	packet_peer_stream->put_var(0);
 
-	if (mouse_mode != Input::MOUSE_MODE_VISIBLE)
+	if (mouse_mode != Input::MOUSE_MODE_VISIBLE) {
 		Input::get_singleton()->set_mouse_mode(mouse_mode);
+	}
 }
 
 void ScriptDebuggerRemote::_get_output() {
-
-	mutex->lock();
+	mutex.lock();
 	if (output_strings.size()) {
-
 		locking = true;
 		packet_peer_stream->put_var("output");
 		packet_peer_stream->put_var(output_strings.size());
@@ -469,109 +457,93 @@ void ScriptDebuggerRemote::_get_output() {
 		errors.pop_front();
 		locking = false;
 	}
-	mutex->unlock();
+	mutex.unlock();
 }
 
 void ScriptDebuggerRemote::line_poll() {
-
 	//the purpose of this is just processing events every now and then when the script might get too busy
 	//otherwise bugs like infinite loops can't be caught
-	if (poll_every % 2048 == 0)
+	if (poll_every % 2048 == 0) {
 		_poll_events();
+	}
 	poll_every++;
 }
 
 void ScriptDebuggerRemote::_err_handler(void *ud, const char *p_func, const char *p_file, int p_line, const char *p_err, const char *p_descr, ErrorHandlerType p_type) {
-
-	if (p_type == ERR_HANDLER_SCRIPT)
+	if (p_type == ERR_HANDLER_SCRIPT) {
 		return; //ignore script errors, those go through debugger
+	}
 
 	Vector<ScriptLanguage::StackInfo> si;
 
 	for (int i = 0; i < ScriptServer::get_language_count(); i++) {
 		si = ScriptServer::get_language(i)->debug_get_current_stack_info();
-		if (si.size())
+		if (si.size()) {
 			break;
+		}
 	}
 
 	ScriptDebuggerRemote *sdr = (ScriptDebuggerRemote *)ud;
-	sdr->send_error(p_func, p_file, p_line, p_err, p_descr, p_type, si);
+	sdr->send_error(String::utf8(p_func), String::utf8(p_file), p_line, String::utf8(p_err), String::utf8(p_descr), p_type, si);
 }
 
 bool ScriptDebuggerRemote::_parse_live_edit(const Array &p_command) {
-
 #ifdef DEBUG_ENABLED
 
 	String cmdstr = p_command[0];
-	if (!scene_tree || !cmdstr.begins_with("live_"))
+	if (!scene_tree || !cmdstr.begins_with("live_")) {
 		return false;
+	}
 
 	if (cmdstr == "live_set_root") {
-
 		scene_tree->_live_edit_root_func(p_command[1], p_command[2]);
 
 	} else if (cmdstr == "live_node_path") {
-
 		scene_tree->_live_edit_node_path_func(p_command[1], p_command[2]);
 
 	} else if (cmdstr == "live_res_path") {
-
 		scene_tree->_live_edit_res_path_func(p_command[1], p_command[2]);
 
 	} else if (cmdstr == "live_node_prop_res") {
-
 		scene_tree->_live_edit_node_set_res_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_node_prop") {
-
 		scene_tree->_live_edit_node_set_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_res_prop_res") {
-
 		scene_tree->_live_edit_res_set_res_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_res_prop") {
-
 		scene_tree->_live_edit_res_set_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_node_call") {
-
-		scene_tree->_live_edit_node_call_func(p_command[1], p_command[2], p_command[3], p_command[4], p_command[5], p_command[6], p_command[7]);
+		scene_tree->_live_edit_node_call_func(p_command[1], p_command[2], p_command[3], p_command[4], p_command[5], p_command[6], p_command[7], p_command[8], p_command[9], p_command[10]);
 
 	} else if (cmdstr == "live_res_call") {
-
-		scene_tree->_live_edit_res_call_func(p_command[1], p_command[2], p_command[3], p_command[4], p_command[5], p_command[6], p_command[7]);
+		scene_tree->_live_edit_res_call_func(p_command[1], p_command[2], p_command[3], p_command[4], p_command[5], p_command[6], p_command[7], p_command[8], p_command[9], p_command[10]);
 
 	} else if (cmdstr == "live_create_node") {
-
 		scene_tree->_live_edit_create_node_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_instance_node") {
-
 		scene_tree->_live_edit_instance_node_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_remove_node") {
-
 		scene_tree->_live_edit_remove_node_func(p_command[1]);
 
 	} else if (cmdstr == "live_remove_and_keep_node") {
-
 		scene_tree->_live_edit_remove_and_keep_node_func(p_command[1], p_command[2]);
 
 	} else if (cmdstr == "live_restore_node") {
-
 		scene_tree->_live_edit_restore_node_func(p_command[1], p_command[2], p_command[3]);
 
 	} else if (cmdstr == "live_duplicate_node") {
-
 		scene_tree->_live_edit_duplicate_node_func(p_command[1], p_command[2]);
 
 	} else if (cmdstr == "live_reparent_node") {
-
 		scene_tree->_live_edit_reparent_node_func(p_command[1], p_command[2], p_command[3], p_command[4]);
 
 	} else {
-
 		return false;
 	}
 
@@ -583,19 +555,18 @@ bool ScriptDebuggerRemote::_parse_live_edit(const Array &p_command) {
 }
 
 void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
-
 	Object *obj = ObjectDB::get_instance(p_id);
-	if (!obj)
+	if (!obj) {
 		return;
+	}
 
 	typedef Pair<PropertyInfo, Variant> PropertyDesc;
 	List<PropertyDesc> properties;
 
 	if (ScriptInstance *si = obj->get_script_instance()) {
 		if (!si->get_script().is_null()) {
-
-			typedef Map<const Script *, Set<StringName> > ScriptMemberMap;
-			typedef Map<const Script *, Map<StringName, Variant> > ScriptConstantsMap;
+			typedef Map<const Script *, Set<StringName>> ScriptMemberMap;
+			typedef Map<const Script *, Map<StringName, Variant>> ScriptConstantsMap;
 
 			ScriptMemberMap members;
 			members[si->get_script().ptr()] = Set<StringName>();
@@ -607,7 +578,6 @@ void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
 
 			Ref<Script> base = si->get_script()->get_base_script();
 			while (base.is_valid()) {
-
 				members[base.ptr()] = Set<StringName>();
 				base->get_members(&(members[base.ptr()]));
 
@@ -694,7 +664,7 @@ void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
 
 		//only send information that can be sent..
 		int len = 0; //test how big is this to encode
-		encode_variant(var, NULL, len);
+		encode_variant(var, nullptr, len);
 		if (len > packet_peer_stream->get_output_buffer_max_size()) { //limit to max size
 			prop.push_back(PROPERTY_HINT_OBJECT_TOO_BIG);
 			prop.push_back("");
@@ -705,7 +675,7 @@ void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
 			prop.push_back(pi.hint_string);
 			prop.push_back(pi.usage);
 
-			if (!res.is_null()) {
+			if (!res.is_null() && !res->get_path().empty()) {
 				var = res->get_path();
 			}
 
@@ -722,10 +692,10 @@ void ScriptDebuggerRemote::_send_object_id(ObjectID p_id) {
 }
 
 void ScriptDebuggerRemote::_set_object_property(ObjectID p_id, const String &p_property, const Variant &p_value) {
-
 	Object *obj = ObjectDB::get_instance(p_id);
-	if (!obj)
+	if (!obj) {
 		return;
+	}
 
 	String prop_name = p_property;
 	if (p_property.begins_with("Members/")) {
@@ -737,12 +707,10 @@ void ScriptDebuggerRemote::_set_object_property(ObjectID p_id, const String &p_p
 }
 
 void ScriptDebuggerRemote::_poll_events() {
-
 	//this si called from ::idle_poll, happens only when running the game,
 	//does not get called while on debug break
 
 	while (packet_peer_stream->get_available_packet_count() > 0) {
-
 		_get_output();
 
 		//send over output_strings
@@ -762,28 +730,24 @@ void ScriptDebuggerRemote::_poll_events() {
 		//cmd.remove(0);
 
 		if (command == "break") {
-
-			if (get_break_language())
+			if (get_break_language()) {
 				debug(get_break_language());
+			}
 		} else if (command == "request_scene_tree") {
-
 #ifdef DEBUG_ENABLED
-			if (scene_tree)
+			if (scene_tree) {
 				scene_tree->_debugger_request_tree();
+			}
 #endif
 		} else if (command == "request_video_mem") {
-
 			_send_video_memory();
 		} else if (command == "inspect_object") {
-
 			ObjectID id = cmd[1];
 			_send_object_id(id);
 		} else if (command == "set_object_property") {
-
 			_set_object_property(cmd[1], cmd[2], cmd[3]);
 
 		} else if (command == "start_profiling") {
-
 			for (int i = 0; i < ScriptServer::get_language_count(); i++) {
 				ScriptServer::get_language(i)->profiling_start();
 			}
@@ -792,26 +756,23 @@ void ScriptDebuggerRemote::_poll_events() {
 			profiler_function_signature_map.clear();
 			profiling = true;
 			frame_time = 0;
-			idle_time = 0;
+			process_time = 0;
 			physics_time = 0;
 			physics_frame_time = 0;
 
-			print_line("PROFILING ALRIGHT!");
+			print_verbose("Starting profiling.");
 
 		} else if (command == "stop_profiling") {
-
 			for (int i = 0; i < ScriptServer::get_language_count(); i++) {
 				ScriptServer::get_language(i)->profiling_stop();
 			}
 			profiling = false;
 			_send_profiling_data(false);
-			print_line("PROFILING END!");
+			print_verbose("Ending profiling.");
 		} else if (command == "start_network_profiling") {
-
 			multiplayer->profiling_start();
 			profiling_network = true;
 		} else if (command == "stop_network_profiling") {
-
 			multiplayer->profiling_end();
 			profiling_network = false;
 		} else if (command == "override_camera_2D:set") {
@@ -851,12 +812,12 @@ void ScriptDebuggerRemote::_poll_events() {
 		} else if (command == "reload_scripts") {
 			reload_all_scripts = true;
 		} else if (command == "breakpoint") {
-
 			bool set = cmd[3];
-			if (set)
+			if (set) {
 				insert_breakpoint(cmd[2], cmd[1]);
-			else
+			} else {
 				remove_breakpoint(cmd[2], cmd[1]);
+			}
 		} else if (command == "set_skip_breakpoints") {
 			skip_breakpoints = cmd[1];
 		} else {
@@ -866,14 +827,14 @@ void ScriptDebuggerRemote::_poll_events() {
 }
 
 void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
-
 	int ofs = 0;
 
 	for (int i = 0; i < ScriptServer::get_language_count(); i++) {
-		if (p_for_frame)
+		if (p_for_frame) {
 			ofs += ScriptServer::get_language(i)->profiling_get_frame_data(&profile_info.write[ofs], profile_info.size() - ofs);
-		else
+		} else {
 			ofs += ScriptServer::get_language(i)->profiling_get_accumulated_data(&profile_info.write[ofs], profile_info.size() - ofs);
+		}
 	}
 
 	for (int i = 0; i < ofs; i++) {
@@ -889,9 +850,7 @@ void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
 	uint64_t total_script_time = 0;
 
 	for (int i = 0; i < to_send; i++) {
-
 		if (!profiler_function_signature_map.has(profile_info_ptrs[i]->signature)) {
-
 			int idx = profiler_function_signature_map.size();
 			packet_peer_stream->put_var("profile_sig");
 			packet_peer_stream->put_var(2);
@@ -916,18 +875,16 @@ void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
 
 	packet_peer_stream->put_var(Engine::get_singleton()->get_idle_frames()); //total frame time
 	packet_peer_stream->put_var(frame_time); //total frame time
-	packet_peer_stream->put_var(idle_time); //idle frame time
+	packet_peer_stream->put_var(process_time); //idle frame time
 	packet_peer_stream->put_var(physics_time); //fixed frame time
 	packet_peer_stream->put_var(physics_frame_time); //fixed frame time
 
 	packet_peer_stream->put_var(USEC_TO_SEC(total_script_time)); //total script execution time
 
 	if (p_for_frame) {
-
 		packet_peer_stream->put_var(profile_frame_data.size()); //how many profile framedatas to send
 		packet_peer_stream->put_var(to_send); //how many script functions to send
 		for (int i = 0; i < profile_frame_data.size(); i++) {
-
 			packet_peer_stream->put_var(profile_frame_data[i].name);
 			packet_peer_stream->put_var(profile_frame_data[i].data);
 		}
@@ -937,7 +894,6 @@ void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
 	}
 
 	for (int i = 0; i < to_send; i++) {
-
 		int sig_id = -1;
 
 		if (profiler_function_signature_map.has(profile_info_ptrs[i]->signature)) {
@@ -956,24 +912,20 @@ void ScriptDebuggerRemote::_send_profiling_data(bool p_for_frame) {
 }
 
 void ScriptDebuggerRemote::idle_poll() {
-
 	// this function is called every frame, except when there is a debugger break (::debug() in this class)
 	// execution stops and remains in the ::debug function
 
 	_get_output();
 
 	if (requested_quit) {
-
 		packet_peer_stream->put_var("kill_me");
 		packet_peer_stream->put_var(0);
 		requested_quit = false;
 	}
 
 	if (performance) {
-
 		uint64_t pt = OS::get_singleton()->get_ticks_msec();
 		if (pt - last_perf_time > 1000) {
-
 			last_perf_time = pt;
 			int max = performance->get("MONITOR_MAX");
 			Array arr;
@@ -988,7 +940,6 @@ void ScriptDebuggerRemote::idle_poll() {
 	}
 
 	if (profiling) {
-
 		if (skip_profile_frame) {
 			skip_profile_frame = false;
 		} else {
@@ -1010,7 +961,6 @@ void ScriptDebuggerRemote::idle_poll() {
 	}
 
 	if (reload_all_scripts) {
-
 		for (int i = 0; i < ScriptServer::get_language_count(); i++) {
 			ScriptServer::get_language(i)->reload_all_scripts();
 		}
@@ -1050,10 +1000,8 @@ void ScriptDebuggerRemote::_send_network_bandwidth_usage() {
 }
 
 void ScriptDebuggerRemote::send_message(const String &p_message, const Array &p_args) {
-
-	mutex->lock();
+	mutex.lock();
 	if (!locking && tcp_client->is_connected_to_host()) {
-
 		if (messages.size() >= max_messages_per_frame) {
 			n_messages_dropped++;
 		} else {
@@ -1063,11 +1011,10 @@ void ScriptDebuggerRemote::send_message(const String &p_message, const Array &p_
 			messages.push_back(msg);
 		}
 	}
-	mutex->unlock();
+	mutex.unlock();
 }
 
 void ScriptDebuggerRemote::send_error(const String &p_func, const String &p_file, int p_line, const String &p_err, const String &p_descr, ErrorHandlerType p_type, const Vector<ScriptLanguage::StackInfo> &p_stack_info) {
-
 	OutputError oe;
 	oe.error = p_err;
 	oe.error_descr = p_descr;
@@ -1109,10 +1056,9 @@ void ScriptDebuggerRemote::send_error(const String &p_func, const String &p_file
 		err_count++;
 	}
 
-	mutex->lock();
+	mutex.lock();
 
 	if (!locking && tcp_client->is_connected_to_host()) {
-
 		if (oe.warning) {
 			if (warn_count > max_warnings_per_second) {
 				n_warnings_dropped++;
@@ -1128,11 +1074,10 @@ void ScriptDebuggerRemote::send_error(const String &p_func, const String &p_file
 		}
 	}
 
-	mutex->unlock();
+	mutex.unlock();
 }
 
 void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string, bool p_error) {
-
 	ScriptDebuggerRemote *sdr = (ScriptDebuggerRemote *)p_this;
 
 	uint64_t ticks = OS::get_singleton()->get_ticks_usec() / 1000;
@@ -1147,8 +1092,9 @@ void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string,
 	String s = p_string;
 	int allowed_chars = MIN(MAX(sdr->max_cps - sdr->char_count, 0), s.length());
 
-	if (allowed_chars == 0 && s.length() > 0)
+	if (allowed_chars == 0 && s.length() > 0) {
 		return;
+	}
 
 	if (allowed_chars < s.length()) {
 		s = s.substr(0, allowed_chars);
@@ -1157,11 +1103,11 @@ void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string,
 	sdr->char_count += allowed_chars;
 	bool overflowed = sdr->char_count >= sdr->max_cps;
 
-	sdr->mutex->lock();
+	sdr->mutex.lock();
 	if (!sdr->locking && sdr->tcp_client->is_connected_to_host()) {
-
-		if (overflowed)
+		if (overflowed) {
 			s += "[...]";
+		}
 
 		OutputString output_string;
 		output_string.message = s;
@@ -1174,11 +1120,10 @@ void ScriptDebuggerRemote::_print_handler(void *p_this, const String &p_string,
 			sdr->output_strings.push_back(output_string);
 		}
 	}
-	sdr->mutex->unlock();
+	sdr->mutex.unlock();
 }
 
 void ScriptDebuggerRemote::request_quit() {
-
 	requested_quit = true;
 }
 
@@ -1187,11 +1132,9 @@ void ScriptDebuggerRemote::set_multiplayer(Ref<MultiplayerAPI> p_multiplayer) {
 }
 
 bool ScriptDebuggerRemote::is_profiling() const {
-
 	return profiling;
 }
 void ScriptDebuggerRemote::add_profiling_frame_data(const StringName &p_name, const Array &p_data) {
-
 	int idx = -1;
 	for (int i = 0; i < profile_frame_data.size(); i++) {
 		if (profile_frame_data[i].name == p_name) {
@@ -1219,10 +1162,9 @@ void ScriptDebuggerRemote::profiling_end() {
 	//ignores this, uses it via connection
 }
 
-void ScriptDebuggerRemote::profiling_set_frame_times(float p_frame_time, float p_idle_time, float p_physics_time, float p_physics_frame_time) {
-
+void ScriptDebuggerRemote::profiling_set_frame_times(float p_frame_time, float p_process_time, float p_physics_time, float p_physics_frame_time) {
 	frame_time = p_frame_time;
-	idle_time = p_idle_time;
+	process_time = p_process_time;
 	physics_time = p_physics_time;
 	physics_frame_time = p_physics_frame_time;
 }
@@ -1231,7 +1173,11 @@ void ScriptDebuggerRemote::set_skip_breakpoints(bool p_skip_breakpoints) {
 	skip_breakpoints = p_skip_breakpoints;
 }
 
-ScriptDebuggerRemote::ResourceUsageFunc ScriptDebuggerRemote::resource_usage_func = NULL;
+void ScriptDebuggerRemote::set_allow_focus_steal_pid(OS::ProcessID p_pid) {
+	allow_focus_steal_pid = p_pid;
+}
+
+ScriptDebuggerRemote::ResourceUsageFunc ScriptDebuggerRemote::resource_usage_func = nullptr;
 
 ScriptDebuggerRemote::ScriptDebuggerRemote() :
 		profiling(false),
@@ -1246,22 +1192,22 @@ ScriptDebuggerRemote::ScriptDebuggerRemote() :
 		last_net_bandwidth_time(0),
 		performance(Engine::get_singleton()->get_singleton_object("Performance")),
 		requested_quit(false),
-		mutex(Mutex::create()),
 		max_messages_per_frame(GLOBAL_GET("network/limits/debugger_stdout/max_messages_per_frame")),
 		n_messages_dropped(0),
 		max_errors_per_second(GLOBAL_GET("network/limits/debugger_stdout/max_errors_per_second")),
 		max_warnings_per_second(GLOBAL_GET("network/limits/debugger_stdout/max_warnings_per_second")),
 		n_errors_dropped(0),
+		n_warnings_dropped(0),
 		max_cps(GLOBAL_GET("network/limits/debugger_stdout/max_chars_per_second")),
 		char_count(0),
 		err_count(0),
 		warn_count(0),
 		last_msec(0),
 		msec_count(0),
+		allow_focus_steal_pid(0),
 		locking(false),
 		poll_every(0),
-		scene_tree(NULL) {
-
+		scene_tree(nullptr) {
 	packet_peer_stream->set_stream_peer(tcp_client);
 	packet_peer_stream->set_output_buffer_max_size((1024 * 1024 * 8) - 4); // 8 MiB should be way more than enough, minus 4 bytes for separator.
 
@@ -1279,8 +1225,6 @@ ScriptDebuggerRemote::ScriptDebuggerRemote() :
 }
 
 ScriptDebuggerRemote::~ScriptDebuggerRemote() {
-
 	remove_print_handler(&phl);
 	remove_error_handler(&eh);
-	memdelete(mutex);
 }
