@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  file_dialog.cpp                                                      */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  file_dialog.cpp                                                       */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "file_dialog.h"
 
@@ -34,20 +34,18 @@
 #include "core/print_string.h"
 #include "scene/gui/label.h"
 
-FileDialog::GetIconFunc FileDialog::get_icon_func = NULL;
-FileDialog::GetIconFunc FileDialog::get_large_icon_func = NULL;
+FileDialog::GetIconFunc FileDialog::get_icon_func = nullptr;
+FileDialog::GetIconFunc FileDialog::get_large_icon_func = nullptr;
 
-FileDialog::RegisterFunc FileDialog::register_func = NULL;
-FileDialog::RegisterFunc FileDialog::unregister_func = NULL;
+FileDialog::RegisterFunc FileDialog::register_func = nullptr;
+FileDialog::RegisterFunc FileDialog::unregister_func = nullptr;
 
 VBoxContainer *FileDialog::get_vbox() {
 	return vbox;
 }
 
 void FileDialog::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_ENTER_TREE || p_what == NOTIFICATION_THEME_CHANGED) {
-
 		if (p_what == NOTIFICATION_ENTER_TREE) {
 			dir_up->set_icon(get_icon("parent_folder"));
 			refresh->set_icon(get_icon("reload"));
@@ -56,39 +54,37 @@ void FileDialog::_notification(int p_what) {
 
 		Color font_color = get_color("font_color", "ToolButton");
 		Color font_color_hover = get_color("font_color_hover", "ToolButton");
+		Color font_color_focus = get_color("font_color_focus", "ToolButton");
 		Color font_color_pressed = get_color("font_color_pressed", "ToolButton");
 
 		dir_up->add_color_override("icon_color_normal", font_color);
 		dir_up->add_color_override("icon_color_hover", font_color_hover);
+		dir_up->add_color_override("font_color_focus", font_color_focus);
 		dir_up->add_color_override("icon_color_pressed", font_color_pressed);
 
 		refresh->add_color_override("icon_color_normal", font_color);
 		refresh->add_color_override("icon_color_hover", font_color_hover);
+		refresh->add_color_override("font_color_focus", font_color_focus);
 		refresh->add_color_override("icon_color_pressed", font_color_pressed);
 
 		show_hidden->add_color_override("icon_color_normal", font_color);
 		show_hidden->add_color_override("icon_color_hover", font_color_hover);
+		show_hidden->add_color_override("font_color_focus", font_color_focus);
 		show_hidden->add_color_override("icon_color_pressed", font_color_pressed);
 
 	} else if (p_what == NOTIFICATION_POPUP_HIDE) {
-
 		set_process_unhandled_input(false);
 	}
 }
 
 void FileDialog::_unhandled_input(const Ref<InputEvent> &p_event) {
-
 	Ref<InputEventKey> k = p_event;
 	if (k.is_valid() && is_window_modal_on_top()) {
-
 		if (k->is_pressed()) {
-
 			bool handled = true;
 
 			switch (k->get_scancode()) {
-
 				case KEY_H: {
-
 					if (k->get_command()) {
 						set_show_hidden_files(!show_hidden_files);
 					} else {
@@ -97,11 +93,9 @@ void FileDialog::_unhandled_input(const Ref<InputEvent> &p_event) {
 
 				} break;
 				case KEY_F5: {
-
 					invalidate();
 				} break;
 				case KEY_BACKSPACE: {
-
 					_dir_entered("..");
 				} break;
 				default: {
@@ -109,24 +103,22 @@ void FileDialog::_unhandled_input(const Ref<InputEvent> &p_event) {
 				}
 			}
 
-			if (handled)
+			if (handled) {
 				accept_event();
+			}
 		}
 	}
 }
 
 void FileDialog::set_enable_multiple_selection(bool p_enable) {
-
 	tree->set_select_mode(p_enable ? Tree::SELECT_MULTI : Tree::SELECT_SINGLE);
 };
 
 Vector<String> FileDialog::get_selected_files() const {
-
 	Vector<String> list;
 
 	TreeItem *item = tree->get_root();
 	while ((item = tree->get_next_selected(item))) {
-
 		list.push_back(dir_access->get_current_dir().plus_file(item->get_text(0)));
 	};
 
@@ -134,11 +126,17 @@ Vector<String> FileDialog::get_selected_files() const {
 };
 
 void FileDialog::update_dir() {
-
 	dir->set_text(dir_access->get_current_dir_without_drive());
 
 	if (drives->is_visible()) {
-		drives->select(dir_access->get_current_drive());
+		if (dir_access->get_current_dir().is_network_share_path()) {
+			_update_drives(false);
+			drives->add_item(RTR("Network"));
+			drives->set_item_disabled(drives->get_item_count() - 1, true);
+			drives->select(drives->get_item_count() - 1);
+		} else {
+			drives->select(dir_access->get_current_drive());
+		}
 	}
 
 	// Deselect any item, to make "Select Current Folder" button text by default.
@@ -146,7 +144,6 @@ void FileDialog::update_dir() {
 }
 
 void FileDialog::_dir_entered(String p_dir) {
-
 	dir_access->change_dir(p_dir);
 	file->set_text("");
 	invalidate();
@@ -154,7 +151,6 @@ void FileDialog::_dir_entered(String p_dir) {
 }
 
 void FileDialog::_file_entered(const String &p_file) {
-
 	_action_pressed();
 }
 
@@ -165,16 +161,16 @@ void FileDialog::_save_confirm_pressed() {
 }
 
 void FileDialog::_post_popup() {
-
 	ConfirmationDialog::_post_popup();
 	if (invalidated) {
 		update_file_list();
 		invalidated = false;
 	}
-	if (mode == MODE_SAVE_FILE)
+	if (mode == MODE_SAVE_FILE) {
 		file->grab_focus();
-	else
+	} else {
 		tree->grab_focus();
+	}
 
 	set_process_unhandled_input(true);
 
@@ -188,15 +184,12 @@ void FileDialog::_post_popup() {
 }
 
 void FileDialog::_action_pressed() {
-
 	if (mode == MODE_OPEN_FILES) {
-
-		TreeItem *ti = tree->get_next_selected(NULL);
+		TreeItem *ti = tree->get_next_selected(nullptr);
 		String fbase = dir_access->get_current_dir();
 
 		PoolVector<String> files;
 		while (ti) {
-
 			files.push_back(fbase.plus_file(ti->get_text(0)));
 			ti = tree->get_next_selected(ti);
 		}
@@ -209,13 +202,13 @@ void FileDialog::_action_pressed() {
 		return;
 	}
 
-	String f = dir_access->get_current_dir().plus_file(file->get_text());
+	String file_text = file->get_text();
+	String f = file_text.is_abs_path() ? file_text : dir_access->get_current_dir().plus_file(file_text);
 
 	if ((mode == MODE_OPEN_ANY || mode == MODE_OPEN_FILE) && dir_access->file_exists(f)) {
 		emit_signal("file_selected", f);
 		hide();
 	} else if (mode == MODE_OPEN_ANY || mode == MODE_OPEN_DIR) {
-
 		String path = dir_access->get_current_dir();
 
 		path = path.replace("\\", "/");
@@ -232,7 +225,6 @@ void FileDialog::_action_pressed() {
 	}
 
 	if (mode == MODE_SAVE_FILE) {
-
 		bool valid = false;
 
 		if (filter->get_selected() == filter->get_item_count() - 1) {
@@ -240,29 +232,27 @@ void FileDialog::_action_pressed() {
 		} else if (filters.size() > 1 && filter->get_selected() == 0) {
 			// match all filters
 			for (int i = 0; i < filters.size(); i++) {
-
 				String flt = filters[i].get_slice(";", 0);
 				for (int j = 0; j < flt.get_slice_count(","); j++) {
-
 					String str = flt.get_slice(",", j).strip_edges();
 					if (f.match(str)) {
 						valid = true;
 						break;
 					}
 				}
-				if (valid)
+				if (valid) {
 					break;
+				}
 			}
 		} else {
 			int idx = filter->get_selected();
-			if (filters.size() > 1)
+			if (filters.size() > 1) {
 				idx--;
+			}
 			if (idx >= 0 && idx < filters.size()) {
-
 				String flt = filters[idx].get_slice(";", 0);
 				int filterSliceCount = flt.get_slice_count(",");
 				for (int j = 0; j < filterSliceCount; j++) {
-
 					String str = (flt.get_slice(",", j).strip_edges());
 					if (f.match(str)) {
 						valid = true;
@@ -282,16 +272,14 @@ void FileDialog::_action_pressed() {
 		}
 
 		if (!valid) {
-
 			exterr->popup_centered_minsize(Size2(250, 80));
 			return;
 		}
 
 		if (dir_access->file_exists(f)) {
-			confirm_save->set_text(RTR("File Exists, Overwrite?"));
+			confirm_save->set_text(RTR("File exists, overwrite?"));
 			confirm_save->popup_centered(Size2(200, 80));
 		} else {
-
 			emit_signal("file_selected", f);
 			hide();
 		}
@@ -299,44 +287,43 @@ void FileDialog::_action_pressed() {
 }
 
 void FileDialog::_cancel_pressed() {
-
 	file->set_text("");
 	invalidate();
 	hide();
 }
 
 bool FileDialog::_is_open_should_be_disabled() {
-
-	if (mode == MODE_OPEN_ANY || mode == MODE_SAVE_FILE)
+	if (mode == MODE_OPEN_ANY || mode == MODE_SAVE_FILE) {
 		return false;
+	}
 
 	TreeItem *ti = tree->get_next_selected(tree->get_root());
 	while (ti) {
 		TreeItem *prev_ti = ti;
 		ti = tree->get_next_selected(tree->get_root());
-		if (ti == prev_ti)
+		if (ti == prev_ti) {
 			break;
+		}
 	}
 	// We have something that we can't select?
-	if (!ti)
+	if (!ti) {
 		return mode != MODE_OPEN_DIR; // In "Open folder" mode, having nothing selected picks the current folder.
+	}
 
 	Dictionary d = ti->get_metadata(0);
 
 	// Opening a file, but selected a folder? Forbidden.
 	return ((mode == MODE_OPEN_FILE || mode == MODE_OPEN_FILES) && d["dir"]) || // Flipped case, also forbidden.
-		   (mode == MODE_OPEN_DIR && !d["dir"]);
+			(mode == MODE_OPEN_DIR && !d["dir"]);
 }
 
 void FileDialog::_go_up() {
-
 	dir_access->change_dir("..");
 	update_file_list();
 	update_dir();
 }
 
 void FileDialog::deselect_items() {
-
 	// Clear currently selected items in file manager.
 	tree->deselect_all();
 
@@ -345,7 +332,6 @@ void FileDialog::deselect_items() {
 		get_ok()->set_disabled(_is_open_should_be_disabled());
 
 		switch (mode) {
-
 			case MODE_OPEN_FILE:
 			case MODE_OPEN_FILES:
 				get_ok()->set_text(RTR("Open"));
@@ -366,14 +352,13 @@ void FileDialog::_tree_multi_selected(Object *p_object, int p_cell, bool p_selec
 }
 
 void FileDialog::_tree_selected() {
-
 	TreeItem *ti = tree->get_selected();
-	if (!ti)
+	if (!ti) {
 		return;
+	}
 	Dictionary d = ti->get_metadata(0);
 
 	if (!d["dir"]) {
-
 		file->set_text(d["name"]);
 	} else if (mode == MODE_OPEN_DIR) {
 		get_ok()->set_text(RTR("Select This Folder"));
@@ -383,22 +368,21 @@ void FileDialog::_tree_selected() {
 }
 
 void FileDialog::_tree_item_activated() {
-
 	TreeItem *ti = tree->get_selected();
-	if (!ti)
+	if (!ti) {
 		return;
+	}
 
 	Dictionary d = ti->get_metadata(0);
 
 	if (d["dir"]) {
-
 		dir_access->change_dir(d["name"]);
-		if (mode == MODE_OPEN_FILE || mode == MODE_OPEN_FILES || mode == MODE_OPEN_DIR || mode == MODE_OPEN_ANY)
+		if (mode == MODE_OPEN_FILE || mode == MODE_OPEN_FILES || mode == MODE_OPEN_DIR || mode == MODE_OPEN_ANY) {
 			file->set_text("");
+		}
 		call_deferred("_update_file_list");
 		call_deferred("_update_dir");
 	} else {
-
 		_action_pressed();
 	}
 }
@@ -406,17 +390,23 @@ void FileDialog::_tree_item_activated() {
 void FileDialog::update_file_name() {
 	int idx = filter->get_selected() - 1;
 	if ((idx == -1 && filter->get_item_count() == 2) || (filter->get_item_count() > 2 && idx >= 0 && idx < filter->get_item_count() - 2)) {
-		if (idx == -1) idx += 1;
+		if (idx == -1) {
+			idx += 1;
+		}
 		String filter_str = filters[idx];
 		String file_str = file->get_text();
 		String base_name = file_str.get_basename();
-		file_str = base_name + "." + filter_str.strip_edges().to_lower();
+		Vector<String> filter_substr = filter_str.split(";");
+		if (filter_substr.size() >= 2) {
+			file_str = base_name + "." + filter_substr[0].strip_edges().get_extension().to_lower();
+		} else {
+			file_str = base_name + "." + filter_str.strip_edges().get_extension().to_lower();
+		}
 		file->set_text(file_str);
 	}
 }
 
 void FileDialog::update_file_list() {
-
 	tree->clear();
 
 	// Scroll back to the top after opening a directory
@@ -436,17 +426,18 @@ void FileDialog::update_file_list() {
 	String item;
 
 	while ((item = dir_access->get_next()) != "") {
-
-		if (item == "." || item == "..")
+		if (item == "." || item == "..") {
 			continue;
+		}
 
 		is_hidden = dir_access->current_is_hidden();
 
 		if (show_hidden_files || !is_hidden) {
-			if (!dir_access->current_is_dir())
+			if (!dir_access->current_is_dir()) {
 				files.push_back(item);
-			else
+			} else {
 				dirs.push_back(item);
+			}
 		}
 	}
 
@@ -472,28 +463,24 @@ void FileDialog::update_file_list() {
 	List<String> patterns;
 	// build filter
 	if (filter->get_selected() == filter->get_item_count() - 1) {
-
 		// match all
 	} else if (filters.size() > 1 && filter->get_selected() == 0) {
 		// match all filters
 		for (int i = 0; i < filters.size(); i++) {
-
 			String f = filters[i].get_slice(";", 0);
 			for (int j = 0; j < f.get_slice_count(","); j++) {
-
 				patterns.push_back(f.get_slice(",", j).strip_edges());
 			}
 		}
 	} else {
 		int idx = filter->get_selected();
-		if (filters.size() > 1)
+		if (filters.size() > 1) {
 			idx--;
+		}
 
 		if (idx >= 0 && idx < filters.size()) {
-
 			String f = filters[idx].get_slice(";", 0);
 			for (int j = 0; j < f.get_slice_count(","); j++) {
-
 				patterns.push_back(f.get_slice(",", j).strip_edges());
 			}
 		}
@@ -502,12 +489,10 @@ void FileDialog::update_file_list() {
 	String base_dir = dir_access->get_current_dir();
 
 	while (!files.empty()) {
-
 		bool match = patterns.empty();
 		String match_str;
 
 		for (List<String>::Element *E = patterns.front(); E; E = E->next()) {
-
 			if (files.front()->get().matchn(E->get())) {
 				match_str = E->get();
 				match = true;
@@ -520,7 +505,6 @@ void FileDialog::update_file_list() {
 			ti->set_text(0, files.front()->get());
 
 			if (get_icon_func) {
-
 				Ref<Texture> icon = get_icon_func(base_dir.plus_file(files.front()->get()));
 				ti->set_icon(0, icon);
 			} else {
@@ -537,25 +521,25 @@ void FileDialog::update_file_list() {
 			d["dir"] = false;
 			ti->set_metadata(0, d);
 
-			if (file->get_text() == files.front()->get() || match_str == files.front()->get())
+			if (file->get_text() == files.front()->get() || match_str == files.front()->get()) {
 				ti->select(0);
+			}
 		}
 
 		files.pop_front();
 	}
 
-	if (tree->get_root() && tree->get_root()->get_children() && tree->get_selected() == NULL)
+	if (tree->get_root() && tree->get_root()->get_children() && tree->get_selected() == nullptr) {
 		tree->get_root()->get_children()->select(0);
+	}
 }
 
 void FileDialog::_filter_selected(int) {
-
 	update_file_name();
 	update_file_list();
 }
 
 void FileDialog::update_filters() {
-
 	filter->clear();
 
 	if (filters.size() > 1) {
@@ -565,37 +549,38 @@ void FileDialog::update_filters() {
 
 		for (int i = 0; i < MIN(max_filters, filters.size()); i++) {
 			String flt = filters[i].get_slice(";", 0).strip_edges();
-			if (i > 0)
+			if (i > 0) {
 				all_filters += ", ";
+			}
 			all_filters += flt;
 		}
 
-		if (max_filters < filters.size())
+		if (max_filters < filters.size()) {
 			all_filters += ", ...";
+		}
 
 		filter->add_item(RTR("All Recognized") + " (" + all_filters + ")");
 	}
 	for (int i = 0; i < filters.size(); i++) {
-
 		String flt = filters[i].get_slice(";", 0).strip_edges();
 		String desc = filters[i].get_slice(";", 1).strip_edges();
-		if (desc.length())
+		if (desc.length()) {
 			filter->add_item(String(tr(desc)) + " (" + flt + ")");
-		else
+		} else {
 			filter->add_item("(" + flt + ")");
+		}
 	}
 
 	filter->add_item(RTR("All Files (*)"));
 }
 
 void FileDialog::clear_filters() {
-
 	filters.clear();
 	update_filters();
 	invalidate();
 }
 void FileDialog::add_filter(const String &p_filter) {
-
+	ERR_FAIL_COND_MSG(p_filter.begins_with("."), "Filter must be \"filename.extension\", can't start with dot.");
 	filters.push_back(p_filter);
 	update_filters();
 	invalidate();
@@ -612,45 +597,39 @@ Vector<String> FileDialog::get_filters() const {
 }
 
 String FileDialog::get_current_dir() const {
-
 	return dir->get_text();
 }
 String FileDialog::get_current_file() const {
-
 	return file->get_text();
 }
 String FileDialog::get_current_path() const {
-
 	return dir->get_text().plus_file(file->get_text());
 }
 void FileDialog::set_current_dir(const String &p_dir) {
-
 	dir_access->change_dir(p_dir);
 	update_dir();
 	invalidate();
 }
 void FileDialog::set_current_file(const String &p_file) {
-
 	file->set_text(p_file);
 	update_dir();
 	invalidate();
-	int lp = p_file.find_last(".");
+	int lp = p_file.rfind(".");
 	if (lp != -1) {
 		file->select(0, lp);
-		if (file->is_inside_tree() && !get_tree()->is_node_being_edited(file))
+		if (file->is_inside_tree() && !get_tree()->is_node_being_edited(file)) {
 			file->grab_focus();
+		}
 	}
 }
 void FileDialog::set_current_path(const String &p_path) {
-
-	if (!p_path.size())
+	if (!p_path.size()) {
 		return;
-	int pos = MAX(p_path.find_last("/"), p_path.find_last("\\"));
+	}
+	int pos = MAX(p_path.rfind("/"), p_path.rfind("\\"));
 	if (pos == -1) {
-
 		set_current_file(p_path);
 	} else {
-
 		String dir = p_path.substr(0, pos);
 		String file = p_path.substr(pos + 1, p_path.length());
 		set_current_dir(dir);
@@ -667,40 +646,43 @@ bool FileDialog::is_mode_overriding_title() const {
 }
 
 void FileDialog::set_mode(Mode p_mode) {
-
 	ERR_FAIL_INDEX((int)p_mode, 5);
 
 	mode = p_mode;
 	switch (mode) {
-
 		case MODE_OPEN_FILE:
 			get_ok()->set_text(RTR("Open"));
-			if (mode_overrides_title)
+			if (mode_overrides_title) {
 				set_title(RTR("Open a File"));
+			}
 			makedir->hide();
 			break;
 		case MODE_OPEN_FILES:
 			get_ok()->set_text(RTR("Open"));
-			if (mode_overrides_title)
+			if (mode_overrides_title) {
 				set_title(RTR("Open File(s)"));
+			}
 			makedir->hide();
 			break;
 		case MODE_OPEN_DIR:
 			get_ok()->set_text(RTR("Select Current Folder"));
-			if (mode_overrides_title)
+			if (mode_overrides_title) {
 				set_title(RTR("Open a Directory"));
+			}
 			makedir->show();
 			break;
 		case MODE_OPEN_ANY:
 			get_ok()->set_text(RTR("Open"));
-			if (mode_overrides_title)
+			if (mode_overrides_title) {
 				set_title(RTR("Open a File or Directory"));
+			}
 			makedir->show();
 			break;
 		case MODE_SAVE_FILE:
 			get_ok()->set_text(RTR("Save"));
-			if (mode_overrides_title)
+			if (mode_overrides_title) {
 				set_title(RTR("Save a File"));
+			}
 			makedir->show();
 			break;
 	}
@@ -713,27 +695,23 @@ void FileDialog::set_mode(Mode p_mode) {
 }
 
 FileDialog::Mode FileDialog::get_mode() const {
-
 	return mode;
 }
 
 void FileDialog::set_access(Access p_access) {
-
 	ERR_FAIL_INDEX(p_access, 3);
-	if (access == p_access)
+	if (access == p_access) {
 		return;
+	}
 	memdelete(dir_access);
 	switch (p_access) {
 		case ACCESS_FILESYSTEM: {
-
 			dir_access = DirAccess::create(DirAccess::ACCESS_FILESYSTEM);
 		} break;
 		case ACCESS_RESOURCES: {
-
 			dir_access = DirAccess::create(DirAccess::ACCESS_RESOURCES);
 		} break;
 		case ACCESS_USERDATA: {
-
 			dir_access = DirAccess::create(DirAccess::ACCESS_USERDATA);
 		} break;
 	}
@@ -745,7 +723,6 @@ void FileDialog::set_access(Access p_access) {
 }
 
 void FileDialog::invalidate() {
-
 	if (is_visible_in_tree()) {
 		update_file_list();
 		invalidated = false;
@@ -755,15 +732,13 @@ void FileDialog::invalidate() {
 }
 
 FileDialog::Access FileDialog::get_access() const {
-
 	return access;
 }
 
 void FileDialog::_make_dir_confirm() {
-
-	Error err = dir_access->make_dir(makedirname->get_text());
+	Error err = dir_access->make_dir(makedirname->get_text().strip_edges());
 	if (err == OK) {
-		dir_access->change_dir(makedirname->get_text());
+		dir_access->change_dir(makedirname->get_text().strip_edges());
 		invalidate();
 		update_filters();
 		update_dir();
@@ -774,13 +749,11 @@ void FileDialog::_make_dir_confirm() {
 }
 
 void FileDialog::_make_dir() {
-
 	makedialog->popup_centered_minsize(Size2(250, 80));
 	makedirname->grab_focus();
 }
 
 void FileDialog::_select_drive(int p_idx) {
-
 	String d = drives->get_item_text(p_idx);
 	dir_access->change_dir(d);
 	file->set_text("");
@@ -788,8 +761,7 @@ void FileDialog::_select_drive(int p_idx) {
 	update_dir();
 }
 
-void FileDialog::_update_drives() {
-
+void FileDialog::_update_drives(bool p_select) {
 	int dc = dir_access->get_drive_count();
 	if (dc == 0 || access != ACCESS_FILESYSTEM) {
 		drives->hide();
@@ -807,14 +779,15 @@ void FileDialog::_update_drives() {
 			drives->add_item(dir_access->get_drive(i));
 		}
 
-		drives->select(dir_access->get_current_drive());
+		if (p_select) {
+			drives->select(dir_access->get_current_drive());
+		}
 	}
 }
 
 bool FileDialog::default_show_hidden_files = false;
 
 void FileDialog::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_unhandled_input"), &FileDialog::_unhandled_input);
 
 	ClassDB::bind_method(D_METHOD("_tree_multi_selected"), &FileDialog::_tree_multi_selected);
@@ -863,9 +836,9 @@ void FileDialog::_bind_methods() {
 	ADD_PROPERTY(PropertyInfo(Variant::INT, "access", PROPERTY_HINT_ENUM, "Resources,User data,File system"), "set_access", "get_access");
 	ADD_PROPERTY(PropertyInfo(Variant::POOL_STRING_ARRAY, "filters"), "set_filters", "get_filters");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "show_hidden_files"), "set_show_hidden_files", "is_showing_hidden_files");
-	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_dir"), "set_current_dir", "get_current_dir");
-	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_file"), "set_current_file", "get_current_file");
-	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_path"), "set_current_path", "get_current_path");
+	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_dir", PROPERTY_HINT_DIR, "", 0), "set_current_dir", "get_current_dir");
+	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_file", PROPERTY_HINT_FILE, "*", 0), "set_current_file", "get_current_file");
+	ADD_PROPERTY(PropertyInfo(Variant::STRING, "current_path", PROPERTY_HINT_NONE, "", 0), "set_current_path", "get_current_path");
 
 	ADD_SIGNAL(MethodInfo("file_selected", PropertyInfo(Variant::STRING, "path")));
 	ADD_SIGNAL(MethodInfo("files_selected", PropertyInfo(Variant::POOL_STRING_ARRAY, "paths")));
@@ -896,7 +869,6 @@ void FileDialog::set_default_show_hidden_files(bool p_show) {
 }
 
 FileDialog::FileDialog() {
-
 	show_hidden_files = default_show_hidden_files;
 
 	mode_overrides_title = true;
@@ -1009,19 +981,19 @@ FileDialog::FileDialog() {
 	vbox = vbc;
 
 	invalidated = true;
-	if (register_func)
+	if (register_func) {
 		register_func(this);
+	}
 }
 
 FileDialog::~FileDialog() {
-
-	if (unregister_func)
+	if (unregister_func) {
 		unregister_func(this);
+	}
 	memdelete(dir_access);
 }
 
 void LineEditFileChooser::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_browse"), &LineEditFileChooser::_browse);
 	ClassDB::bind_method(D_METHOD("_chosen"), &LineEditFileChooser::_chosen);
 	ClassDB::bind_method(D_METHOD("get_button"), &LineEditFileChooser::get_button);
@@ -1030,18 +1002,15 @@ void LineEditFileChooser::_bind_methods() {
 }
 
 void LineEditFileChooser::_chosen(const String &p_text) {
-
 	line_edit->set_text(p_text);
 	line_edit->emit_signal("text_entered", p_text);
 }
 
 void LineEditFileChooser::_browse() {
-
 	dialog->popup_centered_ratio();
 }
 
 LineEditFileChooser::LineEditFileChooser() {
-
 	line_edit = memnew(LineEdit);
 	add_child(line_edit);
 	line_edit->set_h_size_flags(SIZE_EXPAND_FILL);
