@@ -1,56 +1,56 @@
-/*************************************************************************/
-/*  scroll_container.cpp                                                 */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  scroll_container.cpp                                                  */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "scroll_container.h"
 #include "core/os/os.h"
 #include "scene/main/viewport.h"
 
 bool ScrollContainer::clips_input() const {
-
 	return true;
 }
 
 Size2 ScrollContainer::get_minimum_size() const {
-
 	Ref<StyleBox> sb = get_stylebox("bg");
 	Size2 min_size;
 
 	for (int i = 0; i < get_child_count(); i++) {
-
 		Control *c = Object::cast_to<Control>(get_child(i));
-		if (!c)
+		if (!c) {
 			continue;
-		if (c->is_set_as_toplevel())
+		}
+		if (c->is_set_as_toplevel()) {
 			continue;
-		if (c == h_scroll || c == v_scroll)
+		}
+		if (c == h_scroll || c == v_scroll) {
 			continue;
+		}
 		Size2 minsize = c->get_combined_minimum_size();
 
 		if (!scroll_h) {
@@ -88,14 +88,12 @@ void ScrollContainer::_cancel_drag() {
 }
 
 void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
-
 	double prev_v_scroll = v_scroll->get_value();
 	double prev_h_scroll = h_scroll->get_value();
 
 	Ref<InputEventMouseButton> mb = p_gui_input;
 
 	if (mb.is_valid()) {
-
 		if (mb->get_button_index() == BUTTON_WHEEL_UP && mb->is_pressed()) {
 			// only horizontal is enabled, scroll horizontally
 			if (h_scroll->is_visible() && (!v_scroll->is_visible() || mb->get_shift())) {
@@ -126,17 +124,19 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 			}
 		}
 
-		if (v_scroll->get_value() != prev_v_scroll || h_scroll->get_value() != prev_h_scroll)
+		if (v_scroll->get_value() != prev_v_scroll || h_scroll->get_value() != prev_h_scroll) {
 			accept_event(); //accept event if scroll changed
+		}
 
-		if (!OS::get_singleton()->has_touchscreen_ui_hint())
+		if (!OS::get_singleton()->has_touchscreen_ui_hint()) {
 			return;
+		}
 
-		if (mb->get_button_index() != BUTTON_LEFT)
+		if (mb->get_button_index() != BUTTON_LEFT) {
 			return;
+		}
 
 		if (mb->is_pressed()) {
-
 			if (drag_touching) {
 				_cancel_drag();
 			}
@@ -156,11 +156,9 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 
 		} else {
 			if (drag_touching) {
-
 				if (drag_speed == Vector2()) {
 					_cancel_drag();
 				} else {
-
 					drag_touching_deaccel = true;
 				}
 			}
@@ -170,9 +168,7 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 	Ref<InputEventMouseMotion> mm = p_gui_input;
 
 	if (mm.is_valid()) {
-
 		if (drag_touching && !drag_touching_deaccel) {
-
 			Vector2 motion = Vector2(mm->get_relative().x, mm->get_relative().y);
 			drag_accum -= motion;
 
@@ -186,14 +182,16 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 					drag_accum = -motion;
 				}
 				Vector2 diff = drag_from + drag_accum;
-				if (scroll_h)
+				if (scroll_h) {
 					h_scroll->set_value(diff.x);
-				else
+				} else {
 					drag_accum.x = 0;
-				if (scroll_v)
+				}
+				if (scroll_v) {
 					v_scroll->set_value(diff.y);
-				else
+				} else {
 					drag_accum.y = 0;
+				}
 				time_since_motion = 0;
 			}
 		}
@@ -201,7 +199,6 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 
 	Ref<InputEventPanGesture> pan_gesture = p_gui_input;
 	if (pan_gesture.is_valid()) {
-
 		if (h_scroll->is_visible_in_tree()) {
 			h_scroll->set_value(h_scroll->get_value() + h_scroll->get_page() * pan_gesture->get_delta().x / 8);
 		}
@@ -210,12 +207,12 @@ void ScrollContainer::_gui_input(const Ref<InputEvent> &p_gui_input) {
 		}
 	}
 
-	if (v_scroll->get_value() != prev_v_scroll || h_scroll->get_value() != prev_h_scroll)
+	if (v_scroll->get_value() != prev_v_scroll || h_scroll->get_value() != prev_h_scroll) {
 		accept_event(); //accept event if scroll changed
+	}
 }
 
 void ScrollContainer::_update_scrollbar_position() {
-
 	Size2 hmin = h_scroll->get_combined_minimum_size();
 	Size2 vmin = v_scroll->get_combined_minimum_size();
 
@@ -233,96 +230,100 @@ void ScrollContainer::_update_scrollbar_position() {
 	v_scroll->raise();
 }
 
-void ScrollContainer::_ensure_focused_visible(Control *p_control) {
-
-	if (!follow_focus) {
-		return;
-	}
-
-	if (is_a_parent_of(p_control)) {
-		Rect2 global_rect = get_global_rect();
-		Rect2 other_rect = p_control->get_global_rect();
-		float right_margin = 0;
-		if (v_scroll->is_visible()) {
-			right_margin += v_scroll->get_size().x;
-		}
-		float bottom_margin = 0;
-		if (h_scroll->is_visible()) {
-			bottom_margin += h_scroll->get_size().y;
-		}
-
-		float diff = MAX(MIN(other_rect.position.y, global_rect.position.y), other_rect.position.y + other_rect.size.y - global_rect.size.y + bottom_margin);
-		set_v_scroll(get_v_scroll() + (diff - global_rect.position.y));
-		diff = MAX(MIN(other_rect.position.x, global_rect.position.x), other_rect.position.x + other_rect.size.x - global_rect.size.x + right_margin);
-		set_h_scroll(get_h_scroll() + (diff - global_rect.position.x));
+void ScrollContainer::_gui_focus_changed(Control *p_control) {
+	if (follow_focus && is_a_parent_of(p_control)) {
+		ensure_control_visible(p_control);
 	}
 }
 
+void ScrollContainer::ensure_control_visible(Control *p_control) {
+	ERR_FAIL_COND_MSG(!is_a_parent_of(p_control), "Must be a parent of the control.");
+
+	Rect2 global_rect = get_global_rect();
+	Rect2 other_rect = p_control->get_global_rect();
+	float right_margin = v_scroll->is_visible() ? v_scroll->get_size().x : 0.0f;
+	float bottom_margin = h_scroll->is_visible() ? h_scroll->get_size().y : 0.0f;
+
+	Vector2 diff = Vector2(MAX(MIN(other_rect.position.x, global_rect.position.x), other_rect.position.x + other_rect.size.x - global_rect.size.x + right_margin),
+			MAX(MIN(other_rect.position.y, global_rect.position.y), other_rect.position.y + other_rect.size.y - global_rect.size.y + bottom_margin));
+
+	set_h_scroll(get_h_scroll() + (diff.x - global_rect.position.x));
+	set_v_scroll(get_v_scroll() + (diff.y - global_rect.position.y));
+}
+
+void ScrollContainer::_update_dimensions() {
+	child_max_size = Size2(0, 0);
+	Size2 size = get_size();
+	Point2 ofs;
+
+	Ref<StyleBox> sb = get_stylebox("bg");
+	size -= sb->get_minimum_size();
+	ofs += sb->get_offset();
+
+	if (h_scroll->is_visible_in_tree() && h_scroll->get_parent() == this) { //scrolls may have been moved out for reasons
+		size.y -= h_scroll->get_minimum_size().y;
+	}
+
+	if (v_scroll->is_visible_in_tree() && v_scroll->get_parent() == this) { //scrolls may have been moved out for reasons
+		size.x -= v_scroll->get_minimum_size().x;
+	}
+
+	for (int i = 0; i < get_child_count(); i++) {
+		Control *c = Object::cast_to<Control>(get_child(i));
+		if (!c) {
+			continue;
+		}
+		if (c->is_set_as_toplevel()) {
+			continue;
+		}
+		if (c == h_scroll || c == v_scroll) {
+			continue;
+		}
+		Size2 minsize = c->get_combined_minimum_size();
+		child_max_size.x = MAX(child_max_size.x, minsize.x);
+		child_max_size.y = MAX(child_max_size.y, minsize.y);
+
+		Rect2 r = Rect2(-scroll, minsize);
+		if (!scroll_h || (!h_scroll->is_visible_in_tree() && c->get_h_size_flags() & SIZE_EXPAND)) {
+			r.position.x = 0;
+			if (c->get_h_size_flags() & SIZE_EXPAND) {
+				r.size.width = MAX(size.width, minsize.width);
+			} else {
+				r.size.width = minsize.width;
+			}
+		}
+		if (!scroll_v || (!v_scroll->is_visible_in_tree() && c->get_v_size_flags() & SIZE_EXPAND)) {
+			r.position.y = 0;
+			if (c->get_v_size_flags() & SIZE_EXPAND) {
+				r.size.height = MAX(size.height, minsize.height);
+			} else {
+				r.size.height = minsize.height;
+			}
+		}
+		r.position += ofs;
+		fit_child_in_rect(c, r);
+	}
+
+	update();
+}
+
 void ScrollContainer::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_ENTER_TREE || p_what == NOTIFICATION_THEME_CHANGED) {
-
 		call_deferred("_update_scrollbar_position");
 	};
 
 	if (p_what == NOTIFICATION_READY) {
-
-		get_viewport()->connect("gui_focus_changed", this, "_ensure_focused_visible");
+		Viewport *viewport = get_viewport();
+		ERR_FAIL_COND(!viewport);
+		viewport->connect("gui_focus_changed", this, "_gui_focus_changed");
+		_update_dimensions();
 	}
 
 	if (p_what == NOTIFICATION_SORT_CHILDREN) {
-
-		child_max_size = Size2(0, 0);
-		Size2 size = get_size();
-		Point2 ofs;
-
-		Ref<StyleBox> sb = get_stylebox("bg");
-		size -= sb->get_minimum_size();
-		ofs += sb->get_offset();
-
-		if (h_scroll->is_visible_in_tree() && h_scroll->get_parent() == this) //scrolls may have been moved out for reasons
-			size.y -= h_scroll->get_minimum_size().y;
-
-		if (v_scroll->is_visible_in_tree() && v_scroll->get_parent() == this) //scrolls may have been moved out for reasons
-			size.x -= v_scroll->get_minimum_size().x;
-
-		for (int i = 0; i < get_child_count(); i++) {
-
-			Control *c = Object::cast_to<Control>(get_child(i));
-			if (!c)
-				continue;
-			if (c->is_set_as_toplevel())
-				continue;
-			if (c == h_scroll || c == v_scroll)
-				continue;
-			Size2 minsize = c->get_combined_minimum_size();
-			child_max_size.x = MAX(child_max_size.x, minsize.x);
-			child_max_size.y = MAX(child_max_size.y, minsize.y);
-
-			Rect2 r = Rect2(-scroll, minsize);
-			if (!scroll_h || (!h_scroll->is_visible_in_tree() && c->get_h_size_flags() & SIZE_EXPAND)) {
-				r.position.x = 0;
-				if (c->get_h_size_flags() & SIZE_EXPAND)
-					r.size.width = MAX(size.width, minsize.width);
-				else
-					r.size.width = minsize.width;
-			}
-			if (!scroll_v || (!v_scroll->is_visible_in_tree() && c->get_v_size_flags() & SIZE_EXPAND)) {
-				r.position.y = 0;
-				if (c->get_v_size_flags() & SIZE_EXPAND)
-					r.size.height = MAX(size.height, minsize.height);
-				else
-					r.size.height = minsize.height;
-			}
-			r.position += ofs;
-			fit_child_in_rect(c, r);
-		}
-
-		update();
+		_update_dimensions();
 	};
 
 	if (p_what == NOTIFICATION_DRAW) {
-
 		Ref<StyleBox> sb = get_stylebox("bg");
 		draw_style_box(sb, Rect2(Vector2(), get_size()));
 
@@ -330,11 +331,8 @@ void ScrollContainer::_notification(int p_what) {
 	}
 
 	if (p_what == NOTIFICATION_INTERNAL_PHYSICS_PROCESS) {
-
 		if (drag_touching) {
-
 			if (drag_touching_deaccel) {
-
 				Vector2 pos = Vector2(h_scroll->get_value(), v_scroll->get_value());
 				pos += drag_speed * get_physics_process_delta_time();
 
@@ -359,10 +357,12 @@ void ScrollContainer::_notification(int p_what) {
 					turnoff_v = true;
 				}
 
-				if (scroll_h)
+				if (scroll_h) {
 					h_scroll->set_value(pos.x);
-				if (scroll_v)
+				}
+				if (scroll_v) {
 					v_scroll->set_value(pos.y);
+				}
 
 				float sgn_x = drag_speed.x < 0 ? -1 : 1;
 				float val_x = Math::abs(drag_speed.x);
@@ -387,9 +387,7 @@ void ScrollContainer::_notification(int p_what) {
 				}
 
 			} else {
-
 				if (time_since_motion == 0 || time_since_motion > 0.1) {
-
 					Vector2 diff = drag_accum - last_drag_accum;
 					last_drag_accum = drag_accum;
 					drag_speed = diff / get_physics_process_delta_time();
@@ -402,7 +400,6 @@ void ScrollContainer::_notification(int p_what) {
 };
 
 void ScrollContainer::update_scrollbars() {
-
 	Size2 size = get_size();
 	Ref<StyleBox> sb = get_stylebox("bg");
 	size -= sb->get_minimum_size();
@@ -421,14 +418,13 @@ void ScrollContainer::update_scrollbars() {
 	bool hide_scroll_v = !scroll_v || min.height <= size.height;
 	bool hide_scroll_h = !scroll_h || min.width <= size.width;
 
+	v_scroll->set_max(min.height);
 	if (hide_scroll_v) {
-
+		v_scroll->set_page(size.height);
 		v_scroll->hide();
 		scroll.y = 0;
 	} else {
-
 		v_scroll->show();
-		v_scroll->set_max(min.height);
 		if (hide_scroll_h) {
 			v_scroll->set_page(size.height);
 		} else {
@@ -438,14 +434,13 @@ void ScrollContainer::update_scrollbars() {
 		scroll.y = v_scroll->get_value();
 	}
 
+	h_scroll->set_max(min.width);
 	if (hide_scroll_h) {
-
+		h_scroll->set_page(size.width);
 		h_scroll->hide();
 		scroll.x = 0;
 	} else {
-
 		h_scroll->show();
-		h_scroll->set_max(min.width);
 		if (hide_scroll_v) {
 			h_scroll->set_page(size.width);
 		} else {
@@ -461,7 +456,6 @@ void ScrollContainer::update_scrollbars() {
 }
 
 void ScrollContainer::_scroll_moved(float) {
-
 	scroll.x = h_scroll->get_value();
 	scroll.y = v_scroll->get_value();
 	queue_sort();
@@ -480,7 +474,6 @@ void ScrollContainer::set_enable_h_scroll(bool p_enable) {
 }
 
 bool ScrollContainer::is_h_scroll_enabled() const {
-
 	return scroll_h;
 }
 
@@ -495,26 +488,21 @@ void ScrollContainer::set_enable_v_scroll(bool p_enable) {
 }
 
 bool ScrollContainer::is_v_scroll_enabled() const {
-
 	return scroll_v;
 }
 
 int ScrollContainer::get_v_scroll() const {
-
 	return v_scroll->get_value();
 }
 void ScrollContainer::set_v_scroll(int p_pos) {
-
 	v_scroll->set_value(p_pos);
 	_cancel_drag();
 }
 
 int ScrollContainer::get_h_scroll() const {
-
 	return h_scroll->get_value();
 }
 void ScrollContainer::set_h_scroll(int p_pos) {
-
 	h_scroll->set_value(p_pos);
 	_cancel_drag();
 }
@@ -536,48 +524,52 @@ void ScrollContainer::set_follow_focus(bool p_follow) {
 }
 
 String ScrollContainer::get_configuration_warning() const {
+	String warning = Control::get_configuration_warning();
 
 	int found = 0;
 
 	for (int i = 0; i < get_child_count(); i++) {
-
 		Control *c = Object::cast_to<Control>(get_child(i));
-		if (!c)
+		if (!c) {
 			continue;
-		if (c->is_set_as_toplevel())
+		}
+		if (c->is_set_as_toplevel()) {
 			continue;
-		if (c == h_scroll || c == v_scroll)
+		}
+		if (c == h_scroll || c == v_scroll) {
 			continue;
+		}
 
 		found++;
 	}
 
-	if (found != 1)
-		return TTR("ScrollContainer is intended to work with a single child control.\nUse a container as child (VBox, HBox, etc.), or a Control and set the custom minimum size manually.");
-	else
-		return "";
+	if (found != 1) {
+		if (warning != String()) {
+			warning += "\n\n";
+		}
+		warning += TTR("ScrollContainer is intended to work with a single child control.\nUse a container as child (VBox, HBox, etc.), or a Control and set the custom minimum size manually.");
+	}
+
+	return warning;
 }
 
 HScrollBar *ScrollContainer::get_h_scrollbar() {
-
 	return h_scroll;
 }
 
 VScrollBar *ScrollContainer::get_v_scrollbar() {
-
 	return v_scroll;
 }
 
 void ScrollContainer::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_scroll_moved"), &ScrollContainer::_scroll_moved);
 	ClassDB::bind_method(D_METHOD("_gui_input"), &ScrollContainer::_gui_input);
+	ClassDB::bind_method(D_METHOD("_gui_focus_changed"), &ScrollContainer::_gui_focus_changed);
 	ClassDB::bind_method(D_METHOD("set_enable_h_scroll", "enable"), &ScrollContainer::set_enable_h_scroll);
 	ClassDB::bind_method(D_METHOD("is_h_scroll_enabled"), &ScrollContainer::is_h_scroll_enabled);
 	ClassDB::bind_method(D_METHOD("set_enable_v_scroll", "enable"), &ScrollContainer::set_enable_v_scroll);
 	ClassDB::bind_method(D_METHOD("is_v_scroll_enabled"), &ScrollContainer::is_v_scroll_enabled);
 	ClassDB::bind_method(D_METHOD("_update_scrollbar_position"), &ScrollContainer::_update_scrollbar_position);
-	ClassDB::bind_method(D_METHOD("_ensure_focused_visible"), &ScrollContainer::_ensure_focused_visible);
 	ClassDB::bind_method(D_METHOD("set_h_scroll", "value"), &ScrollContainer::set_h_scroll);
 	ClassDB::bind_method(D_METHOD("get_h_scroll"), &ScrollContainer::get_h_scroll);
 	ClassDB::bind_method(D_METHOD("set_v_scroll", "value"), &ScrollContainer::set_v_scroll);
@@ -589,6 +581,7 @@ void ScrollContainer::_bind_methods() {
 
 	ClassDB::bind_method(D_METHOD("get_h_scrollbar"), &ScrollContainer::get_h_scrollbar);
 	ClassDB::bind_method(D_METHOD("get_v_scrollbar"), &ScrollContainer::get_v_scrollbar);
+	ClassDB::bind_method(D_METHOD("ensure_control_visible", "control"), &ScrollContainer::ensure_control_visible);
 
 	ADD_SIGNAL(MethodInfo("scroll_started"));
 	ADD_SIGNAL(MethodInfo("scroll_ended"));
@@ -606,7 +599,6 @@ void ScrollContainer::_bind_methods() {
 };
 
 ScrollContainer::ScrollContainer() {
-
 	h_scroll = memnew(HScrollBar);
 	h_scroll->set_name("_h_scroll");
 	add_child(h_scroll);
