@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  split_container.cpp                                                  */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  split_container.cpp                                                   */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "split_container.h"
 
@@ -34,23 +34,25 @@
 #include "margin_container.h"
 
 Control *SplitContainer::_getch(int p_idx) const {
-
 	int idx = 0;
 
 	for (int i = 0; i < get_child_count(); i++) {
 		Control *c = Object::cast_to<Control>(get_child(i));
-		if (!c || !c->is_visible_in_tree())
+		if (!c || !c->is_visible()) {
 			continue;
-		if (c->is_set_as_toplevel())
+		}
+		if (c->is_set_as_toplevel()) {
 			continue;
+		}
 
-		if (idx == p_idx)
+		if (idx == p_idx) {
 			return c;
+		}
 
 		idx++;
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void SplitContainer::_resort() {
@@ -119,7 +121,6 @@ void SplitContainer::_resort() {
 }
 
 Size2 SplitContainer::get_minimum_size() const {
-
 	/* Calculate MINIMUM SIZE */
 
 	Size2i minimum;
@@ -128,26 +129,24 @@ Size2 SplitContainer::get_minimum_size() const {
 	sep = (dragger_visibility != DRAGGER_HIDDEN_COLLAPSED) ? MAX(sep, vertical ? g->get_height() : g->get_width()) : 0;
 
 	for (int i = 0; i < 2; i++) {
-
-		if (!_getch(i))
+		if (!_getch(i)) {
 			break;
+		}
 
 		if (i == 1) {
-
-			if (vertical)
+			if (vertical) {
 				minimum.height += sep;
-			else
+			} else {
 				minimum.width += sep;
+			}
 		}
 
 		Size2 ms = _getch(i)->get_combined_minimum_size();
 
 		if (vertical) {
-
 			minimum.height += ms.height;
 			minimum.width = MAX(minimum.width, ms.width);
 		} else {
-
 			minimum.width += ms.width;
 			minimum.height = MAX(minimum.height, ms.height);
 		}
@@ -157,80 +156,71 @@ Size2 SplitContainer::get_minimum_size() const {
 }
 
 void SplitContainer::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_SORT_CHILDREN: {
-
 			_resort();
 		} break;
 		case NOTIFICATION_MOUSE_EXIT: {
-
 			mouse_inside = false;
-			if (get_constant("autohide"))
+			if (get_constant("autohide")) {
 				update();
+			}
 		} break;
 		case NOTIFICATION_DRAW: {
-
-			if (!_getch(0) || !_getch(1))
+			if (!_getch(0) || !_getch(1)) {
 				return;
+			}
 
-			if (collapsed || (!dragging && !mouse_inside && get_constant("autohide")))
+			if (collapsed || (!dragging && !mouse_inside && get_constant("autohide"))) {
 				return;
+			}
 
-			if (dragger_visibility != DRAGGER_VISIBLE)
+			if (dragger_visibility != DRAGGER_VISIBLE) {
 				return;
+			}
 
 			int sep = dragger_visibility != DRAGGER_HIDDEN_COLLAPSED ? get_constant("separation") : 0;
 			Ref<Texture> tex = get_icon("grabber");
 			Size2 size = get_size();
 
-			if (vertical)
+			if (vertical) {
 				draw_texture(tex, Point2i((size.x - tex->get_width()) / 2, middle_sep + (sep - tex->get_height()) / 2));
-			else
+			} else {
 				draw_texture(tex, Point2i(middle_sep + (sep - tex->get_width()) / 2, (size.y - tex->get_height()) / 2));
+			}
 		} break;
 		case NOTIFICATION_THEME_CHANGED: {
-
 			minimum_size_changed();
 		} break;
 	}
 }
 
 void SplitContainer::_gui_input(const Ref<InputEvent> &p_event) {
-
-	if (collapsed || !_getch(0) || !_getch(1) || dragger_visibility != DRAGGER_VISIBLE)
+	if (collapsed || !_getch(0) || !_getch(1) || dragger_visibility != DRAGGER_VISIBLE) {
 		return;
+	}
 
 	Ref<InputEventMouseButton> mb = p_event;
 
 	if (mb.is_valid()) {
-
 		if (mb->get_button_index() == BUTTON_LEFT) {
-
 			if (mb->is_pressed()) {
-
 				int sep = get_constant("separation");
 
 				if (vertical) {
-
 					if (mb->get_position().y > middle_sep && mb->get_position().y < middle_sep + sep) {
-
 						dragging = true;
 						drag_from = mb->get_position().y;
 						drag_ofs = split_offset;
 					}
 				} else {
-
 					if (mb->get_position().x > middle_sep && mb->get_position().x < middle_sep + sep) {
-
 						dragging = true;
 						drag_from = mb->get_position().x;
 						drag_ofs = split_offset;
 					}
 				}
 			} else {
-
 				dragging = false;
 			}
 		}
@@ -239,22 +229,23 @@ void SplitContainer::_gui_input(const Ref<InputEvent> &p_event) {
 	Ref<InputEventMouseMotion> mm = p_event;
 
 	if (mm.is_valid()) {
-
 		bool mouse_inside_state = false;
-		if (vertical)
+		if (vertical) {
 			mouse_inside_state = mm->get_position().y > middle_sep && mm->get_position().y < middle_sep + get_constant("separation");
-		else
+		} else {
 			mouse_inside_state = mm->get_position().x > middle_sep && mm->get_position().x < middle_sep + get_constant("separation");
-
-		if (mouse_inside != mouse_inside_state) {
-
-			mouse_inside = mouse_inside_state;
-			if (get_constant("autohide"))
-				update();
 		}
 
-		if (!dragging)
+		if (mouse_inside != mouse_inside_state) {
+			mouse_inside = mouse_inside_state;
+			if (get_constant("autohide")) {
+				update();
+			}
+		}
+
+		if (!dragging) {
 			return;
+		}
 
 		split_offset = drag_ofs + ((vertical ? mm->get_position().y : mm->get_position().x) - drag_from);
 		should_clamp_split_offset = true;
@@ -264,22 +255,21 @@ void SplitContainer::_gui_input(const Ref<InputEvent> &p_event) {
 }
 
 Control::CursorShape SplitContainer::get_cursor_shape(const Point2 &p_pos) const {
-
-	if (dragging)
+	if (dragging) {
 		return (vertical ? CURSOR_VSPLIT : CURSOR_HSPLIT);
+	}
 
 	if (!collapsed && _getch(0) && _getch(1) && dragger_visibility == DRAGGER_VISIBLE) {
-
 		int sep = get_constant("separation");
 
 		if (vertical) {
-
-			if (p_pos.y > middle_sep && p_pos.y < middle_sep + sep)
+			if (p_pos.y > middle_sep && p_pos.y < middle_sep + sep) {
 				return CURSOR_VSPLIT;
+			}
 		} else {
-
-			if (p_pos.x > middle_sep && p_pos.x < middle_sep + sep)
+			if (p_pos.x > middle_sep && p_pos.x < middle_sep + sep) {
 				return CURSOR_HSPLIT;
+			}
 		}
 	}
 
@@ -287,9 +277,9 @@ Control::CursorShape SplitContainer::get_cursor_shape(const Point2 &p_pos) const
 }
 
 void SplitContainer::set_split_offset(int p_offset) {
-
-	if (split_offset == p_offset)
+	if (split_offset == p_offset) {
 		return;
+	}
 
 	split_offset = p_offset;
 
@@ -297,7 +287,6 @@ void SplitContainer::set_split_offset(int p_offset) {
 }
 
 int SplitContainer::get_split_offset() const {
-
 	return split_offset;
 }
 
@@ -308,33 +297,29 @@ void SplitContainer::clamp_split_offset() {
 }
 
 void SplitContainer::set_collapsed(bool p_collapsed) {
-
-	if (collapsed == p_collapsed)
+	if (collapsed == p_collapsed) {
 		return;
+	}
 
 	collapsed = p_collapsed;
 	queue_sort();
 }
 
 void SplitContainer::set_dragger_visibility(DraggerVisibility p_visibility) {
-
 	dragger_visibility = p_visibility;
 	queue_sort();
 	update();
 }
 
 SplitContainer::DraggerVisibility SplitContainer::get_dragger_visibility() const {
-
 	return dragger_visibility;
 }
 
 bool SplitContainer::is_collapsed() const {
-
 	return collapsed;
 }
 
 void SplitContainer::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_gui_input"), &SplitContainer::_gui_input);
 
 	ClassDB::bind_method(D_METHOD("set_split_offset", "offset"), &SplitContainer::set_split_offset);
@@ -359,7 +344,6 @@ void SplitContainer::_bind_methods() {
 }
 
 SplitContainer::SplitContainer(bool p_vertical) {
-
 	mouse_inside = false;
 	split_offset = 0;
 	should_clamp_split_offset = false;
