@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  font.cpp                                                             */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  font.cpp                                                              */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "font.h"
 
@@ -65,11 +65,11 @@ void Font::draw(RID p_canvas_item, const Point2 &p_pos, const String &p_text, co
 	int chars_drawn = 0;
 	bool with_outline = has_outline();
 	for (int i = 0; i < p_text.length(); i++) {
-
 		int width = get_char_size(p_text[i]).width;
 
-		if (p_clip_w >= 0 && (ofs.x + width) > p_clip_w)
+		if (p_clip_w >= 0 && (ofs.x + width) > p_clip_w) {
 			break; //clip
+		}
 
 		ofs.x += draw_char(p_canvas_item, p_pos + ofs, p_text[i], p_text[i + 1], with_outline ? p_outline_modulate : p_modulate, with_outline);
 		++chars_drawn;
@@ -84,12 +84,10 @@ void Font::draw(RID p_canvas_item, const Point2 &p_pos, const String &p_text, co
 }
 
 void Font::update_changes() {
-
 	emit_changed();
 }
 
 void Font::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("draw", "canvas_item", "position", "string", "modulate", "clip_w", "outline_modulate"), &Font::draw, DEFVAL(Color(1, 1, 1)), DEFVAL(-1), DEFVAL(Color(1, 1, 1)));
 	ClassDB::bind_method(D_METHOD("get_ascent"), &Font::get_ascent);
 	ClassDB::bind_method(D_METHOD("get_descent"), &Font::get_descent);
@@ -100,7 +98,19 @@ void Font::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("get_wordwrap_string_size", "string", "width"), &Font::get_wordwrap_string_size);
 	ClassDB::bind_method(D_METHOD("has_outline"), &Font::has_outline);
 	ClassDB::bind_method(D_METHOD("draw_char", "canvas_item", "position", "char", "next", "modulate", "outline"), &Font::draw_char, DEFVAL(-1), DEFVAL(Color(1, 1, 1)), DEFVAL(false));
+
+	ClassDB::bind_method(D_METHOD("get_char_texture", "char", "next", "outline"), &Font::get_char_texture, DEFVAL(0), DEFVAL(false));
+	ClassDB::bind_method(D_METHOD("get_char_texture_size", "char", "next", "outline"), &Font::get_char_texture_size, DEFVAL(0), DEFVAL(false));
+	ClassDB::bind_method(D_METHOD("get_char_tx_offset", "char", "next", "outline"), &Font::get_char_tx_offset, DEFVAL(0), DEFVAL(false));
+	ClassDB::bind_method(D_METHOD("get_char_tx_size", "char", "next", "outline"), &Font::get_char_tx_size, DEFVAL(0), DEFVAL(false));
+	ClassDB::bind_method(D_METHOD("get_char_tx_uv_rect", "char", "next", "outline"), &Font::get_char_tx_uv_rect, DEFVAL(0), DEFVAL(false));
+
 	ClassDB::bind_method(D_METHOD("update_changes"), &Font::update_changes);
+	ClassDB::bind_method(D_METHOD("get_char_contours", "char", "next"), &Font::get_char_contours, DEFVAL(0));
+
+	BIND_ENUM_CONSTANT(CONTOUR_CURVE_TAG_ON);
+	BIND_ENUM_CONSTANT(CONTOUR_CURVE_TAG_OFF_CONIC);
+	BIND_ENUM_CONSTANT(CONTOUR_CURVE_TAG_OFF_CUBIC);
 }
 
 Font::Font() {
@@ -109,30 +119,27 @@ Font::Font() {
 /////////////////////////////////////////////////////////////////
 
 void BitmapFont::_set_chars(const PoolVector<int> &p_chars) {
-
 	int len = p_chars.size();
 	//char 1 charsize 1 texture, 4 rect, 2 align, advance 1
 	ERR_FAIL_COND(len % 9);
-	if (!len)
+	if (!len) {
 		return; //none to do
+	}
 	int chars = len / 9;
 
 	PoolVector<int>::Read r = p_chars.read();
 	for (int i = 0; i < chars; i++) {
-
 		const int *data = &r[i * 9];
 		add_char(data[0], data[1], Rect2(data[2], data[3], data[4], data[5]), Size2(data[6], data[7]), data[8]);
 	}
 }
 
 PoolVector<int> BitmapFont::_get_chars() const {
-
 	PoolVector<int> chars;
 
-	const CharType *key = NULL;
+	const int32_t *key = nullptr;
 
 	while ((key = char_map.next(key))) {
-
 		const Character *c = char_map.getptr(*key);
 		ERR_FAIL_COND_V(!c, PoolVector<int>());
 		chars.push_back(*key);
@@ -151,26 +158,23 @@ PoolVector<int> BitmapFont::_get_chars() const {
 }
 
 void BitmapFont::_set_kernings(const PoolVector<int> &p_kernings) {
-
 	int len = p_kernings.size();
 	ERR_FAIL_COND(len % 3);
-	if (!len)
+	if (!len) {
 		return;
+	}
 	PoolVector<int>::Read r = p_kernings.read();
 
 	for (int i = 0; i < len / 3; i++) {
-
 		const int *data = &r[i * 3];
 		add_kerning_pair(data[0], data[1], data[2]);
 	}
 }
 
 PoolVector<int> BitmapFont::_get_kernings() const {
-
 	PoolVector<int> kernings;
 
 	for (Map<KerningPairKey, int>::Element *E = kerning_map.front(); E; E = E->next()) {
-
 		kernings.push_back(E->key().A);
 		kernings.push_back(E->key().B);
 		kernings.push_back(E->get());
@@ -180,7 +184,6 @@ PoolVector<int> BitmapFont::_get_kernings() const {
 }
 
 void BitmapFont::_set_textures(const Vector<Variant> &p_textures) {
-
 	textures.clear();
 	for (int i = 0; i < p_textures.size(); i++) {
 		Ref<Texture> tex = p_textures[i];
@@ -190,10 +193,10 @@ void BitmapFont::_set_textures(const Vector<Variant> &p_textures) {
 }
 
 Vector<Variant> BitmapFont::_get_textures() const {
-
 	Vector<Variant> rtextures;
-	for (int i = 0; i < textures.size(); i++)
+	for (int i = 0; i < textures.size(); i++) {
 		rtextures.push_back(textures[i].get_ref_ptr());
+	}
 	return rtextures;
 }
 
@@ -208,7 +211,6 @@ Error BitmapFont::create_from_fnt(const String &p_file) {
 	clear();
 
 	while (true) {
-
 		String line = f->get_line();
 
 		int delimiter = line.find(" ");
@@ -216,59 +218,62 @@ Error BitmapFont::create_from_fnt(const String &p_file) {
 		int pos = delimiter + 1;
 		Map<String, String> keys;
 
-		while (pos < line.size() && line[pos] == ' ')
+		while (pos < line.size() && line[pos] == ' ') {
 			pos++;
+		}
 
 		while (pos < line.size()) {
-
 			int eq = line.find("=", pos);
-			if (eq == -1)
+			if (eq == -1) {
 				break;
+			}
 			String key = line.substr(pos, eq - pos);
 			int end = -1;
 			String value;
 			if (line[eq + 1] == '"') {
 				end = line.find("\"", eq + 2);
-				if (end == -1)
+				if (end == -1) {
 					break;
+				}
 				value = line.substr(eq + 2, end - 1 - eq - 1);
 				pos = end + 1;
 			} else {
 				end = line.find(" ", eq + 1);
-				if (end == -1)
+				if (end == -1) {
 					end = line.size();
+				}
 
 				value = line.substr(eq + 1, end - eq);
 
 				pos = end;
 			}
 
-			while (pos < line.size() && line[pos] == ' ')
+			while (pos < line.size() && line[pos] == ' ') {
 				pos++;
+			}
 
 			keys[key] = value;
 		}
 
 		if (type == "info") {
-
-			if (keys.has("face"))
+			if (keys.has("face")) {
 				set_name(keys["face"]);
+			}
 			/*
 			if (keys.has("size"))
 				font->set_height(keys["size"].to_int());
 			*/
 
 		} else if (type == "common") {
-
-			if (keys.has("lineHeight"))
+			if (keys.has("lineHeight")) {
 				set_height(keys["lineHeight"].to_int());
-			if (keys.has("base"))
+			}
+			if (keys.has("base")) {
 				set_ascent(keys["base"].to_int());
+			}
 
 		} else if (type == "page") {
-
 			if (keys.has("file")) {
-
 				String base_dir = p_file.get_base_dir();
 				String file = base_dir.plus_file(keys["file"]);
 				Ref<Texture> tex = ResourceLoader::load(file);
@@ -279,55 +284,66 @@ Error BitmapFont::create_from_fnt(const String &p_file) {
 				}
 			}
 		} else if (type == "char") {
-
-			CharType idx = 0;
-			if (keys.has("id"))
+			int32_t idx = 0;
+			if (keys.has("id")) {
 				idx = keys["id"].to_int();
+			}
 
 			Rect2 rect;
 
-			if (keys.has("x"))
+			if (keys.has("x")) {
 				rect.position.x = keys["x"].to_int();
-			if (keys.has("y"))
+			}
+			if (keys.has("y")) {
 				rect.position.y = keys["y"].to_int();
-			if (keys.has("width"))
+			}
+			if (keys.has("width")) {
 				rect.size.width = keys["width"].to_int();
-			if (keys.has("height"))
+			}
+			if (keys.has("height")) {
 				rect.size.height = keys["height"].to_int();
+			}
 
 			Point2 ofs;
 
-			if (keys.has("xoffset"))
+			if (keys.has("xoffset")) {
 				ofs.x = keys["xoffset"].to_int();
-			if (keys.has("yoffset"))
+			}
+			if (keys.has("yoffset")) {
 				ofs.y = keys["yoffset"].to_int();
+			}
 
 			int texture = 0;
-			if (keys.has("page"))
+			if (keys.has("page")) {
 				texture = keys["page"].to_int();
+			}
 			int advance = -1;
-			if (keys.has("xadvance"))
+			if (keys.has("xadvance")) {
 				advance = keys["xadvance"].to_int();
+			}
 
 			add_char(idx, texture, rect, ofs, advance);
 
 		} else if (type == "kerning") {
-
-			CharType first = 0, second = 0;
+			int32_t first = 0, second = 0;
 			int k = 0;
 
-			if (keys.has("first"))
+			if (keys.has("first")) {
 				first = keys["first"].to_int();
-			if (keys.has("second"))
+			}
+			if (keys.has("second")) {
 				second = keys["second"].to_int();
-			if (keys.has("amount"))
+			}
+			if (keys.has("amount")) {
 				k = keys["amount"].to_int();
+			}
 
 			add_kerning_pair(first, second, -k);
 		}
 
-		if (f->eof_reached())
+		if (f->eof_reached()) {
 			break;
+		}
 	}
 
 	memdelete(f);
@@ -336,65 +352,53 @@ Error BitmapFont::create_from_fnt(const String &p_file) {
 }
 
 void BitmapFont::set_height(float p_height) {
-
 	height = p_height;
 }
 float BitmapFont::get_height() const {
-
 	return height;
 }
 
 void BitmapFont::set_ascent(float p_ascent) {
-
 	ascent = p_ascent;
 }
 float BitmapFont::get_ascent() const {
-
 	return ascent;
 }
 float BitmapFont::get_descent() const {
-
 	return height - ascent;
 }
 
 void BitmapFont::add_texture(const Ref<Texture> &p_texture) {
-
 	ERR_FAIL_COND_MSG(p_texture.is_null(), "It's not a reference to a valid Texture object.");
 	textures.push_back(p_texture);
 }
 
 int BitmapFont::get_texture_count() const {
-
 	return textures.size();
 };
 
 Ref<Texture> BitmapFont::get_texture(int p_idx) const {
-
 	ERR_FAIL_INDEX_V(p_idx, textures.size(), Ref<Texture>());
 	return textures[p_idx];
 };
 
 int BitmapFont::get_character_count() const {
-
 	return char_map.size();
 };
 
-Vector<CharType> BitmapFont::get_char_keys() const {
-
-	Vector<CharType> chars;
+Vector<int32_t> BitmapFont::get_char_keys() const {
+	Vector<int32_t> chars;
 	chars.resize(char_map.size());
-	const CharType *ct = NULL;
+	const int32_t *ct = nullptr;
 	int count = 0;
 	while ((ct = char_map.next(ct))) {
-
 		chars.write[count++] = *ct;
 	};
 
 	return chars;
 };
 
-BitmapFont::Character BitmapFont::get_character(CharType p_char) const {
-
+BitmapFont::Character BitmapFont::get_character(int32_t p_char) const {
 	if (!char_map.has(p_char)) {
 		ERR_FAIL_V(Character());
 	};
@@ -402,10 +406,10 @@ BitmapFont::Character BitmapFont::get_character(CharType p_char) const {
 	return char_map[p_char];
 };
 
-void BitmapFont::add_char(CharType p_char, int p_texture_idx, const Rect2 &p_rect, const Size2 &p_align, float p_advance) {
-
-	if (p_advance < 0)
+void BitmapFont::add_char(int32_t p_char, int p_texture_idx, const Rect2 &p_rect, const Size2 &p_align, float p_advance) {
+	if (p_advance < 0) {
 		p_advance = p_rect.size.width;
+	}
 
 	Character c;
 	c.rect = p_rect;
@@ -417,23 +421,19 @@ void BitmapFont::add_char(CharType p_char, int p_texture_idx, const Rect2 &p_rec
 	char_map[p_char] = c;
 }
 
-void BitmapFont::add_kerning_pair(CharType p_A, CharType p_B, int p_kerning) {
-
+void BitmapFont::add_kerning_pair(int32_t p_A, int32_t p_B, int p_kerning) {
 	KerningPairKey kpk;
 	kpk.A = p_A;
 	kpk.B = p_B;
 
 	if (p_kerning == 0 && kerning_map.has(kpk)) {
-
 		kerning_map.erase(kpk);
 	} else {
-
 		kerning_map[kpk] = p_kerning;
 	}
 }
 
 Vector<BitmapFont::KerningPairKey> BitmapFont::get_kerning_pair_keys() const {
-
 	Vector<BitmapFont::KerningPairKey> ret;
 	ret.resize(kerning_map.size());
 	int i = 0;
@@ -445,32 +445,29 @@ Vector<BitmapFont::KerningPairKey> BitmapFont::get_kerning_pair_keys() const {
 	return ret;
 }
 
-int BitmapFont::get_kerning_pair(CharType p_A, CharType p_B) const {
-
+int BitmapFont::get_kerning_pair(int32_t p_A, int32_t p_B) const {
 	KerningPairKey kpk;
 	kpk.A = p_A;
 	kpk.B = p_B;
 
 	const Map<KerningPairKey, int>::Element *E = kerning_map.find(kpk);
-	if (E)
+	if (E) {
 		return E->get();
+	}
 
 	return 0;
 }
 
 void BitmapFont::set_distance_field_hint(bool p_distance_field) {
-
 	distance_field_hint = p_distance_field;
 	emit_changed();
 }
 
 bool BitmapFont::is_distance_field_hint() const {
-
 	return distance_field_hint;
 }
 
 void BitmapFont::clear() {
-
 	height = 1;
 	ascent = 0;
 	char_map.clear();
@@ -480,16 +477,15 @@ void BitmapFont::clear() {
 }
 
 Size2 Font::get_string_size(const String &p_string) const {
-
 	float w = 0;
 
 	int l = p_string.length();
-	if (l == 0)
+	if (l == 0) {
 		return Size2(0, get_height());
+	}
 	const CharType *sptr = &p_string[0];
 
 	for (int i = 0; i < l; i++) {
-
 		w += get_char_size(sptr[i], sptr[i + 1]).width;
 	}
 
@@ -497,12 +493,12 @@ Size2 Font::get_string_size(const String &p_string) const {
 }
 
 Size2 Font::get_wordwrap_string_size(const String &p_string, float p_width) const {
-
 	ERR_FAIL_COND_V(p_width <= 0, Vector2(0, get_height()));
 
 	int l = p_string.length();
-	if (l == 0)
+	if (l == 0) {
 		return Size2(p_width, get_height());
+	}
 
 	float line_w = 0;
 	float h = 0;
@@ -528,8 +524,7 @@ Size2 Font::get_wordwrap_string_size(const String &p_string, float p_width) cons
 }
 
 void BitmapFont::set_fallback(const Ref<BitmapFont> &p_fallback) {
-
-	for (Ref<BitmapFont> fallback_child = p_fallback; fallback_child != NULL; fallback_child = fallback_child->get_fallback()) {
+	for (Ref<BitmapFont> fallback_child = p_fallback; fallback_child != nullptr; fallback_child = fallback_child->get_fallback()) {
 		ERR_FAIL_COND_MSG(fallback_child == this, "Can't set as fallback one of its parents to prevent crashes due to recursive loop.");
 	}
 
@@ -537,17 +532,158 @@ void BitmapFont::set_fallback(const Ref<BitmapFont> &p_fallback) {
 }
 
 Ref<BitmapFont> BitmapFont::get_fallback() const {
-
 	return fallback;
 }
 
-float BitmapFont::draw_char(RID p_canvas_item, const Point2 &p_pos, CharType p_char, CharType p_next, const Color &p_modulate, bool p_outline) const {
+RID BitmapFont::get_char_texture(CharType p_char, CharType p_next, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return RID();
+	}
 
-	const Character *c = char_map.getptr(p_char);
+	const Character *c = char_map.getptr(ch);
 
 	if (!c) {
-		if (fallback.is_valid())
+		if (fallback.is_valid()) {
+			return fallback->get_char_texture(p_char, p_next, p_outline);
+		}
+		return RID();
+	}
+
+	ERR_FAIL_COND_V(c->texture_idx < -1 || c->texture_idx >= textures.size(), RID());
+	if (!p_outline && c->texture_idx != -1) {
+		return textures[c->texture_idx]->get_rid();
+	} else {
+		return RID();
+	}
+}
+
+Size2 BitmapFont::get_char_texture_size(CharType p_char, CharType p_next, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return Size2();
+	}
+
+	const Character *c = char_map.getptr(ch);
+
+	if (!c) {
+		if (fallback.is_valid()) {
+			return fallback->get_char_texture_size(p_char, p_next, p_outline);
+		}
+		return Size2();
+	}
+
+	ERR_FAIL_COND_V(c->texture_idx < -1 || c->texture_idx >= textures.size(), Size2());
+	if (!p_outline && c->texture_idx != -1) {
+		return textures[c->texture_idx]->get_size();
+	} else {
+		return Size2();
+	}
+}
+
+Vector2 BitmapFont::get_char_tx_offset(CharType p_char, CharType p_next, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return Vector2();
+	}
+
+	const Character *c = char_map.getptr(ch);
+
+	if (!c) {
+		if (fallback.is_valid()) {
+			return fallback->get_char_tx_offset(p_char, p_next, p_outline);
+		}
+		return Vector2();
+	}
+
+	ERR_FAIL_COND_V(c->texture_idx < -1 || c->texture_idx >= textures.size(), Vector2());
+	if (!p_outline && c->texture_idx != -1) {
+		Point2 cpos;
+		cpos.x += c->h_align;
+		cpos.y -= ascent;
+		cpos.y += c->v_align;
+		return cpos;
+	} else {
+		return Vector2();
+	}
+}
+
+Size2 BitmapFont::get_char_tx_size(CharType p_char, CharType p_next, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return Size2();
+	}
+
+	const Character *c = char_map.getptr(ch);
+
+	if (!c) {
+		if (fallback.is_valid()) {
+			return fallback->get_char_tx_size(p_char, p_next, p_outline);
+		}
+		return Size2();
+	}
+
+	ERR_FAIL_COND_V(c->texture_idx < -1 || c->texture_idx >= textures.size(), Size2());
+	if (!p_outline && c->texture_idx != -1) {
+		return c->rect.size;
+	} else {
+		return Size2();
+	}
+}
+
+Rect2 BitmapFont::get_char_tx_uv_rect(CharType p_char, CharType p_next, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return Rect2();
+	}
+
+	const Character *c = char_map.getptr(ch);
+
+	if (!c) {
+		if (fallback.is_valid()) {
+			return fallback->get_char_tx_uv_rect(p_char, p_next, p_outline);
+		}
+		return Rect2();
+	}
+
+	ERR_FAIL_COND_V(c->texture_idx < -1 || c->texture_idx >= textures.size(), Rect2());
+	if (!p_outline && c->texture_idx != -1) {
+		return c->rect;
+	} else {
+		return Rect2();
+	}
+}
+
+float BitmapFont::draw_char(RID p_canvas_item, const Point2 &p_pos, CharType p_char, CharType p_next, const Color &p_modulate, bool p_outline) const {
+	int32_t ch = p_char;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return 0;
+	}
+
+	const Character *c = char_map.getptr(ch);
+
+	if (!c) {
+		if (fallback.is_valid()) {
 			return fallback->draw_char(p_canvas_item, p_pos, p_char, p_next, p_modulate, p_outline);
+		}
 		return 0;
 	}
 
@@ -564,27 +700,37 @@ float BitmapFont::draw_char(RID p_canvas_item, const Point2 &p_pos, CharType p_c
 }
 
 Size2 BitmapFont::get_char_size(CharType p_char, CharType p_next) const {
+	int32_t ch = p_char;
+	bool skip_kerning = false;
+	if (((p_char & 0xfffffc00) == 0xd800) && (p_next & 0xfffffc00) == 0xdc00) { // decode surrogate pair.
+		ch = (p_char << 10UL) + p_next - ((0xd800 << 10UL) + 0xdc00 - 0x10000);
+		skip_kerning = true;
+	}
+	if ((p_char & 0xfffffc00) == 0xdc00) { // skip trail surrogate.
+		return Size2();
+	}
 
-	const Character *c = char_map.getptr(p_char);
+	const Character *c = char_map.getptr(ch);
 
 	if (!c) {
-		if (fallback.is_valid())
+		if (fallback.is_valid()) {
 			return fallback->get_char_size(p_char, p_next);
+		}
 		return Size2();
 	}
 
 	Size2 ret(c->advance, c->rect.size.y);
 
-	if (p_next) {
+	if (!skip_kerning) {
+		if (p_next) {
+			KerningPairKey kpk;
+			kpk.A = p_char;
+			kpk.B = p_next;
 
-		KerningPairKey kpk;
-		kpk.A = p_char;
-		kpk.B = p_next;
-
-		const Map<KerningPairKey, int>::Element *E = kerning_map.find(kpk);
-		if (E) {
-
-			ret.width -= E->get();
+			const Map<KerningPairKey, int>::Element *E = kerning_map.find(kpk);
+			if (E) {
+				ret.width -= E->get();
+			}
 		}
 	}
 
@@ -592,7 +738,6 @@ Size2 BitmapFont::get_char_size(CharType p_char, CharType p_next) const {
 }
 
 void BitmapFont::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("create_from_fnt", "path"), &BitmapFont::create_from_fnt);
 	ClassDB::bind_method(D_METHOD("set_height", "px"), &BitmapFont::set_height);
 
@@ -634,21 +779,19 @@ void BitmapFont::_bind_methods() {
 }
 
 BitmapFont::BitmapFont() {
-
 	clear();
 }
 
 BitmapFont::~BitmapFont() {
-
 	clear();
 }
 
 ////////////
 
 RES ResourceFormatLoaderBMFont::load(const String &p_path, const String &p_original_path, Error *r_error) {
-
-	if (r_error)
+	if (r_error) {
 		*r_error = ERR_FILE_CANT_OPEN;
+	}
 
 	Ref<BitmapFont> font;
 	font.instance();
@@ -656,8 +799,9 @@ RES ResourceFormatLoaderBMFont::load(const String &p_path, const String &p_origi
 	Error err = font->create_from_fnt(p_path);
 
 	if (err) {
-		if (r_error)
+		if (r_error) {
 			*r_error = err;
+		}
 		return RES();
 	}
 
@@ -665,19 +809,17 @@ RES ResourceFormatLoaderBMFont::load(const String &p_path, const String &p_origi
 }
 
 void ResourceFormatLoaderBMFont::get_recognized_extensions(List<String> *p_extensions) const {
-
 	p_extensions->push_back("fnt");
 }
 
 bool ResourceFormatLoaderBMFont::handles_type(const String &p_type) const {
-
 	return (p_type == "BitmapFont");
 }
 
 String ResourceFormatLoaderBMFont::get_resource_type(const String &p_path) const {
-
 	String el = p_path.get_extension().to_lower();
-	if (el == "fnt")
+	if (el == "fnt") {
 		return "BitmapFont";
+	}
 	return "";
 }
