@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  resource_format_text.cpp                                             */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  resource_format_text.cpp                                              */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "resource_format_text.h"
 
@@ -46,17 +46,14 @@
 ///
 
 void ResourceInteractiveLoaderText::set_local_path(const String &p_local_path) {
-
 	res_path = p_local_path;
 }
 
 Ref<Resource> ResourceInteractiveLoaderText::get_resource() {
-
 	return resource;
 }
 
 Error ResourceInteractiveLoaderText::_parse_sub_resource_dummy(DummyReadData *p_data, VariantParser::Stream *p_stream, Ref<Resource> &r_res, int &line, String &r_err_str) {
-
 	VariantParser::Token token;
 	VariantParser::get_token(p_stream, token, line, r_err_str);
 	if (token.type != VariantParser::TK_NUMBER) {
@@ -86,7 +83,6 @@ Error ResourceInteractiveLoaderText::_parse_sub_resource_dummy(DummyReadData *p_
 }
 
 Error ResourceInteractiveLoaderText::_parse_ext_resource_dummy(DummyReadData *p_data, VariantParser::Stream *p_stream, Ref<Resource> &r_res, int &line, String &r_err_str) {
-
 	VariantParser::Token token;
 	VariantParser::get_token(p_stream, token, line, r_err_str);
 	if (token.type != VariantParser::TK_NUMBER) {
@@ -110,7 +106,6 @@ Error ResourceInteractiveLoaderText::_parse_ext_resource_dummy(DummyReadData *p_
 }
 
 Error ResourceInteractiveLoaderText::_parse_sub_resource(VariantParser::Stream *p_stream, Ref<Resource> &r_res, int &line, String &r_err_str) {
-
 	VariantParser::Token token;
 	VariantParser::get_token(p_stream, token, line, r_err_str);
 	if (token.type != VariantParser::TK_NUMBER) {
@@ -120,19 +115,8 @@ Error ResourceInteractiveLoaderText::_parse_sub_resource(VariantParser::Stream *
 
 	int index = token.value;
 
-	String path = local_path + "::" + itos(index);
-
-	if (!ignore_resource_parsing) {
-
-		if (!ResourceCache::has(path)) {
-			r_err_str = "Can't load cached sub-resource: " + path;
-			return ERR_PARSE_ERROR;
-		}
-
-		r_res = RES(ResourceCache::get(path));
-	} else {
-		r_res = RES();
-	}
+	ERR_FAIL_COND_V(!int_resources.has(index), ERR_INVALID_PARAMETER);
+	r_res = int_resources[index];
 
 	VariantParser::get_token(p_stream, token, line, r_err_str);
 	if (token.type != VariantParser::TK_PARENTHESIS_CLOSE) {
@@ -144,7 +128,6 @@ Error ResourceInteractiveLoaderText::_parse_sub_resource(VariantParser::Stream *
 }
 
 Error ResourceInteractiveLoaderText::_parse_ext_resource(VariantParser::Stream *p_stream, Ref<Resource> &r_res, int &line, String &r_err_str) {
-
 	VariantParser::Token token;
 	VariantParser::get_token(p_stream, token, line, r_err_str);
 	if (token.type != VariantParser::TK_NUMBER) {
@@ -155,7 +138,6 @@ Error ResourceInteractiveLoaderText::_parse_ext_resource(VariantParser::Stream *
 	int id = token.value;
 
 	if (!ignore_resource_parsing) {
-
 		if (!ext_resources.has(id)) {
 			r_err_str = "Can't load cached ext-resource #" + itos(id);
 			return ERR_PARSE_ERROR;
@@ -192,9 +174,7 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 	packed_scene.instance();
 
 	while (true) {
-
 		if (next_tag.name == "node") {
-
 			int parent = -1;
 			int owner = -1;
 			int type = -1;
@@ -220,7 +200,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 			}
 
 			if (next_tag.fields.has("instance")) {
-
 				instance = packed_scene->get_state()->add_value(next_tag.fields["instance"]);
 
 				if (packed_scene->get_state()->get_node_count() == 0 && parent == -1) {
@@ -230,7 +209,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 			}
 
 			if (next_tag.fields.has("instance_placeholder")) {
-
 				String path = next_tag.fields["instance_placeholder"];
 
 				int path_v = packed_scene->get_state()->add_value(path);
@@ -248,8 +226,9 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 			if (next_tag.fields.has("owner")) {
 				owner = packed_scene->get_state()->add_node_path(next_tag.fields["owner"]);
 			} else {
-				if (parent != -1 && !(type == SceneState::TYPE_INSTANCED && instance == -1))
+				if (parent != -1 && !(type == SceneState::TYPE_INSTANCED && instance == -1)) {
 					owner = 0; //if no owner, owner is root
+				}
 			}
 
 			if (next_tag.fields.has("index")) {
@@ -259,7 +238,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 			int node_id = packed_scene->get_state()->add_node(parent, owner, type, name, instance, index);
 
 			if (next_tag.fields.has("groups")) {
-
 				Array groups = next_tag.fields["groups"];
 				for (int i = 0; i < groups.size(); i++) {
 					packed_scene->get_state()->add_node_group(node_id, packed_scene->get_state()->add_name(groups[i]));
@@ -267,7 +245,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 			}
 
 			while (true) {
-
 				String assign;
 				Variant value;
 
@@ -292,7 +269,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 				}
 			}
 		} else if (next_tag.name == "connection") {
-
 			if (!next_tag.fields.has("from")) {
 				error = ERR_FILE_CORRUPT;
 				error_text = "missing 'from' field from connection tag";
@@ -356,7 +332,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 				}
 			}
 		} else if (next_tag.name == "editable") {
-
 			if (!next_tag.fields.has("path")) {
 				error = ERR_FILE_CORRUPT;
 				error_text = "missing 'path' field from connection tag";
@@ -379,7 +354,6 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 				}
 			}
 		} else {
-
 			error = ERR_FILE_CORRUPT;
 			_printerr();
 			return Ref<PackedScene>();
@@ -390,12 +364,11 @@ Ref<PackedScene> ResourceInteractiveLoaderText::_parse_node_tag(VariantParser::R
 }
 
 Error ResourceInteractiveLoaderText::poll() {
-
-	if (error != OK)
+	if (error != OK) {
 		return error;
+	}
 
 	if (next_tag.name == "ext_resource") {
-
 		if (!next_tag.fields.has("path")) {
 			error = ERR_FILE_CORRUPT;
 			error_text = "Missing 'path' in external resource tag";
@@ -433,7 +406,6 @@ Error ResourceInteractiveLoaderText::poll() {
 		RES res = ResourceLoader::load(path, type);
 
 		if (res.is_null()) {
-
 			if (ResourceLoader::get_abort_on_missing_resources()) {
 				error = ERR_FILE_CORRUPT;
 				error_text = "[ext_resource] referenced nonexistent resource at: " + path;
@@ -443,8 +415,6 @@ Error ResourceInteractiveLoaderText::poll() {
 				ResourceLoader::notify_dependency_error(local_path, path, type);
 			}
 		} else {
-
-			resource_cache.push_back(res);
 #ifdef TOOLS_ENABLED
 			//remember ID for saving
 			res->set_id_for_path(local_path, index);
@@ -452,6 +422,7 @@ Error ResourceInteractiveLoaderText::poll() {
 		}
 
 		ExtResource er;
+		er.cache = res;
 		er.path = path;
 		er.type = type;
 		ext_resources[index] = er;
@@ -466,7 +437,6 @@ Error ResourceInteractiveLoaderText::poll() {
 		return error;
 
 	} else if (next_tag.name == "sub_resource") {
-
 		if (!next_tag.fields.has("type")) {
 			error = ERR_FILE_CORRUPT;
 			error_text = "Missing 'type' in external resource tag";
@@ -486,15 +456,18 @@ Error ResourceInteractiveLoaderText::poll() {
 
 		String path = local_path + "::" + itos(id);
 
-		//bool exists=ResourceCache::has(path);
-
 		Ref<Resource> res;
 
-		if (!ResourceCache::has(path)) { //only if it doesn't exist
+		bool do_assign = false;
 
+		if (ResourceCache::has(path)) {
+			//cached, do not assign
+			Resource *r = ResourceCache::get(path);
+			res = Ref<Resource>(r);
+		} else {
+			//create
 			Object *obj = ClassDB::instance(type);
 			if (!obj) {
-
 				error_text += "Can't create sub resource of type: " + type;
 				_printerr();
 				error = ERR_FILE_CORRUPT;
@@ -503,7 +476,6 @@ Error ResourceInteractiveLoaderText::poll() {
 
 			Resource *r = Object::cast_to<Resource>(obj);
 			if (!r) {
-
 				error_text += "Can't create sub resource of type, because not a resource: " + type;
 				_printerr();
 				error = ERR_FILE_CORRUPT;
@@ -511,14 +483,18 @@ Error ResourceInteractiveLoaderText::poll() {
 			}
 
 			res = Ref<Resource>(r);
-			resource_cache.push_back(res);
+			do_assign = true;
+		}
+
+		int_resources[id] = res; //always assign int resources
+		if (do_assign) {
 			res->set_path(path);
+			res->set_subindex(id);
 		}
 
 		resource_current++;
 
 		while (true) {
-
 			String assign;
 			Variant value;
 
@@ -530,12 +506,11 @@ Error ResourceInteractiveLoaderText::poll() {
 			}
 
 			if (assign != String()) {
-				if (res.is_valid()) {
+				if (do_assign) {
 					res->set(assign, value);
 				}
 				//it's assignment
 			} else if (next_tag.name != String()) {
-
 				error = OK;
 				break;
 			} else {
@@ -549,9 +524,7 @@ Error ResourceInteractiveLoaderText::poll() {
 		return OK;
 
 	} else if (next_tag.name == "resource") {
-
 		if (is_scene) {
-
 			error_text += "found the 'resource' tag on a scene file!";
 			_printerr();
 			error = ERR_FILE_CORRUPT;
@@ -560,7 +533,6 @@ Error ResourceInteractiveLoaderText::poll() {
 
 		Object *obj = ClassDB::instance(res_type);
 		if (!obj) {
-
 			error_text += "Can't create sub resource of type: " + res_type;
 			_printerr();
 			error = ERR_FILE_CORRUPT;
@@ -569,7 +541,6 @@ Error ResourceInteractiveLoaderText::poll() {
 
 		Resource *r = Object::cast_to<Resource>(obj);
 		if (!r) {
-
 			error_text += "Can't create sub resource of type, because not a resource: " + res_type;
 			_printerr();
 			error = ERR_FILE_CORRUPT;
@@ -581,7 +552,6 @@ Error ResourceInteractiveLoaderText::poll() {
 		resource_current++;
 
 		while (true) {
-
 			String assign;
 			Variant value;
 
@@ -603,7 +573,6 @@ Error ResourceInteractiveLoaderText::poll() {
 				resource->set(assign, value);
 				//it's assignment
 			} else if (next_tag.name != String()) {
-
 				error = ERR_FILE_CORRUPT;
 				error_text = "Extra tag found when parsing main resource file";
 				_printerr();
@@ -617,9 +586,7 @@ Error ResourceInteractiveLoaderText::poll() {
 		return OK;
 
 	} else if (next_tag.name == "node") {
-
 		if (!is_scene) {
-
 			error_text += "found the 'node' tag on a resource file!";
 			_printerr();
 			error = ERR_FILE_CORRUPT;
@@ -628,8 +595,9 @@ Error ResourceInteractiveLoaderText::poll() {
 
 		Ref<PackedScene> packed_scene = _parse_node_tag(rp);
 
-		if (!packed_scene.is_valid())
+		if (!packed_scene.is_valid()) {
 			return error;
+		}
 
 		error = ERR_FILE_EOF;
 		//get it here
@@ -651,16 +619,13 @@ Error ResourceInteractiveLoaderText::poll() {
 }
 
 int ResourceInteractiveLoaderText::get_stage() const {
-
 	return resource_current;
 }
 int ResourceInteractiveLoaderText::get_stage_count() const {
-
 	return resources_total; //+ext_resources;
 }
 
 void ResourceInteractiveLoaderText::set_translation_remapped(bool p_remapped) {
-
 	translation_remapped = p_remapped;
 }
 
@@ -669,18 +634,15 @@ ResourceInteractiveLoaderText::ResourceInteractiveLoaderText() {
 }
 
 ResourceInteractiveLoaderText::~ResourceInteractiveLoaderText() {
-
 	memdelete(f);
 }
 
 void ResourceInteractiveLoaderText::get_dependencies(FileAccess *p_f, List<String> *p_dependencies, bool p_add_types) {
-
 	open(p_f);
 	ignore_resource_parsing = true;
 	ERR_FAIL_COND(error != OK);
 
 	while (next_tag.name == "ext_resource") {
-
 		if (!next_tag.fields.has("type")) {
 			error = ERR_FILE_CORRUPT;
 			error_text = "Missing 'type' in external resource tag";
@@ -721,20 +683,18 @@ void ResourceInteractiveLoaderText::get_dependencies(FileAccess *p_f, List<Strin
 }
 
 Error ResourceInteractiveLoaderText::rename_dependencies(FileAccess *p_f, const String &p_path, const Map<String, String> &p_map) {
-
 	open(p_f, true);
 	ERR_FAIL_COND_V(error != OK, error);
 	ignore_resource_parsing = true;
 	//FileAccess
 
-	FileAccess *fw = NULL;
+	FileAccess *fw = nullptr;
 
 	String base_path = local_path.get_base_dir();
 
 	uint64_t tag_end = f->get_position();
 
 	while (true) {
-
 		Error err = VariantParser::parse_tag(&stream, lines, error_text, next_tag, &rp);
 
 		if (err != OK) {
@@ -746,17 +706,15 @@ Error ResourceInteractiveLoaderText::rename_dependencies(FileAccess *p_f, const
 		}
 
 		if (next_tag.name != "ext_resource") {
-
 			//nothing was done
-			if (!fw)
+			if (!fw) {
 				return OK;
+			}
 
 			break;
 
 		} else {
-
 			if (!fw) {
-
 				fw = FileAccess::open(p_path + ".depren", FileAccess::WRITE);
 				if (is_scene) {
 					fw->store_line("[gd_scene load_steps=" + itos(resources_total) + " format=" + itos(FORMAT_VERSION) + "]\n");
@@ -800,6 +758,11 @@ Error ResourceInteractiveLoaderText::rename_dependencies(FileAccess *p_f, const
 	f->seek(tag_end);
 
 	uint8_t c = f->get_8();
+	if (c == '\n' && !f->eof_reached()) {
+		// Skip first newline character since we added one
+		c = f->get_8();
+	}
+
 	while (!f->eof_reached()) {
 		fw->store_8(c);
 		c = f->get_8();
@@ -823,7 +786,6 @@ Error ResourceInteractiveLoaderText::rename_dependencies(FileAccess *p_f, const
 }
 
 void ResourceInteractiveLoaderText::open(FileAccess *p_f, bool p_skip_first_tag) {
-
 	error = OK;
 
 	lines = 1;
@@ -838,7 +800,6 @@ void ResourceInteractiveLoaderText::open(FileAccess *p_f, bool p_skip_first_tag)
 	Error err = VariantParser::parse_tag(&stream, lines, error_text, tag);
 
 	if (err) {
-
 		error = err;
 		_printerr();
 		return;
@@ -881,7 +842,6 @@ void ResourceInteractiveLoaderText::open(FileAccess *p_f, bool p_skip_first_tag)
 	}
 
 	if (!p_skip_first_tag) {
-
 		err = VariantParser::parse_tag(&stream, lines, error_text, next_tag, &rp);
 
 		if (err) {
@@ -893,12 +853,10 @@ void ResourceInteractiveLoaderText::open(FileAccess *p_f, bool p_skip_first_tag)
 
 	rp.ext_func = _parse_ext_resources;
 	rp.sub_func = _parse_sub_resources;
-	rp.func = NULL;
 	rp.userdata = this;
 }
 
 static void bs_save_unicode_string(FileAccess *f, const String &p_string, bool p_bit_on_len = false) {
-
 	CharString utf8 = p_string.utf8();
 	if (p_bit_on_len) {
 		f->store_32((utf8.length() + 1) | 0x80000000);
@@ -909,9 +867,9 @@ static void bs_save_unicode_string(FileAccess *f, const String &p_string, bool p
 }
 
 Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const String &p_path) {
-
-	if (error)
+	if (error) {
 		return error;
+	}
 
 	FileAccessRef wf = FileAccess::open(p_path, FileAccess::WRITE);
 	if (!wf) {
@@ -931,11 +889,12 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 
 	bs_save_unicode_string(wf.f, is_scene ? "PackedScene" : resource_type);
 	wf->store_64(0); //offset to import metadata, this is no longer used
-	for (int i = 0; i < 14; i++)
+	for (int i = 0; i < 14; i++) {
 		wf->store_32(0); // reserved
+	}
 
 	wf->store_32(0); //string table size, will not be in use
-	size_t ext_res_count_pos = wf->get_position();
+	uint64_t ext_res_count_pos = wf->get_position();
 
 	wf->store_32(0); //zero ext resources, still parsing them
 
@@ -948,7 +907,6 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 	rp.userdata = &dummy_read;
 
 	while (next_tag.name == "ext_resource") {
-
 		if (!next_tag.fields.has("path")) {
 			error = ERR_FILE_CORRUPT;
 			error_text = "Missing 'path' in external resource tag";
@@ -999,7 +957,7 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 
 	//now, save resources to a separate file, for now
 
-	size_t sub_res_count_pos = wf->get_position();
+	uint64_t sub_res_count_pos = wf->get_position();
 	wf->store_32(0); //zero sub resources, still parsing them
 
 	String temp_file = p_path + ".temp";
@@ -1008,11 +966,10 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 		return ERR_CANT_OPEN;
 	}
 
-	Vector<size_t> local_offsets;
-	Vector<size_t> local_pointers_pos;
+	Vector<uint64_t> local_offsets;
+	Vector<uint64_t> local_pointers_pos;
 
 	while (next_tag.name == "sub_resource" || next_tag.name == "resource") {
-
 		String type;
 		int id = -1;
 		bool main_res;
@@ -1048,13 +1005,12 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 		wf->store_64(0); //temp local offset
 
 		bs_save_unicode_string(wf2, type);
-		size_t propcount_ofs = wf2->get_position();
+		uint64_t propcount_ofs = wf2->get_position();
 		wf2->store_32(0);
 
 		int prop_count = 0;
 
 		while (true) {
-
 			String assign;
 			Variant value;
 
@@ -1071,14 +1027,12 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 			}
 
 			if (assign != String()) {
-
 				Map<StringName, int> empty_string_map; //unused
 				bs_save_unicode_string(wf2, assign, true);
 				ResourceFormatSaverBinaryInstance::write_variant(wf2, value, dummy_read.resource_set, dummy_read.external_resources, empty_string_map);
 				prop_count++;
 
 			} else if (next_tag.name != String()) {
-
 				error = OK;
 				break;
 			} else {
@@ -1098,7 +1052,6 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 		//this is a node, must save one more!
 
 		if (!is_scene) {
-
 			error_text += "found the 'node' tag on a resource file!";
 			_printerr();
 			error = ERR_FILE_CORRUPT;
@@ -1107,8 +1060,9 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 
 		Ref<PackedScene> packed_scene = _parse_node_tag(rp);
 
-		if (!packed_scene.is_valid())
+		if (!packed_scene.is_valid()) {
 			return error;
+		}
 
 		error = OK;
 		//get it here
@@ -1121,15 +1075,15 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 
 		local_offsets.push_back(wf2->get_position());
 		bs_save_unicode_string(wf2, "PackedScene");
-		size_t propcount_ofs = wf2->get_position();
+		uint64_t propcount_ofs = wf2->get_position();
 		wf2->store_32(0);
 
 		int prop_count = 0;
 
 		for (List<PropertyInfo>::Element *E = props.front(); E; E = E->next()) {
-
-			if (!(E->get().usage & PROPERTY_USAGE_STORAGE))
+			if (!(E->get().usage & PROPERTY_USAGE_STORAGE)) {
 				continue;
+			}
 
 			String name = E->get().name;
 			Variant value = packed_scene->get(name);
@@ -1147,7 +1101,7 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 
 	wf2->close();
 
-	size_t offset_from = wf->get_position();
+	uint64_t offset_from = wf->get_position();
 	wf->seek(sub_res_count_pos); //plus one because the saved one
 	wf->store_32(local_offsets.size());
 
@@ -1173,7 +1127,6 @@ Error ResourceInteractiveLoaderText::save_as_binary(FileAccess *p_f, const Strin
 }
 
 String ResourceInteractiveLoaderText::recognize(FileAccess *p_f) {
-
 	error = OK;
 
 	lines = 1;
@@ -1200,11 +1153,13 @@ String ResourceInteractiveLoaderText::recognize(FileAccess *p_f) {
 		}
 	}
 
-	if (tag.name == "gd_scene")
+	if (tag.name == "gd_scene") {
 		return "PackedScene";
+	}
 
-	if (tag.name != "gd_resource")
+	if (tag.name != "gd_resource") {
 		return "";
+	}
 
 	if (!tag.fields.has("type")) {
 		error_text = "Missing 'type' field in 'gd_resource' tag";
@@ -1218,9 +1173,9 @@ String ResourceInteractiveLoaderText::recognize(FileAccess *p_f) {
 /////////////////////
 
 Ref<ResourceInteractiveLoader> ResourceFormatLoaderText::load_interactive(const String &p_path, const String &p_original_path, Error *r_error) {
-
-	if (r_error)
+	if (r_error) {
 		*r_error = ERR_CANT_OPEN;
+	}
 
 	Error err;
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ, &err);
@@ -1238,41 +1193,41 @@ Ref<ResourceInteractiveLoader> ResourceFormatLoaderText::load_interactive(const
 }
 
 void ResourceFormatLoaderText::get_recognized_extensions_for_type(const String &p_type, List<String> *p_extensions) const {
-
 	if (p_type == "") {
 		get_recognized_extensions(p_extensions);
 		return;
 	}
 
-	if (p_type == "PackedScene")
+	if (ClassDB::is_parent_class("PackedScene", p_type)) {
 		p_extensions->push_back("tscn");
-	else
+	}
+
+	// Don't allow .tres for PackedScenes.
+	if (p_type != "PackedScene") {
 		p_extensions->push_back("tres");
+	}
 }
 
 void ResourceFormatLoaderText::get_recognized_extensions(List<String> *p_extensions) const {
-
 	p_extensions->push_back("tscn");
 	p_extensions->push_back("tres");
 }
 
 bool ResourceFormatLoaderText::handles_type(const String &p_type) const {
-
 	return true;
 }
 String ResourceFormatLoaderText::get_resource_type(const String &p_path) const {
-
 	String ext = p_path.get_extension().to_lower();
-	if (ext == "tscn")
+	if (ext == "tscn") {
 		return "PackedScene";
-	else if (ext != "tres")
+	} else if (ext != "tres") {
 		return String();
+	}
 
-	//for anyhting else must test..
+	//for anything else must test..
 
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ);
 	if (!f) {
-
 		return ""; //could not rwead
 	}
 
@@ -1285,10 +1240,8 @@ String ResourceFormatLoaderText::get_resource_type(const String &p_path) const {
 }
 
 void ResourceFormatLoaderText::get_dependencies(const String &p_path, List<String> *p_dependencies, bool p_add_types) {
-
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ);
 	if (!f) {
-
 		ERR_FAIL();
 	}
 
@@ -1300,10 +1253,8 @@ void ResourceFormatLoaderText::get_dependencies(const String &p_path, List<Strin
 }
 
 Error ResourceFormatLoaderText::rename_dependencies(const String &p_path, const Map<String, String> &p_map) {
-
 	FileAccess *f = FileAccess::open(p_path, FileAccess::READ);
 	if (!f) {
-
 		ERR_FAIL_V(ERR_CANT_OPEN);
 	}
 
@@ -1314,10 +1265,9 @@ Error ResourceFormatLoaderText::rename_dependencies(const String &p_path, const
 	return ria->rename_dependencies(f, p_path, p_map);
 }
 
-ResourceFormatLoaderText *ResourceFormatLoaderText::singleton = NULL;
+ResourceFormatLoaderText *ResourceFormatLoaderText::singleton = nullptr;
 
 Error ResourceFormatLoaderText::convert_file_to_binary(const String &p_src_path, const String &p_dst_path) {
-
 	Error err;
 	FileAccess *f = FileAccess::open(p_src_path, FileAccess::READ, &err);
 
@@ -1344,18 +1294,14 @@ Error ResourceFormatLoaderText::convert_file_to_binary(const String &p_src_path,
 /*****************************************************************************************************/
 
 String ResourceFormatSaverTextInstance::_write_resources(void *ud, const RES &p_resource) {
-
 	ResourceFormatSaverTextInstance *rsi = (ResourceFormatSaverTextInstance *)ud;
 	return rsi->_write_resource(p_resource);
 }
 
 String ResourceFormatSaverTextInstance::_write_resource(const RES &res) {
-
 	if (external_resources.has(res)) {
-
 		return "ExtResource( " + itos(external_resources[res]) + " )";
 	} else {
-
 		if (internal_resources.has(res)) {
 			return "SubResource( " + itos(internal_resources[res]) + " )";
 		} else if (res->get_path().length() && res->get_path().find("::") == -1) {
@@ -1373,18 +1319,17 @@ String ResourceFormatSaverTextInstance::_write_resource(const RES &res) {
 }
 
 void ResourceFormatSaverTextInstance::_find_resources(const Variant &p_variant, bool p_main) {
-
 	switch (p_variant.get_type()) {
 		case Variant::OBJECT: {
-
 			RES res = p_variant.operator RefPtr();
 
-			if (res.is_null() || external_resources.has(res))
+			if (res.is_null() || external_resources.has(res)) {
 				return;
+			}
 
 			if (!p_main && (!bundle_resources) && res->get_path().length() && res->get_path().find("::") == -1) {
 				if (res->get_path() == local_path) {
-					ERR_PRINTS("Circular reference to resource being saved found: '" + local_path + "' will be null next time it's loaded.");
+					ERR_PRINT("Circular reference to resource being saved found: '" + local_path + "' will be null next time it's loaded.");
 					return;
 				}
 				int index = external_resources.size();
@@ -1392,8 +1337,9 @@ void ResourceFormatSaverTextInstance::_find_resources(const Variant &p_variant,
 				return;
 			}
 
-			if (resource_set.has(res))
+			if (resource_set.has(res)) {
 				return;
+			}
 
 			List<PropertyInfo> property_list;
 
@@ -1403,11 +1349,9 @@ void ResourceFormatSaverTextInstance::_find_resources(const Variant &p_variant,
 			List<PropertyInfo>::Element *I = property_list.front();
 
 			while (I) {
-
 				PropertyInfo pi = I->get();
 
 				if (pi.usage & PROPERTY_USAGE_STORAGE) {
-
 					Variant v = res->get(I->get().name);
 
 					if (pi.usage & PROPERTY_USAGE_RESOURCE_NOT_PERSISTENT) {
@@ -1433,23 +1377,19 @@ void ResourceFormatSaverTextInstance::_find_resources(const Variant &p_variant,
 
 		} break;
 		case Variant::ARRAY: {
-
 			Array varray = p_variant;
 			int len = varray.size();
 			for (int i = 0; i < len; i++) {
-
 				const Variant &v = varray.get(i);
 				_find_resources(v);
 			}
 
 		} break;
 		case Variant::DICTIONARY: {
-
 			Dictionary d = p_variant;
 			List<Variant> keys;
 			d.get_key_list(&keys);
 			for (List<Variant>::Element *E = keys.front(); E; E = E->next()) {
-
 				Variant v = d[E->get()];
 				_find_resources(v);
 			}
@@ -1460,7 +1400,6 @@ void ResourceFormatSaverTextInstance::_find_resources(const Variant &p_variant,
 }
 
 Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_resource, uint32_t p_flags) {
-
 	if (p_path.ends_with(".tscn")) {
 		packed_scene = p_resource;
 	}
@@ -1486,8 +1425,9 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 	if (packed_scene.is_valid()) {
 		//add instances to external resources if saving a packed scene
 		for (int i = 0; i < packed_scene->get_state()->get_node_count(); i++) {
-			if (packed_scene->get_state()->is_node_instance_placeholder(i))
+			if (packed_scene->get_state()->is_node_instance_placeholder(i)) {
 				continue;
+			}
 
 			Ref<PackedScene> instance = packed_scene->get_state()->get_node_instance(i);
 			if (instance.is_valid() && !external_resources.has(instance)) {
@@ -1499,8 +1439,9 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 
 	{
 		String title = packed_scene.is_valid() ? "[gd_scene " : "[gd_resource ";
-		if (packed_scene.is_null())
+		if (packed_scene.is_null()) {
 			title += "type=\"" + p_resource->get_class() + "\" ";
+		}
 		int load_steps = saved_resources.size() + external_resources.size();
 		/*
 		if (packed_scene.is_valid()) {
@@ -1557,7 +1498,6 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 	Vector<ResourceSort> sorted_er;
 
 	for (Map<RES, int>::Element *E = external_resources.front(); E; E = E->next()) {
-
 		ResourceSort rs;
 		rs.resource = E->key();
 		rs.index = E->get();
@@ -1572,16 +1512,15 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 		f->store_string("[ext_resource path=\"" + p + "\" type=\"" + sorted_er[i].resource->get_save_class() + "\" id=" + itos(sorted_er[i].index) + "]\n"); //bundled
 	}
 
-	if (external_resources.size())
+	if (external_resources.size()) {
 		f->store_line(String()); //separate
+	}
 
 	Set<int> used_indices;
 
 	for (List<RES>::Element *E = saved_resources.front(); E; E = E->next()) {
-
 		RES res = E->get();
 		if (E->next() && (res->get_path() == "" || res->get_path().find("::") != -1)) {
-
 			if (res->get_subindex() != 0) {
 				if (used_indices.has(res->get_subindex())) {
 					res->set_subindex(0); //repeated
@@ -1593,13 +1532,13 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 	}
 
 	for (List<RES>::Element *E = saved_resources.front(); E; E = E->next()) {
-
 		RES res = E->get();
 		ERR_CONTINUE(!resource_set.has(res));
-		bool main = (E->next() == NULL);
+		bool main = (E->next() == nullptr);
 
-		if (main && packed_scene.is_valid())
+		if (main && packed_scene.is_valid()) {
 			break; //save as a scene
+		}
 
 		if (main) {
 			f->store_line("[resource]");
@@ -1632,12 +1571,11 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 		res->get_property_list(&property_list);
 		//property_list.sort();
 		for (List<PropertyInfo>::Element *PE = property_list.front(); PE; PE = PE->next()) {
-
-			if (skip_editor && PE->get().name.begins_with("__editor"))
+			if (skip_editor && PE->get().name.begins_with("__editor")) {
 				continue;
+			}
 
 			if (PE->get().usage & PROPERTY_USAGE_STORAGE) {
-
 				String name = PE->get().name;
 				Variant value;
 				if (PE->get().usage & PROPERTY_USAGE_RESOURCE_NOT_PERSISTENT) {
@@ -1656,8 +1594,9 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 					continue;
 				}
 
-				if (PE->get().type == Variant::OBJECT && value.is_zero() && !(PE->get().usage & PROPERTY_USAGE_STORE_IF_NULL))
+				if (PE->get().type == Variant::OBJECT && value.is_zero() && !(PE->get().usage & PROPERTY_USAGE_STORE_IF_NULL)) {
 					continue;
+				}
 
 				String vars;
 				VariantWriter::write_to_string(value, vars, _write_resources, this);
@@ -1665,15 +1604,15 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 			}
 		}
 
-		if (E->next())
+		if (E->next()) {
 			f->store_line(String());
+		}
 	}
 
 	if (packed_scene.is_valid()) {
 		//if this is a scene, save nodes and connections!
 		Ref<SceneState> state = packed_scene->get_state();
 		for (int i = 0; i < state->get_node_count(); i++) {
-
 			StringName type = state->get_node_type(i);
 			StringName name = state->get_node_name(i);
 			int index = state->get_node_index(i);
@@ -1699,10 +1638,16 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 			}
 
 			if (groups.size()) {
+				// Write all groups on the same line as they're part of a section header.
+				// This improves readability while not impacting VCS friendliness too much,
+				// since it's rare to have more than 5 groups assigned to a single node.
 				groups.sort_custom<StringName::AlphCompare>();
-				String sgroups = " groups=[\n";
+				String sgroups = " groups=[";
 				for (int j = 0; j < groups.size(); j++) {
-					sgroups += "\"" + String(groups[j]).c_escape() + "\",\n";
+					sgroups += "\"" + String(groups[j]).c_escape() + "\"";
+					if (j < groups.size() - 1) {
+						sgroups += ", ";
+					}
 				}
 				sgroups += "]";
 				header += sgroups;
@@ -1711,7 +1656,6 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 			f->store_string(header);
 
 			if (instance_placeholder != String()) {
-
 				String vars;
 				f->store_string(" instance_placeholder=");
 				VariantWriter::write_to_string(instance_placeholder, vars, _write_resources, this);
@@ -1719,7 +1663,6 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 			}
 
 			if (instance.is_valid()) {
-
 				String vars;
 				f->store_string(" instance=");
 				VariantWriter::write_to_string(instance, vars, _write_resources, this);
@@ -1729,18 +1672,21 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 			f->store_line("]");
 
 			for (int j = 0; j < state->get_node_property_count(i); j++) {
-
 				String vars;
 				VariantWriter::write_to_string(state->get_node_property_value(i, j), vars, _write_resources, this);
 
 				f->store_string(String(state->get_node_property_name(i, j)).property_name_encode() + " = " + vars + "\n");
 			}
 
-			if (i < state->get_node_count() - 1)
+			if (i < state->get_node_count() - 1) {
 				f->store_line(String());
+			}
 		}
 
 		for (int i = 0; i < state->get_connection_count(); i++) {
+			if (i == 0) {
+				f->store_line("");
+			}
 
 			String connstr = "[connection";
 			connstr += " signal=\"" + String(state->get_connection_signal(i)) + "\"";
@@ -1765,7 +1711,10 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 
 		Vector<NodePath> editable_instances = state->get_editable_instances();
 		for (int i = 0; i < editable_instances.size(); i++) {
-			f->store_line("\n[editable path=\"" + editable_instances[i].operator String() + "\"]");
+			if (i == 0) {
+				f->store_line("");
+			}
+			f->store_line("[editable path=\"" + editable_instances[i].operator String() + "\"]");
 		}
 	}
 
@@ -1781,7 +1730,6 @@ Error ResourceFormatSaverTextInstance::save(const String &p_path, const RES &p_r
 }
 
 Error ResourceFormatSaverText::save(const String &p_path, const RES &p_resource, uint32_t p_flags) {
-
 	if (p_path.ends_with(".sct") && p_resource->get_class() != "PackedScene") {
 		return ERR_FILE_UNRECOGNIZED;
 	}
@@ -1791,18 +1739,17 @@ Error ResourceFormatSaverText::save(const String &p_path, const RES &p_resource,
 }
 
 bool ResourceFormatSaverText::recognize(const RES &p_resource) const {
-
 	return true; // all recognized!
 }
 void ResourceFormatSaverText::get_recognized_extensions(const RES &p_resource, List<String> *p_extensions) const {
-
-	if (p_resource->get_class() == "PackedScene")
+	if (p_resource->get_class() == "PackedScene") {
 		p_extensions->push_back("tscn"); //text scene
-	else
+	} else {
 		p_extensions->push_back("tres"); //text resource
+	}
 }
 
-ResourceFormatSaverText *ResourceFormatSaverText::singleton = NULL;
+ResourceFormatSaverText *ResourceFormatSaverText::singleton = nullptr;
 ResourceFormatSaverText::ResourceFormatSaverText() {
 	singleton = this;
 }
