@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  shader.cpp                                                           */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  shader.cpp                                                            */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "shader.h"
 #include "core/os/file_access.h"
@@ -36,12 +36,10 @@
 #include "texture.h"
 
 Shader::Mode Shader::get_mode() const {
-
 	return mode;
 }
 
 void Shader::set_code(const String &p_code) {
-
 	String type = ShaderLanguage::get_shader_type(p_code);
 
 	if (type == "canvas_item") {
@@ -59,13 +57,11 @@ void Shader::set_code(const String &p_code) {
 }
 
 String Shader::get_code() const {
-
 	_update_shader();
 	return VisualServer::get_singleton()->shader_get_code(shader);
 }
 
 void Shader::get_param_list(List<PropertyInfo> *p_params) const {
-
 	_update_shader();
 
 	List<PropertyInfo> local;
@@ -74,7 +70,6 @@ void Shader::get_param_list(List<PropertyInfo> *p_params) const {
 	params_cache_dirty = false;
 
 	for (List<PropertyInfo>::Element *E = local.front(); E; E = E->next()) {
-
 		PropertyInfo pi = E->get();
 		if (default_textures.has(pi.name)) { //do not show default textures
 			continue;
@@ -82,24 +77,22 @@ void Shader::get_param_list(List<PropertyInfo> *p_params) const {
 		pi.name = "shader_param/" + pi.name;
 		params_cache[pi.name] = E->get().name;
 		if (p_params) {
-
 			//small little hack
-			if (pi.type == Variant::_RID)
+			if (pi.type == Variant::_RID) {
 				pi.type = Variant::OBJECT;
+			}
 			p_params->push_back(pi);
 		}
 	}
 }
 
 RID Shader::get_rid() const {
-
 	_update_shader();
 
 	return shader;
 }
 
 void Shader::set_default_texture_param(const StringName &p_param, const Ref<Texture> &p_texture) {
-
 	if (p_texture.is_valid()) {
 		default_textures[p_param] = p_texture;
 		VS::get_singleton()->shader_set_default_texture_param(shader, p_param, p_texture->get_rid());
@@ -112,17 +105,15 @@ void Shader::set_default_texture_param(const StringName &p_param, const Ref<Text
 }
 
 Ref<Texture> Shader::get_default_texture_param(const StringName &p_param) const {
-
-	if (default_textures.has(p_param))
+	if (default_textures.has(p_param)) {
 		return default_textures[p_param];
-	else
+	} else {
 		return Ref<Texture>();
+	}
 }
 
 void Shader::get_default_texture_param_list(List<StringName> *r_textures) const {
-
-	for (const Map<StringName, Ref<Texture> >::Element *E = default_textures.front(); E; E = E->next()) {
-
+	for (const Map<StringName, Ref<Texture>>::Element *E = default_textures.front(); E; E = E->next()) {
 		r_textures->push_back(E->key());
 	}
 }
@@ -149,7 +140,6 @@ bool Shader::is_text_shader() const {
 }
 
 bool Shader::has_param(const StringName &p_param) const {
-
 	return params_cache.has("shader_param/" + p_param);
 }
 
@@ -157,7 +147,6 @@ void Shader::_update_shader() const {
 }
 
 void Shader::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("get_mode"), &Shader::get_mode);
 
 	ClassDB::bind_method(D_METHOD("set_code", "code"), &Shader::set_code);
@@ -182,22 +171,20 @@ void Shader::_bind_methods() {
 }
 
 Shader::Shader() {
-
 	mode = MODE_SPATIAL;
 	shader = VisualServer::get_singleton()->shader_create();
 	params_cache_dirty = true;
 }
 
 Shader::~Shader() {
-
 	VisualServer::get_singleton()->free(shader);
 }
 ////////////
 
 RES ResourceFormatLoaderShader::load(const String &p_path, const String &p_original_path, Error *r_error) {
-
-	if (r_error)
+	if (r_error) {
 		*r_error = ERR_FILE_CANT_OPEN;
+	}
 
 	Ref<Shader> shader;
 	shader.instance();
@@ -209,32 +196,31 @@ RES ResourceFormatLoaderShader::load(const String &p_path, const String &p_origi
 
 	shader->set_code(str);
 
-	if (r_error)
+	if (r_error) {
 		*r_error = OK;
+	}
 
 	return shader;
 }
 
 void ResourceFormatLoaderShader::get_recognized_extensions(List<String> *p_extensions) const {
-
+	p_extensions->push_back("gdshader");
 	p_extensions->push_back("shader");
 }
 
 bool ResourceFormatLoaderShader::handles_type(const String &p_type) const {
-
 	return (p_type == "Shader");
 }
 
 String ResourceFormatLoaderShader::get_resource_type(const String &p_path) const {
-
 	String el = p_path.get_extension().to_lower();
-	if (el == "shader")
+	if (el == "gdshader" || el == "shader") {
 		return "Shader";
+	}
 	return "";
 }
 
 Error ResourceFormatSaverShader::save(const String &p_path, const RES &p_resource, uint32_t p_flags) {
-
 	Ref<Shader> shader = p_resource;
 	ERR_FAIL_COND_V(shader.is_null(), ERR_INVALID_PARAMETER);
 
@@ -257,14 +243,14 @@ Error ResourceFormatSaverShader::save(const String &p_path, const RES &p_resourc
 }
 
 void ResourceFormatSaverShader::get_recognized_extensions(const RES &p_resource, List<String> *p_extensions) const {
-
 	if (const Shader *shader = Object::cast_to<Shader>(*p_resource)) {
 		if (shader->is_text_shader()) {
+			p_extensions->push_back("gdshader");
 			p_extensions->push_back("shader");
 		}
 	}
 }
-bool ResourceFormatSaverShader::recognize(const RES &p_resource) const {
 
+bool ResourceFormatSaverShader::recognize(const RES &p_resource) const {
 	return p_resource->get_class_name() == "Shader"; //only shader, not inherited
 }
