@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  world_2d.cpp                                                         */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  world_2d.cpp                                                          */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 
 #include "world_2d.h"
 
@@ -34,13 +34,12 @@
 #include "scene/2d/camera_2d.h"
 #include "scene/2d/visibility_notifier_2d.h"
 #include "scene/main/viewport.h"
+#include "servers/navigation_2d_server.h"
 #include "servers/physics_2d_server.h"
 #include "servers/visual_server.h"
 
 struct SpatialIndexer2D {
-
 	struct CellRef {
-
 		int ref;
 
 		_FORCE_INLINE_ int inc() {
@@ -58,7 +57,6 @@ struct SpatialIndexer2D {
 	};
 
 	struct CellKey {
-
 		union {
 			struct {
 				int32_t x;
@@ -74,7 +72,6 @@ struct SpatialIndexer2D {
 	};
 
 	struct CellData {
-
 		Map<VisibilityNotifier2D *, CellRef> notifiers;
 	};
 
@@ -84,7 +81,6 @@ struct SpatialIndexer2D {
 	Map<VisibilityNotifier2D *, Rect2> notifiers;
 
 	struct ViewportData {
-
 		Map<VisibilityNotifier2D *, uint64_t> notifiers;
 		Rect2 rect;
 	};
@@ -96,30 +92,25 @@ struct SpatialIndexer2D {
 	uint64_t pass;
 
 	void _notifier_update_cells(VisibilityNotifier2D *p_notifier, const Rect2 &p_rect, bool p_add) {
-
 		Point2i begin = p_rect.position;
 		begin /= cell_size;
 		Point2i end = p_rect.position + p_rect.size;
 		end /= cell_size;
 		for (int i = begin.x; i <= end.x; i++) {
-
 			for (int j = begin.y; j <= end.y; j++) {
-
 				CellKey ck;
 				ck.x = i;
 				ck.y = j;
 				Map<CellKey, CellData>::Element *E = cells.find(ck);
 
 				if (p_add) {
-
-					if (!E)
+					if (!E) {
 						E = cells.insert(ck, CellData());
+					}
 					E->get().notifiers[p_notifier].inc();
 				} else {
-
 					ERR_CONTINUE(!E);
 					if (E->get().notifiers[p_notifier].dec() == 0) {
-
 						E->get().notifiers.erase(p_notifier);
 						if (E->get().notifiers.empty()) {
 							cells.erase(E);
@@ -131,7 +122,6 @@ struct SpatialIndexer2D {
 	}
 
 	void _notifier_add(VisibilityNotifier2D *p_notifier, const Rect2 &p_rect) {
-
 		ERR_FAIL_COND(notifiers.has(p_notifier));
 		notifiers[p_notifier] = p_rect;
 		_notifier_update_cells(p_notifier, p_rect, true);
@@ -139,11 +129,11 @@ struct SpatialIndexer2D {
 	}
 
 	void _notifier_update(VisibilityNotifier2D *p_notifier, const Rect2 &p_rect) {
-
 		Map<VisibilityNotifier2D *, Rect2>::Element *E = notifiers.find(p_notifier);
 		ERR_FAIL_COND(!E);
-		if (E->get() == p_rect)
+		if (E->get() == p_rect) {
 			return;
+		}
 
 		_notifier_update_cells(p_notifier, p_rect, true);
 		_notifier_update_cells(p_notifier, E->get(), false);
@@ -152,7 +142,6 @@ struct SpatialIndexer2D {
 	}
 
 	void _notifier_remove(VisibilityNotifier2D *p_notifier) {
-
 		Map<VisibilityNotifier2D *, Rect2>::Element *E = notifiers.find(p_notifier);
 		ERR_FAIL_COND(!E);
 		_notifier_update_cells(p_notifier, E->get(), false);
@@ -160,7 +149,6 @@ struct SpatialIndexer2D {
 
 		List<Viewport *> removed;
 		for (Map<Viewport *, ViewportData>::Element *F = viewports.front(); F; F = F->next()) {
-
 			Map<VisibilityNotifier2D *, uint64_t>::Element *G = F->get().notifiers.find(p_notifier);
 
 			if (G) {
@@ -170,7 +158,6 @@ struct SpatialIndexer2D {
 		}
 
 		while (!removed.empty()) {
-
 			p_notifier->_exit_viewport(removed.front()->get());
 			removed.pop_front();
 		}
@@ -179,7 +166,6 @@ struct SpatialIndexer2D {
 	}
 
 	void _add_viewport(Viewport *p_viewport, const Rect2 &p_rect) {
-
 		ERR_FAIL_COND(viewports.has(p_viewport));
 		ViewportData vd;
 		vd.rect = p_rect;
@@ -188,11 +174,11 @@ struct SpatialIndexer2D {
 	}
 
 	void _update_viewport(Viewport *p_viewport, const Rect2 &p_rect) {
-
 		Map<Viewport *, ViewportData>::Element *E = viewports.find(p_viewport);
 		ERR_FAIL_COND(!E);
-		if (E->get().rect == p_rect)
+		if (E->get().rect == p_rect) {
 			return;
+		}
 		E->get().rect = p_rect;
 		changed = true;
 	}
@@ -201,7 +187,6 @@ struct SpatialIndexer2D {
 		ERR_FAIL_COND(!viewports.has(p_viewport));
 		List<VisibilityNotifier2D *> removed;
 		for (Map<VisibilityNotifier2D *, uint64_t>::Element *E = viewports[p_viewport].notifiers.front(); E; E = E->next()) {
-
 			removed.push_back(E->key());
 		}
 
@@ -214,12 +199,11 @@ struct SpatialIndexer2D {
 	}
 
 	void _update() {
-
-		if (!changed)
+		if (!changed) {
 			return;
+		}
 
 		for (Map<Viewport *, ViewportData>::Element *E = viewports.front(); E; E = E->next()) {
-
 			Point2i begin = E->get().rect.position;
 			begin /= cell_size;
 			Point2i end = E->get().rect.position + E->get().rect.size;
@@ -231,24 +215,22 @@ struct SpatialIndexer2D {
 			uint64_t visible_cells = (uint64_t)(end.x - begin.x) * (uint64_t)(end.y - begin.y);
 
 			if (visible_cells > 10000) {
-
 				//well you zoomed out a lot, it's your problem. To avoid freezing in the for loops below, we'll manually check cell by cell
 
 				for (Map<CellKey, CellData>::Element *F = cells.front(); F; F = F->next()) {
-
 					const CellKey &ck = F->key();
 
-					if (ck.x < begin.x || ck.x > end.x)
+					if (ck.x < begin.x || ck.x > end.x) {
 						continue;
-					if (ck.y < begin.y || ck.y > end.y)
+					}
+					if (ck.y < begin.y || ck.y > end.y) {
 						continue;
+					}
 
 					//notifiers in cell
 					for (Map<VisibilityNotifier2D *, CellRef>::Element *G = F->get().notifiers.front(); G; G = G->next()) {
-
 						Map<VisibilityNotifier2D *, uint64_t>::Element *H = E->get().notifiers.find(G->key());
 						if (!H) {
-
 							H = E->get().notifiers.insert(G->key(), pass);
 							added.push_back(G->key());
 						} else {
@@ -258,12 +240,9 @@ struct SpatialIndexer2D {
 				}
 
 			} else {
-
 				//check cells in grid fashion
 				for (int i = begin.x; i <= end.x; i++) {
-
 					for (int j = begin.y; j <= end.y; j++) {
-
 						CellKey ck;
 						ck.x = i;
 						ck.y = j;
@@ -275,10 +254,8 @@ struct SpatialIndexer2D {
 
 						//notifiers in cell
 						for (Map<VisibilityNotifier2D *, CellRef>::Element *G = F->get().notifiers.front(); G; G = G->next()) {
-
 							Map<VisibilityNotifier2D *, uint64_t>::Element *H = E->get().notifiers.find(G->key());
 							if (!H) {
-
 								H = E->get().notifiers.insert(G->key(), pass);
 								added.push_back(G->key());
 							} else {
@@ -290,9 +267,9 @@ struct SpatialIndexer2D {
 			}
 
 			for (Map<VisibilityNotifier2D *, uint64_t>::Element *F = E->get().notifiers.front(); F; F = F->next()) {
-
-				if (F->get() != pass)
+				if (F->get() != pass) {
 					removed.push_back(F->key());
+				}
 			}
 
 			while (!added.empty()) {
@@ -311,7 +288,6 @@ struct SpatialIndexer2D {
 	}
 
 	SpatialIndexer2D() {
-
 		pass = 0;
 		changed = false;
 		cell_size = GLOBAL_DEF("world/2d/cell_size", 100);
@@ -319,75 +295,68 @@ struct SpatialIndexer2D {
 };
 
 void World2D::_register_viewport(Viewport *p_viewport, const Rect2 &p_rect) {
-
 	indexer->_add_viewport(p_viewport, p_rect);
 }
 
 void World2D::_update_viewport(Viewport *p_viewport, const Rect2 &p_rect) {
-
 	indexer->_update_viewport(p_viewport, p_rect);
 }
 void World2D::_remove_viewport(Viewport *p_viewport) {
-
 	indexer->_remove_viewport(p_viewport);
 }
 
 void World2D::_register_notifier(VisibilityNotifier2D *p_notifier, const Rect2 &p_rect) {
-
 	indexer->_notifier_add(p_notifier, p_rect);
 }
 void World2D::_update_notifier(VisibilityNotifier2D *p_notifier, const Rect2 &p_rect) {
-
 	indexer->_notifier_update(p_notifier, p_rect);
 }
 void World2D::_remove_notifier(VisibilityNotifier2D *p_notifier) {
-
 	indexer->_notifier_remove(p_notifier);
 }
 
 void World2D::_update() {
-
 	indexer->_update();
 }
 
 RID World2D::get_canvas() {
-
 	return canvas;
 }
 
 RID World2D::get_space() {
-
 	return space;
 }
 
-void World2D::get_viewport_list(List<Viewport *> *r_viewports) {
+RID World2D::get_navigation_map() const {
+	return navigation_map;
+}
 
+void World2D::get_viewport_list(List<Viewport *> *r_viewports) {
 	for (Map<Viewport *, SpatialIndexer2D::ViewportData>::Element *E = indexer->viewports.front(); E; E = E->next()) {
 		r_viewports->push_back(E->key());
 	}
 }
 
 void World2D::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("get_canvas"), &World2D::get_canvas);
 	ClassDB::bind_method(D_METHOD("get_space"), &World2D::get_space);
+	ClassDB::bind_method(D_METHOD("get_navigation_map"), &World2D::get_navigation_map);
 
 	ClassDB::bind_method(D_METHOD("get_direct_space_state"), &World2D::get_direct_space_state);
 
 	ADD_PROPERTY(PropertyInfo(Variant::_RID, "canvas", PROPERTY_HINT_NONE, "", 0), "", "get_canvas");
 	ADD_PROPERTY(PropertyInfo(Variant::_RID, "space", PROPERTY_HINT_NONE, "", 0), "", "get_space");
+	ADD_PROPERTY(PropertyInfo(Variant::_RID, "navigation_map", PROPERTY_HINT_NONE, "", 0), "", "get_navigation_map");
 	ADD_PROPERTY(PropertyInfo(Variant::OBJECT, "direct_space_state", PROPERTY_HINT_RESOURCE_TYPE, "Physics2DDirectSpaceState", 0), "", "get_direct_space_state");
 }
 
 Physics2DDirectSpaceState *World2D::get_direct_space_state() {
-
 	return Physics2DServer::get_singleton()->space_get_direct_state(space);
 }
 
 World2D::World2D() {
-
-	canvas = VisualServer::get_singleton()->canvas_create();
-	space = Physics2DServer::get_singleton()->space_create();
+	canvas = RID_PRIME(VisualServer::get_singleton()->canvas_create());
+	space = RID_PRIME(Physics2DServer::get_singleton()->space_create());
 
 	//set space2D to be more friendly with pixels than meters, by adjusting some constants
 	Physics2DServer::get_singleton()->space_set_active(space, true);
@@ -397,12 +366,20 @@ World2D::World2D() {
 	ProjectSettings::get_singleton()->set_custom_property_info("physics/2d/default_linear_damp", PropertyInfo(Variant::REAL, "physics/2d/default_linear_damp", PROPERTY_HINT_RANGE, "-1,100,0.001,or_greater"));
 	Physics2DServer::get_singleton()->area_set_param(space, Physics2DServer::AREA_PARAM_ANGULAR_DAMP, GLOBAL_DEF("physics/2d/default_angular_damp", 1.0));
 	ProjectSettings::get_singleton()->set_custom_property_info("physics/2d/default_angular_damp", PropertyInfo(Variant::REAL, "physics/2d/default_angular_damp", PROPERTY_HINT_RANGE, "-1,100,0.001,or_greater"));
+
+	// Create default navigation map
+	navigation_map = Navigation2DServer::get_singleton()->map_create();
+	Navigation2DServer::get_singleton()->map_set_active(navigation_map, true);
+	Navigation2DServer::get_singleton()->map_set_cell_size(navigation_map, GLOBAL_DEF("navigation/2d/default_cell_size", 1.0));
+	Navigation2DServer::get_singleton()->map_set_cell_height(navigation_map, GLOBAL_DEF("navigation/2d/default_cell_height", 1.0));
+	Navigation2DServer::get_singleton()->map_set_edge_connection_margin(navigation_map, GLOBAL_DEF("navigation/2d/default_edge_connection_margin", 1.0));
+
 	indexer = memnew(SpatialIndexer2D);
 }
 
 World2D::~World2D() {
-
 	VisualServer::get_singleton()->free(canvas);
 	Physics2DServer::get_singleton()->free(space);
+	Navigation2DServer::get_singleton()->free(navigation_map);
 	memdelete(indexer);
 }
