@@ -1,32 +1,32 @@
-/*************************************************************************/
-/*  godot.cpp                                                            */
-/*************************************************************************/
-/*                       This file is part of:                           */
-/*                           GODOT ENGINE                                */
-/*                      https://godotengine.org                          */
-/*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
-/*                                                                       */
-/* Permission is hereby granted, free of charge, to any person obtaining */
-/* a copy of this software and associated documentation files (the       */
-/* "Software"), to deal in the Software without restriction, including   */
-/* without limitation the rights to use, copy, modify, merge, publish,   */
-/* distribute, sublicense, and/or sell copies of the Software, and to    */
-/* permit persons to whom the Software is furnished to do so, subject to */
-/* the following conditions:                                             */
-/*                                                                       */
-/* The above copyright notice and this permission notice shall be        */
-/* included in all copies or substantial portions of the Software.       */
-/*                                                                       */
-/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       */
-/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    */
-/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*/
-/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY  */
-/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,  */
-/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE     */
-/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                */
-/*************************************************************************/
+/**************************************************************************/
+/*  godot.cpp                                                             */
+/**************************************************************************/
+/*                         This file is part of:                          */
+/*                             GODOT ENGINE                               */
+/*                        https://godotengine.org                         */
+/**************************************************************************/
+/* Copyright (c) 2014-present Godot Engine contributors (see AUTHORS.md). */
+/* Copyright (c) 2007-2014 Juan Linietsky, Ariel Manzur.                  */
+/*                                                                        */
+/* Permission is hereby granted, free of charge, to any person obtaining  */
+/* a copy of this software and associated documentation files (the        */
+/* "Software"), to deal in the Software without restriction, including    */
+/* without limitation the rights to use, copy, modify, merge, publish,    */
+/* distribute, sublicense, and/or sell copies of the Software, and to     */
+/* permit persons to whom the Software is furnished to do so, subject to  */
+/* the following conditions:                                              */
+/*                                                                        */
+/* The above copyright notice and this permission notice shall be         */
+/* included in all copies or substantial portions of the Software.        */
+/*                                                                        */
+/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,        */
+/* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF     */
+/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. */
+/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY   */
+/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   */
+/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE      */
+/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
+/**************************************************************************/
 /**
  @file  godot.cpp
  @brief ENet Godot specific functions
@@ -95,8 +95,9 @@ public:
 
 	Error recvfrom(uint8_t *p_buffer, int p_len, int &r_read, IP_Address &r_ip, uint16_t &r_port) {
 		Error err = sock->poll(NetSocket::POLL_TYPE_IN, 0);
-		if (err != OK)
+		if (err != OK) {
 			return err;
+}
 		return sock->recvfrom(p_buffer, p_len, r_read, r_ip, r_port);
 	}
 
@@ -186,32 +187,36 @@ public:
 			connected = true;
 		}
 		dtls->poll();
-		if (dtls->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING)
+		if (dtls->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING) {
 			return ERR_BUSY;
-		else if (dtls->get_status() != PacketPeerDTLS::STATUS_CONNECTED)
+		} else if (dtls->get_status() != PacketPeerDTLS::STATUS_CONNECTED) {
 			return FAILED;
+}
 		r_sent = p_len;
 		return dtls->put_packet(p_buffer, p_len);
 	}
 
 	Error recvfrom(uint8_t *p_buffer, int p_len, int &r_read, IP_Address &r_ip, uint16_t &r_port) {
 		dtls->poll();
-		if (dtls->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING)
+		if (dtls->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING) {
 			return ERR_BUSY;
-		if (dtls->get_status() != PacketPeerDTLS::STATUS_CONNECTED)
+}
+		if (dtls->get_status() != PacketPeerDTLS::STATUS_CONNECTED) {
 			return FAILED;
+}
 		int pc = dtls->get_available_packet_count();
-		if (pc == 0)
+		if (pc == 0) {
 			return ERR_BUSY;
-		else if (pc < 0)
+		} else if (pc < 0) {
 			return FAILED;
+}
 
 		const uint8_t *buffer;
 		Error err = dtls->get_packet(&buffer, r_read);
 		ERR_FAIL_COND_V(err != OK, err);
 		ERR_FAIL_COND_V(p_len < r_read, ERR_OUT_OF_MEMORY);
 
-		copymem(p_buffer, buffer, r_read);
+		memcpy(p_buffer, buffer, r_read);
 		r_ip = udp->get_packet_address();
 		r_port = udp->get_packet_port();
 		return err;
@@ -264,12 +269,13 @@ public:
 		ERR_FAIL_COND_V(!peers.has(key), ERR_UNAVAILABLE);
 		Ref<PacketPeerDTLS> peer = peers[key];
 		Error err = peer->put_packet(p_buffer, p_len);
-		if (err == OK)
+		if (err == OK) {
 			r_sent = p_len;
-		else if (err == ERR_BUSY)
+		} else if (err == ERR_BUSY) {
 			r_sent = 0;
-		else
+		} else {
 			r_sent = -1;
+}
 		return err;
 	}
 
@@ -295,9 +301,9 @@ public:
 			Ref<PacketPeerDTLS> peer = E->get();
 			peer->poll();
 
-			if (peer->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING)
+			if (peer->get_status() == PacketPeerDTLS::STATUS_HANDSHAKING) {
 				continue;
-			else if (peer->get_status() != PacketPeerDTLS::STATUS_CONNECTED) {
+			} else if (peer->get_status() != PacketPeerDTLS::STATUS_CONNECTED) {
 				// Peer disconnected, removing it.
 				remove.push_back(E->key());
 				continue;
@@ -316,7 +322,7 @@ public:
 				Vector<String> s = E->key().rsplit(":", false, 1);
 				ERR_CONTINUE(s.size() != 2); // BUG!
 
-				copymem(p_buffer, buffer, r_read);
+				memcpy(p_buffer, buffer, r_read);
 				r_ip = s[0];
 				r_port = s[1].to_int();
 				break; // err = OK
@@ -444,7 +450,7 @@ void enet_socket_destroy(ENetSocket socket) {
 
 int enet_socket_send(ENetSocket socket, const ENetAddress *address, const ENetBuffer *buffers, size_t bufferCount) {
 
-	ERR_FAIL_COND_V(address == NULL, -1);
+	ERR_FAIL_COND_V(address == nullptr, -1);
 
 	ENetGodotSocket *sock = (ENetGodotSocket *)socket;
 	IP_Address dest;
@@ -494,11 +500,13 @@ int enet_socket_receive(ENetSocket socket, ENetAddress *address, ENetBuffer *buf
 	IP_Address ip;
 
 	Error err = sock->recvfrom((uint8_t *)buffers[0].data, buffers[0].dataLength, read, ip, address->port);
-	if (err == ERR_BUSY)
+	if (err == ERR_BUSY) {
 		return 0;
+}
 
-	if (err != OK)
+	if (err != OK) {
 		return -1;
+}
 
 	enet_address_set_ip(address, ip.get_ipv6(), 16);
 
@@ -544,7 +552,7 @@ int enet_socket_connect(ENetSocket socket, const ENetAddress *address) {
 
 ENetSocket enet_socket_accept(ENetSocket socket, ENetAddress *address) {
 
-	return NULL;
+	return nullptr;
 }
 
 int enet_socket_shutdown(ENetSocket socket, ENetSocketShutdown how) {
